@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -570,15 +570,17 @@ class Field:
             self.writePermission = wp
         else:
             self.writePermission = 'write'
-        if (self.type == 'Ref') and not self.isBack and self.back:
-            # We must initialise the corresponding back reference
-            self.back.klass = klass
-            self.back.init(self.back.attribute, self.klass, appName)
-        if self.type in ('List', 'Dict'):
-            for subName, subField in self.fields:
-                fullName = '%s_%s' % (name, subName)
-                subField.init(fullName, klass, appName)
-                subField.name = '%s*%s' % (name, subName)
+        # Field-specific lazy initialisation
+        if self.type == 'Ref':
+            if not self.isBack and self.back:
+                # Initialise the corresponding back reference
+                self.back.klass = klass
+                self.back.init(self.back.attribute, self.klass, appName)
+        elif self.type in ('List', 'Dict'):
+            subs = self.fields
+            if not callable(subs):
+                # Lazy-initialise sub-fields
+                self.lazyInitSubFields(subs, klass, appName)
         elif self.type == 'Switch':
             for case, fields in self.fields:
                 for name, field in fields:
@@ -850,7 +852,7 @@ class Field:
            default field layouts when they are not available.'''
         return self.getDefaultLayouts() or Layouts.defaults(self)
 
-    def getCss(self, layoutType, res, config):
+    def getCss(self, obj, layoutType, res, config):
         '''This method completes the list p_res with the names of CSS files
            that are required for displaying widgets of self's type on a given
            p_layoutType. p_res is not a set because order of inclusion of CSS
@@ -860,7 +862,7 @@ class Field:
                 if fileName not in res:
                     res.append(fileName)
 
-    def getJs(self, layoutType, res, config):
+    def getJs(self, obj, layoutType, res, config):
         '''This method completes the list p_res with the names of Javascript
            files that are required for displaying widgets of self's type on a
            given p_layoutType. p_res is not a set because order of inclusion of
@@ -1341,10 +1343,16 @@ class Field:
         tool.log(SEARCH_DONE % len(r))
         return r
 
-    def keepCheckedResults(self, req, objects):
+    def keepCheckedResults(self, req, objects, unchecked=None):
         '''Among p_objects as retrieved via m_getSearchResults, keep only those
            being checked, according to m_getCheckedInfo.'''
-        ids, unchecked = self.getCheckedInfo(req)
+        # If p_unchecked is not None, we already know the IDs of the objects
+        # being unchecked.
+        if unchecked:
+            ids = unchecked
+            unchecked = True
+        else:
+            ids, unchecked = self.getCheckedInfo(req)
         i = len(objects) - 1
         # Remove, from search results, unchecked objects
         while i >= 0:
@@ -1463,11 +1471,15 @@ class Field:
             # Retrieve the outer, inner fields and row number
             name, px = rq['px'].rsplit(':', 1)
             outer, inner, i = name.split('*')
+            i = int(i)
             outer = obj.getAppyType(outer)
-            # Update the value
+            # Update the value. Clone the row to ensure the ZODB will detect the
+            # object change. Indeed, rows are instances of not-persistent class
+            # appy.Object.
             value = outer.getValue(obj)
-            row = value[int(i)]
+            row = value[i]
             setattr(row, inner, requestValue)
+            value[i] = row.clone()
             # Store the complete value again on p_obj
             setattr(obj.aq_base, outer.name, value)
         # Update obj's last modification date
