@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -53,6 +53,18 @@ class ActionInitiator(Initiator):
         if self.field.result == 'redirect':
             self.backFromPopupUrl = msg
 
+# ------------------------------------------------------------------------------
+class Multi:
+    '''Management of muti-actions = actions that execute on a set of objects,
+       and not on a single one.'''
+
+    def __init__(self, back, check):
+        # The DOM node to ajax-refresh once the multi-action has been executed
+        self.back = back
+        # The DOM node storing the un/checked objects onto which the
+        # multi-action must be executed.
+        self.check = check
+
 # ------------------------------------------------------------------------------
 class Action(Field):
     '''An action is a Python method that can be triggered by the user on a
@@ -68,8 +80,8 @@ class Action(Field):
     pxView = pxCell = Px('''
      <form var="isFake=field.isFake(zobj, _);
                 formId='%s_%s_form' % (zobj.id, name);
-                label=_(field.labelId);
-                multi=multi|False;
+                label=_('label', field=field);
+                multi=multi|None;
                 className=className|zobj.portal_type;
                 inputTitle=field.getInputTitle(zobj, label);
                 inputLabel=field.getInputLabel(label, layoutType);
@@ -314,18 +326,19 @@ class Action(Field):
         else:
             # Submitting the form will really trigger the action
             return '%s/onExecuteAction' % zobj.absolute_url()
-    
+
     def getOnClick(self, zobj, name, req, layoutType, q, multi):
         '''Gets the JS code to execute when the action button is clicked'''
         # Determine the ID of the form to submit
         formId = '%s_%s_form' % (zobj.id, name)
         # Determine the back hook and check hook (if multi)
         if multi:
-            back = 'queryResult'
-            check = q(req.search)
+            back = multi.back
+            check = multi.check
         else:
             back = (layoutType == 'cell') and zobj.id or None
-            check = 'null'
+            check = None
+        check = check and q(check) or 'null'
         if not self.options:
             # Determine the parameters for executing the action
             showComment = (self.confirm == 'comment') and 'true' or 'false'
@@ -347,6 +360,32 @@ class Action(Field):
                  (target.onClick, q(formId), check)
         return js
 
+    def getTargetObjects(self, obj, req):
+        '''In the context of a multi-action, retrieve the list of target objects
+           onto which the action must be applied.'''
+        # If semantics indicates that checked objects must be kept, retrieve
+        # these objects.
+        tool = obj.tool
+        ids, unchecked = self.getCheckedInfo(req)
+        if not unchecked:
+            r = []
+            for id in ids:
+                cobj = tool.getObject(id)
+                if cobj: # The object may have been deleted in the meanwhile
+                    r.append(cobj)
+            return r
+        # Get all the potential target objects, from a Search or Ref
+        searchParams = req['searchParams']
+        if searchParams:
+            # Re-trigger the search
+            r = self.getSearchResults(tool.o, req['searchParams'])
+        else:
+            # Get objets from a ref
+            r = getattr(obj, req['fieldName'].split('*',1)[0])
+        # Remove those not being checked in the UI
+        self.keepCheckedResults(req, r, unchecked=ids)
+        return r
+
     def __call__(self, obj, options=None):
         '''Calls the action on p_obj. Returns a tuple (b_success, s_message)'''
         # Get args to give to method(s) in self.action
@@ -359,12 +398,7 @@ class Action(Field):
         if self.confirm == 'comment':
             args['comment'] = req.get('popupComment')
         if multi:
-            # Re-trigger the search to get the list of objects
-            tool = obj.o.getTool()
-            objects = self.getSearchResults(tool, req['searchParams'])
-            # Remove those not being checked in the UI
-            self.keepCheckedResults(req, objects)
-            args['objects'] = objects
+            args['objects'] = self.getTargetObjects(obj, req)
         # Call method(s) in self.action
         if type(self.action) in sutils.sequenceTypes:
             # There are multiple methods
@@ -488,4 +522,10 @@ class Action(Field):
                 obj.setMessageCookie()
             else:
                 return obj.goto(msg)
+
+    def getMulti(self, back, check):
+        '''In the context of a multi-action, create and return a Multi instance
+           defining useful info about the action, like the back and check
+           hooks.'''
+        return Multi(back, check)
 # ------------------------------------------------------------------------------
