@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -228,13 +228,13 @@ class Date(Field):
           mapping, generateLabel, label, sdefault, scolspan, swidth, sheight,
           persist, False, view, cell, edit, xml, translations)
 
-    def getCss(self, layoutType, res, config):
+    def getCss(self, obj, layoutType, res, config):
         # CSS files are only required if the calendar must be shown
-        if self.calendar: Field.getCss(self, layoutType, res, config)
+        if self.calendar: Field.getCss(self, obj, layoutType, res, config)
 
-    def getJs(self, layoutType, res, config):
+    def getJs(self, obj, layoutType, res, config):
         # Javascript files are only required if the calendar must be shown
-        if self.calendar: Field.getJs(self, layoutType, res, config)
+        if self.calendar: Field.getJs(self, obj, layoutType, res, config)
 
     def getSelectableYears(self, obj):
         '''Gets the list of years one may select for this field'''
