@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -113,6 +113,7 @@ class List(Field):
                  tableId='list_%s' % name" if="isEdit or value"
             id=":tableId" width=":field.width" class="grid"
             var2="subFields=field.getSubFields(zobj, layoutType);
+                  swidths=field.getWidths(subFields);
                   totals=field.createTotals(isEdit);
                   showTotals=not isEdit and totals">
       <!-- Header -->
@@ -120,7 +121,7 @@ class List(Field):
        <tr valign="middle">
         <th if="showTotals"></th>
         <th for="name, sub in subFields" if="sub"
-            width=":field.widths[loop.name.nb]">
+            width=":swidths[loop.name.nb]">
          <x var="field=sub">::field.getListHeader(_ctx_)</x></th>
         <!-- Icon for adding a new row -->
         <th if="isEdit">
@@ -165,29 +166,48 @@ class List(Field):
          edit, xml, translations)
         self.validable = True
         # Tuple of elements of the form (name, Field instance) determining the
-        # format of every element in the list.
+        # format of every element in the list. It can also be a method returning
+        # this tuple. In that case, however, no i18n label will be generated for
+        # the sub-fields: these latters must be created with a value for
+        # attribute "translations".
         self.fields = fields
-        # Force some layouting for sub-fields, if subLayouts are given. So the
-        # one who wants freedom on tuning layouts at the field level must
-        # specify subLayouts=None.
-        if subLayouts:
-            for name, field in self.fields:
-                field.layouts = field.formatLayouts(subLayouts)
+        # Force some layouting for sub-fields, if p_subLayouts are passed. If
+        # you want freedom to tune layouts at the field level, you must specify
+        # p_subLayouts=None.
+        self.subLayouts = subLayouts
         # One may specify the width of every column in the list. Indeed, using
-        # widths and layouts of sub-fields may not be sufficient.
-        self.computeWidths(widths)
+        # widths and layouts at the sub-field level may not be sufficient.
+        self.widths = widths
+        # Initialize sub-fields, if statically known
+        if not callable(fields):
+            self.initSubFields(fields)
         # When deleting a row, must we display a popup for confirming it ?
         self.deleteConfirm = deleteConfirm
         # If you want to specify additional rows representing totals, give in
         # "totalRows" a list of Totals instances (see above).
         self.totalRows = totalRows or []
 
-    def computeWidths(self, widths):
-        '''Set given p_widths or compute default ones if not given'''
-        if not widths:
-            self.widths = [''] * len(self.fields)
-        else:
-            self.widths = widths
+    def initSubFields(self, subFields):
+        '''Initialises sub-fields' attributes'''
+        # Initialise sub-layouts
+        subLayouts = self.subLayouts
+        if subLayouts:
+            for name, field in subFields:
+                field.layouts = field.formatLayouts(subLayouts)
+
+    def lazyInitSubFields(self, subFields, klass, appName):
+        '''Lazy-initialise sub-fields, when known'''
+        # It can be at server startup, if sub-fields are statically defined, or
+        # everytime a List field is solicited, if sub-fields are dynamically
+        # computed.
+        for subName, subField in subFields:
+            fullName = '%s_%s' % (self.name, subName)
+            subField.init(fullName, klass, appName)
+            subField.name = '%s*%s' % (self.name, subName)
+
+    def getWidths(self, subFields):
+        '''Get the widths to appply to these p_subFields'''
+        return self.widths or [''] * len(subFields)
 
     def getDefaultLayouts(self):
         '''Returns default layouts for a List field'''
@@ -199,18 +219,30 @@ class List(Field):
         for n, field in self.fields:
             if n == name: return field
 
-    def getSubFields(self, obj, layoutType):
-        '''Returns the sub-fields (name, Field) that are showable among
-           field.fields on the given p_layoutType. Fields that cannot appear in
-           the result are nevertheless present as a tuple (name, None). This
-           way, it keeps a nice layouting of the table.'''
-        res = []
-        for n, field in self.fields:
-            elem = (n, None)
+    def getSubFields(self, obj, layoutType=None):
+        '''Returns the sub-fields, as a list of tuples ~[(s_name, Field)]~,
+           being showable, among p_self.fields on this p_layoutType. Fields that
+           cannot appear in the result are nevertheless present as a tuple
+           (s_name, None). This way, it keeps a nice layouting of the table.'''
+        fields = self.fields
+        if callable(fields):
+            # Dynamically computed fields: get and completely initialise them
+            aobj = obj.appy()
+            fields = self.getAttribute(aobj, 'fields') # Involves caching
+            self.initSubFields(fields)
+            self.lazyInitSubFields(fields, aobj.klass,
+                                   obj.getTool().getAppName())
+        # Stop here if no p_layoutType is passed
+        if layoutType is None: return fields
+        # Retain only sub-fields being showable on this p_layoutType
+        r = []
+        for name, field in fields:
             if field.isShowable(obj, layoutType):
-                elem = (n, field)
-            res.append(elem)
-        return res
+                elem = (name, field)
+            else:
+                elem = (name, None)
+            r.append(elem)
+        return r
 
     def getRequestValue(self, obj, requestName=None):
         '''Concatenates the list from distinct form elements in the request'''
@@ -225,7 +257,7 @@ class List(Field):
             row = Object()
             rowId = key.split('*')[-1]
             if rowId == '-1': continue # Ignore the template row
-            for subName, subField in self.fields:
+            for subName, subField in self.getSubFields(obj):
                 keyName = '%s*%s*%s' % (name, subName, rowId)
                 if request.has_key(keyName + subField.getRequestSuffix()):
                     v = subField.getRequestValue(obj, requestName=keyName)
@@ -266,7 +298,7 @@ class List(Field):
         '''Gets a ready-to-store Object instance representing a single row,
            from p_requestValue.'''
         res = Object()
-        for name, field in self.fields:
+        for name, field in self.getSubFields(obj):
             if not hasattr(requestValue, name) and \
                not field.isShowable(obj, 'edit'):
                 # Some fields, although present on "edit", may not be part of
@@ -297,15 +329,17 @@ class List(Field):
         r = getattr(obj.aq_base, self.name, None)
         if r: return copy.deepcopy(r)
 
-    def getCss(self, layoutType, res, config):
+    def getCss(self, obj, layoutType, res, config):
         '''Gets the CSS required by sub-fields if any'''
-        for name, field in self.fields:
-            field.getCss(layoutType, res, config)
+        for name, field in self.getSubFields(obj, layoutType):
+            if field:
+                field.getCss(obj, layoutType, res, config)
 
-    def getJs(self, layoutType, res, config):
+    def getJs(self, obj, layoutType, res, config):
         '''Gets the JS required by sub-fields if any'''
-        for name, field in self.fields:
-            field.getJs(layoutType, res, config)
+        for name, field in self.getSubFields(obj, layoutType):
+            if field:
+                field.getJs(obj, layoutType, res, config)
 
     def jsDeleteConfirm(self, q, tableId):
         '''Gets the JS code to call when the user wants to delete a row'''
@@ -317,7 +351,7 @@ class List(Field):
         i = -1
         for row in value:
             i += 1
-            for name, subField in self.fields:
+            for name, subField in self.getSubFields(obj):
                 message = subField.validate(obj, getattr(row, name, None))
                 if message:
                     setattr(errors, '%s*%d' % (subField.name, i), message)
