@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -216,7 +216,7 @@ class Pod(Field):
       downloadDisposition='attachment', forceOoCall=False,
       optimalColumnWidths=False, distributeColumns=None, script=None,
       pdfOptions='ExportNotes=True', tabbedCR=False, fonts=None, confirm=False,
-      raiseOnError=False, action=None, beforeAction=None):
+      raiseOnError=False, action=None, beforeAction=None, multiObjects=False):
         # Param "template" stores the path to the pod template(s). If there is
         # a single template, a string is expected. Else, a list or tuple of
         # strings is expected. Every such path must be relative to your
@@ -430,6 +430,14 @@ class Pod(Field):
         # generated, set a method in parameter "beforeAction". This method's
         # signature must be the same as for parameter "action" hereabove.
         self.beforeAction = beforeAction
+        # If p_multiObjects is True, this POD will not be produced, based on a
+        # single object, but on multiple objects, being the result of some
+        # search. In that case, attribute "show" must be (or evaluate to)
+        # 'query' in order to be taken into account. Indeed, such PODs have only
+        # sense on the 'query' layout, visible on search results. Note that is
+        # is not sufficient to define show='query': you must explicitly set
+        # multiObjects=True, too.
+        self.multiObjects = multiObjects
         # Call the base constructor
         Field.__init__(self, None, (0,1), None, None, show, page, group,
           layouts, move, False, True, None, False, specificReadPermission,
