@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -36,7 +36,7 @@ class Selection:
     '''If you want to have dynamically computed possible values for a Select
        field, use a Selection instance.'''
 
-    def __init__(self, methodName):
+    def __init__(self, methodName, single=None):
         # p_methodName must be the name of a method that will be called every
         # time Appy will need to get the list of possible values for the related
         # field. It must correspond to an instance method of the class defining
@@ -47,8 +47,23 @@ class Selection:
         # method to produce an i18n version of "text" if needed.
         self.methodName = methodName
 
+        # While the method whose name is p_methodName is called on edit and
+        # filter layouts to list the possible values to select, the method you
+        # specify in p_single allows to produce the translated text for a single
+        # value. It accepts a single arg, being one of the values as stored by
+        # the select field, and must return its translated text. If you don't
+        # specify a p_single method, p_methodName will be used instead, but
+        # useless processing will take place, because p_method produces texts
+        # for the complete list of possible values. Moreover, a stored select
+        # value may not be among the list of values as returned by p_method
+        # anymore.
+        self.single = single
+
     def getText(self, obj, value, field, language=None):
         '''Gets the text that corresponds to p_value'''
+        # Use method p_single if specified
+        if self.single: return self.single(obj.appy(), value)
+        # Use p_self.methodName
         if language:
             withTranslations = language
         else:
@@ -97,7 +112,7 @@ class Select(Field):
               title=":text">:ztool.truncateValue(text, charsWidth)</option>
      </select>
      <x if="not isSelect">
-      <div for="val, text in possibleValues">
+      <div for="val, text in possibleValues" class="flex1">
        <input type=":field.render" name=":name" id=":val" value=":val"
               class=":masterCss" onchange=":field.getOnChange(zobj, layoutType)"
               checked=":field.isSelected(zobj, name, val, rawValue)"/>
@@ -305,7 +320,10 @@ class Select(Field):
                                                        complete=True)
             elif not className:
                 # ... or from the database if we are editing an object
-                masterValues = master.getValue(obj)
+                if master.type == 'Ref':
+                    masterValues = master.getValue(obj, noListIfSingleObj=True)
+                else:
+                    masterValues = master.getValue(obj)
             else:
                 # We don't have any master value
                 masterValues = None
