@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -23,9 +23,8 @@ from appy.shared.diff import HtmlDiff
 from appy.shared.data import countries
 from appy.shared import utils as sutils
 from appy.fields.select import Selection
-from appy.shared.xml_parser import Escape
-from appy.shared.xml_parser import XhtmlCleaner
 from appy.gen.indexer import XhtmlTextExtractor, ValidValue
+from appy.shared.xml_parser import Escape,XmlParser,XhtmlCleaner,StringCleaner
 
 # ------------------------------------------------------------------------------
 digit  = re.compile('[0-9]')
@@ -40,6 +39,7 @@ NOT_IMPLEMENTED = 'This feature is not implemented yet'
 WRONG_READONLY = 'A read-only field must have a format being LINE.'
 SELECT_MULTILINGUAL = "A selection field can't be multilingual."
 PASSWORD_MULTILINGUAL = "A password or captcha field can't be multilingual."
+UNPARSABLE = 'Unparsable: %s'
 
 # ------------------------------------------------------------------------------
 class String(Field):
@@ -820,6 +820,25 @@ class String(Field):
                                        raiseOnError=False)
         return extractor.parse('<p>%s</p>' % value)
 
+    def repair(self, obj):
+        '''Checks whether this XHTML p_value is SAX-parsable. If not, try to
+           remove illegal chars. The method returns True if the field content
+           has been repaired.'''
+        value = self.getValue(obj)
+        if not value: return
+        if not StringCleaner.isParsable(value):
+            # Remove illegal chars
+            new = StringCleaner.clean(value)
+            if not StringCleaner.isParsable(new):
+                # Still unparsable... try adding an ending p
+                anew = new.strip() + '</p>'
+                if not StringCleaner.isParsable(anew):
+                    raise Exception(UNPARSABLE % new)
+                else:
+                    new = anew
+            setattr(obj, self.name, new)
+            return True
+
     def getIndexValue(self, obj, forSearch=False):
         '''Pure text must be extracted from rich content; multilingual content
            must be concatenated.'''
@@ -1099,8 +1118,8 @@ class String(Field):
             return 'XhtmlIndex'
         return Field.getIndexType(self)
 
-    def getJs(self, layoutType, res, config):
-        if (self.format == String.XHTML) and (layoutType in ('edit', 'view')):
+    def getJs(self, obj, layoutType, res, config):
+        if self.format == String.XHTML and layoutType in ('edit', 'view'):
             # Compute the URL to ckeditor CDN
             ckUrl = String.cdnUrl % (config.ckVersion, config.ckDistribution)
             if ckUrl not in res: res.append(ckUrl)
