@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -14,11 +14,12 @@
 # You should have received a copy of the GNU General Public License along with
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
-import types, string, copy
+import copy
+
 from group import Group
 from appy.px import Px
-from appy.shared.utils import sequenceTypes
 from appy.gen.utils import User
+from appy.shared.utils import sequenceTypes
 
 # Default Appy permissions -----------------------------------------------------
 r, w, d = ('read', 'write', 'delete')
@@ -106,6 +107,10 @@ class State:
             elif isinstance(roles, dict):
                 for name, role in roles.iteritems():
                     roles[name] = self.getRole(role)
+            elif callable(roles):
+                # v_roles is a method: it is not possible to standardize it. It
+                # will be called at check time.
+                pass
             else:
                 # "roles" is a list or tuple, or None (nobody may have this
                 # permission).
@@ -127,7 +132,9 @@ class State:
         if type(roles) not in sequenceTypes: roles = (roles,)
         if isinstance(permissions, basestring): permissions = (permissions,)
         for perm, existingRoles in self.permissions.iteritems():
-            if permissions and (perm not in permissions): continue
+            if permissions and perm not in permissions: continue
+            # It is not possible to add p_roles if v_existingRoles is a method
+            if callable(existingRoles): continue
             for role in roles:
                 # Do nothing if "role" is already among existing roles
                 name = isinstance(role, basestring) and role or role.name
