@@ -333,7 +333,10 @@ class FieldDescriptor:
 
     def walkList(self):
         '''Generate List-specific i18n labels'''
-        for name, field in self.appyType.fields:
+        subFields = self.appyType.fields
+        # Do not generate anything if the list of sub-fields is dynamic
+        if callable(subFields): return
+        for name, field in subFields:
             if not self.mustGenerateLabels(field): continue
             prefix = '%s_%s_%s' % (self.classDescr.name, self.fieldName, name)
             self.generateLabels(field, prefix, forceLabel=True)
