@@ -1262,11 +1262,11 @@ class ToolMixin(BaseMixin):
         login = user.login # May have been transformed by a source
         # Authenticate the user if required. In the case of SSO, authentication
         # has already been performed by the reverse proxy.
-        if authenticate and (place != 'sso'):
-            if (user.state == 'inactive') or \
-               (not user.checkPassword(password)) or \
-               (cfg.authContext and cfg.authContext.isMandatory(tool) and \
-                not ctx and (place != 'basic')):
+        if authenticate and place != 'sso':
+            if user.state == 'inactive' or not user.checkPassword(password) or \
+               (cfg.authContext and \
+                cfg.authContext.isMandatory(tool, authenticate) and \
+                not ctx and place != 'basic'):
                 # Disable the user
                 self._disableUser(req)
                 return
@@ -1445,8 +1445,16 @@ class ToolMixin(BaseMixin):
     def getResultPodFields(self, contentType):
         '''Finds, among fields defined on p_contentType, which ones are Pod
            fields that need to be shown on a page displaying query results.'''
-        return [f for f in self.getAllAppyTypes(contentType) \
-                if (f.type == 'Pod') and (f.show == 'query')]
+        r = []
+        for field in self.getAllAppyTypes(contentType):
+            if field.type != 'Pod' or not field.multiObjects: continue
+            if callable(field.show):
+                show = field.show(self.appy())
+            else:
+                show = field.show
+            if show == 'query':
+                r.append(field)
+        return r
 
     def formatDate(self, date, format=None, withHour=True, language=None):
         '''Returns p_date formatted as specified by p_format, or tool.dateFormat
