@@ -35,6 +35,10 @@ class PageWrapper(AbstractWrapper):
         if self.isEmpty('expression'): return Show.V_
         return self.user.hasRole('Manager')
 
+    def showEditors(self):
+        '''Show field "editors" only to Managers'''
+        return self.user.hasRole('Manager')
+
     def mayView(self):
         '''In addition to the workflow, evaluating p_self.expression, if
            defined, determines p_self's visibility.'''
@@ -49,7 +53,40 @@ class PageWrapper(AbstractWrapper):
             self.log(message, type='error')
             return True
 
+    def linkEditor(self, user, sub=True):
+        '''Set this new editor as p_self's owner'''
+        self.addLocalRole(user.login, 'Owner')
+        if sub:
+            for page in self.pages:
+                page.linkEditor(user)
+
+    def unlinkEditor(self, user, sub=True):
+        '''ungrant to p_user the possibility to edit p_self'''
+        if user.login != self.o.creator:
+            self.deleteLocalRole(user.login, 'Owner')
+        if sub:
+            for page in self.pages:
+                page.unlinkEditor(user)
+
+    def setEditors(self, editors=None):
+        '''Set editors and related local roles on this page (p_self) and
+           sub-pages.'''
+        # Set p_editors if passed
+        if editors is not None:
+            self.editors = editors
+        else:
+            editors = self.editors
+        # Set local roles
+        self.resetLocalRoles()
+        for user in editors:
+            self.linkEditor(user, sub=False)
+        # Update sub-pages, too
+        for page in self.pages:
+            page.setEditors(editors)
+
     def onEdit(self, created):
+        # Set local roles depending on page creator and editors
+        self.setEditors()
         return self._callCustom('onEdit', created)
 
     def getMergedContent(self, level=1):
