@@ -66,6 +66,20 @@ class ToolWrapper(AbstractWrapper):
       <input type="hidden" name="action" value="generate"/>
      </form>''')
 
+    pxPageSelector = Px('''
+      <select var="pages=[p for p in tool.pages if p.o.mayView()]" if="pages"
+              onchange="gotoURL(this)">
+       <option value="">:_('goto_link')</option>
+       <option for="page in pages" value=":page.url"
+               selected=":page == obj">:page.title</option>
+      </select>''',
+
+     js='''
+       function gotoURL(select) {
+         var url = select.value;
+         if (url) goto(url);
+       }''')
+
     pxPageBottom = Px('''
      <script type="text/javascript">:'initSlaves(%s,%s)' % \
                     (q(zobj.absolute_url()), q(layoutType))</script>''')
@@ -566,10 +580,8 @@ class ToolWrapper(AbstractWrapper):
         '''Synchronizes the local User copies with a distant LDAP user base'''
         cfg, context = self.o.getProductConfig(True).getLdap()
         if not cfg: raise Exception('LDAP config not found.')
-        counts = cfg.synchronizeUsers(self, sso=context)
-        msg = 'LDAP users: %d created, %d updated, %d untouched, ' \
-              '%d invalid entries.' % counts
-        return True, msg
+        message = cfg.synchronizeUsers(self, sso=context)
+        return True, message
 
     def showSynchronizeUsers(self):
         '''Show this button only if a LDAP connection exists and is enabled'''
@@ -634,4 +646,15 @@ class ToolWrapper(AbstractWrapper):
             # to the tool.
             r = self.allows('write')
         return r
+
+    def searchAll(self):
+        '''Allows to retrieve search results from a peer site'''
+        if not self.user.hasRole('Manager'): self.raiseUnauthorized()
+        # Retrieve the name of the class for which instances must be searched
+        className = self.request.get('className')
+        if not className:
+            raise Exception('Specify the name of the class in key "className".')
+        sortBy = self.request.get('sortBy') or 'Created'
+        return ['%s/xml' % o.url \
+                for o in self.search(className, noSecurity=True, sortBy=sortBy)]
 # ------------------------------------------------------------------------------
