@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -294,9 +294,11 @@ class FileBuffer(Buffer):
 
 # ------------------------------------------------------------------------------
 class MemoryBuffer(Buffer):
+    '''Buffer whose content is loaded in RAM'''
+
     class Rex:
         '''Regular expressions in use for a memory buffer'''
-        part = '(for|if|else|with|meta-if)\s*(.*)'
+        part = '(for|if|else|with\+|with|meta-if)\s*(.*)'
         action = re.compile('(?:(\w+)\s*\:\s*)?do\s+(\w+)(-)?(?:\s+%s)?' % part)
         subAction = re.compile(part)
         for_ = re.compile('\s*([\w\-_,\s]+)\s+in\s+(.*)')
@@ -304,6 +306,9 @@ class MemoryBuffer(Buffer):
         var_ = re.compile('@?[\w\-_]+')
         from_ = re.compile('from(\+)?\s+(.*)')
 
+    # Existing variants for the "with" statement
+    withVariants = {'with': None, 'with+': None}
+
     def __init__(self, env, parent):
         Buffer.__init__(self, env, parent)
         self.content = u''
@@ -483,8 +488,8 @@ class MemoryBuffer(Buffer):
         # Change buffer position wrt parent
         self.parent.pushSubBuffer(self)
 
-    def addElement(self, elem, elemType='pod'):
-        if elemType == 'pod':
+    def addElement(self, elem, pod=True):
+        if pod:
             elem = PodElement.create(elem)
         self.elements[self.getLength()] = elem
         if isinstance(elem, Cell) or isinstance(elem, Table):
@@ -574,7 +579,9 @@ class MemoryBuffer(Buffer):
         '''Creates an Action instance, depending on p_actionType'''
         if actionType == 'if':
             r = actions.If(statementName, self, subExpr, podElem, minus)
-            if main:
+            # In the case of multi-statements, only the first "if" will be
+            # connectable to an "else" statement.
+            if main or not self.action.getAction('If'):
                 self.env.ifActions.append(r)
                 if r.name:
                     # We must register this action as a named action
@@ -606,9 +613,10 @@ class MemoryBuffer(Buffer):
             iters, subExpr = forRes.groups()
             iters = self._getForIterators(iters)
             r = actions.For(statementName, self, subExpr, podElem, minus, iters)
-        elif actionType == 'with':
-            variables = self._getVariables(subExpr)
-            r = actions.Variables(statementName, self, podElem, minus,variables)
+        elif actionType in self.withVariants:
+            r = actions.Variables(statementName, self, podElem, minus,
+                                  self._getVariables(subExpr),
+                                  lasting=actionType.endswith('+'))
         elif actionType == 'meta-if':
             r = actions.MetaIf(self, subExpr, podElem, minus, statements)
         else:
@@ -629,9 +637,9 @@ class MemoryBuffer(Buffer):
             if not aRes:
                 raise ParsingError(BAD_STATEMENT % main)
             statementName, podElem, minus, actionType, subExpr = aRes.groups()
-            if not (podElem in PodElement.POD_ELEMS):
+            if podElem not in PodElement.POD_ELEMS:
                 raise ParsingError(BAD_ELEMENT % podElem)
-            if minus and (not podElem in PodElement.MINUS_ELEMS):
+            if minus and podElem not in PodElement.MINUS_ELEMS:
                 raise ParsingError(BAD_MINUS % (podElem,PodElement.MINUS_ELEMS))
             # Find the target element in the buffer
             i = self.getIndex(podElem)
@@ -813,7 +821,7 @@ class MemoryBuffer(Buffer):
         return pos + 1
 
     def getStopIndex(self, removeMainElems):
-        '''This method returns the stop index of the buffer part I must dump.'''
+        '''Return the stop index of the buffer part that must be dumped'''
         if removeMainElems:
             ns = self.env.namespaces
             deepestElem = self.action.elem.DEEPEST_TO_REMOVE
@@ -838,8 +846,9 @@ class MemoryBuffer(Buffer):
 
     reTagContent = re.compile('<(?P<p>[\w-]+):(?P<f>[\w-]+)(.*?)>.*</(?P=p):' \
                               '(?P=f)>', re.S)
+
     def evaluate(self, result, context, subElements=True,
-                 removeMainElems=False):
+                 removeMainElems=False, slice=None):
         '''Evaluates this buffer given the current p_context and add the result
            into p_result. With pod, p_result is the root file buffer; with px
            it is a memory buffer.'''
@@ -850,14 +859,17 @@ class MemoryBuffer(Buffer):
             else:
                 g = res.group
                 g1, g2, g3 = g(1), g(2), g(3)
-                if self.env.parser.caller.protection and (g2 == 'table-cell'):
+                if self.env.parser.caller.protection and g2 == 'table-cell':
                     # Attribute "protected" must be removed
                     g3 = g3.replace('table:protected=" "', '')
                 r = '<%s:%s%s></%s:%s>' % (g1, g2, g3, g1, g2)
                 result.write(r)
         else:
             if removeMainElems: self.removeAutomaticExpressions()
-            currentIndex = self.getStartIndex(removeMainElems)
+            if slice:
+                currentIndex = slice[0]
+            else:
+                currentIndex = self.getStartIndex(removeMainElems)
             for index, evalEntry in BufferIterator(self):
                 result.write(self.content[currentIndex:index])
                 currentIndex = index + 1
@@ -886,7 +898,10 @@ class MemoryBuffer(Buffer):
                         evalEntry.action.execute(result, context)
                     else:
                         result.write(evalEntry.content)
-            stopIndex = self.getStopIndex(removeMainElems)
+            if slice:
+                stopIndex = slice[1]
+            else:
+                stopIndex = self.getStopIndex(removeMainElems)
             if currentIndex < (stopIndex-1):
                 result.write(self.content[currentIndex:stopIndex])
 
