@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -34,12 +34,22 @@ FILE_NOT_FOUND = "'%s' does not exist or is not a file."
 PDF_TO_IMG_ERROR = 'A PDF file could not be converted into images. Please ' \
                    'ensure that Ghostscript (gs) is installed on your ' \
                    'system and the "gs" program is in the path.'
-CONVERT_ERROR = 'Program "convert" (imagemagick) must be installed and ' \
-  'in the path for performing this operation. If you wanted to convert a ' \
-  'SVG file into a PNG file, conversion of SVG files must also be enabled. ' \
-  'On Ubuntu: apt-get install librsvg2-bin'
 TO_PDF_ERROR = 'ConvertImporter error while converting a doc to PDF: %s.'
 
+# ------------------------------------------------------------------------------
+class ImageMagick:
+    '''Allows to perform command-line calls to ImageMagick'''
+
+    # The wrapper around any error message
+    error = 'Error while executing ImageMagick command: %s. %s'
+
+    @classmethod
+    def run(class_, command):
+        '''Executes this ImageMagick p_command, passed a list'''
+        out, err = sutils.executeCommand(command)
+        if err: raise Exception(class_.error % (' '.join(command), err))
+        return out
+
 # ------------------------------------------------------------------------------
 class DocImporter:
     '''Base class used for importing external content into a pod template (an
@@ -516,7 +526,7 @@ class ImageImporter(DocImporter):
         # The image has not already been imported: copy it
         if not at.startswith('http'):
             shutil.copy(at, importPath)
-            # Ensure we can modify the image (with imagemagick)
+            # Ensure we can modify the image (with ImageMagick)
             os.chmod(importPath, stat.S_IREAD | stat.S_IWRITE)
             return importPath
         # The image has (maybe) been retrieved from a HTTP GET
@@ -550,16 +560,15 @@ class ImageImporter(DocImporter):
         '''Read, with ImageMagick, EXIF metadata in the image, and apply a
            rotation (still with ImageMagick) if required.'''
         # Indeed, LO will not interpret EXIF metadata when reading the image
-        cmd = ['identify', '-format', '%[EXIF:*]', self.importPath]
-        out, err = sutils.executeCommand(cmd)
-        if err: raise Exception(CONVERT_ERROR)
+        path = self.importPath
+        cmd = ['identify', '-format', '%[EXIF:*]', path]
+        out = ImageMagick.run(cmd)
         if out:
             match = self.orientationRex.search(out)
             if match:
                 code = int(match.group(1))
-                cmd = ['convert', self.importPath,
-                       '-auto-orient', self.importPath]
-                out, err = sutils.executeCommand(cmd)
+                cmd = ['convert', path, '-auto-orient', path]
+                ImageMagick.run(cmd)
 
     def init(self, anchor, wrapInPara, size, sizeUnit, maxWidth, cssAttrs,
              keepRatio, convertOptions):
@@ -597,11 +606,9 @@ class ImageImporter(DocImporter):
                 image = Image(self.importPath, self.format)
                 options = self.convertOptions(image)
             if options:
-                # Ensure we have the right to modify the file@self.importPath
                 cmd = ['convert', self.importPath] + options.split() + \
                       [self.importPath]
-                out, err = sutils.executeCommand(cmd)
-                if err: raise Exception(CONVERT_ERROR)
+                ImageMagick.run(cmd)
                 transformed = True
         # Avoid creating an Image instance twice if no transformation occurred
         if image and not transformed:
@@ -669,9 +676,7 @@ class ImageImporter(DocImporter):
         # In the case of SVG files, perform an image conversion to PNG
         if imagePath.endswith('.svg'):
             newImportPath = os.path.splitext(self.importPath)[0] + '.png'
-            out, err = sutils.executeCommand(['convert', self.importPath,
-                                             newImportPath])
-            if err: raise Exception(CONVERT_ERROR)
+            ImageMagick.run(['convert', self.importPath, newImportPath])
             os.remove(self.importPath)
             self.importPath = newImportPath
             imagePath = os.path.splitext(imagePath)[0] + '.png'
