@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -23,9 +23,10 @@ from appy.pod import PodError
 class PodElement:
     OD_TO_POD = {'p': 'Text', 'h': 'Title', 'list-item': 'Item',
                  'section': 'Section', 'table': 'Table', 'table-row': 'Row',
-                 'table-cell': 'Cell', None: 'Expression', 'frame': 'Frame'}
+                 'table-cell': 'Cell', None: 'Expression', 'frame': 'Frame',
+                 'text': 'Doc'}
     POD_ELEMS = ('text', 'title', 'item', 'section', 'table', 'row', 'cell',
-                 'frame')
+                 'frame', 'doc')
     subTags = []
 
     # Elements for which the '-' operator can be applied
@@ -79,6 +80,14 @@ class Table(PodElement):
 class Frame(PodElement):
     OD = XmlElement('frame', nsUri=ns.NS_DRAW)
 
+class Doc(PodElement):
+    '''Represents the base tag for a complete ODT document'''
+    OD = XmlElement('text', nsUri=ns.NS_OFFICE)
+    # End tag for the main tag representing the document
+    END_TAG = '</office:text>'
+    # End tag for the sub-tag containing sequence declarations
+    SD_END_TAG = '</text:sequence-decls>'
+
 class Expression(PodElement):
     '''Represents a Python expression that is found in a pod or px.'''
     OD = None
