@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -22,14 +22,16 @@ from appy.pod.elements import *
 
 # ------------------------------------------------------------------------------
 class OdTable:
-    '''Informations about the currently parsed Open Document (Od)table.'''
+    '''Informations about the currently parsed Open Document (Od)table'''
+
     def __init__(self):
         self.nbOfColumns = 0
         self.nbOfRows = 0
         self.curColIndex = None
         self.curRowAttrs = None
+
     def isOneCell(self):
-        return (self.nbOfColumns == 1) and (self.nbOfRows == 1)
+        return self.nbOfColumns == 1 and self.nbOfRows == 1
 
 class OdInsert:
     '''While parsing an odt/pod file, we may need to insert a specific odt chunk
@@ -225,7 +227,7 @@ class PodEnvironment(OdfEnvironment):
                              for holder in self.expressionsHolders]
         self.impactableElems = (Text.OD.elem, Title.OD.elem, Item.OD.elem,
                                 Table.OD.elem, Row.OD.elem, Cell.OD.elem,
-                                Section.OD.elem, Frame.OD.elem)
+                                Section.OD.elem, Frame.OD.elem, Doc.OD.elem)
         self.inserts = self.transformInserts()
 
     def getExpression(self, elem):
