@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -23,6 +23,7 @@ from appy.pod import PodError
 from appy.pod.lo_pool import LoPool
 from appy.shared.zip import unzip, zip
 from appy.pod.buffers import FileBuffer
+from appy.shared.xml_parser import Escape
 from appy.pod import styles_manager as sm
 from appy.pod.converter import FILE_TYPES
 from appy.pod import doc_importers as imps
@@ -532,7 +533,8 @@ class Renderer:
                 css = ' class="%s"' % css
             else:
                 css = ''
-            r[i] = '<p%s>%s%s%s%s</p>' % (css, pre, opening, r[i], closing)
+            r[i] = '<p%s>%s%s%s%s</p>' % (css, pre, opening,
+                                          Escape.xhtml(r[i]), closing)
             i -= 1
         return self.renderXhtml(''.join(r), stylesMapping=stylesMapping)
 
