@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -134,7 +134,7 @@ class HtmlElement(Element):
         self.innerStyle = None
         # We must know if a style must be applied on inner paragraphs within
         # "li" tags.
-        if (elem == 'li') and attrs.has_key('class'):
+        if elem == 'li' and attrs.has_key('class'):
             self.paraStyle = attrs['class']
         else:
             self.paraStyle = None # Will also be set on "ol" and "ul" tags.
@@ -203,7 +203,7 @@ class HtmlElement(Element):
         # Special case: check elements that can't be found within a "li".
         # Instead of being noted as "conflictual", note that we must keep
         # these element's contents but remove the surrounding tags.
-        if (parent.elem == 'li') and (self.elem in PARA_TAGS):
+        if parent.elem == 'li' and self.elem in PARA_TAGS:
             self.removeTag = True
             parent.addInnerCssStyles(self)
             return ()
@@ -269,9 +269,25 @@ class HtmlElement(Element):
                 dump(' text:style-name="%s"' % styleName)
             else:
                 # Check if a style must be applied on 'p' tags
-                if self.innerCssStyles:
-                    p.cssStyles.merge(self.innerCssStyles)
-                innerStyle = listElem.paraStyle or env.itemStyles[itemStyle]
+                innerStyles = self.innerCssStyles
+                paraStyle = None
+                if innerStyles:
+                    p.cssStyles.merge(innerStyles)
+                    class_ = getattr(innerStyles, 'classes', None)
+                    if class_:
+                        # Take the last CSS class if several are defined
+                        paraStyle = class_.split()[-1]
+                # Some remarks here.
+                # 1) The CSS class defined on the "p" tag, when present, takes
+                #    precedence over the one defined at the list level (itself
+                #    copied from the applicable ListProperties instance. This
+                #    precedence rule is hard-coded: it is not possible to choose
+                #    which one prevails, as opposed to what happens with
+                #    TableProperties instances via boolean attribute "prevails".
+                # 2) The CSS class potentially defined at the level of the "li"
+                #    tag is currently ignored.
+                innerStyle = paraStyle or listElem.paraStyle or \
+                             env.itemStyles[itemStyle]
                 if innerStyle:
                     p.cssStyles.addClass(innerStyle)
                 dump(env.getOdfAttributes(p))
@@ -725,8 +741,9 @@ class XhtmlEnvironment(XmlEnvironment):
         # self.mergedInnerStyles) ?
         self.mergedCount = 0
         # The width (in cm, as a float) of pages (margins excluded) within the
-        # current document.
-        self.pageWidth = renderer.stylesManager.pageLayout.getWidth()
+        # current document. Within ODS templates, such width seems not be
+        # present (may have no sense for a sheet).
+        self.pageWidth = renderer.stylesManager.pageLayout.getWidth() or 17.0
         # It may be required to enable the POD post-processor (=PPP). But did we
         # already do it ?
         self.pppEnabled = False
