@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -93,7 +93,7 @@ class PxParser(XmlParser):
             # later). This way, when encountering the corresponding end element,
             # we will be able to check whether the end element corresponds to
             # the main element or to a sub-element.
-            e.currentBuffer.addElement(elem, elemType='px')
+            e.currentBuffer.addElement(elem, pod=False)
         if elem != 'x':
             # Dump the start element and its attributes. But as a preamble,
             # manage special attributes that could not be dumped at all, like
