@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -586,9 +586,10 @@ class XmlMarshaller:
     atFiles = ('image', 'file') # Types of archetypes fields that contain files
     typesMap = {'list': 'list', 'PersistentList': 'list', 'LazyMap': 'list',
                 'UserList': 'list', 'dict': 'dict', 'PersistentMapping': 'dict',
-                'UserDict': 'dict', 'IOBTree': 'dict', 'FileInfo': 'file',
-                'bool': 'bool', 'int': 'int', 'float': 'float', 'long': 'long',
-                'tuple': 'tuple', 'DateTime': 'DateTime'}
+                'UserDict': 'dict', 'IOBTree': 'dict', 'OOBTree': 'dict',
+                'FileInfo': 'file', 'bool': 'bool', 'int': 'int', 'float':
+                'float', 'long': 'long', 'tuple': 'tuple',
+                'DateTime': 'DateTime'}
 
     def __init__(self, cdata=False, dumpUnicode=False, conversionFunctions={},
                  dumpXmlPrologue=True, rootTag='xmlPythonData', namespaces={},
@@ -621,7 +622,13 @@ class XmlMarshaller:
         # '*':s_prefix will indicate a default prefix that will be applied to
         # any tag that does not have it own key in this dict.
         self.namespacedTags = namespacedTags
-        self.objectType = None # Will be given by method m_marshal
+        self.objectType = None # Will be given by method m_marshall
+        # For binaries, content or disk path is marshalled
+        self.marshallBinaries = True
+        # If p_self.marshallBinaries is False, in order to compute absolute
+        # paths to marshalled files, the base database folder, where those files
+        # reside, will be set here.
+        self.databaseFolder = None
 
     def getTagName(self, name):
         '''Returns the name of tag p_name as will be dumped. It can be p_name,
@@ -641,6 +648,10 @@ class XmlMarshaller:
         iType = type(instance)
         if iType == types.InstanceType:
             return True
+        elif iType.__name__ == 'Event' and \
+             iType.__bases__[0].__name__ == 'Persistent':
+            # An event from the calendar field
+            return True
         elif iType.__name__ == 'ImplicitAcquirerWrapper':
             # This is the case with Archetype instances
             return True
@@ -670,7 +681,11 @@ class XmlMarshaller:
         '''Dumps a string into the result.'''
         if self.cdata: res.write('<![CDATA[')
         if isinstance(s, str):
-            s = s.decode('utf-8')
+            try:
+                s = s.decode('utf-8')
+            except UnicodeDecodeError:
+                # Wrongly-encoded user input ?
+                s = s.decode('latin-1')
         # Replace special chars by XML entities
         for c in s:
             if self.xmlEntities.has_key(c):
@@ -681,7 +696,7 @@ class XmlMarshaller:
 
     def dumpFile(self, res, v):
         '''Dumps a file into the result'''
-        if not v: return
+        if not v or v.size == 0 or not self.marshallBinaries: return
         w = res.write
         # p_value contains the (possibly binary) content of a file. We will
         # encode it in Base64, in one or several parts.
@@ -819,6 +834,10 @@ class XmlMarshaller:
                 res.write(' name="')
                 self.dumpString(res, fileName)
                 res.write('"')
+            if not self.marshallBinaries:
+                # Dump the file location
+                res.write(' location="%s/%s/%s"' % (self.databaseFolder,
+                                          fieldValue.fsPath, fieldValue.fsName))
         res.write('>')
         # Dump the field value
         self.dumpValue(res, fieldValue, fType, isRef=(fieldType=='ref'))
@@ -908,6 +927,9 @@ class XmlMarshaller:
                 modified = getattr(instance, 'modified', instance.created)
                 self.dumpField(res, 'modified', modified)
                 appyi = instance.appy()
+                self.marshallBinaries = appyi.config.marshallBinaries
+                if not self.marshallBinaries:
+                    self.databaseFolder = appyi.tool.getDbFolder()
                 for field in instance.getAppyTypes('xml', None):
                     # Dump only needed fields
                     if field.name in self.fieldsToExclude: continue
@@ -1109,7 +1131,7 @@ class XhtmlCleaner(XmlParser):
     attrsToAdd = {'table': {'cellspacing':'0', 'cellpadding':'6', 'border':'1'},
                   'tr':    {'valign': 'top'}}
     # Tags that require a line break to be inserted after them.
-    lineBreakTags = ('p', 'li', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'td')
+    lineBreakTags = ('p', 'li', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'td', 'th')
     # No-end tags
     noEndTags = ('br', 'img')
 
@@ -1261,4 +1283,31 @@ class XhtmlToText(XmlParser):
     def endElement(self, elem):
         '''Dumps a carriage return every time a paragraph is encountered'''
         if elem in self.paraTags: self.res.append('\n')
+
+# ------------------------------------------------------------------------------
+class StringCleaner:
+    '''Ensure a string does not contain any char that would provoke SAX parser
+       errors. Also propose a method for producing clean strings, for which
+       those chars were removed.'''
+
+    # Chars (some of the ASCII control characters) provoking SAX parse errors in
+    # strings, once being part of a XML data structure that must be parsed with
+    # a SAX parser.
+    numbers = (1, 2, 3, 4, 5, 6, 7, 8, 10, 16, 19, '0e')
+    chars = '|'.join(['\\x%s' % str(n).zfill(2) for n in numbers])
+    illegal = re.compile(chars)
+
+    @classmethod
+    def isParsable(class_, s):
+        '''Returns True if p_s is SAX-parsable'''
+        try:
+            XmlParser().parse('<x>%s</x>' % s)
+            return True
+        except Exception:
+            return False
+
+    @classmethod
+    def clean(class_, s):
+        '''Return p_s whose illegal chars were removed'''
+        return class_.illegal.sub('', s)
 # ------------------------------------------------------------------------------
