@@ -3,7 +3,7 @@
 '''Contacts LibreOffice (LO) in server mode for test purposes'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -20,6 +20,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+import sys
 from pathlib import Path
 
 from appy.pod import test
@@ -38,12 +39,14 @@ TESTS = {
 HELP_SERVER = 'The hostname of the machine running LibreOffice. Defaults to ' \
   '"%s".' % DEFAULT_SERVER
 HELP_PORT = 'The port where LibreOffice listens. Defaults to %d.' % DEFAULT_PORT
-HELP_TEST = 'The test to execute. Available tests are %s.' % \
+HELP_TEST = 'The test to execute. Available tests are %s. ' \
+            'Default test is "convertToPdf".' % \
             ', '.join(['"%s" (%s)' % (k, v) for k, v in TESTS.items()])
 TEST_NOT_FOUND = 'Test %s does not exist.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class LO(Program):
+    '''Tests LO'''
 
     def defineArguments(self):
         '''Define the allowed arguments for this program'''
@@ -72,6 +75,7 @@ class LO(Program):
             converter = Converter(str(odtFile), 'pdf', server=self.server,
                                   port=self.port, verbose=True)
             converter.run()
+            return True
         except converter.Error as err:
             print(err)
 
@@ -87,6 +91,7 @@ class LO(Program):
                             overwriteExisting=True, forceOoCall=True)
         print('Rendering %s...' % str(template))
         renderer.run()
+        return True
         print('Result produced in %s.' % str(result))
 
     def run(self):
@@ -96,15 +101,17 @@ class LO(Program):
         # Abort if the specified test does not exist
         if not hasattr(self, self.test):
             print(TEST_NOT_FOUND % self.test)
-            return
+            sys.exit(1)
         method = getattr(self, self.test)
         if not callable(method):
             print(TEST_NOT_FOUND % self.test)
-            return
+            sys.exit(1)
         # Get the path to the POD test files
         self.filesPath = Path(test.__file__).parent / 'templates'
         # Execute the test
-        return method()
+        success = method()
+        if not success:
+            sys.exit(1)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 if __name__ == '__main__': LO().run()
