@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,7 +16,7 @@
 # You should have received a copy of the GNU General Public License along with
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import os, shutil
+import os, shutil, sys
 from pathlib import Path
 
 import appy
@@ -25,13 +25,12 @@ from appy.bin import Program
 from appy.utils.path import chown
 from appy.utils.loc import Counter
 from appy.tr.updater import Updater
-from appy.deploy import parseSpecifier
-from appy.deploy.subversion import Subversion
+from appy.deploy.repository import Repository
 
 # File templates - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # <site>/bin/site
 site = """
-#!/usr/bin/python3
+#!{self.python}
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import sys
@@ -63,9 +62,15 @@ if __name__ == '__main__':
 
 # <site>/bin/deploy
 deploy = """
-#!/usr/bin/python3
+#!{self.python}
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import sys
+
+lib = '{self.folder}/lib'
+if lib not in sys.path:
+    sys.path.insert(0, lib)
+
 from appy.bin.deploy import Deploy
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -97,12 +102,12 @@ def complete(c):
 # <app>/__init__.py
 appInit = '''
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+import appy
 from appy.database import log
 from appy.server import guard, scheduler, backup
 from appy import database, model, ui, server, deploy, peer
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import appy
 class Config(appy.Config):
     server = server.Config()
     security = guard.Config()
@@ -127,7 +132,7 @@ except ImportError:
 
 # <app>/make
 appMake = '''
-#!/usr/bin/python3
+#!{self.python}
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from pathlib import Path
 from appy.bin.make import App
@@ -146,7 +151,7 @@ from %s import Config as c
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # Declare this extension
 from pathlib import Path
-c.declareExt(Path('%s'))
+c.declareExt(Path(__path__[0]))
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # Apply ext's changes to this instance
@@ -194,6 +199,8 @@ class Site(Target):
         # The folder or distant specifier for the app and its ext
         self.app = app
         self.ext = ext
+        # The path to the currently running Python interpreter
+        self.python = sys.executable
         # The port on which the site will listen
         self.port = port
         # The owner of the site tree (if None, the user executing the script
@@ -215,15 +222,14 @@ class Site(Target):
         self.folder.mkdir(parents=True)
         # Create the sub-folders
         for name in Site.folders: (self.folder/name).mkdir()
-        # If the app or ext are distant, get, in lib/<appOrExtName>, a copy from
-        # the server.
+        # If the app or ext are distant, download, in lib/<appOrExtName>, a copy
+        # from the code.
         lib = self.folder / 'lib'
         if not isinstance(self.app, Path):
-            self.app = Subversion(self.app).checkout(lib)
+            self.app = self.app.download(lib)
         ext = self.ext
         if ext:
-            self.ext = ext if isinstance(ext, Path) \
-                           else Subversion(ext).checkout(lib)
+            self.ext = ext if isinstance(ext, Path) else ext.download(lib)
         else:
             self.ext = ''
         # Integrate dependencies into the site
@@ -234,8 +240,8 @@ class Site(Target):
                     # A dependency to a local package. Symlink it in <site>/lib.
                     os.symlink(dep, lib / dep.name)
                 else:
-                    # A specifier. Checkout a copy in <site>/lib.
-                    Subversion(dep).checkout(lib)
+                    # A remote repo: download a copy <site>/lib
+                    dep.download(lib)
         # Create <site>/bin/site and <site>/bin/deploy
         self.createFile(self.folder/'bin'/'site', site, 0o770)
         self.createFile(self.folder/'bin'/'deploy', deploy, 0o770)
@@ -261,6 +267,8 @@ class App(Target):
         if not folder.is_absolute():
             folder = folder.resolve()
         self.folder = folder
+        # The path to the currently running Python interpreter
+        self.python = sys.executable
 
     def create(self):
         '''Creates a new app'''
@@ -317,7 +325,7 @@ class Ext(App):
         # Create <ext>/__init__.py
         path = self.folder / '__init__.py'
         if not path.exists():
-            self.createFile(path, extInit % (self.app.name, str(self.folder)))
+            self.createFile(path, extInit % self.app.name)
         # Create base folders
         for name in App.subFolders:
             folder = self.folder / name
@@ -332,36 +340,40 @@ class Ext(App):
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Make(Program):
     '''This program allows to create or update an app or a site'''
+
     # We can create/update the following artefacts
     allowedTargets = ('app', 'site', 'ext')
 
     # Help messages
-    APP_S        = 'url=<url>,[name=<name>,login=<login>,password=<password>]'
-    HELP_TARGET  = 'can be "app" for creating/updating an app, "site" for ' \
-                   'creating a site or "ext" for creating an extension to an ' \
-                   'app.'
-    HELP_FOLDER  = 'folder is the absolute path to the base folder for the ' \
-                   'app, site or ext.'
-    HELP_A       = 'Mandatory when target is [site] or [ext], specify here ' \
-                   'the absolute path to the related app (if the app is to ' \
-                   'be found locally), or a distant app specifier of the ' \
-                   'form %s. Currently, a distant specifier must point to a ' \
-                   'Subversion repository.' % APP_S
-    HELP_P       = '[site] the port on which the site will listen. Defaults ' \
-                   'to 8000.'
-    HELP_O       = "[site, unix/linux only] the owner of the site. If " \
-                   "specified, the whole site tree will be chown'ed to this " \
-                   "owner. Optionally, you may specify the group by " \
-                   "suffixing the owner by a colon and the group name, ie: " \
-                   "appy:appy."
-    HELP_D       = '[site] dependencies, as a series of local paths or ' \
-                   'specifiers. For every local path, a symlink will be ' \
-                   'created in <site>/lib. For every specifier, a local copy ' \
-                   'will created in <site>/lib.'
-    HELP_E       = '[site] The absolute path to the optional extension to ' \
-                   'the app specified via option "-a" (if the ext is to be ' \
-                   'found locally) or a distant ext specifier of the form ' \
-                   '"%s".' % APP_S
+    APP_S   = 'url=<url>,[type=<type>,name=<name>,login=<login>,' \
+              'password=<password>]'
+    HELP_TG = 'can be "app" for creating/updating an app, "site" for ' \
+              'creating a site or "ext" for creating an extension to an app.'
+    HELP_F  = 'folder is the absolute path to the base folder for the app, ' \
+              'site or ext.'
+    HELP_A  = 'Mandatory when target is [site] or [ext], specify here the ' \
+              'absolute path to the related app (if the app is to be found ' \
+              'locally), or a distant app specifier of the form %s. ' \
+              'A distant specifier must point to a Git (type=git) or ' \
+              'Subversion (type=svn) repository. For a git specifier, ' \
+              'attributes "login" and "password" are useless: only ' \
+              'public-key authentication is supported. For a git specifier, ' \
+              'mentioning "type=git" is useless: it is the default value. ' \
+              'In a specifier, attribute "url" must always be the first one; ' \
+              'attribute "password", when included, must always be the last ' \
+              'one.' % APP_S
+    HELP_P  = '[site] the port on which the site will listen. Defaults to 8000.'
+    HELP_O  = "[site, unix/linux only] the owner of the site. If specified, " \
+              "the whole site tree will be chown'ed to this owner. " \
+              "Optionally, you may specify the group by suffixing the owner " \
+              "by a colon and the group name, ie: appy:appy."
+    HELP_D  = '[site] dependencies, as a series of local paths or ' \
+              'specifiers. For every local path, a symlink will be created ' \
+              'in <site>/lib. For every specifier, a local copy will be ' \
+              'created in <site>/lib.'
+    HELP_E  = '[site] The absolute path to the optional extension to the app ' \
+              'specified via option "-a" (if the ext is to be found locally) ' \
+              'or a distant ext specifier of the form "%s".' % APP_S
 
     # Error messages
     WRONG_TARGET = 'Wrong target "%s".'
@@ -377,8 +389,8 @@ class Make(Program):
         '''Define the allowed arguments for this program'''
         add = self.parser.add_argument
         # Positional arguments, common to all targets (app and site)
-        add('target', help=Make.HELP_TARGET)
-        add('folder', help=Make.HELP_FOLDER)
+        add('target', help=Make.HELP_TG)
+        add('folder', help=Make.HELP_F)
         # Optional arguments specific to "site"
         add('-a', '--app',  dest='app',  help=Make.HELP_A)
         add('-p', '--port', dest='port', help=Make.HELP_P)
@@ -391,11 +403,11 @@ class Make(Program):
         '''Get the path to a local app (or ext if p_isExt is True) or the
            distant specifier else.'''
         if not app and not isExt: self.exit(self.NO_APP)
-        if ',' in app:
+        if app.startswith('url='):
             # A specifier
             if specifierDisallowed: self.exit(self.SPEC_APP_KO)
-            r = parseSpecifier(app)
-            if not r.url:
+            r = Repository.parse(app)
+            if r is None:
                 print(self.WRONG_APP_S % app)
                 self.exit(self.APP_S_KO % app)
         else:
@@ -414,10 +426,10 @@ class Make(Program):
         length = len(dependencies)
         while i < length:
             dep = dependencies[i]
-            if ',' in dep:
+            if dep.startswith('url='):
                 # A specifier
-                dep = parseSpecifier(dep)
-                if not dep.url:
+                dep = Repository.parse(dep)
+                if dep is None:
                     print(self.WRONG_APP_S % dep)
                     self.exit(self.APP_S_KO % dep)
             else:
