@@ -1,7 +1,7 @@
 '''Appy module managing database files'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -18,7 +18,7 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import os, time, pathlib
+import os, sys, time, pathlib
 
 from DateTime import DateTime
 from zc.lockfile import LockError
@@ -78,9 +78,9 @@ class Config:
         #   "filePath"       is <site>/var/appy.fs
         #   "binariesFolder" is <site>/var
 
-        # What roles can unlock locked pages ? By default, only a Manager can
-        # do it. You can place here roles expressed as strings or Role
-        # instances, global or local.
+        # What roles can unlock pages ? By default, only a Manager can do it.
+        # You can place here roles expressed as strings or Role instances,
+        # global or local.
         self.unlockers = ['Manager']
         # Number of times a transaction is retried after a conflict error has
         # occurred.
@@ -136,6 +136,13 @@ class Config:
             # Execute method named p_method on the tool
             database.run(method, handler, logger)
 
+    def getPidPath(self):
+        '''Return a pathlib.Path instance to the "pid" file, which, when the
+           server is started in "start" mode, stores its process ID, in order
+           to be able to stop him afterwards.'''
+        # The choice was made to create the "pid" besides the DB file
+        return self.filePath.parent / 'pid'
+
     def getDatabaseSize(self, formatted=False):
         '''Returns the database size'''
         r = os.stat(self.filePath).st_size
@@ -189,6 +196,16 @@ class Database:
         '''Returns the list of active connections to the database'''
         return self.db.connectionDebugInfo()
 
+    def listTransactions(self, start=None, end=None):
+        '''Lists the database transactions that occurred between this p_start
+           and p_end dates.'''
+        # p_start and/or p_end, if passed, must be DateTime instances
+        # If p_start is None, it will represent the start of the database life.
+        # If p_end is None, it will represent the end of times.
+        start = start.timeTime() if start else 0
+        end = end.timeTime() if end else sys.maxsize
+        return self.db.undoLog(start, end)
+
     def commit(self, handler):
         '''Commits the current transaction related to p_handler'''
         transaction.commit()
@@ -202,6 +219,12 @@ class Database:
         # Close the p_connection if given
         if connection: connection.close()
 
+    def undo(self, handler, transactionIds):
+        '''Undo transactions whose IDs are in p_transactionIds'''
+        for id in transactionIds:
+            self.db.undo(id)
+            handler.dbConnection.sync()
+
     def close(self, abort=False):
         '''Closes the database'''
         # Must we first abort any ongoing transaction ?
@@ -654,7 +677,9 @@ class Database:
         # Call the custom "onEditEarly" if available. This method is called
         # *before* potentially linking p_o to its initiator.
         if isTemp:
-            multicall(o, 'onEditEarly', False, None)
+            # At this step, o's container is not there yet. p_initiator is
+            # passed if m_onEditEarly requires it.
+            multicall(o, 'onEditEarly', False, initiator)
             if not self.exists(o=o): return
         # Manage the relationship between the initiator and the new object
         if isTemp and initiator:
@@ -721,6 +746,7 @@ class Database:
                 if historize and field.back.getAttribute(tied, 'historized'):
                     className = o.translate(tied.class_.name)
                     tied.history.add('Unlink', deletion=True,
+                                     field=field.back.name,
                                      comment='%s: %s' % (className, title))
         if not isTemp:
             # Unindex p_o if it was indexed
