@@ -2,7 +2,7 @@
    of a given class from an Appy app's model.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -243,7 +243,7 @@ class Catalog(PersistentMapping):
         '''Gets the field whose name is p_name'''
         return handler.server.model.classes.get(self.name).fields.get(name)
 
-    def getSorted(self, r, sortBy, sortOrder):
+    def getSorted(self, r, sortBy, sortOrder, maxResults=None):
         '''Gets, from search results p_r (=a IITreeSet of IIDs), a list of
            sorted IIDs.'''
         # If the result must not be sorted, return the result set as-is
@@ -252,10 +252,14 @@ class Catalog(PersistentMapping):
         # Find the sort index
         sortIndex = self.getIndex(sortBy)
         # Get sorted results
-        return Sorter(sortIndex, r, reverse=sortOrder=='desc').run()
+        res = Sorter(sortIndex, r, reverse=sortOrder=='desc').run()
+        # Truncate the result if required
+        if res and maxResults is not None:
+            res = res[:maxResults]
+        return res
 
     def search(self, handler, secure=False, sortBy=None, sortOrder='asc',
-               **fields):
+               maxResults=None, **fields):
         '''Performs a search in this catalog. Returns a IITreeSet object if
            results are found, None else.'''
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -270,12 +274,15 @@ class Catalog(PersistentMapping):
         # sortOrder  | Can be "asc" (ascending, the defaut) or "desc"
         #            | (descending).
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # maxResults | If specified, the search results (after potentially being
+        #            | sorted) will be truncated to this number.
+        #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # fields     | Keyword args must correspond to valid indexed field names
         #            | on p_className. For every such arg, the specified value
         #            | must be a valid value according to the field definition.
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Add the security-related search parameter if p_secure is True
-        if secure and ('allowed' not in fields):
+        if secure and 'allowed' not in fields:
             fields['allowed'] = handler.guard.userAllowed
         # The result set, as a IITreeSet instance (or None)
         r = None
@@ -294,6 +301,8 @@ class Catalog(PersistentMapping):
         # Browse search values from p_fields. Those values are implicitly
         # AND-ed: as soon as one value does not match, there is no match at all.
         for name, value in fields.items():
+            # Ignore values being None
+            if value is None: continue
             if isinstance(value, Operator) and value.negative:
                 # A negative operator. Do not manage it now, store it for later.
                 neg = (name, value)
@@ -313,7 +322,8 @@ class Catalog(PersistentMapping):
         # If there was no positive arg at all, take, as basis for the search,
         # all instances from this class' catalog.
         r = self.all if not positive else r
-        if not r or not negative: return self.getSorted(r, sortBy, sortOrder)
+        if not r or not negative:
+            return self.getSorted(r, sortBy, sortOrder, maxResults)
         # Apply negative args
         for name, value in negative:
             field = self.getField(name, handler)
@@ -322,7 +332,7 @@ class Catalog(PersistentMapping):
                 r = difference(r, ids)
                 if not r: return
         # Convert the IITreeSet to a sorted list of IIDs and return it
-        return self.getSorted(r, sortBy, sortOrder)
+        return self.getSorted(r, sortBy, sortOrder, maxResults)
 
     def unindexObject(self, iid, log=None):
         '''Unindexes object having this p_iid from this catalog. Returns True if
