@@ -1,7 +1,7 @@
 '''An index for Ref fields'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -45,7 +45,14 @@ class RefIndex(Index):
     @classmethod
     def toTerm(class_, value, field):
         '''Ensure p_value is an object's IID'''
-        return value if isinstance(value, int) else value.iid
+        if isinstance(value, int):
+            r = value
+        elif isinstance(value, str):
+            # Allow p_value to be an IID encoded as a string
+            r = int(value)
+        else:
+            r = value.iid
+        return r
 
     @classmethod
     def toString(class_, o, value):
