@@ -1,7 +1,7 @@
 '''Deployment system for Appy sites and apps'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -20,25 +20,23 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import os, sys
 from pathlib import Path
-from appy.model.utils import Object as O
-from appy.deploy.subversion import Subversion
+
+from appy.deploy.repository import Repository
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-def parseSpecifier(specifier):
-    '''Parses a string specifier of the form <attr>=<value>[,<attr>=<value>]'''
-    r = O()
-    for part in specifier.split(','):
-        name, value = part.split('=', 1)
-        r[name] = value
-    return r
+T_EXEC    = 'Executing :: %s'
+T_LIST    = 'Available target(s) for app "%s", from reference site "%s":\n%s'
+NO_CONFIG = 'The "deploy" config was not found in config.deploy.'
+NO_TARGET = 'No target was found on config.deploy.targets.'
+TARGET_KO = 'Target "%s" not found. Available target(s): %s.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Target:
-    '''Represents a app deployed on a site on a distant machine'''
+    '''Represents an app deployed on a site on a distant machine'''
 
     def __init__(self, sshHost, sshPort=22, sshLogin='root', sshKey=None,
                  sitePath=None, sitePort=8000, siteApp=None, siteExt=None,
-                 siteOwner='appy:appy', siteDependencies=None):
+                 siteOwner='appy:appy', siteDependencies=None, default=False):
         # The name of the distant host, for establishing a SSH connection
         self.sshHost = sshHost
         # The port for the SSH connection
@@ -53,20 +51,27 @@ class Target:
         self.sitePath = sitePath
         # The port on which this site will listen
         self.sitePort = sitePort
-        # The specifier of the distant app and ext. For example:
-        #    url=https://svn.forge.pallavi.be/appy-python-3/trunk,name=appy
+        # Instances representing the distant repos where the app and ext reside.
+        # Must be instances of classes appy.deploy.git.Git or
+        # appy.deploy.subversion.Subversion.
         self.siteApp = siteApp
         self.siteExt = siteExt
         # The owner of the distant site. Typically: appy:appy.
         self.siteOwner = siteOwner
         # A list of Python dependencies to install on the distant app, in its
-        # "lib" folder. For every dependency, a specifier must be used, with the
-        # same syntax as attributes "siteApp/siteExt" hereabove.
+        # "lib" folder. Every dependency must be specified via a Repository
+        # instance, from one of the concrete classes as mentioned hereabove (see
+        # attributes p_self.siteApp and p_self.siteExt).
         self.siteDependencies = siteDependencies or []
+        # Is this target the default target ? It is not mandatory to have a
+        # default target. It is useful if you want to launch deployer commands
+        # without specfying arg "-t <target>": the default target will be
+        # automatically chosen.
+        self.default = default
 
     def __repr__(self):
         '''p_self's string representation'''
-        return '<Target %s:%d>' % (self.sshHost, self.sshPort)
+        return '<Target %s:%d@%s>' % (self.sshHost, self.sshPort, self.sitePath)
 
     def execute(self, command):
         '''Executes p_command on this target'''
@@ -77,7 +82,7 @@ class Target:
         if self.sshKey: r.insert(1, '-i %s' % self.sshKey)
         # Build the complete command
         r = ' '.join(r)
-        print('Executing: %s...' % r)
+        print(T_EXEC % r)
         os.system(r)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -90,11 +95,6 @@ class Config:
         # be defined at key "default".
         self.targets = {}
 
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-NO_CONFIG = 'The "deploy" config was not found in config.deploy.'
-NO_TARGET = 'No target was found on config.deploy.targets.'
-TARGET_KO = 'Target "%s" not found. Available target(s): %s.'
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Deployer:
     '''App deployer'''
@@ -103,15 +103,19 @@ class Deployer:
     apt = 'DEBIAN_FRONTEND=noninteractive apt-get -yq install'
 
     # OS packages being Appy dependencies
-    osDependencies = 'libreoffice subversion python3-pip apache2 imagemagick'
+    osDependencies = 'libreoffice git python3-pip apache2 imagemagick'
 
-    def __init__(self, appPath, sitePath, command, targetName='default'):
+    # Commands for which a target is not required
+    noTargetCommands = ('list',)
+
+    def __init__(self, appPath, sitePath, command, targetName=None):
         # The path to the app
         self.appPath = appPath
         # The path to the reference, local site, containing targets definition
         self.sitePath = sitePath
-        # The chosen target (name)
-        self.targetName = targetName or 'default'
+        # The chosen target (name). If p_targetName is None, the default target
+        # (if there is one) will be selected.
+        self.targetName = targetName
         self.target = None # Will hold a Target instance
         # The command to execute
         self.command = command
@@ -137,12 +141,15 @@ class Deployer:
     def list(self):
         '''Lists the available targets on the config'''
         infos = []
+        default = None
+        i = 1
         for name, target in self.config.deploy.targets.items():
-            info = '*** Target: %s\n%s' % (name, target)
+            suffix = ' [default]' if target.default else ''
+            info = '%s. %s - %s%s' % (str(i).zfill(2), name, target, suffix)
             infos.append(info)
-        infos = '\n\n'.join(infos)
-        print('Available target(s) for app "%s", from reference site "%s":\n' \
-              '%s' % (self.appPath.name, self.sitePath.name, infos))
+            i += 1
+        infos = '\n'.join(infos)
+        print(T_LIST % (self.appPath.name, self.sitePath.name, infos))
 
     def info(self):
         '''Retrieve info about the target OS'''
@@ -156,9 +163,9 @@ class Deployer:
           # Install required dependencies via Aptitude
           '%s %s' % (self.apt, self.osDependencies),
           # Install Appy and dependencies via pip
-          'pip3 install appy',
+          'pip3 install appy -U',
           # Create special user "appy"
-          'adduser --disabled-password --gecos \'\' appy'
+          'adduser --disabled-password --gecos appy appy'
         ]
         target.execute(';'.join(commands))
 
@@ -167,15 +174,15 @@ class Deployer:
         t = self.target
         # Build args to appy/bin/make
         q = self.quote
-        args = [q(t.sitePath), q('-a'), q(t.siteApp), q('-p'), q(t.sitePort),
-                q('-o'), q(t.siteOwner)]
+        args = [q(t.sitePath), q('-a'), q(t.siteApp.asSpecifier()), q('-p'),
+                q(t.sitePort), q('-o'), q(t.siteOwner)]
         if t.siteExt:
             args.append(q('-e'))
-            args.append(q(t.siteExt))
+            args.append(q(t.siteExt.asSpecifier()))
         if t.siteDependencies:
             args.append(q('-d'))
             for dep in t.siteDependencies:
-                args.append(q(dep))
+                args.append(q(dep.asSpecifier()))
         # Build the statements to pass to the distant Python interpreter
         statements = [
           'import sys', 'from appy.bin.make import Make',
@@ -190,26 +197,33 @@ class Deployer:
         '''Performs an update of all software known to the site and coming from
            external sources (app and dependencies) and (re)starts the site.'''
         target = self.target
-        # Build the commands to update the app, ext and dependencies
+        # (1) Build the set of commands to update the app, ext and dependencies
         commands = []
+        siteOwner = target.siteOwner
         lib = Path(target.sitePath) / 'lib'
         for name in ('App', 'Ext'):
-            spec = getattr(target, 'site%s' % name)
-            if not spec: continue
-            svn = Subversion(parseSpecifier(spec))
-            svnUp, local = svn.build('update', lib)
-            commands.append(svnUp)
+            repo = getattr(target, 'site%s' % name)
+            if repo:
+                command, folder = repo.getUpdateCommand(lib)
+                commands.append(command)
+                commands.append('chown -R %s %s' % (siteOwner, folder))
         # Update dependencies
-        for dep in target.siteDependencies:
-            svn = Subversion(parseSpecifier(dep))
-            svnUp, local = svn.build('update', lib)
-            commands.append(svnUp)
-        # Build the command to restart the distant site
+        for repo in target.siteDependencies:
+            command, folder = repo.getUpdateCommand(lib)
+            commands.append(command)
+            commands.append('chown -R %s %s' % (siteOwner, folder))
+        # Run those commands as the main SSH user: else, agent forwarding will
+        # not be allowed and will prevent to update repositories using public
+        # key authentication.
+        command = '%s %s' % (Repository.getEnvironment(), ';'.join(commands))
+        target.execute(command)
+        # (2) Build the command to restart the distant site
+        commands = []
         restart = '%s/bin/site restart' % target.sitePath
         commands.append(restart)
         commands.append(self.tail % (100, target.sitePath))
-        # The commands will be ran with target.siteOwner
-        owner = target.siteOwner.split(':')[0]
+        # These commands will be ran with target.siteOwner
+        owner = siteOwner.split(':')[0]
         command = "su %s -c '%s'" % (owner, ';'.join(commands))
         target.execute(command)
 
@@ -222,6 +236,19 @@ class Deployer:
     #                             Main method
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
+    def getTarget(self, targets):
+        '''Return the target onto which the command must be applied'''
+        name = self.targetName
+        r = None
+        if name:
+            r = targets.get(name)
+        else:
+            for target in targets.values():
+                if target.default:
+                    r = target
+                    break
+        return r
+
     def run(self):
         '''Performs p_self.command on the specified p_self.targetName'''
         # Add the relevant paths to sys.path
@@ -238,10 +265,9 @@ class Deployer:
             print(NO_TARGET)
             sys.exit(1)
         # Get the target
-        name = self.targetName
-        target = self.target = targets.get(name)
-        if not target:
-            print(TARGET_KO % (name, ', '.join(targets)))
+        target = self.target = self.getTarget(targets)
+        if not target and self.command not in Deployer.noTargetCommands:
+            print(TARGET_KO % (self.targetName, ', '.join(targets)))
             sys.exit(1)
         getattr(self, self.command)()
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
