@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -95,8 +95,10 @@ class Base:
     # for displaying base information about an object.
     titleAttributes = {'multiplicity': (1,1), 'show': Show.EX, 'indexed': True,
       'searchable': True, 'filterField': searchable,
-      # This method allows to produce an index value suitable for sorting
-      'indexValue': lambda o, v: Normalize.text(o.getShownValue(),
+      # The following lambda allows to produce an index value suitable for
+      # sorting. In the call to m_getShownValue, language 'en' is forced in
+      # order to bypass the user language and always have a deterministic value.
+      'indexValue': lambda o, v: Normalize.text(o.getShownValue(language='en'),
                                                 keepBlank=False) if v else ''}
 
     title = String(**titleAttributes)
@@ -109,6 +111,10 @@ class Base:
         '''Sets p_login as p_self's creator'''
         self.history[-1].login = login
 
+    def getCreator(self, title=True):
+        '''Return the User instance (or its p_title) behind p_self.creator'''
+        return self.history[-1].getUser(self, title=title)
+
     # Object's creation and last modification dates
     p['show'] = Show.VE_
     created = Date(default=lambda o: o.history[-1].date, persist=False, **p)
@@ -151,6 +157,7 @@ class Base:
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     recordAttributes = {'method': History.view, 'label':'Base', 'layouts':'f',
+      'xml': lambda o, value: o.history,
       'page':Page('history', label='Base_page_history',
                   sticky=True, icon='history.svg',
                   show=lambda o: 'view' if o.user.hasRole('Manager') else None)}
@@ -174,6 +181,7 @@ class Base:
     # from any object in the database, to (i) a series of heavily-demanded
     # objects, like the tool or the currently logged user, or (ii) commonly
     # used functions.
+
     # ~~~ All these properties will have this setter, raising an exception
     def raiseReadOnly(self, v): raise Exception(RO_EXPR)
 
@@ -199,6 +207,9 @@ class Base:
     # The object (base) URL
     url = property(lambda o: o.H().server.getUrl(o), raiseReadOnly)
 
+    # An access to the developer's cache object, stored on the handler
+    cache = property(lambda o: o.H().cache, raiseReadOnly)
+
     # A potential initiator object
     initiator = property(lambda o: o.getInitiator(), raiseReadOnly)
 
@@ -220,10 +231,16 @@ class Base:
     # The current traversal
     traversal = property(lambda o: o.H().traversal, raiseReadOnly)
 
+    # The translated name of the current object state
+    stateLabel = property(lambda o: o.getLabel(o.state, field=False))
+
     # The catalog storing indexed data for this object
     traverse['catalog'] = 'Manager'
     catalog = property(lambda o: o.getCatalog(), raiseReadOnly)
 
+    # A reference the the database object
+    database = property(lambda o: o.H().server.database, raiseReadOnly)
+
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Base methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -243,9 +260,11 @@ class Base:
         name = self.__class__.__bases__[0].__name__
         return classes.get(name)
 
-    def getWorkflow(self):
-        '''Get the appy.model.meta.Workflow corresponding to p_self'''
-        return self.getClass().workflow
+    def getWorkflow(self, python=False):
+        '''Get the appy.model.meta.Workflow corresponding to p_self, or the True
+           Python class if p_python is True)'''
+        r = self.getClass().workflow
+        return r.python if python else r
 
     def do(self, name, comment=None, doAction=True, doHistory=True,
            secure=False, doSay=False, data=None):
@@ -289,9 +308,15 @@ class Base:
     def getObject(self, id, logMissing=False):
         '''Gets an object given its p_id'''
         if not id: raise Exception(NO_ID)
-        handler = self.H()
-        return handler.server.database.getObject(handler, id,
-                                                 logMissing=logMissing)
+        return self.database.getObject(self.H(), id, logMissing=logMissing)
+
+    def getRefObject(self):
+        '''This method returns the source object, if we are in the context of
+           performing some action (like a search) in the context of a Ref.'''
+        # Information about such Ref is found in the request
+        info = self.req.ref or Criteria.readFromRequest(self.H()).get('_ref')
+        if not info: return
+        return self.getObject(info.split(':', 1)[0])
 
     def __repr__(self):
         '''Returns the class name and Appy object ID'''
@@ -309,9 +334,10 @@ class Base:
         '''Adds p_message to the global message returned in the response'''
         self.resp.addMessage(message)
 
-    def goto(self, url=None, message=None):
-        '''Return to some p_url or to the referer page'''
-        self.resp.goto(url, message)
+    def goto(self, url=None, message=None, fromPopup=False):
+        '''Return to some p_url or to the referer page. If you want to come back
+           from a popup, explicitly set p_fromPopup to True.'''
+        self.resp.goto(url, message, fromPopup)
 
     def gotoSet(self):
         '''Return True if m_goto has already been called and the response code
@@ -343,6 +369,13 @@ class Base:
         H = self.H()
         return H.server.buildUrl(H, name=name, base=base, ram=ram, bg=bg)
 
+    def buildSvg(self, name, bg=False):
+        '''Variant of m_buildUrl being specific to SVG files. Indeed, SVG files
+           are exclusively RAM-cached.'''
+        # p_name must not contain the ".svg" file extension
+        H = self.H()
+        return H.server.buildUrl(H, '%s.svg' % name, ram=True, bg=bg)
+
     def getUrl(self, sub=None, **params):
         '''See doc in appy.server.Server::getUrl'''
         return self.H().server.getUrl(self, sub=sub, **params)
@@ -379,11 +412,11 @@ class Base:
         # Browse p_self's fields, or p_fields if passed
         for field in (fields or self.class_.fields.values()):
             # Ignore fields not being on p_page
-            if page and (field.pageName != page): continue
+            if page and field.pageName != page: continue
             # Ignore fields not being on any page in p_phase
             if phase and (field.page and (field.page.phase != phase)): continue
             # Ignore fields not being of p_type
-            if type and (field.type != type): continue
+            if type and field.type != type: continue
             # Ignore fields that can't be rendered on p_layout
             if not field.isRenderable(layout): continue
             # Ignore unshowable fields 
@@ -429,9 +462,9 @@ class Base:
             if collect:
                 # Collect JS/CSS files
                 if css is None: css = []
-                field.getCss(layout, css)
+                field.getCss(self, layout, css)
                 if js is None: js = []
-                field.getJs(layout, js, config)
+                field.getJs(self, layout, js, config)
             # Insert the field directly in the result if it is not in a group
             if not field.group:
                 r.append(field)
@@ -463,21 +496,23 @@ class Base:
             value = field.getStoredValue(self, name)
         return field.isEmptyValue(self, value)
 
-    def getLabel(self, name, field=True):
+    def getLabel(self, name, field=True, language=None):
         '''Gets the translated label for field (if p_field is True) or workflow
            state or transition (if p_field is False) named p_name.'''
         # Translate the name of a field
         if field:
             field = self.getField(name, raiseError=True)
-            return self.translate('labelId', field=field)
+            return self.translate('label', field=field, language=language)
         # Translate the name of a state or transition
-        return self.translate('%s_%s' % (self.getWorkflow().name, name))
+        return self.translate('%s_%s' % (self.getWorkflow().name, name),
+                              language=language)
 
-    def getShownValue(self, name='title', layout='view', language=None):
+    def getShownValue(self, name='title', layout='view', language=None,
+                      value=None):
         '''Call field.getShownValue on field named p_name'''
         field = self.getField(name)
-        return field.getShownValue(self, field.getValue(self), layout,
-                                   language=language)
+        value = field.getValue(self) if value is None else value
+        return field.getShownValue(self, value, layout, language=language)
 
     def getValue(self, name, type=None, layout='view', language=None,
                  value=None, bypassDefault=False):
@@ -571,9 +606,8 @@ class Base:
            - an attribute set on referred objects ("title", by default);
            - a method that will be called on every tied object, will receive
              every such object as unique arg and will return a value that will
-             represent its order among all tied objects. This return value will
-             then be returned by the standard method given to the "key" param of
-             the standard list.sort method;
+             represent its order among all tied objects. This mimics usage of
+             the "key" param of the standard list.sort method;
            - None. If None, default sorting will occur, using the method stored
              in field.insert.
         '''
@@ -594,7 +628,7 @@ class Base:
             keyMethod = lambda o: field.getSortValue(o)
         else:
             # Sort according to a custom method
-            keyMethod = lambda o: sortKey(o)
+            keyMethod = sortKey
         # Perform the sort
         objects.sort(key=keyMethod, reverse=reverse)
 
@@ -624,7 +658,7 @@ class Base:
         '''Computes the container ID ("cid") for p_self'''
         r = None
         for field in self.class_.fields.values():
-            if (field.type == 'Ref') and field.isBack and field.back.composite:
+            if field.type == 'Ref' and field.isBack and field.back.composite:
                 container = getattr(self, field.name, None)
                 if container:
                     try:
@@ -680,13 +714,12 @@ class Base:
            p_className, or of self's class if not specified.'''
         if className and not isinstance(className, str): raise Exception(NO_STR)
         handler = self.H()
-        database = handler.server.database
         className = className or self.class_.name
-        return database.search(handler, className, **kwargs)
+        return self.database.search(handler, className, **kwargs)
 
     def search1(self, className=None, **kwargs):
         '''Performs a search for a single object'''
-        r = self.search(className, **kwargs)
+        r = self.search(className, maxResults=1, **kwargs)
         return r[0] if r else None
 
     def count(self, className=None, **kwargs):
@@ -742,9 +775,7 @@ class Base:
            class. Returns True if at least one change has been actually
            performed in at least one index.'''
         if self.isTemp(): raise Exception(INDEX_TEMP)
-        handler = self.H()
-        database = handler.server.database
-        return database.reindexObject(handler, self, **kwargs)
+        return self.database.reindexObject(self.H(), self, **kwargs)
 
     def getCatalog(self, raiseError=False):
         '''Returns the catalog storing p_self (if indexed)'''
@@ -764,7 +795,7 @@ class Base:
       <x if="breadcrumb.sup">::breadcrumb.sup</x>
       <x for="part in breadcrumb.parts"
          var2="link=not loop.part.last and part.view">
-       <img if="not loop.part.first" src=":url('arrowsA.svg')"
+       <img if="not loop.part.first" src=":svg('arrowsA')"
             class="iconS" style="transform: rotate(270deg)"/>
        <!-- Display the title or a link to the object -->
        <span if="not link" class="title">::part.title</span>
@@ -813,7 +844,7 @@ class Base:
         <a class="clickable"
            onclick=":'submitAppyForm(%s,%s,%s)' % \
                       (q('save'), q(current.name), q('view'))">
-         <img src=":url('save.svg')" class=":picto"/>
+         <img src=":svg('save')" class=":picto"/>
          <div style=":'display:%s' % config.ui.pageDisplay">::label</div>
         </a>
        </div>
@@ -825,7 +856,7 @@ class Base:
         <a class="clickable" name="cancel"
            onclick=":'submitAppyForm(%s,%s,%s)' % \
                       (q('cancel'), q(current.name), q('view'))">
-         <img src=":url('cancel.svg')" class=":picto"/>
+         <img src=":svg('cancel')" class=":picto"/>
          <div style=":'display:%s' % config.ui.pageDisplay">::label</div>
         </a>
        </div>
@@ -836,7 +867,7 @@ class Base:
                not o.Lock.isSet(o, user, page)" var2="label=_('object_delete')">
        <div>
         <a class="clickable" onclick=":'onDeleteObject(%s)' % q(o.url)">
-         <img src=":url('delete.svg')" class=":picto"/>
+         <img src=":svg('delete')" class=":picto"/>
          <div style=":'display:%s' % config.ui.pageDisplay">::label</div>
         </a>
        </div>
@@ -866,8 +897,8 @@ class Base:
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def create(self, _name, secure=False, raiseOnWrongAttribute=True,
-               executeMethods=True, initialComment=None, initialState=None,
-               **kwargs):
+               executeMethods=True, indexIt=True, initialComment=None,
+               initialState=None, **kwargs):
         '''Create a new instance of a Appy class'''
         # If p__name is the name of a field, the created object will be linked
         # to p_self via this field. Else, p__name must correspond to a root
@@ -904,10 +935,9 @@ class Base:
             className = _name
             field = None
         # Create the object
-        handler = self.H()
-        database = handler.server.database
-        o = database.new(handler, className, id=id, secure=secure,
-                       initialComment=initialComment, initialState=initialState)
+        o = self.database.new(self.H(), className, id=id, secure=secure,
+                              initialComment=initialComment,
+                              initialState=initialState)
         # Set object attributes
         for name, value in kwargs.items():
             try:
@@ -923,7 +953,9 @@ class Base:
         # initialise its object after it's been linked to its initiator object.
         if executeMethods: multicall(o, 'onEdit', False, True)
         # Index the object (when relevant) and return it
-        if o.class_.isIndexable():
+        if indexIt and o.class_.isIndexable():
+            # p_indexIt may be False if the object is, in a first step, created
+            # "empty" and field values are filled afterwards.
             o.reindex()
         return o
 
@@ -982,7 +1014,7 @@ class Base:
 
     def delete(self, historize=False, executeMethods=True):
         '''Deletes p_self (see homonym method on the database object'''
-        return self.H().server.database.delete(self, historize, executeMethods)
+        return self.database.delete(self, historize, executeMethods)
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Translation method
@@ -1056,22 +1088,27 @@ class Base:
     # PXs
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    # The object, as shown in a list of referred (tied) objects
+    # The object, as shown in a list of referred (tied) objects. In this PX:
+    # v_io refers to the source object ;
+    # v_o  refers to the tied object.
     pxTied = Px('''
-     <tr var="ifield=initiator.field;
-              *=ifield.initPx(initiator.o, req, _ctx_) if _ctx_.ajaxSingle
-                                                       else None;
-              id=o.iid;
-              objectIndex=ifield.getIndexOf(initiator.o, o, raiseError=False);
+     <tr var="io=initiator.o;
+              ifield=initiator.field;
+              *=ifield.initPx(io, req, _ctx_) if _ctx_.ajaxSingle else None;
+              ohook=ifield.getHookFor(o);
+              backHook=ohook if layout == 'cell' else None;
+              target=ui.LinkTarget(ifield.class_, backHook, ifield.viaPopup);
+              objectIndex=ifield.getIndexOf(io, o, raiseError=False);
+              cbId='%s_%s' % (hook, currentNumber);
               mayView=guard.mayView(o);
-              cbId='%s_%s' % (hook, currentNumber)"
-         id=":id" valign=":ifield.rowAlign" class=":rowCss">
+              id=o.iid"
+         id=":ohook" valign=":ifield.rowAlign" class=":rowCss">
       <td for="column in columns" width=":column.width" align=":column.align"
           var2="field=column.field; isCB=field=='_checkboxes'"
           class=":selector.cbClass if isCB and selector else ''">
 
        <!-- Special column with the tied element's number -->
-       <x if="(objectIndex!=None) and (field=='_number')">:ifield.pxNumber</x>
+       <x if="objectIndex != None and field == '_number'">:ifield.pxNumber</x>
 
        <!-- Special column containing the tied element's checkbox -->
        <input if="isCB and mayView" type="checkbox" name=":hook" id=":cbId"
@@ -1083,17 +1120,17 @@ class Base:
       </td>
 
       <!-- Store data in this tr node allowing to ajax-refresh it -->
-      <script>:ifield.getAjaxDataRow(o, hook, rowCss=rowCss, \
+      <script>:ifield.getAjaxDataRow(o, ohook, hook, rowCss=rowCss, \
                currentNumber=currentNumber, cbChecked=cbId, ajaxSingle=True, \
-               initiator=':%d:%s' % (initiator.o.iid, ifield.name))</script>
+               initiator=':%d:%s' % (io.iid, ifield.name))</script>
      </tr>''')
 
     # The object, as shown in a list of search results
     pxResult = Px('''
      <tr valign=":uiSearch.search.rowAlign" class=":rowCss"
-         var="id=o.iid; mayView=guard.mayView(o); backHook=str(id);
-              cbId='%s_%s' % (mode.checkboxesId, currentNumber)"
-         id=":id">
+         var="id=o.iid; ohook=mode.getHookFor(o); mayView=guard.mayView(o);
+              backHook=ohook; cbId='%s_%s' % (mode.checkboxesId, currentNumber)"
+         id=":ohook">
       <td for="column in mode.columns"
           var2="field=column.field; isCB=field=='_checkboxes'"
           id=":'field_%s' % field.name|field"
@@ -1110,7 +1147,7 @@ class Base:
       </td>
 
       <!-- Store data in this tr node allowing to ajax-refresh it -->
-      <script>:mode.getAjaxDataRow(o, rowCss=rowCss, \
+      <script>:mode.getAjaxDataRow(o, ohook, rowCss=rowCss, \
                currentNumber=currentNumber, cbChecked=cbId)</script>
      </tr>''')
 
@@ -1152,6 +1189,8 @@ class Base:
        <input type="hidden" name="popup" value=":popup"/>
        <input type="hidden" name="page" value=":page"/>
        <input type="hidden" name="nav" value=":req.nav or ''"/>
+       <input type="hidden" name="referer"
+              value=":req.referer or o.referer or ''"/>
        <input type="hidden" name="_get_" value=":req._get_ or ''"/>
        <input type="hidden" name="gotoPage" value=""/>
        <input type="hidden" name="gotoLayout" value=""/>
@@ -1230,10 +1269,10 @@ class Base:
         # (p_self) created from a Ref field on a source object (the initiator).
         # The method returns None if there is no initiator.
         req = self.req
-        nav = req.nav or req.onav
-        if not nav or (nav == 'no'): return
         if not search:
             # Try to get a "classic" initiator, from the "[o]nav" key
+            nav = req.nav or req.onav
+            if not nav or nav == 'no': return
             fieldType, info = nav.split('.', 1)
             field = eval(fieldType.capitalize())
             if field.initiator:
@@ -1246,14 +1285,14 @@ class Base:
             ref = req._ref or req.ref
             if not ref:
                 criteria = Criteria.readFromRequest(self.H())
-                if criteria and ('_ref' in criteria): ref = criteria['_ref']
+                if criteria and '_ref' in criteria: ref = criteria['_ref']
             if ref:
                 return self.getObject(ref.split(':')[0])
 
     def getFolder(self, create=False, withRelative=False):
         '''Gets, as a pathlib.Path instance, the folder where binary files
            related to this object are stored on the filesystem.'''
-        return self.H().server.database.getFolder(self, create, withRelative)
+        return self.database.getFolder(self, create, withRelative)
 
     def cancel(self, initiator=None, popup=False, isTemp=False):
         '''Redirect the user after an object edition has been canceled'''
@@ -1278,17 +1317,7 @@ class Base:
         if popup: return Iframe.goBack(self.tool, initiator)
         # Determine, if not defined yet, the URL to go back to
         if self.gotoSet(): return
-        if not url:
-            if initiator:
-                # Go back to the initiator page
-                url = initiator.getUrl()
-            elif isTemp:
-                # Go back to home page
-                url = self.tool.computeHomePage()
-            else:
-                # Return to p_self's view page
-                url = self.getUrl(sub='view', page=page, nav=req.nav or 'no')
-        self.goto(url)
+        self.goto(url or self.req.referer)
 
     traverse['save'] = 'perm:write'
     def save(self):
@@ -1312,7 +1341,7 @@ class Base:
         # validation failed or a confirmation is required.
         if r: return r
         # Update the object in the database
-        text = self.H().server.database.update(self, validator, initiator)
+        text = self.database.update(self, validator, initiator)
         # Recompute cached info on the guard: things may have changed
         guard = self.guard
         guard.cache()
@@ -1375,7 +1404,7 @@ class Base:
             # Let the initiator complete URL parameters when relevant
             initiator.updateParameters(params)
         # Create a temp object
-        o = handler.server.database.new(handler, className, temp=True)
+        o = self.database.new(handler, className, temp=True)
         # Populate p_o's edit form fields with a template object if present
         templateId = req.template
         if templateId: params['template'] = templateId
