@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -23,6 +23,7 @@ from persistent.list import PersistentList
 from appy.px import Px
 from appy import utils
 from appy.database import catalog
+from appy.model.utils import Fake
 from appy.tr import FieldTranslations
 from appy.utils import string as sutils
 from appy.model.utils import Object as O
@@ -38,36 +39,63 @@ XSS_DETECTED = 'Detected Javascript in user input.'
 XSS_WARNING  = 'Your behaviour is considered a security attack. System ' \
                'administrator has been warned.'
 UNINDEXED    = 'Field "%s": cannot retrieve catalog version of unindexed field.'
+AJAX_EDITED  = 'Ajax-edited %s%s on %s.'
+METH_ERROR   = 'Method %s:\n%s'
+DEF_ERR      = 'Field %d.%s: error. %s.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Show:
     '''Provides some frequently used values for field's "show" attributes'''
+
+    # Attribute Field:show allows to specify one or several layouts onto which a
+    # field can be rendered. Here are the possible layouts.
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # "view"    | The standard page rendering an object and its fields
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # "edit"    | The standard page rendering the form allowing to edit an
+    #           | object and its fields.
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # "result"  | A column within a list of objects being search results or tied
+    #           | objects, or some similar zone on alternate views, like a grid.
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # "sub"     | On search/ref results, this zone corresponds to the range of
+    #           | icons and/or buttons present besides every object, near its
+    #           | title.
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # "buttons" | On an object view/edit page, it corresponds to the sub-zone
+    #           | containing the range of icons, actions or other fields.
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    # "xml"     | Special, non-graphical layout representing the (marshalled)
+    #           | XML view of an object.
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
     # As a convention, a trailing underscore indicates a negation
 
-    # All layouts but edit. To use for users that can consult the field value
+    # All layouts but "edit". To use for users that can consult the field value
     # but cannot modify it. Also used for fields like some Ref fields that are
     # never manipulated via "edit" layouts.
-    E_ = ('view', 'result', 'xml')
-    # A variant, without "result"
-    ER_ = ('view', 'xml')
-    # A variant, with layout "buttons" instead of "result". "buttons"
-    # corresponds, on lists, to the range of icons and/or buttons present in the
-    # "title" column.
-    E_B = ('view', 'buttons', 'xml')
-    B = ('edit', 'view', 'buttons', 'xml')
+    E_  = ('view', 'result', 'xml')
+    ER_ = ('view', 'xml') # A variant, without "result"
+    E_S = ('view', 'sub', 'xml') # A variant, with "sub" instead of "result"
+    S   = ('edit', 'view', 'sub', 'xml')
+    BS  = ('buttons', 'sub') # Both in pxButtons and within lists of objects
+    BR  = ('buttons', 'result') # Appropriate for some fields like pods
+
+    # All layouts but "view". To use typically when, on "view", the field value
+    # is already shown as a part of some custom widget.
+    V_  = ('edit', 'result', 'xml')
+
+    # All layouts but "view" and "edit'. To use when you need both E_ and V_.
+    RX  = VE_ = ('result', 'xml')
 
-    # All layouts but view. To use typically when, on view, the field value is
-    # already shown as a part of some custom widget.
-    V_ = ('edit', 'result', 'xml')
-    # All layouts but view and edit. To use when you need both E_ and V_.
-    RX = VE_ = ('result', 'xml')
     # This set is used for showing workflow transitions in lists of objects
-    TR = ('view', 'result')
+    TR  = ('view', 'result')
+
     # This is, a.o., for custom widgets whose edit and view variants are not
     # that different, but that cannot be shown elsewhere (result, xml, etc).
-    VE = ('view', 'edit')
-    VX = ('view', 'xml')
-    EX = ('edit', 'xml')
+    VE  = ('view', 'edit')
+    VX  = ('view', 'xml')
+    EX  = ('edit', 'xml')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Initiator:
@@ -222,7 +250,7 @@ class Field:
       inRequest=field.valueIsInRequest(o, req, name, layout);
       error=req['%s_error' % name] or handler.validator.errors.get(name)|None;
       isMultiple=field.isMultiValued();
-      masterCss=field.slaves and ('master_%s' % name) or '';
+      masterCss='master_%s' % name if field.slaves else '';
       tagCss=tagCss|None;
       tagCss=field.getTagCss(tagCss, layout);
       o=o or tool;
@@ -247,26 +275,25 @@ class Field:
         # Call pxRender
         return self.pxRender(context)
 
-    # Show the field content for some object on a list of referred objects
+    # Show the field content for some object ("o") on a list of referred objects
     pxTied = Px('''
      <!-- The "title" field -->
      <x if="field.name == 'title'">
       <x if="mayView">
-       <x if="not ifield.menuUrlMethod or \
+       <x if="not ifield.menuUrlMethod or
               (layout != 'cell')">:ifield.pxObjectTitle</x>
-       <x if="ifield.menuUrlMethod and (layout == 'cell')"
+       <x if="ifield.menuUrlMethod and layout == 'cell'"
           var2="target,url=ifield.getMenuUrl(o, \
            tied)">::ui.Title.get(o, target=target, baseUrl=url)</x>
        <x if="not selector and guard.mayAct(o)">:ifield.pxObjectActions</x>
       </x>
       <div if="not mayView">
-       <img src=":url('password.svg')" class="iconS"/>
+       <img src=":svg('password')" class="iconS"/>
        <x>:_('unauthorized')</x></div>
      </x>
 
      <!-- Any other field -->
-     <x if="mayView and (field.name != 'title') and \
-            field.isShowable(o, 'result')"
+     <x if="mayView and field.name != 'title' and field.isShowable(o, 'result')"
         var2="layout='cell';
               fieldName=field.name;
               minimal=not field.inlineEdit">:field.pxRender</x>''')
@@ -276,29 +303,30 @@ class Field:
      <!-- The "title" field -->
      <x if="field.name == 'title'"
         var2="navInfo=mode.getNavInfo(currentNumber); target=mode.target;
-              backHook=backHook|None">
+             showActions=not popup and uiSearch.showActions and guard.mayAct(o);
+            backHook=backHook|None">
       <x if="mayView"
          var2="pageName=o.getDefaultPage('view');
                selectJs=popup and uiSearch.initiator.jsSelectOne(q,cbId) or ''">
        <x if="hasattr(o, 'getSupTitle')">::o.getSupTitle(navInfo)</x>
-       <x>::ui.Title.get(o, mode=mode.titleMode, nav=navInfo, target=target, \
-          page=pageName, popup=popup, selectJs=selectJs, highlight=True, \
-          backHook=backHook, pageLayout=uiSearch.search.pageLayoutOnView)</x>
+       <x>::ui.Title.get(o, mode=mode.titleMode, nav=navInfo, target=target,
+          page=pageName, popup=popup, selectJs=selectJs,
+          pageLayout=uiSearch.search.pageLayoutOnView,
+          highlight=True, backHook=backHook)</x>
        <span if="hasattr(o,'getSubTitle')"
              class=":class_.getCssFor(o,'subTitle')"
              style=":'display:inline' if mode.showSubTitles else 'display:none'"
              name="subTitle">::uiSearch.highlight(o.getSubTitle())</span>
 
        <!-- Actions -->
-       <div if="not popup and uiSearch.showActions and guard.mayAct(o)"
-            class=":ui.getActionsCss(uiSearch.actionsDisplay)"
-            var2="layout='buttons';
+       <div if="showActions" class=":ui.getActionsCss(uiSearch.actionsDisplay)"
+            var2="layout='sub';
                   editable=guard.mayEdit(o);
                   locked=o.Lock.isSet(o, user, 'main')">
 
-        <!-- Fields on layout "buttons" -->
-        <x if="not popup and (uiSearch.showActions == 'all')"
-           var2="fields=o.getFields('buttons', 'main'); layout='cell'">
+        <!-- Fields on layout "sub" -->
+        <x if="not popup and uiSearch.showActions == 'all'"
+           var2="fields=o.getFields(layout, 'main'); layout='cell'">
          <!-- Call cell and not pxRender to avoid having a table -->
          <x for="field in fields"
             var2="name=field.name;
@@ -312,30 +340,29 @@ class Field:
         <!-- Edit -->
         <x if="editable">
          <a if="not locked"
-            var2="linkInPopup=popup or (target.target != '_self')"
+            var2="linkInPopup=popup or target.target != '_self'"
             target=":target.target"
-            onclick=":target.getOnClick(backHook or str(o.iid))"
+            onclick=":target.getOnClick(backHook or ohook)"
             href=":o.getUrl(sub='edit', page=o.getDefaultPage('edit'), \
                             nav=navInfo, popup=linkInPopup)">
-          <img src=":url('edit.svg')" class="iconS" title=":_('object_edit')"/>
+          <img src=":svg('edit')" class="iconS" title=":_('object_edit')"/>
          </a>
          <x if="locked" var2="lockStyle='iconS'; page='main'">::o.Lock.px</x>
         </x>
 
         <!-- Delete -->
         <img if="not locked and guard.mayDelete(o)" class="clickable iconS"
-             src=":url('deleteS.svg')" title=":_('object_delete')"
+             src=":svg('deleteS')" title=":_('object_delete')"
              onClick=":'onDeleteObject(%s)' % q(o.url)"/>
        </div>
       </x>
       <x if="not mayView">
-       <img src=":url('password.svg')" class="iconS"/>
+       <img src=":svg('password')" class="iconS"/>
        <x>:_('unauthorized')</x>
       </x>
      </x>
      <!-- Any other field -->
-     <x if="mayView and (field.name != 'title') and \
-            field.isShowable(o, 'result')"
+     <x if="mayView and field.name != 'title' and field.isShowable(o, 'result')"
         var2="layout='cell';
               fieldName=field.name;
               minimal=not field.inlineEdit">:field.pxRender</x>''')
@@ -354,7 +381,7 @@ class Field:
     # Displays validation-error-related info about a field
     def validationPx(c):
         '''Python implementation of validation PX'''
-        url = c.url('warning.svg')
+        url = c.svg('warning')
         if c.error:
             r = '<abbr title="%s"><img src="%s" class="iconS"/></abbr>' % \
                 (c.error, url)
@@ -367,23 +394,18 @@ class Field:
     # Displays the fact that a field is required
     pxRequired = Px(lambda c: '<span class="required">*</span>')
 
-    # Button for showing changes to the field
+    # Button for showing / hiding changes to the field
     pxChanges = Px('''
-     <div if="zobj.hasHistory(name)" style="margin-bottom: 5px">
-      <!-- Button for showing the field version containing changes -->
-      <input if="not showChanges"
-             var2="label=_('changes_show');
-                   css=ztool.getButtonCss(label)" type="button" class=":css"
-             value=":label" style=":url('changes', bg=True)"
-             onclick=":'askField(%s,%s,%s,null,%s)' % \
-                       (q(tagId), q(obj.url), q('view'), q('True'))"/>
-      <!-- Button for showing the field version without changes -->
-      <input if="showChanges"
-             var2="label=_('changes_hide');
-                  css=ztool.getButtonCss(label)" type="button" class=":css"
-             value=":label" style=":url('changesNo', bg=True)"
-             onclick=":'askField(%s,%s,%s,null,%s)' % \
-                       (q(tagId), q(obj.url), q('view'), q('False'))"/>
+     <div if="not o.history.isEmpty(name)" style="margin-bottom: 5px">
+      <input type="button"
+        var="suffix='hide' if showChanges else 'show';
+             label=_('changes_%s' % suffix);
+             css=ui.Button.getCss(label);
+             icon='changesNo' if showChanges else 'changes';
+             bp='False' if showChanges else 'True'"
+        class=":css" value=":label" style=":url(icon, bg=True)"
+        onclick=":'askField(%s,%s,%s,null,%s)' % (q(tagId), q(o.url),
+                                                  q('view'), q(bp))"/>
      </div>''')
 
     # Widget for filtering object values on query results
@@ -396,7 +418,7 @@ class Field:
              value=":mode.filters.get(name, '')"
              onkeydown=":'if (event.keyCode==13) document.getElementById(%s).
                           click()' % q(filterIdIcon)"/>
-      <img id=":filterIdIcon" class="clickable iconS" src=":url('funnel.svg')"
+      <img id=":filterIdIcon" class="clickable iconS" src=":svg('funnel')"
            onclick=":'askBunchFiltered(%s,%s)' % (q(mode.hook), q(name))"/>
      </x>''')
 
@@ -419,8 +441,8 @@ class Field:
         self.default = default
         # Default value on layout "edit". If None, self.default is used instead.
         self.defaultOnEdit = defaultOnEdit
-        # Must the field be visible or not?
-        self.show = show
+        # Must the field be visible or not ?
+        self.show = show    
         # When displaying/editing the whole object, on what page and phase must
         # this field value appear ?
         if page:
@@ -555,19 +577,19 @@ class Field:
         # use instead translations already defined on another field, use the
         # following attribute "label". A label is made of 2 parts: the prefix,
         # based on class name, and the name, which is the field name by default.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # If "label" is... | it will be understood as...
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # a string         | a new prefix = a new class name. Your field will
         #                  | use the label of an homonym field on another class.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # a tuple          | a tuple (prefix, name), defining both a new prefix
         #                  | and a new field name. Your field will get the label
         #                  | of a field from another class, having another name.
         #                  | And if you want to reuse labels of another field
         #                  | defined on the same class, use a tuple of the form
         #                  |                 (None, name)
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.label = label
         # When you specify a default value "for search" (= "sdefault"), on a
         # search screen, in the search field corresponding to this field, this
@@ -637,16 +659,23 @@ class Field:
             self.descrId = '%s_descr' % self.labelId
             self.helpId  = '%s_help' % self.labelId
 
-    def __repr__(self):
+    def __repr__(self, start='<', end='>'):
         '''String representation for this field'''
+        # If p_self's repr goes to the UI, using surrounding chars "<" and ">"
+        # may cause XHTML escaping problems.
         name = '%s::%s' % (self.container.name, self.name) \
                if hasattr(self, 'container') else 'uninit'
-        return '<field %s (%s)>' % (name, self.__class__.__name__.lower())
+        return '%sfield %s (%s)%s' % (start, name,
+                                      self.__class__.__name__.lower(), end)
+
+    def asString(self):
+        '''p_self's alternate short string representation'''
+        return self.__repr__(start='', end='')
 
     def isMultiValued(self):
         '''Does this type definition allow to define multiple values?'''
         maxOccurs = self.multiplicity[1]
-        return (maxOccurs is None) or (maxOccurs > 1)
+        return maxOccurs is None or maxOccurs > 1
 
     def isSortable(self, inRefs=False):
         '''Can fields of this type be used for sorting purposes, when sorting
@@ -671,7 +700,7 @@ class Field:
     def isShowable(self, o, layout):
         '''When displaying p_o on a given p_layout, must we show this field ?'''
         # Check if the user has the permission to view or edit the field
-        perm = self.writePermission if (layout=='edit') else self.readPermission
+        perm = self.writePermission if layout == 'edit' else self.readPermission
         if not o.allows(perm): return
         # Evaluate self.show
         if callable(self.show):
@@ -798,7 +827,7 @@ class Field:
         #   are given to it, as its unique parameter.
         self.masterValue = utils.initMasterValue(masterValue)
 
-    def getCss(self, layout, r):
+    def getCss(self, o, layout, r):
         '''Complete list p_r with the names of CSS files that are required for
            displaying widgets of self's type on a given p_layout. p_r is not a
            set because order of inclusion of CSS files may be important.'''
@@ -807,7 +836,7 @@ class Field:
                 if name not in r:
                     r.append(name)
 
-    def getJs(self, layout, r, config):
+    def getJs(self, o, layout, r, config):
         '''Completes list p_r with the names of Javascript files that are
            required for displaying widgets of self's type on a given p_layout.
            p_r is not a set because order of inclusion of Javascript files may
@@ -842,41 +871,46 @@ class Field:
                 # Access the inner value
                 if r:
                     if outerField.outerType == PersistentList:
-                        # The row ID is the index if an element from a list
+                        # The row ID is the index of an element from a list
                         rowId = int(rowId)
                         if rowId < len(r):
                             r = getattr(r[rowId], name, None)
                         else:
                             r = None
                     else:
-                        # The row ID is a key from a dict
-                        r = r.get(rowId, None)
+                        # The row ID is a key from a dict. Remove the -d- prefix
+                        r = r.get(rowId[3:], None)
                         if r: r = r.get(name, None)
             # Return an empty string if fromRequest is True
-            if fromRequest and (r is None): r = ''
+            if fromRequest and r is None: r = ''
         else:
             r = self.getRequestValue(o, self.name) if fromRequest \
                 else o.values.get(self.name)
         return r
 
     def getRequestValue(self, o, requestName=None):
-        '''Gets a value for this field as carried in the request. In the
-           simplest cases, the request value is a single value whose name in the
-           request is the name of the field.
+        '''Gets a value for this field as carried in the request'''
+        #  In the simplest cases, the request value is a single value whose name
+        # in the request is the name of the field.
 
-           Sometimes, several request values must be combined (ie: see the
-           overriden method in the Date class).
+        # Sometimes, several request values must be combined (ie: see the
+        # overriden method in the Date class).
 
-           Sometimes (ie, a field within a List/Dict), the name of the request
-           value(s) representing the field value does not correspond to the
-           field name (ie: the request name includes information about
-           the container field). In this case, p_requestName must be used for
-           searching into the request, instead of the field name (self.name).'''
-        return o.req[requestName or self.name]
+        # Sometimes (ie, a field within a List/Dict), the name of the request
+        # value(s) representing the field value does not correspond to the field
+        # name (ie: the request name includes information about the container
+        # field). In this case, p_requestName must be used for searching into
+        # the request, instead of the field name (self.name).
+
+        # Ensure multiplicities are respected
+        r = o.req[requestName or self.name]
+        if not isinstance(r, __builtins__['list']) and self.isMultiValued():
+            r = [] if r is None else [r]
+        return r
 
     def setRequestValue(self, o):
-        '''Sets, in the request, field value on p_obj in its "request" form
-           (=the way the value is carried in the request).'''
+        '''Sets, in the request, field value on p_o in its "request" form (=the
+           way the value is carried in the request).'''
         # Get a copy of the field value on p_obj and put it in the request
         value = self.getCopyValue(o)
         if value is not None:
@@ -895,17 +929,17 @@ class Field:
         '''Gets, on p_o(bject), the value for this field (p_self)'''
 
         # Possible values for parameters are described hereafter.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # name    | p_name can be different from self.name if p_self is a
         #         | sub-field into a List: in this case, it includes the row
         #         | number.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # layout  | In most cases, we don't care about the layout for getting
         #         | the value of p_self on p_o. One exception is that a default
         #         | value can be defined specifically on layout "edit". In that
         #         | case, p_layout must be available in order to determine the
         #         | possible specific default value for this layout.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # single  | This attribute, although defined here at the most abstract
         #         | level, has only sense for "multi-values", like values from
         #         | the Ref field. Such values are always stored as lists or
@@ -914,7 +948,7 @@ class Field:
         #         | This can be achieved with p_single being True. If p_single
         #         | is None or False, the retrieved value will be the whole
         #         | list or tuple.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Get the value from the database
         value = self.getStoredValue(o, name)
         if self.isEmptyValue(o, value):
@@ -936,11 +970,11 @@ class Field:
                     # but it they depend on values set at (b), and are cached
                     # and indexed, (c) will get the wrong, cached value.
                     default = self.callMethod(o, default, cache=False)
-                except Exception:
-                    # Already logged. Here I do not raise the exception,
-                    # because it can be raised as the result of reindexing
-                    # the object in situations that are not foreseen by
-                    # method in self.default.
+                except Exception as e:
+                    o.log(DEF_ERR % (o.iid, self.name, str(e)), type='error')
+                    # Do not raise the exception, because it can be raised as
+                    # the result of reindexing the object in situations that are
+                    # not foreseen by v_default.
                     default = None
             return default
         return value
@@ -1016,7 +1050,7 @@ class Field:
         else:
             r = self.getUniFormattedValue(o, value, contentLanguage=language)
         # If p_r is a list, transform it into a HTML "ul" tag
-        if type(r) in utils.sequenceTypes:
+        if isinstance(r, utils.sequenceTypes):
             r = '<ul>%s</ul>' % ''.join(['<li>%s</li>' % v for v in r])
         elif not r:
             r = empty
@@ -1074,7 +1108,7 @@ class Field:
                   (hook, o.url, layout)
         if inlineEdit == 'icon':
             r = '<img src="%s" title="%s" class="inlineIcon iconS" %s/>%s' % \
-                (o.buildUrl('edit.svg'), o.translate('object_edit'), onClick,
+                (o.buildSvg('edit'), o.translate('object_edit'), onClick,
                  value)
         else:
             r = '<span class="editable" %s>%s</span>' % (onClick, value)
@@ -1083,7 +1117,7 @@ class Field:
     def getFakeObject(self, value, req):
         '''Returns a fake object, to mimic an object storing this p_value for
            field p_self.'''
-        return O(values={self.name: value}, req=req, id=0, iid=0)
+        return Fake(self, value, req)
 
     def getIndexType(self, class_=False):
         '''Returns the name of the class corresponding to the index used to
@@ -1234,10 +1268,14 @@ class Field:
            Javascript function that will be called when its value changes (in
            order to update slaves).'''
         if not self.slaves: return ''
+        url = o.url
         # When the field is on a search screen, we need p_className
-        name = "'%s'" % className if className else 'null'
-        return "updateSlaves(this,null,'%s','%s',%s,true)" % \
-               (o.url, layout, name)
+        if className:
+            name = "'%s'" % className
+            url = '%s/@%s' % (url, className)
+        else:
+            name = 'null'
+        return "updateSlaves(this,null,'%s','%s',%s,true)" % (url, layout, name)
 
     def isEmptyValue(self, o, value):
         '''Returns True if the p_value must be considered as an empty value'''
@@ -1363,17 +1401,21 @@ class Field:
             # Retrieve the outer, inner fields and row number
             name, px = req.px.rsplit(':', 1)
             outer, inner, i = name.split('*')
+            i = int(i)
             outer = o.getField(outer)
-            # Update the value
+            # Update the value. Clone the row to ensure the ZODB will detect the
+            # object change. Indeed, rows are instances of not-persistent class
+            # appy.Object.
             value = outer.getValue(o)
-            row = value[int(i)]
+            row = value[i]
             setattr(row, inner, reqValue)
+            value[i] = row.clone()
             # Store the complete value again on p_o
             o.values[outer.name] = value
         # Update obj's last modification date
         o.modified = DateTime()
         o.reindex()
-        o.log('ajax-edited %s%s on %s.' % (self.name, part, o.id))
+        o.log(AJAX_EDITED % (self.name, part, o.id))
 
     def callMethod(self, o, method, cache=True):
         '''This method is used to call a p_method on p_o. p_method is part of
@@ -1393,7 +1435,7 @@ class Field:
             try:
                 return method(o, self)
             except Exception as e:
-                o.log('method %s:\n%s' % (method.__name__, tb), type='error')
+                o.log(METH_ERROR % (method.__name__, tb), type='error')
                 # Raise the initial error
                 raise te
 
