@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -55,6 +55,22 @@ class ActionInitiator(Initiator):
             r = msg
         return r
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+class Multi:
+    '''Management of muti-actions = actions that execute on a set of objects,
+       and not on a single one.'''
+
+    def __init__(self, back, check):
+        # The DOM node to ajax-refresh once the multi-action has been executed
+        self.back = back
+        # The DOM node storing the un/checked objects onto which the
+        # multi-action must be executed.
+        if check and ',' in check:
+            # This is a search being embedded into another field. In that
+            # case, the hook is simply 'search'.
+            check = 'search'
+        self.check = check
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Action(Field):
     '''An action is a Python method that can be triggered by the user on a
@@ -84,8 +100,8 @@ class Action(Field):
     view = cell = query = Px('''
      <form var="isFake=field.isFake(o, _);  
                 formId='%d_%s_form' % (o.iid, name);
-                label=_(field.labelId);
-                multi=multi|False;
+                label=_('label', field=field);
+                multi=multi|None;
                 className=className|o.class_.name;
                 inputTitle=field.getInputTitle(o, label);
                 inputLabel=field.getInputLabel(label, layout);
@@ -127,9 +143,9 @@ class Action(Field):
       <!-- As picto -->
       <a if="asPicto" class=":'help fake' if isFake else 'clickable'"
          title=":isFake if isFake else ''"
-         onclick=":field.getOnClick(o, name, req, layout, q, multi) \
-                   if not isFake else ''">
-       <img src=":url(field.icon, base=field.iconBase)" class=":picto"/>
+         onclick=":field.getOnClick(_ctx_) if not isFake else ''">
+       <img src=":url(field.icon, base=field.iconBase, ram=field.iconRam)"
+            class=":picto"/>
        <div style=":'display:%s' % config.ui.pageDisplay">::inputTitle</div>
       </a>
 
@@ -138,13 +154,15 @@ class Action(Field):
        <!-- The button for executing the action -->
        <input if="not isFake" type="button" class=":css" title=":inputTitle"
               value=":inputLabel"
-              style=":url(field.sicon, base=field.siconBase, bg='18px 18px')"
-              onclick=":field.getOnClick(o, name, req, layout, q, multi)"/>
+              style=":url(field.sicon, base=field.siconBase,
+                          ram=field.siconRam, bg='18px 18px')"
+              onclick=":field.getOnClick(_ctx_)"/>
 
        <!-- ... or the fake button -->
        <input if="isFake" type="button" class=":'%s fake' % css"
               title=":isFake" value=":inputLabel"
-              style=":url(field.sicon, base=field.siconBase, bg='18px 18px')"/>
+              style=":url(field.sicon, base=field.siconBase,
+                          ram=field.siconRam, bg='18px 18px')"/>
       </x>
      </form>''')
 
@@ -163,18 +181,18 @@ class Action(Field):
         # Attribute "action" must hold a method or a list/tuple of methods.
         # In most cases, every method will be called without arg, but there are
         # exceptions (see parameters "options" and "confirm").
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # If the such method returns...
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #      None      | The return status is implicitly considered to be a
         #                | success and a standard translated message will be
         #                | shown to the user.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #     success    | It is a boolean value representing the status of the
         #                | action: success (True) or failure (False). Depending
         #                | on its status, a different standard message will be
         #                | returned to the user.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (success, msg) | Is is a 2-tuple. The "success" part (a boolean
         #                | value) has exactly the same meaning as the
         #                | hereabove-described "success" case. The "message"
@@ -182,49 +200,61 @@ class Action(Field):
         #                | custom translated message, potentially
         #                | XHTML-formatted, containing human-readable details
         #                | about the action success or failure.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # When several methods are specified, their individual results will be
         # merged, ie, to return a concatenated set of messages to the user.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.action = action
-        # ----------------------------------------------------------------------
+
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Attribute "result" can hold the following values.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "computation"  | (the default case) the action will simply compute
         #                | things and redirect the user to the same page, with
         #                | some status message about execution of the action;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #    "file"      | the result is the binary content of a file that the
         #                | user will download. In that case, the "message" part
         #                | of the method result must be an open file handler;
         #                | after the action has been executed, Appy will close
         #                | it;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #   "redirect"   | the action will lead to the user being redirected to
         #                | some other page. The URL of this page must be given
         #                | in the "message" part of the method result. If
         #                | "message" is None, we can't determine where to
         #                | redirect and we will fallback to case "computation".
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.result = result
+
         # If self.result is "file", the "disposition" for downloading the file
         # is defined in self.downloadDisposition and can be "attachment" or
         # "inline".
         self.downloadDisposition = downloadDisposition
+
         # If "confirm" is True, a popup will ask the user if he is really sure
         # about triggering this action. If "confirm" is "comment", the same
         # effect will be achieved, but the popup will contain a field allowing
         # to enter a comment; this comment will be available to self.action's
         # method(s), via a parameter named "comment".
         self.confirm = confirm
-        # If you want to specify an icon being in folder <yourApp>/static,
-        # specify "<yourApp>/<yourImage>". Else, default icon "appy/action.svg"
-        # will be used.
-        self.icon, self.iconBase = utils.iconParts(icon or 'action.svg')
+
+        # If you want to use a specific (SVG) icon for this action, store it in
+        # folder <yourApp>/static and set its name (ie "myimage.svg") in
+        # parameter "icon". Optimized SVG images are recommended. You can use
+        # another image format (ie, PNG); in that case, though, you must prefix
+        # the image name with you app name. For example, if your app is named
+        # "MyApp" and your icon is MyApp/static/myimage.png, attribute "icon"
+        # must hold "MyApp/myimage.png". If you do not specify any image,
+        # default icon action.svg will be used (it is stored in appy/ui/static).
+        icon = icon or 'action.svg'
+        self.icon, self.iconBase, self.iconRam = utils.iconParts(icon)
+
         # You may specify, in attribute "sicon", an alternate icon suitable when
-        # rendered as a small icon.
+        # rendered as a small icon. Optimized SVG images are recommended.
         sicon = sicon or icon or 'action.svg'
-        self.sicon, self.siconBase = utils.iconParts(sicon)
+        self.sicon, self.siconBase, self.siconRam = utils.iconParts(sicon)
+
         # Call the base constructor
         Field.__init__(self, None, (0,1), default, defaultOnEdit, show, page,
           group, layouts, move, False, True, None, None, False, None,
@@ -232,12 +262,14 @@ class Action(Field):
           masterValue, focus, historized, mapping, generateLabel, label, None,
           None, None, None, False, False, view, cell, edit, xml, translations)
         self.validable = False
+
         # There are various ways to render the action in the ui:
         # "button"   (the default) as a button;
         # "icon"     as an icon on layouts where compacity is a priority
         #            (ie, within lists of objects) but still as a button on the
         #            "view" layout.
         self.render = render
+
         # An action may receive options: once the user clicks on the action's
         # icon or button, a form is shown, allowing to choose options. In order
         # to achieve this, specify an Appy class in field "options". self.action
@@ -245,13 +277,14 @@ class Action(Field):
         # named "option". After the action has been executed, this instance will
         # be deleted.
         self.options = options
+
         # By default, an action is performed on a single object: self.action is
         # an instance method (or a list of instance methods) executing on a
         # single instance of the class defining this action. It is also possible
         # to define an action on a list of objects being results of a search
         # (= a "multi-action"). Here are the steps to follow to define such a
         # multi-action.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  1 | Define an Action field on your class, with its "show" attribute
         #    | being "query" (or a method returning it) in order to tell Appy
         #    | that the action will not be rendered on an object's standard
@@ -263,7 +296,7 @@ class Action(Field):
         #    | tool as single arg, instead of an instance of your class. In
         #    | order to clearly identify this little cheat, name this first arg
         #    | "tool" instead of "self".
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  2 | Defining an action field on the "query" layout, as explained in
         #    | the previous step, prevents the field from being shown on the
         #    | standard object's layouts, but does not automatically display the
@@ -301,7 +334,7 @@ class Action(Field):
         #    | parameters when appropriate. Recall that attribute
         #    | "searchAdvanced" can itself be dynamic: it can be defined as a
         #    | static method accepting the tool as unique arg.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  3 | The method(s) defined in p_self.action will have these
         #    | particularities:
         #    | - as for a "show" method, it will receive the tool as first arg,
@@ -311,11 +344,11 @@ class Action(Field):
         #    | Moreover, if your search is defined with checkboxes=True, your
         #    | method(s) "objects" will contain only objects being checked in
         #    | the UI.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Note that you can use a multi-action having an "options" attribute. In
         # that case, method(s) in parameter "action" must both have args
         # "options" and "objects".
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # If the action can't be executed, but, instead of not showing the
         # corresponding button at all in the UI, you prefer to show a "fake"
         # button with an explanation about why the action can't be currently
@@ -323,6 +356,7 @@ class Action(Field):
         # returning the explanation as a string, or True if you simply want a
         # standard message to appear.
         self.fake = fake
+
         # Ensure validity of parameter values
         self.checkParameters()
 
@@ -349,23 +383,22 @@ class Action(Field):
             # of the class in the path if p_self is a multi-action.
             part = '@%s/' % self.container.name if layout == 'query' else ''
             return '%s/%s%s/perform' % (o.url, part, self.name)
-    
-    def getOnClick(self, o, name, req, layout, q, multi):
+
+    def getOnClick(self, ctx):
         '''Gets the JS code to execute when the action button is clicked'''
         # Determine the ID of the form to submit
-        formId = '%d_%s_form' % (o.iid, name)
+        o = ctx.o
+        q = ctx.q
+        formId = '%d_%s_form' % (o.iid, ctx.name)
         # Determine the back hook and check hook (if multi)
+        multi = ctx.multi
         if multi:
-            back = 'searchResults'
-            check = req.search
-            if ',' in check:
-                # This is a search being embedded into another field. In that
-                # case, the hook is simply 'search'.
-                check = 'search'
-            check = q(check)
+            back = multi.back
+            check = multi.check
         else:
-            back = str(o.iid) if layout == 'cell' else None
-            check = 'null'
+            back = ctx.ohook if ctx.layout == 'cell' else None
+            check = None
+        check = q(check) if check else 'null'
         if not self.options:
             # Determine the parameters for executing the action
             showComment = 'true' if self.confirm == 'comment' else 'false'
@@ -390,6 +423,32 @@ class Action(Field):
                  (target.onClick, q(formId), check)
         return js
 
+    def getTargetObjects(self, o, req):
+        '''In the context of a multi-action, retrieve the list of target objects
+           onto which the action must be applied.'''
+        # If semantics indicates that checked objects must be kept, retrieve
+        # these objects.
+        tool = o.tool
+        ids, unchecked = Search.getCheckedInfo(req)
+        if not unchecked:
+            r = []
+            for iid in ids:
+                cobj = tool.getObject(iid)
+                if cobj: # The object may have been deleted in the meanwhile
+                    r.append(cobj)
+            return r
+        # Get all the potential target objects, from a Search or Ref
+        searchParams = req.searchParams
+        if searchParams:
+            # Replay the search to get the list of objects
+            r = Search.replay(tool, searchParams)
+        else:
+            # Get objets from a ref
+            r = getattr(o, req.fieldName.split('*',1)[0])
+        # Remove those not being checked in the UI
+        Search.keepCheckedResults(req, r, unchecked=ids)
+        return r
+
     def execute(self, o, options=None):
         '''Execute the action on p_o. Returns a tuple (b_success, s_message)'''
         # Get args to give to method(s) in self.action
@@ -401,11 +460,7 @@ class Action(Field):
         # Is that a multi-action ? A multi-action is an action to perform on a
         # list of objects instead of a single object.
         if req.multi:
-            # Replay the search to get the list of objects
-            objects = Search.replay(o.tool, req.searchParams)
-            # Remove those not being checked in the UI
-            Search.keepCheckedResults(req, objects)
-            args['objects'] = objects
+            args['objects'] = self.getTargetObjects(o, req)
         # Call method(s) in self.action
         if type(self.action) in utils.sequenceTypes:
             # There are multiple methods
@@ -447,7 +502,7 @@ class Action(Field):
 
     def isShowable(self, o, layout):
         '''Never show actions on an "edit" p_layout'''
-        return False if layout == 'edit' else Field.isShowable(self, o, layout)
+        return False if layout == 'edit' else super().isShowable(o, layout)
 
     def isFake(self, o, _):
         '''Must the shown button be a fake button ? If yes, the return value is
@@ -469,7 +524,7 @@ class Action(Field):
            action = the content of attribute "value" for the "input" field.'''
         # An icon is a button rendered without "value", excepted on the "view"
         # layout, where we still display it.
-        if (self.render == 'icon') and (layout != 'view'): return ''
+        if self.render == 'icon' and layout != 'view': return ''
         return label
 
     def getConfirmText(self, o):
@@ -514,7 +569,7 @@ class Action(Field):
             r = msg
         # Stop here if p_minimal is True
         if minimal: return r
-        if (result == 'computation') or not success:
+        if result == 'computation' or not success:
             # If we are called from an Ajax request, simply return msg
             if handler.isAjax():
                 r = msg
@@ -533,4 +588,10 @@ class Action(Field):
                 return o.goto(msg)
         else:
             return r
+
+    def getMulti(self, back, check):
+        '''In the context of a multi-action, create and return a Multi instance
+           defining useful info about the action, like the back and check
+           hooks.'''
+        return Multi(back, check)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
