@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -28,26 +28,42 @@ class Boolean(Field):
 
     class Layouts(Layouts):
         '''Boolean-specific layouts'''
+        es = 'f;lrv;-'
+
         # Default layout (render = "checkbox") ("b" stands for "base"), followed
         # by the "grid" variant (if the field is in a group with "grid" style).
-        b   = Layouts(edit=Layout('f;lrv;-', width=None), view='lf')
-        g   = Layouts(edit=Layout('frvl', width=None), view='fl')
-        d   = Layouts(edit=Layout('flrv;=d', width=None), view='lf')
+
+        b   = Layouts(edit=Layout(es,        width=None),   view='lf')
+        g   = Layouts(edit=Layout('frvl',    width=None),   view='fl')
+        d   = Layouts(edit=Layout('flrv;=d', width=None),   view='lf')
+
         # *d*escription also visible on "view"
-        dv  = Layouts(edit=d['edit'], view='lf-d')
-        h   = Layouts(edit=Layout('flhv', width=None), view='lf')
-        gd  = Layouts(edit=Layout('f;dv-', width=None), view='fl')
+
+        dv  = Layouts(edit=d['edit'],                       view='lf-d')
+        h   = Layouts(edit=Layout('flhv',    width=None),   view='lf')
+        gd  = Layouts(edit=Layout('f;dv-',   width=None),   view='fl')
+
+        # The base layout, plus bottom space
+
+        bs = Layouts(edit=Layout(es,         width=None,
+                                 css_class='bottomSpaceS'), view='lf')
+
         # The "long" version of the previous layout (if the description is
         # long), with vertical alignment on top instead of middle.
-        gdl = Layouts(edit=Layout('f;dv=', width=None), view='fl')
+
+        gdl = Layouts(edit=Layout('f;dv=',   width=None),   view='fl')
+
         # Centered layout, no description
-        c   = Layouts(edit='flrv|', view='lf|')
+
+        c   = Layouts(edit='flrv|',                         view='lf|')
+
         # Layout for radio buttons (render = "radios")
-        r   = Layouts(edit='f', view='f')
-        rl  = Layouts(edit='l-f', view='lf')
-        rld = Layouts(edit='l-d-f', view='lf')
-        grl = Layouts(edit='fl', view='fl')
-        gdr = Layouts(edit=Layout('d-fv=', width=None), view='fl')
+
+        r   = Layouts(edit='f',                             view='f')
+        rl  = Layouts(edit='l-f',                           view='lf')
+        rld = Layouts(edit='l-d-f',                         view='lf')
+        grl = Layouts(edit='fl',                            view='fl')
+        gdr = Layouts(edit=Layout('d-fv=',   width=None),   view='fl')
 
         @classmethod
         def getDefault(class_, field):
