@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -89,7 +89,7 @@ class Date(Field):
       <!-- The icon for displaying the calendar popup -->
       <x if="field.calendar">
        <input type="hidden" id=":name" name=":name"/>
-       <img id=":'%s_img' % name" src=":url('calendar.svg')" class="iconS"/>
+       <img id=":'%s_img' % name" src=":svg('calendar')" class="iconS"/>
        <script>::field.getJsInit(name, years)</script>
       </x>
 
@@ -104,8 +104,7 @@ class Date(Field):
        <tr var="fromName='%s_from' % name;
                 dayFromName='%s_from_day' % name;
                 monthFromName='%s_from_month' % name">
-        <td width="10px">&nbsp;</td>
-        <td><label>:_('search_from')</label></td>
+        <td><label>:_('date_from')</label></td>
         <td>
          <select id=":dayFromName" name=":dayFromName">
           <option value="">--</option>
@@ -125,7 +124,7 @@ class Date(Field):
          <!-- The icon for displaying the calendar popup -->
          <x if="field.calendar">
           <input type="hidden" id=":fromName" name=":fromName"/>
-          <img id=":'%s_img' % fromName" src=":url('calendar.svg')"
+          <img id=":'%s_img' % fromName" src=":svg('calendar')"
                class="iconS"/>
           <script>::field.getJsInit(fromName, years)</script>
          </x>
@@ -141,8 +140,7 @@ class Date(Field):
                 dayToName='%s_to_day' % name;
                 monthToName='%s_to_month' % name;
                 yearToName='%s_to_year' % name">
-        <td></td>
-        <td><label>:_('search_to')</label>&nbsp;&nbsp;&nbsp;&nbsp;</td>
+        <td><label>:_('date_to')</label></td>
         <td height="20px">
          <select id=":dayToName" name=":dayToName">
           <option value="">--</option>
@@ -162,8 +160,7 @@ class Date(Field):
          <!-- The icon for displaying the calendar popup -->
          <x if="field.calendar">
           <input type="hidden" id=":toName" name=":toName"/>
-          <img id=":'%s_img' % toName" src=":url('calendar.svg')"
-               class="iconS"/>
+          <img id=":'%s_img' % toName" src=":svg('calendar')" class="iconS"/>
           <script>::field.getJsInit(toName, years)</script>
          </x>
          <!-- Hour and minutes when relevant -->
@@ -220,18 +217,18 @@ class Date(Field):
           focus, historized, mapping, generateLabel, label, sdefault, scolspan,
           swidth, sheight, persist, False, view, cell, edit, xml, translations)
 
-    def getCss(self, layout, r):
+    def getCss(self, o, layout, r):
         '''CSS files are only required if the calendar must be shown'''
-        if self.calendar: Field.getCss(self, layout, r)
+        if self.calendar: Field.getCss(self, o, layout, r)
 
-    def getJs(self, layout, r, config):
+    def getJs(self, o, layout, r, config):
         '''Javascript files are only required if the calendar must be shown'''
-        if self.calendar: Field.getJs(self, layout, r, config)
+        if self.calendar: Field.getJs(self, o, layout, r, config)
 
     def getSelectableYears(self, o):
         '''Gets the list of years one may select for this field'''
         startYear = self.getAttribute(o, 'startYear')
-        r = range(startYear, self.endYear + 1)
+        r = list(range(startYear, self.endYear + 1))
         if self.reverseYears: r.reverse()
         return r
 
