@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -57,9 +57,9 @@ class Dict(List):
 
     # PX for rendering a single row
     pxRow = Px('''
-     <!-- Render a separatation row -->
+     <!-- Render a separation row -->
      <x if="not rowId">::text.get(field)</x>
-     <tr if="rowId" valign="top" class=":loop.rowId.odd and 'even' or 'odd'">
+     <tr if="rowId" valign="top" class=":'even' if loop.rId.odd else 'odd'">
       <x>:field.pxFirstCell</x>
       <td class="discreet">::text</td>
       <td for="subName, field in subFields" if="field" align="center"
@@ -72,15 +72,16 @@ class Dict(List):
             id=":'list_%s' % name" class="grid" width=":field.width"
             var2="keys=field.keys(o);
                   subFields=field.getSubFields(o, layout);
+                  swidths=field.getWidths(subFields);
                   o=alto|o">
       <!-- Header -->
       <tr valign="bottom">
-       <th width=":field.widths[0]"></th>
+       <th width=":swidths[0]"></th>
        <th for="subName, sub in subFields" if="sub"
-           width=":field.widths[loop.subName.nb + 1]">::_(sub.labelId)</th>
+           width=":swidths[loop.subName.nb + 1]">::sub.getListHeader(_ctx_)</th>
       </tr>
       <!-- Rows of data -->
-      <x for="rowId, text in keys">:field.pxRow</x>
+      <x for="rId, text in keys" var2="rowId='-d-%s' % rId">:field.pxRow</x>
      </table>''')
 
     def __init__(self, keys, fields, validator=None, multiplicity=(0,1),
@@ -110,9 +111,9 @@ class Dict(List):
         # that will determine content and style for the separator row.
         self.keys = keys
 
-    def computeWidths(self, widths):
-        '''Set given p_widths or compute default ones if not given'''
-        self.widths = widths or ([''] * (len(self.fields) + 1))
+    def getWidths(self, subFields):
+        '''Get the widths to appply to these p_subFields'''
+        return self.widths or [''] * (len(subFields) + 1)
 
     def getStorableValue(self, o, value, single=False):
         '''Gets p_value in a form that can be stored in the database'''
@@ -158,7 +159,7 @@ class Dict(List):
     def subValidate(self, o, value, errors):
         '''Validates inner fields'''
         for key, row in value.items():
-            for name, subField in self.fields:
+            for name, subField in self.getSubFields(o):
                 message = subField.validate(o, getattr(row, name, None))
                 if message:
                     setattr(errors, '%s*%s' % (subField.name, key), message)
