@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -155,8 +155,8 @@ class FileInfo:
 
     def getExtension(self):
         '''Get the file extension from the MIME type or from the upload name'''
-        if self.mimeType in mimeTypesExts:
-            return mimeTypesExts[self.mimeType]
+        if self.mimeType in utils.mimeTypesExts:
+            return utils.mimeTypesExts[self.mimeType]
         elif self.uploadName:
             parts = os.path.splitext(self.uploadName)
             if len(parts) > 1:
@@ -196,7 +196,7 @@ class FileInfo:
         # an identifier within the database p_folder.
         ext = utils.mimeTypesExts.get(self.mimeType) or 'bin'
         self.fsName = '%s.%s' % (name, ext)
-        self.uploadName = self.normalizeFileName(niceName)
+        self.uploadName = self.normalizeFileName(niceName or 'file')
         # Write the file on disk (and compute/get its size in bytes)
         path = str(folder / self.fsName)
         f = open(path, 'wb')
@@ -217,6 +217,8 @@ class FileInfo:
         else:
             # Write value[1] on disk
             content = value[1]
+            if isinstance(content, str):
+                content = content.encode('utf-8')
             if isinstance(content, bytes):
                 self.size = len(content)
                 f.write(content)
@@ -230,12 +232,17 @@ class FileInfo:
         '''Copies the "external" file stored at p_filePath in the db-controlled
            file system, for storing a value for p_fieldName.'''
         # Set names for the file
-        filePath = str(filePath) if isinstance(filePath, Path) else filePath
-        name = self.normalizeFileName(filePath)
+        if isinstance(filePath, UnmarshalledFile):
+            name = self.normalizeFileName(filePath.name)
+            location = filePath.location
+            self.mimeType = filePath.type or putils.guessMimeType(location)
+            filePath = location
+        else:
+            filePath = str(filePath) if isinstance(filePath, Path) else filePath
+            name = self.normalizeFileName(filePath)
+            self.mimeType = putils.guessMimeType(filePath)
         self.uploadName = name
         self.fsName = '%s%s' % (fieldName, os.path.splitext(name)[1])
-        # Set mimeType
-        self.mimeType = putils.guessMimeType(filePath)
         # Copy the file
         fsName = dbFolder / self.fsName
         shutil.copyfile(filePath, fsName)
@@ -284,12 +291,16 @@ class FileInfo:
         if self.mimeType not in File.resizableImages: return
         # Get the absolute path to the file on disk
         path = folder / self.fsName
+        spath = str(path)
         # Get the width, in pixels, that will be used to resize it
         width = sutils.Normalize.digit(str(width))
-        utils.convert(path, '-resize %sx%s>' % (width, width))
-        o.log(RESIZED % (path, width))
+        try:
+            utils.ImageMagick.convert(spath, '-resize %sx%s>' % (width, width))
+            o.log(RESIZED % (spath, width))
+        except Exception as err:
+            o.log(err, type='warning')
         # (re)compute p_self's attributes, after resizing
-        self.setAttributes(str(path))
+        self.setAttributes(spath)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class File(Field):
@@ -300,9 +311,16 @@ class File(Field):
 
     # MIME types for images
     imageMimeTypes = ('image/png', 'image/jpeg', 'image/gif')
+
     # MIME types for resizable images
     resizableImages = imageMimeTypes
 
+    # Classes that may contain information about a binary file
+    objectTypes = (Object, FileInfo, UnmarshalledFile)
+
+    # Types that may represent paths to a binary file
+    pathTypes = (str, Path)
+
     class Layouts(Layouts):
         '''File-specific layouts'''
         b = Layouts(edit='lrv-f', view='l-f')
@@ -368,7 +386,7 @@ class File(Field):
         # (self.icon) on "buttons" and "result" layouts.
         self.render = render
         # Icon to use when this file is rendered as an icon
-        self.icon = icon
+        self.icon, self.iconBase, self.iconRam = utils.iconParts(icon)
         # In "nameStorer", you can specify the name of another field that will
         # store the file name. This field must be a String belonging to the same
         # class as p_self. As soon as, in the UI, a file is selected in p_self's
@@ -442,10 +460,11 @@ class File(Field):
         return o.req['%s_action' % self.name] != 'keep'
 
     def getJsOnChange(self):
-        '''Gets the JS code for updaing the name storer when defined'''
+        '''Gets the JS code for updating the name storer when defined'''
         storer = self.nameStorer
         if not storer: return ''
-        return 'updateFileNameStorer(this, "%s")' % storer
+        name = storer.name if isinstance(storer, Field) else storer
+        return 'updateFileNameStorer(this,"%s")' % name
 
     def getStyle(self):
         '''Get the content of the "style" attribute of the "input" tag on the
@@ -479,8 +498,8 @@ class File(Field):
         # For non-images, display a link for downloading it, as an icon when
         # relevant.
         if self.render == 'icon':
-            content = '<img src="%s" title="%s"/>' % \
-                      (o.buildUrl(self.icon), title)
+            iurl = o.buildUrl(self.icon, base=self.iconBase, ram=self.iconRam)
+            content = '<img src="%s" title="%s"/>' % (iurl, title)
             # On "view", we have place, so display "title" besides the icon
             suffix = title if layout == 'view' else ''
         else:
@@ -522,8 +541,10 @@ class File(Field):
         #    | the same attributes, representing a file unmarshalled from XML
         #    | content.
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        #  b | a string or pathlib.Path instance. In this case, it represents
-        #    | the path of a file on disk;
+        #  b | a string, a pathlib.Path instance or an instance of
+        #    | appy.xml.unmarshalled.UnmarshalledFile having attribute
+        #    | "location". In theses cases, p_value represents the path of a
+        #    | file on disk;
         #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #  c | a 2-tuple (name, content) where:
         #    | - "name"     is the name of the file (ie "myFile.odt")
@@ -545,13 +566,13 @@ class File(Field):
             if info: info.removeFile(folder)
             # Store the new file. As a preamble, create a FileInfo instance.
             info = FileInfo(relative)
-            if isinstance(value, Object) or isinstance(value, FileInfo) or \
-               isinstance(value, UnmarshalledFile):
-                # Cases (a) and (e)
-                info.writeFile(self.name, value, folder, config=o.config)
-            elif isinstance(value, str) or isinstance(value, Path):
+            if isinstance(value, File.pathTypes) or \
+               (isinstance(value, UnmarshalledFile) and value.location):
                 # Case (b)
                 info.copyFile(self.name, value, folder)
+            elif isinstance(value, File.objectTypes):
+                # Cases (a) and (e)
+                info.writeFile(self.name, value, folder, config=o.config)
             else:
                 # Cases (c) and (d): extract file name, content and MIME type
                 name = type = None
