@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -215,7 +215,7 @@ class UiGroup:
 
     # Render a help icon for a group
     pxHelp = Px(lambda c: '<abbr title="%s"><img class="iconS" src="%s"/>' \
-                    '</abbr>' % (c._('help', field=c.field), c.url('help.svg')))
+                    '</abbr>' % (c._('help', field=c.field), c.svg('help')))
 
     # Render the group title, description and help
     pxHeader = Px('''
