@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -152,6 +152,23 @@ class Hour(Field):
         hours = int(minutes / 60.0)
         return hours, mins
 
+    @classmethod
+    def addMinutes(class_, value, minutes):
+        '''Adds this number of p_minutes to p_value ~(i_hours, i_minutes)~ and
+           returns the result as a tuple of the same format as p_value.'''
+        hour, mins = value
+        h, m = class_.fromMinutes(minutes)
+        hour += h
+        mins += m
+        # If, after the addition, the v_hour part is higher than 23, it goes
+        # "through 0". For example, (23,55) + 62 = (00,57)
+        if mins >= 60:
+            mins -= 60
+            hour += 1
+            if hour > 23:
+                hour = hour % 24
+        return hour, mins
+
     @classmethod
     def fromString(class_, s, sep=':'):
         '''Converts the string representation p_s of an hour (ie, "1:15",
@@ -220,6 +237,18 @@ class Hour(Field):
             r = ((start, (24,0)), ((0,0), end))
         return r
 
+    @classmethod
+    def inRange(class_, value, start, end):
+        '''Is the p_value within range [start, end] ? All parameters have the
+           form ~(i_hour, i_minutes)~'''
+        # Note that p_value must be between (00,00) and (23,59)
+        if end > start:
+            r = value >= start and value <= end
+        else:
+            r = class_.inRange(value, start, (23,59)) or \
+                class_.inRange(value, (0,0), end)
+        return r
+
     @classmethod
     def intersection(class_, rangeA, rangeB, recurse=True):
         '''Compute and return the number of minutes being common to p_rangeA and
