@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -94,8 +94,8 @@ class List(Field):
 
     # PX for rendering a single row
     pxRow = Px('''
-     <tr valign="top" style=":(rowId == -1) and 'display: none' or ''"
-         class=":loop.row.odd and 'odd' or 'even'">
+     <tr valign="top" style=":'display:none' if rowId == -1 else ''"
+         class=":'odd' if loop.row.odd else 'even'">
       <x if="not isCell">:field.pxFirstCell</x>
       <td if="showTotals"></td>
       <td for="info in subFields" if="info[1]"
@@ -105,13 +105,13 @@ class List(Field):
                 fieldName='%s*%d' % (field.name, rowId)">:field.pxRender</td>
       <!-- Icons -->
       <td if="isEdit" align=":dright">
-       <img class="clickable iconS" src=":url('deleteS.svg')"
+       <img class="clickable iconS" src=":svg('deleteS')"
             title=":_('object_delete')"
             onclick=":field.jsDeleteConfirm(q, tableId)"/>
-       <img class="clickable iconS" src=":url('arrow.svg')"
+       <img class="clickable iconS" src=":svg('arrow')"
             style="transform: rotate(180deg)" title=":_('move_up')"
             onclick=":'moveRow(%s,%s,this)' % (q('up'), q(tableId))"/>
-       <img class="clickable iconS" src=":url('arrow.svg')"
+       <img class="clickable iconS" src=":svg('arrow')"
             title=":_('move_down')"
             onclick=":'moveRow(%s,%s,this)' % (q('down'), q(tableId))"/>
        <input class="clickable" type="image" tabindex="0"
@@ -133,6 +133,7 @@ class List(Field):
                  tableId='list_%s' % name" if="isEdit or value"
             id=":tableId" width=":field.width" class="grid"
             var2="subFields=field.getSubFields(o, layout);
+                  swidths=field.getWidths(subFields);
                   totals=field.createTotals(isEdit);
                   showTotals=not isEdit and totals">
       <!-- Header -->
@@ -140,7 +141,7 @@ class List(Field):
        <tr valign="middle">
         <th if="showTotals"></th>
         <th for="name, sub in subFields" if="sub"
-            width=":field.widths[loop.name.nb]">
+            width=":swidths[loop.name.nb]">
          <x var="field=sub">::field.getListHeader(_ctx_)</x></th>
         <!-- Icon for adding a new row -->
         <th if="isEdit">
@@ -313,51 +314,79 @@ class List(Field):
          None, None, None, True, False, view, cell, edit, xml, translations)
         self.validable = True
         # Tuple of elements of the form (name, Field instance) determining the
-        # format of every element in the list.
+        # format of every element in the list. It can also be a method returning
+        # this tuple. In that case, however, no i18n label will be generated for
+        # the sub-fields: these latters must be created with a value for
+        # attribute "translations".
         self.fields = fields
-        # Force some layouting for sub-fields, if subLayouts are given. So the
-        # one who wants freedom on tuning layouts at the field level must
-        # specify subLayouts=None.
-        if subLayouts:
-            for name, field in self.fields:
-                field.layouts = subLayouts
+        # Force some layouting for sub-fields, if p_subLayouts are passed. If
+        # you want freedom to tune layouts at the field level, you must specify
+        # p_subLayouts=None.
+        self.subLayouts = subLayouts
         # One may specify the width of every column in the list. Indeed, using
-        # widths and layouts of sub-fields may not be sufficient.
-        self.computeWidths(widths)
+        # widths and layouts at the sub-field level may not be sufficient.
+        self.widths = widths
+        # Initialize sub-fields, if statically known
+        if not callable(fields):
+            self.initSubFields(fields)
         # When deleting a row, must we display a popup for confirming it ?
         self.deleteConfirm = deleteConfirm
         # If you want to specify additional rows representing totals, give in
         # "totalRows" a list of Totals instances (see above).
         self.totalRows = totalRows or []
 
+    def initSubFields(self, subFields):
+        '''Initialises sub-fields' attributes'''
+        # Initialise sub-layouts
+        subLayouts = self.subLayouts
+        if subLayouts:
+            for name, field in subFields:
+                field.layouts = Layouts.getFor(self, subLayouts)
+
+    def lazyInitSubFields(self, subFields, class_):
+        '''Lazy-initialise sub-fields, when known'''
+        # It can be at server startup, if sub-fields are statically defined, or
+        # everytime a List field is solicited, if sub-fields are dynamically
+        # computed.
+        for sub, field in subFields:
+            fullName = '%s_%s' % (self.name, sub)
+            field.init(class_, fullName)
+            field.name = '%s*%s' % (self.name, sub)
+
     def init(self, class_, name):
         '''List-specific lazy initialisation'''
         Field.init(self, class_, name)
-        for sub, field in self.fields:
-            fullName = '%s_%s' % (name, sub)
-            field.init(class_, fullName)
-            field.name = '%s*%s' % (name, sub)
+        subs = self.fields
+        if not callable(subs):
+            self.lazyInitSubFields(subs, class_)
 
-    def computeWidths(self, widths):
-        '''Set given p_widths or compute default ones if not given'''
-        self.widths = widths or ([''] * len(self.fields))
+    def getWidths(self, subFields):
+        '''Get the widths to appply to these p_subFields'''
+        return self.widths or [''] * len(subFields)
 
     def getField(self, name):
         '''Gets the field definition whose name is p_name'''
         for n, field in self.fields:
             if n == name: return field
 
-    def getSubFields(self, o, layout):
-        '''Returns the sub-fields (name, Field) that are showable among
-           field.fields on the given p_layout. Fields that cannot appear in
-           the result are nevertheless present as a tuple (name, None). This
-           way, it keeps a nice layouting of the table.'''
+    def getSubFields(self, o, layout=None):
+        '''Returns the sub-fields, as a list of tuples ~[(s_name, Field)]~,
+           being showable, among p_self.fields on this p_layout. Fields that
+           cannot appear in the result are nevertheless present as a tuple
+           (s_name, None). This way, it keeps a nice layouting of the table.'''
+        fields = self.fields
+        if callable(fields):
+            # Dynamically computed fields: get and completely initialise them
+            fields = self.getAttribute(o, 'fields') # Involves caching
+            self.initSubFields(fields)
+            self.lazyInitSubFields(fields, o.class_)
+        # Stop here if no p_layout is passed
+        if layout is None: return fields
+        # Retain only sub-fields being showable on this p_layout
         r = []
-        for n, field in self.fields:
-            elem = (n, None)
-            if field.isShowable(o, layout):
-                elem = (n, field)
-            r.append(elem)
+        for n, field in fields:
+            f = field if field.isShowable(o, layout) else None
+            r.append((n, f))
         return r
 
     def getRequestValue(self, o, requestName=None):
@@ -373,7 +402,7 @@ class List(Field):
             row = O()
             rowId = key.split('*')[-1]
             if rowId == '-1': continue # Ignore the template row
-            for subName, subField in self.fields:
+            for subName, subField in self.getSubFields(o):
                 keyName = '%s*%s*%s' % (name, subName, rowId)
                 if keyName + subField.getRequestSuffix() in req:
                     v = subField.getRequestValue(o, requestName=keyName)
@@ -381,6 +410,9 @@ class List(Field):
             if rowId.isdigit():
                 rowId = int(rowId)
                 isDict = False
+            else:
+                # Remove the -d- prefix
+                rowId = rowId[3:]
             r[rowId] = row
         # Produce a sorted list (List only)
         if not isDict:
@@ -414,7 +446,7 @@ class List(Field):
         '''Gets a ready-to-store Object instance representing a single row,
            from p_requestValue.'''
         r = O()
-        for name, field in self.fields:
+        for name, field in self.getSubFields(o):
             if not hasattr(requestValue, name) and \
                not field.isShowable(o, 'edit'):
                 # Some fields, although present on "edit", may not be part of
@@ -425,7 +457,8 @@ class List(Field):
                 continue
             subValue = getattr(requestValue, name, None)
             try:
-                setattr(r, name, field.getStorableValue(o, subValue))
+                setattr(r, name,
+                        field.getStorableValue(o, subValue, single=True))
             except ValueError:
                 # The value for this field for this specific row is incorrect.
                 # It can happen in the process of validating the whole List
@@ -445,15 +478,17 @@ class List(Field):
         r = getattr(o, self.name, None)
         if r: return copy.deepcopy(r)
 
-    def getCss(self, layout, r):
+    def getCss(self, o, layout, r):
         '''Gets the CSS required by sub-fields if any'''
-        for name, field in self.fields:
-            field.getCss(layout, r)
+        for name, field in self.getSubFields(o, layout):
+            if field:
+                field.getCss(o, layout, r)
 
-    def getJs(self, layout, r, config):
+    def getJs(self, o, layout, r, config):
         '''Gets the JS required by sub-fields if any'''
-        for name, field in self.fields:
-            field.getJs(layout, r, config)
+        for name, field in self.getSubFields(o, layout):
+            if field:
+                field.getJs(o, layout, r, config)
 
     def jsDeleteConfirm(self, q, tableId):
         '''Gets the JS code to call when the user wants to delete a row'''
@@ -465,7 +500,7 @@ class List(Field):
         i = -1
         for row in value:
             i += 1
-            for name, subField in self.fields:
+            for name, subField in self.getSubFields(o):
                 message = subField.validate(o, getattr(row, name, None))
                 if message:
                     setattr(errors, '%s*%d' % (subField.name, i), message)
