@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -24,8 +24,9 @@ from appy.model.utils import Object as O
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PARAM_EMPTY = 'Ogone: parameter "%s" has no value and has been ignored.'
-RESP_KO     = 'Ogone response SHA failed. Params: %s'
-RESP_KO_ERR = 'Failure, fraud? An administrator has been contacted.'
+RESP_OK     = '%d - Ogone answered: %s'
+RESP_KO     = '%d - Ogone response SHA failed. Params: %s'
+RESP_KO_ERR = 'An administrator has been contacted.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -35,7 +36,10 @@ class Config:
        Config class.'''
 
     # The URL to redirect the user to for performing an on-line payment
-    payUrl = 'https://ogone.%s.v-psp.com/ncol/%s/orderstandard_utf8.asp'
+    payUrls = {
+      'test': 'https://ogone.test.v-psp.com/ncol/test/orderstandard_utf8.asp',
+      'prod': 'https://secure.ogone.com/ncol/prod/orderstandard_utf8.asp'
+    }
 
     # Fields below are those that must be sent in every request to Ogone
     sendable = ('pspid', 'currency', 'language')
@@ -43,13 +47,20 @@ class Config:
     # Default parameters sent back by Ingenico to your app, after a payment has
     # been performed, being part of the HTTP GET request (payment accepted,
     # refused, etc).
+
+    # ! Compared to the predefined list as defined by Ingenico, params "CN" and
+    # "BRAND" have been removed. Indeed, it may contains spaces; these spaces
+    # are converted by Ingenico into + signs, but Appy does not convert them
+    # back to spaces for HTTP GET requests.
+
     defaultBackParams = asDict(
-      ('ORDERID', 'CURRENCY', 'AMOUNT', 'PM', 'ACCEPTANCE', 'STATUS', 'CARDNO',
-       'ED', 'CN', 'TRXDATE', 'PAYID', 'PAYIDSUB', 'NCERROR', 'BRAND', 'IPCTY',
-       'CCCTY', 'ECI', 'CVCCHECK', 'AAVCHECK', 'VC', 'DCC_INDICATOR',
-       'DCC_EXCHRATE', 'DCC_EXCHRATETS', 'DCC_CONVCCY', 'DCC_CONVAMOUNT',
-       'DCC_VALIDHOURS', 'DCC_EXCHRATESOURCE', 'DCC_MARGINPERCENTAGE',
-       'DCC_COMMPERCENTAGE', 'IP')
+      ('AAVCHECK', 'ACCEPTANCE', 'ALIAS', 'AMOUNT', 'CARDNO', 'CCCTY',
+       'COMPLUS', 'CREATION_STATUS', 'CURRENCY', 'CVCCHECK',
+       'DCC_COMMPERCENTAGE', 'DCC_CONVAMOUNT', 'DCC_CONVCCY', 'DCC_EXCHRATE',
+       'DCC_EXCHRATESOURCE', 'DCC_EXCHRATETS', 'DCC_INDICATOR',
+       'DCC_MARGINPERCENTAGE', 'DCC_VALIDHOURS', 'ECI', 'ED', 'IP', 'IPCTY',
+       'NCERROR', 'ORDERID', 'PAYID', 'PAYIDSUB', 'PM', 'STATUS',
+       'SUBSCRIPTION_ID', 'TRXDATE', 'VC')
     )
 
     def __init__(self):
@@ -94,10 +105,10 @@ class Ogone(Field):
     # Return status codes
     statuses = {
      '0': 'payment_abandoned', # Abandoned or aborted for an unknown reason
-     '1': 'payment_canceled',  # Canceled by the customer
-     '2': 'payment_refused',   # Payment refused
-     '5': 'payment_accepted',  # Payment authorized (in a 2-step process)
-     '9': 'payment_accepted',  # Payment accepted
+     '1': 'payment_canceled' , # Canceled by the customer
+     '2': 'payment_refused'  , # Payment refused
+     '5': 'payment_accepted' , # Payment authorized (in a 2-step process)
+     '9': 'payment_accepted' , # Payment accepted
     }
 
     # The "view" and "cell" layouts are used to display a button for performing
@@ -112,7 +123,7 @@ class Ogone(Field):
 
      <form var="env=config.ogone.env; label=_('pay')"
            method="post" id="form1" name="form1"
-           action=":config.ogone.payUrl % (env, env)" class="inline">
+           action=":config.ogone.payUrls[env]" class="inline">
 
        <!-- The list of hidden fields required by Ogone -->
        <input type="hidden" for="key,val in value.items()"
@@ -121,11 +132,11 @@ class Ogone(Field):
        <!-- Submit button on most layouts -->
        <input if="layout != 'view'" type="submit" id="submit2" name="submit2"
               value=":label" class="button buttonSmall pay"
-              style=":url('pay.svg', bg='18px 18px')"/>
+              style=":svg('pay', bg='18px 18px')"/>
 
        <!-- Submit picto -->
        <x if="layout == 'view'">
-        <input type="image" id="submit2" name="submit2" src=":url('pay.svg')"
+        <input type="image" id="submit2" name="submit2" src=":svg('pay')"
                value=":label" class="pictoC" title=":label"/>
         <div align="center">:label</div>
        </x>
@@ -239,7 +250,7 @@ class Ogone(Field):
             # Upperize the name of the parameter
             name = name.upper()
             # Keep it only if mentioned in the config and not empty
-            if (name not in configParams) or not value: continue
+            if not value or name not in configParams: continue
             setattr(r, name, value)
         return r
 
@@ -250,13 +261,15 @@ class Ogone(Field):
         cfg = o.config.ogone
         req = o.req
         params = self.getBackParameters(req, cfg.backParams)
-        # Call the response method defined in this Ogone field
+        sparams = str(params)
+        # Log and check the response
         if not self.ogoneResponseOk(req.SHASIGN, cfg.shaOutKey, params):
-            o.log(RESP_KO % str(params))
+            o.log(RESP_KO % (o.iid, sparams))
             raise Exception(RESP_KO_ERR)
-        # Call the field method handling the response received from Ogone.
-        # Redirect the user to the correct page. If the field method returns
-        # some URL, use it. Else, use the p_o's view URL.
+        o.log(RESP_OK % (o.iid, sparams))
+        # Handle the Ogone response by calling the response method defined in
+        # this Ogone field, and redirect the user to the correct page. If the
+        # field method returns some URL, use it. Else, use the p_o's view URL.
         url, message = self.responseMethod(o, params)
         url = url or o.url
         o.H().commit = True
