@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -48,24 +48,36 @@ class Page:
         # page name.
         self.label = label
         # The icon to display. If you want to use:
-        # * one of the standard Appy icons, residing in appy/ui/static, use
-        #   string "appy/<icon_name>", for example "appy/help.svg";
-        # * an icon being in your app's "static" folder, use string
-        #   "<app_name>/<icon_name>";
+        # * one of the standard Appy icons, residing in appy/ui/static, set the
+        #   icon name, for example "help.svg" ;
+        # * an icon being in your app's "static" folder, if it is a SVG file,
+        #   use its name (ie, "myimage.svg"); if it is another format (ie, PNG),
+        #   use string "<app_name>/<icon_name>";
         # * an icon that is chosen dynamically, set a method from the class
         #   where this page is defined. It will be called without arg and must
         #   return the icon URL, or None if the icon is not available (yet).
-        #   Here are some tips.
-        #   1) If you want to build the URL to a standard Appy icon, use, ie:
-        #      o.buildUrl('help.svg')
-        #      ("o" being any instance of any Appy class)
-        #   2) If you want to build the URL to an icon being in your app's
-        #      "static" folder, use, ie:
-        #      o.buildUrl('myIcon.png', base='MyApp')
+        #
+        #   For this latter case, here are some tips.
+        #   1) If you want to build the URL to a standard SVG Appy icon, or to
+        #      a SVG icon being in your app's "static" folder, use, ie:
+        #
+        #                      o.buildSvg('help')
+        #
+        #      ("o" is any instance of any Appy class;
+        #       in the example, the file to use is help.svg, byt the ".svg"
+        #       suffix does not need to be mentioned.)
+        #
+        #   2) If you want to build the URL to a non-SVG icon being in your
+        #      app's "static" folder, use, ie:
+        #
+        #               o.buildUrl('myIcon.png', base='MyApp')
+        #
         #   3) If you want to build the URL to an icon that was uploaded in a
         #      File field, use:
-        #      '%s/<field_name>/download' % o.url
+        # 
+        #                 '%s/<field_name>/download' % o.url
         self.setIcon(icon or 'page.svg')
+
         # Stickyness refers to the complete block of controls that is shown for
         # the current object, when this page is shown. Attribute "sticky" may
         # hold True, False, "edit" or "view", and determines if the block must
@@ -89,8 +101,9 @@ class Page:
         if callable(icon):
             self.icon = icon
             self.iconBase = None
+            self.iconRam = False
         else:
-            self.icon, self.iconBase = iconParts(icon)
+            self.icon, self.iconBase, self.iconRam = iconParts(icon)
 
     @staticmethod
     def get(pageData):
@@ -193,9 +206,10 @@ class UiPage:
             # but no image has been uploaded yet.
             base = None
             icon = 'page.svg'
+            ram = True
         else:
-            base, icon = page.iconBase, page.icon
-        return o.buildUrl(icon, base)
+            base, icon, ram = page.iconBase, page.icon, page.iconRam
+        return o.buildUrl(icon, base, ram=ram)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Phase:
@@ -230,8 +244,8 @@ class UiPhase:
 
       <!-- Siblings navigation -->
       <div var="nav=req.nav"
-           if="not isEdit and (phase.name=='main') and (nav) and (nav != 'no')"
-           var2="self=ui.Siblings.get(nav, tool, _ctx_)">:self.pxNavigate</div>
+        if="not isEdit and phase.name == 'main' and nav and nav != 'no'"
+        var2="self=ui.Siblings.get(nav,tool,_ctx_)">:self.pxNavigate|'⚠'</div>
 
       <!-- Page(s) within the phase -->
       <div if="not isEdit" for="pag in phase.pages.values()"
@@ -245,7 +259,7 @@ class UiPhase:
                  pictoUrl=pag.iconUrl">
 
         <!-- Main icon -->
-        <div>
+        <div class="pagePicto">
          <a if="viewUrl" href=":viewUrl">
           <img src=":pictoUrl" class=":picto"/>
          </a>
@@ -263,7 +277,7 @@ class UiPhase:
                editable=mayEdit and mayAct and pag.showOnEdit and pag.showEdit">
          <a if="editable and not locked"
             href=":o.getUrl(sub='edit', page=pag.name, popup=popup, nav=nav)">
-          <img src=":url('editPage.svg')" class="pictoT shake"
+          <img src=":svg('editPage')" class="pictoT shake"
                title=":_('object_edit')"/></a>
          <x var="page=pag.name; lockStyle='iconM pictoT'">::o.Lock.px</x>
         </x>
@@ -277,35 +291,38 @@ class UiPhase:
      # Class names ending with "C" are used when controls are *C*ompact
      css='''
       .phase { margin:0 0 30px -30px; display:flex; flex-wrap:wrap;
-               width:fit-content }
-      .phase div, .phase form { display:|pageDisplay| }
+               gap:|pageGap|; width:fit-content }
 
       .phase > div { height:|pageHeight|; width:|pageWidth|; text-align:center;
-        text-transform:uppercase; font-weight:bold; font-size:90%;
+        text-transform:|pageTextTransform|; font-weight:bold;
         display:flex !important; align-items:center; justify-content:center;
         padding:|pagePadding|; background-color:|phaseBgColor|;
         position:relative; padding:0 5px; color:|phaseColor|;
+        border-radius:|pageRadius|; font-size:|pageFontSize|;
+        border-bottom:1px solid |phaseBorderColor|;
         border-right:1px solid |phaseBorderColor|;
-        border-top:1px solid |phaseBorderColor|;
-        border-bottom:1px solid |phaseBorderColor| }
-      
-      .phase a { color:|phaseColor| }
+        border-top:1px solid |phaseBorderColor| }
 
+      .phase a { color:|phaseColor| }  
+      .pagePicto, .phase form { display:|pageDisplay| }
       .phaseC { margin:0 0 20px 0; border-left:1px solid |phaseBorderColor| }
-      .phaseC > div { height:|pageCHeight|; width:|pageCWidth|; font-size:9pt;
-                      margin:|pageMargin|; padding:|pageCPadding| }
+      .phaseC > div { height:|pageCHeight|; width:|pageCWidth|;
+        font-size:|pageCFontSize|; padding:|pageCPadding|;
+        text-transform:|pageCTextTransform|;
+        border-radius:|pageCRadius| }
 
       .picto { width:|pictoSize|; height:|pictoSize|; margin:|pictoMargin| }
       .pictoT { position:|epictoPosition|; top:8px; right:8px; width:22px;
                 padding: |epictoPadding| }
       .pictoC, .pictoB { width:|pictoCSize|; height:|pictoCSize|;
                          margin:|pictoCMargin| }
-      .inverted { filter:invert(66%) }
+      .filtered { filter:invert(66%) }
 
       .currentPage { background-color:|phaseBgcColor| !important;
                      color:|phaseCcolor| }
-      .currentPage img { filter:invert(66%) }
-      .currentPage a, .currentPage div { color:|brightColor| }
+      .currentPage img { filter:|phaseFilter| }
+      .currentPage div { color:|brightColor| }
+      .currentPage a { color:|phaseCcolor| }
 
       // Animation for the "edit" icon
       @keyframes shakeImage {
@@ -421,9 +438,9 @@ class UiPhases:
         if (div.id == 'currentPage') return;
         // Switch the page to dark mode
         var classes = div.classList,
-            inverted = classes.contains('currentPage');
-        if (on && !inverted) classes.add('currentPage');
-        else if (!on && inverted) classes.remove('currentPage');
+            filtered = classes.contains('currentPage');
+        if (on && !filtered) classes.add('currentPage');
+        else if (!on && filtered) classes.remove('currentPage');
       }
 
       function initPages(rowIndex) {
