@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -189,19 +189,20 @@ class Pod(Field):
 
     # Icon allowing to generate a given template in a given format
     pxIcon = Px('''
-     <a var="fmtSuffix='●' if frozen else '';
-             linkId=field.getLinkId(confirm, id, info, fmt);
-             linkGetter='getNode(%s)' % q(linkId) if confirm else 'this';
-             js='generatePod(%s,%s,%s,%s,%s,%s,null,%s)' % \
-                 (linkGetter, q(o.url), q(name), q(info.template), q(fmt), \
-                 q(tool.Search.encodeForReplay(req, layout)), gc)"
-        onclick=":confirm and 'askConfirm(%s,%s,%s)' % (q('script'), \
-                 q(js,False), field.labelId) or js"
-        title=":field.getIconTitle(o, fmt, frozen)" class="clickable"
-        style="position: relative" id=":linkId">
-      <img src=":url('download.svg')" class="iconS"/>
+     <div class="clickable iflex1"
+       var="fmtSuffix='●' if frozen else '';
+            linkId=field.getLinkId(confirm, id, info, fmt);
+            linkGetter='getNode(%s)' % q(linkId) if confirm else 'this';
+            js='generatePod(%s,%s,%s,%s,%s,%s,null,%s)' %
+                (linkGetter, q(o.url), q(name), q(info.template), q(fmt),
+                q(tool.Search.encodeForReplay(req, layout)), gc)"
+       onclick=":'askConfirm(%s,%s,%s)' % (q('script'), q(js,False),
+                   field.labelId) if confirm else js"
+       title=":field.getIconTitle(o, fmt, frozen)"
+        id=":linkId">
+      <img src=":svg('download')" class="iconS"/>
       <span class="fmt">:fmt.upper() + fmtSuffix</span>
-     </a>''')
+     </div>''')
 
     view = cell = Px('''
      <script if="field.confirm">::field.getJsConfirmVar(o)</script>
@@ -283,15 +284,8 @@ class Pod(Field):
       <br if="lineBreak"/>
      </x>''',
 
-     css='''.phase .fmt { position: absolute; left: 27%; bottom: -80%;
-                          font-size: 85% }
-            .list .fmt { font-weight: normal; margin-right: -8px }
-            @keyframes movedown {
-              from { top: 0px  }
-              50% { top: -5px }
-              to { top: 0px }
-            }
-            .waiting { animation: movedown 0.7s ease infinite }''',
+     css='''.phase .fmt { position: absolute; bottom:11%; left:28% }
+            .list .fmt { font-weight: normal }''',
 
      js='''
       podDownloadStatus = function(node, data) {
@@ -349,7 +343,7 @@ class Pod(Field):
         var data = {'class': node.className,
                     'onclick': node.attributes.onclick.value};
         node.setAttribute('onclick', '');
-        node.className = 'waiting';
+        node.className = 'waiting iflex1';
         // Set the pod download cookie. "False" means: not downloaded yet.
         createCookie('podDownload', 'False');
         // Set a timer that will check the cookie value
@@ -393,7 +387,7 @@ class Pod(Field):
       downloadDisposition='attachment', forceOoCall=False,
       optimalColumnWidths=False, distributeColumns=None, script=None,
       pdfOptions='ExportNotes=True', tabbedCR=False, fonts=None, confirm=False,
-      raiseOnError=False, action=None, beforeAction=None):
+      raiseOnError=False, action=None, beforeAction=None, multiObjects=False):
         # Param "template" stores the path to the pod template(s). If there is
         # a single template, a string is expected. Else, a list or tuple of
         # strings is expected. Every such path must be relative to your
@@ -609,6 +603,14 @@ class Pod(Field):
         # generated, set a method in parameter "beforeAction". This method's
         # signature must be the same as for parameter "action" hereabove.
         self.beforeAction = beforeAction
+        # If p_multiObjects is True, this POD will not be produced, based on a
+        # single object, but on multiple objects, being the result of some
+        # search. In that case, attribute "show" must be (or evaluate to)
+        # 'query' in order to be taken into account. Indeed, such PODs have only
+        # sense on the 'query' layout, visible on search results. Note that is
+        # is not sufficient to define show='query': you must explicitly set
+        # multiObjects=True, too.
+        self.multiObjects = multiObjects
         # Call the base constructor
         Field.__init__(self, None, (0,1), None, None, show, page, group,
           layouts, move, False, True, None, None, False, None, readPermission,
@@ -896,7 +898,9 @@ class Pod(Field):
         # If the pod document is related to a search, replay it and put the
         # result in the pod context.
         if queryData:
-            podContext['objects'] = Search.replay(tool, queryData)
+            search, objects = Search.replay(tool, queryData, returnSearch=True)
+            podContext['search'] = search
+            podContext['objects'] = objects
             podContext['queryData'] = queryData.split(':')
             podContext['_ref'] = Search.getRefInfo(tool, nameOnly=False)
         # Add the field-specific context if present
