@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -21,18 +21,18 @@
 UPLOADED = 'file "%s" uploaded in Ref "%d.%s" via rich "%s".'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import sys
+import sys, re
 from xml.sax._exceptions import SAXParseException
 
 from appy.px import Px
 from appy.ui.layout import Layouts
 from appy.model.fields import Field
-from appy.xml.cleaner import Cleaner
 from appy.utils.diff import HtmlDiff
 from appy.utils.inject import Injector
 from appy.model.fields.text import Text
 from appy.utils import string as sutils
 from appy.ui.layout import Layouts, Layout
+from appy.xml.cleaner import Cleaner, StringCleaner
 from appy.model.fields.multilingual import Multilingual
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -105,8 +105,8 @@ class Rich(Multilingual, Field):
     viewUni = cellUni = Px('''
      <x var="inlineEdit=field.getAttribute(o, 'inlineEdit');
              css=field.getAttribute(o, 'viewCss');
-             mayAjaxEdit=inlineEdit and (layout != 'cell') and not \
-                       showChanges and guard.mayEdit(o, field.writePermission);
+             mayAjaxEdit=inlineEdit and layout != 'cell' and not showChanges \
+                         and guard.mayEdit(o, field.writePermission);
              value=field.injectContent(o, value) if field.inject else value">
       <div if="not mayAjaxEdit"
            class=":css">::value or field.valueIfEmpty</div>
@@ -139,9 +139,9 @@ class Rich(Multilingual, Field):
       generateLabel=None, label=None, sdefault='', scolspan=1, swidth=None,
       sheight=None, persist=True, styles=defaultStyles, customStyles=None,
       documents=False, spellcheck=False, languages=('en',),
-      languagesLayouts=None, inlineEdit=False, toolbar='Standard', view=None,
-      cell=None, edit=None, xml=None, translations=None, inject=False,
-      valueIfEmpty='', viewCss='xhtml'):
+      languagesLayouts=None, viewSingle=False, inlineEdit=False,
+      toolbar='Standard', view=None, cell=None, edit=None, xml=None,
+      translations=None, inject=False, valueIfEmpty='', viewCss='xhtml'):
         # The list of styles that the user will be able to select in the styles
         # dropdown (within CKEditor) is defined hereafter.
         self.styles = styles
@@ -247,7 +247,7 @@ class Rich(Multilingual, Field):
         self.viewCss = viewCss
         # ~~~
         # Call the base constructors
-        Multilingual.__init__(self, languages, languagesLayouts)
+        Multilingual.__init__(self, languages, languagesLayouts, viewSingle)
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
           show, page, group, layouts, move, indexed, mustIndex, indexValue,
           None, searchable, filterField, readPermission, writePermission, width,
@@ -258,20 +258,29 @@ class Rich(Multilingual, Field):
         if maxChars is None:
             self.maxChars = Rich.NO_MAX
 
-    def getDiffValue(self, o, value, i=None, language=None):
-        '''If p_i is None, p_value is p_self's currently stored value on p_o (or
-           its p_language part if p_language is specified) and the method will
-           return a version of p_value that includes the cumulative diffs
-           between successive versions, from the very first stored version in
-           p_o's history to the currently stored p_value.
+    def getDiffValue(self, o, i=None, language=None):
+        '''If p_i is None, the method returns the content of this field on p_o,
+           rendered with the cumulative diffs between successive versions, from
+           the very first stored version in p_o's history to the currently
+           stored value.
 
-           If p_i is not None, p_value is p_self's value as stored in the entry
-           whose index is p_i+1 in p_o's history, and the method will return a
-           diff between it and p_value's subsequent version, either to be found
-           in a more recent entry in p_o's history or, failing that, the
+           If p_i is not None, it is the index of a Change event in p_o's
+           history, and the method returns a diff between the value before this
+           change was applied, and its subsequent version, either to be found
+           in a more recent change in p_o's history or, failing that, the
            currently stored value on p_o.'''
-        r = last = None
-        for event in o.history.iter(eventType='Change',chronological=True,i=i):
+        # Get the value as currently stored on p_o
+        current = self.getStoredValue(o) or ''
+        if language and current: current = current.get(language) or ''
+        # Initialise data structures
+        if i is None:
+            start = last = r = None
+        else:
+            last = o.history[i]
+            start = i-1 # The event chronologically after the current one
+            r = last.getValue(self.name, language)
+        for event in o.history.iter(eventType='Change', chronological=True, \
+                                    i=start):
             # Ignore irrelevant events
             if not event.hasField(self.name, language=language): continue
             # We have a matching event
@@ -279,9 +288,9 @@ class Rich(Multilingual, Field):
                 # Return the diff between p_value and the newer version stored
                 # in this event.
                 newer = event.getValue(self.name, language)
-                iMsg, dMsg = o.history[i+1].getDiffTexts(o)
-                return HtmlDiff(value, newer, iMsg, dMsg).get()
-            # If we are here, we must produce a cumulative diff
+                itext, dtext = last.getDiffTexts(o)
+                return HtmlDiff(r, newer, itext, dtext).get()
+            # If we are here, a cumulative diff must be produced
             if r is None:
                 # This is the first encountered version
                 r = event.getValue(self.name, language)
@@ -289,16 +298,16 @@ class Rich(Multilingual, Field):
                 # We need to produce the difference between current result
                 # and this version.
                 newer = event.getValue(self.name, language)
-                iMsg, dMsg = last.getDiffTexts(o)
-                r = HtmlDiff(r, newer, iMsg, dMsg).get()
+                itext, dtext = last.getDiffTexts(o)
+                r = HtmlDiff(r, newer, itext, dtext).get()
             # Remember the last walked event
             last = event
-        if not last:
-            # There is no diff to show
-            return value
-        # Now we need to compare the result with the current version
-        iMsg, dMsg = last.getDiffTexts(o)
-        return HtmlDiff(r, value or '', iMsg, dMsg).get()
+        # If no event was walked, there is no diff to show. Return the currently
+        # stored value
+        if not last: return current
+        # The result must be compared with the v_current value
+        itext, dtext = last.getDiffTexts(o)
+        return HtmlDiff(r, current, itext, dtext).get()
 
     def getUniFormattedValue(self, o, value, layout='view', showChanges=False,
                              language=None, contentLanguage=None):
@@ -306,10 +315,11 @@ class Rich(Multilingual, Field):
            specified, p_value is the p_contentLanguage part of a multilingual
            value.'''
         if Field.isEmptyValue(self, o, value) and not showChanges: return ''
-        r = value
         if showChanges:
             # Compute the successive changes that occurred on p_value
-            r = self.getDiffValue(o, r, language=contentLanguage)
+            r = self.getDiffValue(o, language=contentLanguage)
+        else:
+            r = value
         return r
 
     def validateUniValue(self, o, value):
@@ -317,7 +327,10 @@ class Rich(Multilingual, Field):
         try:
             Cleaner().clean(value)
         except SAXParseException as err:
-            return o.translate('bad_xml', mapping={'error': str(err)})
+            # Try after removing problematic chars
+            value = StringCleaner.clean(value)
+            if not StringCleaner.isParsable(value):
+                return o.translate('bad_xml', mapping={'error': str(err)})
 
     def getSearchValue(self, req, value=None):
         '''The input widget for a Rich field is of type "text". So behave like a
@@ -330,7 +343,7 @@ class Rich(Multilingual, Field):
            in p_o's history before p_i, or, if not found, the value as currently
            stored on p_o.'''
         r = Field.getHistoryValue(self, o, value, i, language, empty=None)
-        return empty if not r else self.getDiffValue(o, value, i-1, language)
+        return empty if not r else self.getDiffValue(o, i, language)
 
     def getUniStorableValue(self, o, value):
         '''Gets the p_value as can be stored in the database within p_obj'''
@@ -341,9 +354,15 @@ class Rich(Multilingual, Field):
         try:
             value = Cleaner().clean(value)
         except SAXParseException as err:
-            # Errors while parsing p_value can't prevent the user from storing
-            # it.
-            o.log(XML_ERROR % (self.name, o.id, str(err)), type='warning')
+            # Try to remove chars known for producing SAX errors ...
+            value = StringCleaner.clean(value)
+            # ... then, re-try
+            try:
+                value = Cleaner().clean(value)
+            except SAXParseException as err:
+                # Errors while parsing p_value can't prevent the user from
+                # storing it.
+                o.log(XML_ERROR % (self.name, o.id, str(err)), type='warning')
         # Manage maxChars
         max = self.maxChars
         if max and (len(value) > max): value = value[:max]
@@ -422,9 +441,9 @@ class Rich(Multilingual, Field):
             ck.append('%s: %s' % (k, sv))
         return ', '.join(ck)
 
-    def getJs(self, layout, r, config):
+    def getJs(self, o, layout, r, config):
         '''A rich field depend on CKeditor'''
-        if (layout not in ('edit', 'view')) or \
+        if layout not in ('edit', 'view') or \
            ((layout == 'view') and not self.inlineEdit): return
         # Compute the URL to ckeditor CDN
         ckUrl = Rich.cdnUrl % (config.ui.ckVersion, config.ui.ckDistribution)
