@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -175,7 +175,7 @@ class Select(Field):
      </select>
 
      <!-- As radio buttons or checkboxes -->
-     <x if="not isSelect">
+     <div if="not isSelect" style=":field.getRadioStyle()">
 
       <!-- (Un)check all -->
       <div if="field.render == 'checkbox' and len(possibleValues) &gt; 1"
@@ -185,14 +185,14 @@ class Select(Field):
        <label lfor=":allId">:_('check_uncheck')</label>
       </div>
 
-      <div for="val, text in possibleValues"
+      <div for="val, text in possibleValues" class="flex1"
            var2="vid='%s_%s' % (name, val)">
        <input type=":field.render" name=":name" id=":vid" value=":val"
               class=":masterCss" onchange=":field.getOnChange(o, layout)"
               checked=":field.isSelected(o, name, val, rawValue)"/>
        <label lfor=":vid" class="subLabel">:text</label>
       </div>
-     </x>
+     </div>
      <script if="hostLayout" var2="x=o.Lock.set(o, user, field=field)">:\
       'prepareForAjaxSave(%s,%s,%s,%s)' % \
        (q(name), q(o.iid), q(o.url), q(hostLayout))</script></x>''')
@@ -404,7 +404,7 @@ class Select(Field):
                                                        single=True)
             elif not className:
                 # ... or from the database if we are editing an object
-                masterValues = master.getValue(o)
+                masterValues = master.getValue(o, single=True)
             else:
                 # We don't have any master value
                 masterValues = None
