@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -27,6 +27,7 @@ from appy.ui.layout import Layouts
 from appy.xml.escape import Escape
 from appy.model.fields import Field
 from appy.utils import string as sutils
+from appy.xml.cleaner import StringCleaner
 from appy.ui.layout import Layouts, Layout
 from appy.database.operators import or_, in_
 from appy.database.indexes.text import TextIndex
@@ -444,7 +445,7 @@ class Icon:
         r = '<img class="icon" src="%s" title="%s" name="%s"' \
             ' onmouseover="switchIconBack(this, true)"' \
             ' onmouseout="switchIconBack(this, false)"' \
-            ' data-type="%s" data-data="%s" data-shortcut="%d" ' \
+            ' data-type="%s" data-data="%s" data-shortcut="%s" ' \
             'onclick="useIcon(this)"/>' % \
              (o.buildUrl(self.icon), o.translate(self.label), self.name,
               self.type, self.data or '', shortcut)
@@ -611,11 +612,11 @@ class Text(Multilingual, Field):
                 fdir='row' if inToolbar else 'column'"
            style=":'float:%s;display:flex;flex-direction:%s' % (align, fdir)">
        <div>
-        <img id=":'%s_save' % tid" src=":url('save.svg')"
+        <img id=":'%s_save' % tid" src=":svg('save')"
              class=":'iconS %s' % ('clickable' if inToolbar else 'inlineIcon')"
              title=":_('object_save')"/></div>
        <div>
-        <img id=":'%s_cancel' % tid" src=":url('cancel.svg')"
+        <img id=":'%s_cancel' % tid" src=":svg('cancel')"
              class=":'iconS %s' % ('clickable' if inToolbar else 'inlineIcon')"
              title=":_('object_cancel')"/></div>
       </div>
@@ -659,7 +660,7 @@ class Text(Multilingual, Field):
       edit=None, xml=None, translations=None,
       # Specific attributes
       placeholder=None, languages=('en',), languagesLayouts=None,
-      structured=False, readonly=False):
+      viewSingle=False, structured=False, readonly=False):
         # You can define a placeholder in the following attribute. Please
         # consult the homonym attribute on class String from string.py for more
         # information.
@@ -674,7 +675,7 @@ class Text(Multilingual, Field):
         # "readonly" set.
         self.readonly = readonly
         # Call the base constructors
-        Multilingual.__init__(self, languages, languagesLayouts)
+        Multilingual.__init__(self, languages, languagesLayouts, viewSingle)
         Field.__init__(self, validator, multiplicity, default, defaultOnEdit,
           show, page, group, layouts, move, indexed, mustIndex, indexValue,
           None, searchable, filterField, readPermission, writePermission, width,
@@ -762,7 +763,7 @@ class Text(Multilingual, Field):
         '''Gets the p_value as can be stored in the database within p_o'''
         if not value: return value
         # Clean the value
-        value = value.replace('\r', '')
+        value = StringCleaner.clean(value.replace('\r', ''))
         # Manage maxChars
         max = self.maxChars
         if max and (len(value) > max): value = value[:max]
