@@ -1,7 +1,7 @@
 '''Meta-class for a Appy class'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -50,14 +50,13 @@ from appy.model.workflow.localRoles import LocalRoles
 from appy.model.fields.phase import Page as FieldPage
 
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-UNALLOWED_FIELD_NAME = 'You have defined a field or search named "%s" on ' \
-  'class "%s". This name is not allowed.'
-CREATOR_LOCAL_ROLE = '%s: local role "%s" cannot be used as creator.'
-DUPLICATE_SEARCH_NAME = '%s: duplicate search "%s".'
-SEARCH_FIELD_HOMONYM = '%s: "%s" is both used as search and field name, this ' \
-  'is not allowed.'
-CUSTOM_ID_NOT_STRING = 'The custom ID produced by method "generaeId" must be ' \
-  'a string.'
+NAME_KO     = 'You have defined a field or search named "%s" on class "%s". ' \
+              'This name is not allowed.'
+CR_LOC_ROLE = '%s: local role "%s" cannot be used as creator.'
+DUP_S_NAME  = '%s: duplicate search "%s".'
+S_F_HOM     = '%s: "%s" is both used as search and field name, this is not ' \
+              'allowed.'
+CUS_ID_KO   = 'The custom ID produced by method "generaeId" must be a string.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Class(Meta):
@@ -66,31 +65,11 @@ class Class(Meta):
     # Attributes added on Appy classes
     attributes = {'fields': Field}
 
-    # Reserved attribute names unallowed for naming a Field or a Search
-    unallowedNames = {
-      'id':         None, # The object's identifier
-      'iid':        None, # The object's integer identifier
-      'values':     None, # Dict storing all field values
-      'container':  None, # Info about the objet's container
-      'history':    None, # The object's history
-      'localRoles': None, # The object's local roles
-      'locks':      None, # Locks on object pages
-    }
-
     # Fields being unwanted on search fields, although being indexed. Field
     # "title" is not really "unsearchable", but in most cases, it is preferable
     # to use field "searchable".
     unsearchableFields = ('title', 'allowed', 'cid')
 
-    @classmethod
-    def unallowedName(class_, name):
-        '''Return True if p_name can't be used for naming a Field or Search'''
-        # A name is unallowed if
-        # (a) it corresponds to some standard attribute added at object creation
-        #     on any Appy object (but not declared at the class level);
-        # (b) it corresponds to an attribute or method defined in class Base.
-        return (name in class_.unallowedNames) or (name in Base.__dict__)
-
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Model-construction-time methods
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -161,7 +140,7 @@ class Class(Meta):
             # instances of this class, will not find it.
             setattr(appy.model.meta.class_, genName, self.concrete)
         else:
-            self.type = (self.name in Model.baseClasses) and 'over' or 'app'
+            self.type = 'over' if self.name in Model.baseClasses else 'app'
         # Fields are grouped into pages, themselves grouped into "phases"
         self.phases = collections.OrderedDict()
         # Read fields and static searches
@@ -171,13 +150,26 @@ class Class(Meta):
         self.workflow = None
         # Call the "update" method if present: it allows developers to modify
         # this class. One of the most frequent updates will be to update field
-        # "title".
+        # "title". If this method must add a new field, or replace an existing
+        # field with an alternate version, do not write things like:
+        #
+        #   @staticmethod
+        #   def update(class_):
+        #       class_.fields['title'] = Text(...)
+        #
+        # ... call method m_setField (defined hereafter) instead:
+        #
+        #   @staticmethod
+        #   def update(class_):
+        #       title = Text(...)
+        #       class_.setField('title', title)
         if self.type == 'over':
             # Execute the "update" method on the base class
             base = eval(self.name)
             if hasattr(base, 'update'):
                 base.update(self)
-        if hasattr(self.python, 'update'): self.python.update(self)
+        if hasattr(self.python, 'update'):
+            self.python.update(self)
 
     def getBaseClass(self):
         '''Returns the name of the class being p_self's base class'''
@@ -244,8 +236,8 @@ class Class(Meta):
                     # Ensure the field name is acceptable
                     self.checkAttributeName(name)
                     if not aClass.__module__.startswith('appy.model') and \
-                       (Class.unallowedName(name)):
-                        raise Err(UNALLOWED_FIELD_NAME % (name, self.name))
+                       Meta.unallowedName(name):
+                        raise Err(NAME_KO % (name, self.name))
                     # A field was found.
                     # ~~~
                     # Special fields "title" and "record" must be duplicated.
@@ -263,6 +255,14 @@ class Class(Meta):
                         self.readPage(field.page)
         self.fields = r
 
+    def setField(self, name, field):
+        '''Possibly called by p_self.python's m_update method, this method adds,
+           to p_self.fields, a new p_field named p_name, or replaces the field
+           currently named p_name if the name is already in use.'''
+        self.fields[name] = field
+        # Late-initialise it
+        field.init(self, name)
+
     def readSearches(self):
         '''Create attribute "searches" as an ordered dict storing all static
            searches defined on this class.'''
@@ -273,13 +273,13 @@ class Class(Meta):
             for search in class_.searches:
                 name = search.name
                 # Ensure the search name is acceptable
-                if Class.unallowedName(name):
-                    raise Err(UNALLOWED_FIELD_NAME % (name, self.name))
+                if Meta.unallowedName(name):
+                    raise Err(NAME_KO % (name, self.name))
                 # Ensure the search name is unique, also among fields
                 if name in r:
-                    raise Err(DUPLICATE_SEARCH_NAME % (self.name, name))
+                    raise Err(DUP_S_NAME % (self.name, name))
                 if name in self.fields:
-                    raise Err(SEARCH_FIELD_HOMONYM % (self.name, name))
+                    raise Err(S_F_HOM % (self.name, name))
                 # Ensure the field name is acceptable
                 self.checkAttributeName(name)
                 search.init(self)
@@ -300,7 +300,7 @@ class Class(Meta):
         for creator in creators:
             if isinstance(creator, Role):
                 if creator.local:
-                    error = CREATOR_LOCAL_ROLE % (self.name,creator.name)
+                    error = CR_LOC_ROLE % (self.name,creator.name)
                     raise Model.Error(error)
                 r.append(creator)
             else:
@@ -344,7 +344,7 @@ class Class(Meta):
         if r is None: return
         # Raise an error if the returned ID is not a string
         if not isinstance(r, str):
-            raise Exception(CUSTOM_ID_NOT_STRING)
+            raise Exception(CUS_ID_KO)
         return r
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -414,11 +414,11 @@ class Class(Meta):
     def getCreateLink(self, tool, create, formName,
                       sourceField=None, insert=None):
         '''When instances of p_self must be created from a template (from the
-           UI), this method returns the link allowing to query such templates
+           UI), this method returns the link allowing to search such templates
            from a popup.'''
-        r = '%s/query?class=%s&search=fromSearch&popup=1&fromClass=%s' \
-            '&formName=%s' % (tool.url, create.class_.__name__, self.name,
-                              formName)
+        r = '%s/Search/results?className=%s&search=fromSearch&popup=True&' \
+            'fromClass=%s&formName=%s' % \
+            (tool.url, create.container.name, self.name, formName)
         # When object creation occurs via a Ref field, its coordinates are given
         # in p_sourceField as a string: "sourceObjectId:fieldName".
         if sourceField: r += '&sourceField=%s' % sourceField
@@ -476,11 +476,16 @@ class Class(Meta):
            storing index values for instances of this class ?'''
         return getattr(self.concrete, 'indexable', True)
 
-    def getListPods(self):
+    def getListPods(self, tool):
         '''Finds, among p_self' fields, those being Pod fields showable on
            search results.'''
-        return [f for f in self.fields.values() \
-                if (f.type == 'Pod') and (f.show == 'query')]
+        r = []
+        for field in self.fields.values():
+            if field.type != 'Pod' or not field.multiObjects: continue
+            show = field.show(tool) if callable(field.show) else field.show
+            if show == 'query':
+                r.append(field)
+        return r
 
     def getDynamicSearches(self, tool):
         '''Gets the dynamic searches potentially defined on this class'''
@@ -575,8 +580,8 @@ class Class(Meta):
         # Collect CSS and JS files
         config = tool.config
         for field in fields:
-            field.getCss('edit', css)
-            field.getJs('edit', js, config)
+            field.getCss(None, 'edit', css)
+            field.getJs(None, 'edit', js, config)
         # Returns a gridder allowing to render the search field
         gridder = Gridder(cols=cols)
         return fields, css, js, gridder
@@ -673,17 +678,16 @@ class Class(Meta):
       <input var="css='Small' if fromRef else 'Portlet';
                   label=_('object_add_from')"
          type="button" value=":label" class=":'button%s button' % css"
-         onclick="openPopup('iframePopup')" style=":url('add.svg', bg=True)"/>
+         onclick="openPopup('iframePopup')" style=":svg('add', bg=True)"/>
      </a>''')
 
     # Style characteristics to apply to the "add" button, depending on its type
     addStyles = {
       # The "add" button as it appears in the portlet
-      'portlet': O(css='buttonPortlet button', bgIcon='portletAdd.svg',
+      'portlet': O(css='buttonPortlet button', bgIcon='portletAdd',
                    bgSize='24px 24px'),
       # The "add" button as it may appear elsewhere (ie, on search results)
-      'small'  : O(css='buttonSmall button', bgIcon='add.svg',
-                   bgSize='18px 18px'),
+      'small'  : O(css='buttonSmall button', bgIcon='add', bgSize='18px 18px'),
       # When using pxAdd from a custom PX, let this latter manage button style
       None     : O(css='', bgIcon=None, bgSize=None)
     }
@@ -705,7 +709,7 @@ class Class(Meta):
 
       <!-- Create from an empty form -->
       <input class=":styles.css" value=":text" title=":text" type="submit"
-        style=":url(styles.bgIcon, bg=styles.bgSize) if styles.bgIcon else ''"
+        style=":svg(styles.bgIcon, bg=styles.bgSize) if styles.bgIcon else ''"
         onclick=":target.getOnClick('searchResults')"/>
 
       <!-- Create from a pre-filled form when relevant -->
