@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -18,6 +18,7 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from appy.px import Px
 from appy.model.base import Base
+from appy.model.user import User
 from appy.model.fields import Show
 from appy.model.query import Query
 from appy.xml.escape import Escape
@@ -49,17 +50,18 @@ class Page(Base):
     workflow = Anonymous
 
     pa = {'label': 'Page'}
-    # Pages are not indexed by default
-    indexable = False
 
     @staticmethod
     def getListColumns(tool):
         '''Show minimal info for the anonymous user'''
-        if tool.user.isAnon(): return
+        if tool.user.isAnon(): return ('title',)
         return ('title', 'expression', 'next', 'selectable*100px|',
                 'podable*80px|', 'viewCss*100px|')
     listColumns = getListColumns
 
+    # An alternate, shorter list of columns
+    listColumnsShort = 'title', 'expression', 'next', 'state'
+
     # The POD ouput
     doc = Pod(template='/model/pod/Page.odt', formats=('pdf',), show=False,
               layouts=Pod.Layouts.inline, freezeTemplate=lambda o,tpl: ('pdf',))
@@ -96,7 +98,7 @@ class Page(Base):
                    focus=True, layouts=Info.Layouts.n, **pa)
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    #                            Linked carousel
+    #                           Linked carousel
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     def showCarousel(self):
@@ -117,7 +119,9 @@ class Page(Base):
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
     content = Rich(documents='documents', height='350px', inject=True,
-                   viewCss=lambda o: o.viewCss, layouts=Rich.Layouts.f, **pa)
+                   show=lambda o: 'edit' if o.isEmpty('content') else True,
+                   viewCss=lambda o: o.viewCss, viewSingle=True,
+                   layouts=Rich.Layouts.f, **pa)
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     #                           Linked search
@@ -290,6 +294,8 @@ class Page(Base):
         # p_level is the current level of recursion, while p_r is the current
         # list under creation. Indeed, in order to avoid concatenating lists, at
         # each m_getMergedContent call, the current p_r(esult) is passed.
+        #
+        # This method is notably called by appy/model/pod/Page.odt
         if r is None:
             # We are at the start of a recursive merge: there is no result yet.
             # Create it.
@@ -309,6 +315,40 @@ class Page(Base):
                 page.getMergedContent(r, level=level+1)
         return r
 
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+    #                                Editors
+    #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+    # "Editors" are users being allowed to edit this page and its sub-pages.
+    # These users are granted local role Owner on the page and its sub-pages. If
+    # you use an alternate worklow for pages, ensure role Owner may edit pages.
+
+    def linkEditor(self, user):
+        '''Set this new editor as p_self's owner'''
+        self.localRoles.add(user.login, 'Owner')
+        for page in self.pages:
+            page.linkEditor(user)
+
+    def unlinkEditor(self, user):
+        '''ungrant to p_user the possibility to edit p_self'''
+        if user.login != self.creator:
+            self.localRoles.delete(user.login, 'Owner')
+        for page in self.pages:
+            page.unlinkEditor(user)
+
+    # Users allowed to edit this page and its sub-pages
+    editors = Ref(User, add=False, link='popup', multiplicity=(0,None),
+      afterLink=linkEditor, afterUnlink=unlinkEditor,
+      back=Ref(attribute='editablePages', multiplicity=(0,None),
+               page=FPage('editablePages', show=User.showEPages),
+               layouts=Layouts.dv, actionsDisplay='inline',
+               showHeaders=True, shownInfo=listColumnsShort, label='User'),
+      showHeaders=True, shownInfo=User.listColumnsShort,
+      actionsDisplay='inline', layouts=Layouts.dv,
+      page=FPage('editors',
+                 show=lambda o: 'view' if o.user.hasRole('Manager') else None),
+      **pa)
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Main methods
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
@@ -335,7 +375,7 @@ class Page(Base):
         # A page created from the portlet (if class Page is declared as root)
         # has no container when under creation.
         container = self.container
-        return not container or (container.class_.name == 'Tool')
+        return not container or container.class_.name == 'Tool'
 
     def onEdit(self, created):
         '''Link the page among root pages if created from the portlet'''
