@@ -2,7 +2,7 @@
    searches. A searched is always attached to a given class from the model.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -71,7 +71,7 @@ class UiSearch:
 
       <!-- Pod templates -->
       <x if="not empty and not popup and search.showPods">
-       <table var="fields=class_.getListPods()"
+       <table var="fields=class_.getListPods(tool)"
               if="fields and mode.objects" align=":dright">
         <tr>
          <td var="o=mode.objects[0]"
@@ -85,7 +85,7 @@ class UiSearch:
       <div if="not popup and search.showTitle" class="pageTitle topSpaceS">
        <x>::uiSearch.translated</x>
        <x if="mode.batch and not empty">
-        <img src=":url('arrowsA.svg')" class="iconS"
+        <img src=":svg('arrowsA')" class="iconS"
              style="transform: rotate(270deg)"/>
         <x>:mode.batch.total</x>
        </x>
@@ -121,11 +121,7 @@ class UiSearch:
            align=":search.navAlign">:mode.batch.pxNavigate</div>
 
       <!-- No result -->
-      <x if="empty">
-       <x>::_('query_no_result')</x>
-       <x if="showNewSearch and mode.newSearchUrl"><br/><i class="discreet">
-         <a href=":mode.newSearchUrl">:_('search_new')</a></i></x>
-      </x>
+      <div if="empty" class="topSpace">::_('query_no_result')</div>
     </div></x>''')
 
     def __init__(self, search, tool, ctx, initiator=None, name=None):
@@ -257,7 +253,8 @@ class Search:
                  translatedDescr=None, checkboxes=False, checkboxesDefault=True,
                  container=None, add=False, resultModes=None, shownInfo=None,
                  actions=None, rowAlign='top', pageLayoutOnView=None,
-                 thumbnailCss='thumbnail', **fields):
+                 thumbnailCss='thumbnail', switchModeAlign='left',
+                 gridFiltersAlign='center', **fields):
         # "name" is mandatory, excepted in some special cases (ie, when used as
         # "select" param for a Ref field).
         self.name = name
@@ -299,7 +296,7 @@ class Search:
         # Most of the time, we know what is the class whose instances must be
         # searched. When it is not the case, the p_container can be explicitly
         # specified.
-        if container and (container.__class__.__name__ != 'Class'):
+        if container and container.__class__.__name__ != 'Class':
             container = container.meta
         self.container = container
         # Is it possible to create new "container" instances from this search ?
@@ -322,6 +319,12 @@ class Search:
         # The name of the CSS class to use for rendering thumbnails, when result
         # mode is "grid".
         self.thumbnailCss = thumbnailCss
+        # Alignment for the widget allowing to switch from one mode to another.
+        # Can be any valid value for property "justify-content" in a flex
+        # layout, a.o.: "left", "right" and "center".
+        self.switchModeAlign = switchModeAlign
+        # Alignment of filters on a grid layout
+        self.gridFiltersAlign = gridFiltersAlign
 
     def init(self, class_):
         '''Lazy search initialisation'''
@@ -338,14 +341,17 @@ class Search:
         return r if not callable(r) else \
                tool.H().methods.call(tool, self.show, class_=class_)
 
-    def getSessionKey(self):
+    def getSessionKey(self, name=None):
         '''Returns the name of the key storing siblings of a given object in the
            context of this search.'''
-        return '%s_%s' % (self.container.name, self.name)
+        # The name of the search may be passed in p_name
+        name = name or self.name
+        return '%s_%s' % (self.container.name, name)
 
     mergeAttributes = ('sortBy', 'sortOrder', 'showActions', 'actionsDisplay',
                        'actions', 'maxPerPage', 'resultModes', 'shownInfo',
-                       'checkboxes', 'checkboxesDefault')
+                       'checkboxes', 'checkboxesDefault', 'switchModeAlign',
+                       'gridFiltersAlign')
 
     def merge(self, other):
         '''Merge parameters from another search in p_other'''
@@ -402,15 +408,21 @@ class Search:
         # objects being referenced from p_refObject through p_refField.
         # ~~~
         # If p_other is not None, it is another Search instance whose parameters
-        # will be merged with p_self's parameters.
+        # will be merged with p_self's parameters (including sortBy and
+        # sortOrder).
         # ~~~
         # Prepare search parameters
         className = self.container.name
         params = self.fields.copy()
         # Use the search's default sort key and order if not passed as args
-        if not sortBy:
+        if not sortBy and not other:
             sortBy = self.sortBy
             sortOrder = self.sortOrder
+        # Take p_other into account
+        if other:
+            sortBy = other.sortBy
+            sortOrder = other.sortOrder
+            params.update(other.fields)
         # Take p_filters into account
         if filters:
             for name, value in filters.items():
@@ -475,7 +487,7 @@ class Search:
             # Take into account default params for the advanced search
             advanced = modelClass.getSearchAdvanced(tool)
             if advanced: r.merge(advanced)
-        elif (name == 'allSearch') or not name:
+        elif not name or name == 'allSearch':
             # It is the search for every instance of p_modelClass
             r = Search('allSearch', container=modelClass)
             name = r.name
@@ -493,7 +505,7 @@ class Search:
                 r = o.getField(fieldName).getAttribute(o, 'createVia')
             else:
                 # We are creating a root object
-                tool.model.classes.get(fromClass).getCreateVia(tool)
+                r = tool.model.classes.get(fromClass).getCreateVia(tool)
             initiator = initiators.TemplateInitiator(fromClass, req.formName,
                                                      req.insert, sourceField)
         elif ',' in name:
@@ -548,7 +560,7 @@ class Search:
             if not info:
                 # Try to get it from the "_ref" key within search criteria
                 criteria = Criteria.readFromRequest(tool.H())
-                if criteria and ('_ref' in criteria): info = criteria['_ref']
+                if criteria and '_ref' in criteria: info = criteria['_ref']
         if not info or (sep not in info): return None, None
         id, field = info.split(sep)
         o = tool.getObject(id)
@@ -563,11 +575,11 @@ class Search:
         # the special search named "allSearch".
         if ('search' in req) and not req.search:
             req.search = 'allSearch'
-        if not req.search or (layout == 'cell'): return ''
+        if not req.search or layout == 'cell': return ''
         return ':'.join([(req[key] or '') for key in class_.replayParams])
 
     @classmethod
-    def replay(class_, tool, searchParams):
+    def replay(class_, tool, searchParams, returnSearch=False):
         '''Replays the search whose parameters are in p_searchParams'''
 
         # In some contexts, a Search needs to be "replayed". For example,
@@ -580,6 +592,12 @@ class Search:
         # Search parameters (p_searchParams) are in the format produced by
         # m_encodeForReplay. This method return resulting objects. The operation
         # is logged.
+        #
+        # If p_returnSearch is True, instead of returning the list of matched
+        # objects, the method returns a tuple (uiSearch, objects). Indeed, some
+        # callers (like POD fields) may need to get info about the search as
+        # stored on the UiSearch instance, like its translated name and
+        # description.
         # ~~~
         # Get search parameters as an object
         params = O()
@@ -608,7 +626,8 @@ class Search:
           filters=modelClass.getFilters(tool, params.filters),
           refObject=refObject, refField=refField)
         tool.log(REPLAY_OK % len(r))
-        return r
+        # Also return the UiSearch instance:
+        return (UiSearch(search, tool, O()), r) if returnSearch else r
 
     @classmethod
     def getCheckedInfo(class_, req):
@@ -629,10 +648,16 @@ class Search:
         return ids, req.checkedSem == 'unchecked'
 
     @classmethod
-    def keepCheckedResults(class_, req, objects):
+    def keepCheckedResults(class_, req, objects, unchecked=None):
         '''Among p_objects as retrieved via m_replay, keep only those being
            checked, according to m_getCheckedInfo.'''
-        ids, unchecked = class_.getCheckedInfo(req)
+        # If p_unchecked is not None, we already know the IDs of the objects
+        # being unchecked.
+        if unchecked:
+            ids = unchecked
+            unchecked = True
+        else:
+            ids, unchecked = class_.getCheckedInfo(req)
         i = len(objects) - 1
         # Remove, from search results, unchecked objects
         while i >= 0:
@@ -653,10 +678,11 @@ class Search:
       <x if="not mode.inField">::ui.Globals.getForms(tool)</x>
 
       <!-- Switch mode -->
-      <div if="len(resultModes) &gt; 1" class="modes">
+      <div if="not mode.empty and len(resultModes) &gt; 1" class="modes"
+           style=":'justify-content:%s' % uiSearch.search.switchModeAlign">
        <img for="cls in mode.getClasses(resultModes)"
             name=":cls.name" title=":cls.getText(_)"
-            src=":url(cls.icon)" class="clickable iconL"
+            src=":svg(cls.icon)" class="clickable iconL"
             style=":'display:%s' % ('block' if mode.name!=cls.name else 'none')"
             onclick="switchResultMode(this)"/>
       </div>
@@ -727,7 +753,7 @@ class Search:
       <p if="not objects">::_('query_no_result')</p>
       <div for="o in objects" style="padding: 3px 5px">
        <a href=":search.onSelectLiveResult(o, io, ifield)"
-          var="title=o.getShownValue();
+          var="title=o.getShownValue(layout='xml');
                content=Px.truncateValue(title, width=80)"
           title=":title">:content</a>
       </div>
@@ -762,7 +788,7 @@ class Search:
       <!-- Search title -->
       <div class="pageTitle bottomSpace">
        <x>:_('%s_plural' % className)</x> 
-       <img src=":url('arrowsA.svg')" class="iconS"
+       <img src=":svg('arrowsA')" class="iconS"
             style="transform: rotate(270deg)"/><x>:_('search_title')</x>
       </div>
 
@@ -784,7 +810,7 @@ class Search:
        <input var="label=_('search_button');
                    css=ui.Button.getCss(label, small=False)"
               type="submit" class=":css" value=":label"
-              style=":url('search.svg', bg='18px 18px')"/>
+              style=":svg('search', bg='18px 18px')"/>
       </form>
      </x>''', template=Template.px, hook='content')
 
@@ -805,9 +831,9 @@ class Search:
 
     live = Px('''
      <div var="searchLabel=_('search_button')" class=":liveCss">
-      <img src=":url('search.svg')" class="iconM"/>
+      <img src=":svg('search')" class="iconM"/>
       <div style="position: relative">
-       <input type="text" size="14" name="w_searchable" autocomplete="off"
+       <input type="text" name="w_searchable" autocomplete="off"
               id=":'%s_LSinput' % pre" class="inputSearch"
               title=":searchLabel" placeholder=":searchLabel"
               var="jsCall='onLiveSearchEvent(event,%s,%s,%s)' % \
@@ -822,14 +848,15 @@ class Search:
       </div>
      </div>''',
 
-     css='''.lsSelected { background-color: #d9d7d9 }
-            .lsRef { display: flex }
-            .lsSearch { display: flex; margin-top: 5px; padding: |lsPadding|;
-                        background-color: |lsBgColor| }
-            .lsSearch input[type=text] { background-color: transparent;
-                        border-bottom: |lsBottom| |lsBottomColor| }
-            .lsSearch .dropdown { background-color: |lsrBgColor| }
-            .lsAll { font-size: 80%; font-style: italic }''',
+     css='''.lsSelected { background-color:#d9d7d9 }
+            .lsRef { display:flex }
+            .lsSearch { display:flex; margin-top:5px; padding:|lsPadding|;
+                        background-color:|lsBgColor| }
+            .lsSearch input[type=text] { background-color:|lsInputBgColor|;
+               border-bottom: |lsBottom| |lsBottomColor|;
+               width:|lsInputWidth|; height:|lsInputHeight| }
+            .lsSearch .dropdown { background-color:|lsrBgColor| }
+            .lsAll { font-size:80%; font-style:italic }''',
 
      # Live-search-related functions (LS)
      js='''
