@@ -2,7 +2,7 @@
    calendar, etc.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -19,8 +19,11 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+from DateTime import DateTime
+
 from appy.px import Px
 from appy.model.batch import Batch
+from appy.database.operators import in_
 from appy.utils import string as sutils
 from appy.model.utils import Object as O
 from appy.ui import LinkTarget, Columns, Title
@@ -41,8 +44,9 @@ class Mode:
     pxActions = Px('''
      <table>
       <tr><td for="action in actions"
-            var2="field=action; fieldName=field.name; layout='query';
-                  multi=True">:action.pxRender</td></tr>
+            var2="multi=action.getMulti('queryResult', req.search);
+                  field=action; fieldName=field.name;
+                  layout='query'">:action.pxRender</td></tr>
      </table>''')
 
     @classmethod
@@ -76,8 +80,7 @@ class Mode:
         '''Gets the i18n text corresponding to mode named p_name'''
         name = class_.__name__.lower()
         name = name.rsplit('_', 1)[0] if '_' in name else name
-        return _('result_mode_%s' % name) if name in Mode.concrete \
-                                          else _('custom_%s' % name)
+        return _('result_mode_%s' % name) if name in Mode.concrete else _(name)
 
     @classmethod
     def isGrid(class_, o):
@@ -114,7 +117,8 @@ class Mode:
         # Is the search integrated into another field ?
         self.inField = False
         # The target for "a" tags
-        self.target = LinkTarget(self.class_, popup=uiSearch.search.viaPopup)
+        self.target = LinkTarget(self.class_.python,
+                                 popup=uiSearch.search.viaPopup)
         # How to render links to result objects ?
         self.titleMode = uiSearch.getTitleMode(self.popup)
 
@@ -156,7 +160,7 @@ class Mode:
             if initatorParams: params.update(initatorParams)
         # Add custom search criteria
         if self.criteria:
-            params['criteria'] = self.criteria
+            params['criteria+'] = self.criteria
         # Add the "ref" key if present
         ref = self.tool.req.ref
         if ref:
@@ -176,11 +180,22 @@ class Mode:
         '''To be overridden by subclasses for adding Ajax parameters
            (see m_getAjaxData above)'''
 
-    def getAjaxDataRow(self, o, **params):
+    def getHookFor(self, o):
+        '''When p_o is shown within a list/grid/... of search results, this
+           method computes the ID of the ajax-refreshable hook tag containing
+           p_o.'''
+        # Currently, this tag is only made of the object ID. Tthis could lead to
+        # problems if several searches are shown on the same page. The solution
+        # would be to prefix the ID with the search name. But some searchs do
+        # not have any name or have a long name (being found within other
+        # fields).
+        return str(o.iid)
+
+    def getAjaxDataRow(self, o, rhook, **params):
         '''Initializes an AjaxData object on the DOM node corresponding to the
            row displaying info about p_o within the results.'''
-        return "new AjaxData('%s/pxResult', 'GET', %s, '%d', '%s')"% \
-               (o.url, sutils.getStringFrom(params), o.iid, self.hook)
+        return "new AjaxData('%s/pxResult','GET',%s,'%s','%s')" % \
+               (o.url, sutils.getStringFrom(params), rhook, self.hook)
 
     def getRefUrl(self):
         '''When the search is triggered from a Ref field, this method returns
@@ -192,7 +207,7 @@ class List(Mode):
     '''Displays search results as a table containing one row per object'''
 
     # Icon for switching to this mode
-    icon = 'list.svg'
+    icon = 'list' # .svg
 
     # Name for this mode
     name = 'list'
@@ -205,23 +220,24 @@ class List(Mode):
            var2="field=column.field" width=":column.width" align=":column.align"
            class=":mode.cbClass if field == '_checkboxes' else ''">
         <x if="column.header">
-         <img if="field == '_checkboxes'" src=":url('checkAll.svg')"
+         <img if="field == '_checkboxes'" src=":svg('checkAll')"
               class="clickable iconM" title=":_('check_uncheck')"
               onclick=":'toggleAllCbs(%s)' % q(mode.checkboxesId)"/>
          <x if="not column.special">
           <span class="htitle">::Px.truncateText(_(field.labelId))</span>
           <!-- Sort icons -->
-          <x if="field.isSortable() and (mode.batch.total &gt; 1)">
+          <div if="field.isSortable() and mode.batch.total &gt; 1"
+               class="thIcons">
            <img if="(mode.sortKey != field.name) or (mode.sortOrder == 'desc')"
                 onclick=":'askBunchSorted(%s, %s, %s)' % \
                           (q(mode.hook), q(field.name), q('asc'))"
-                src=":url('asc.svg')" class="clickable iconS"/>
+                src=":svg('asc')" class="clickable iconS"/>
            <img if="(mode.sortKey != field.name) or (mode.sortOrder == 'asc')"
                 onclick=":'askBunchSorted(%s, %s, %s)' % \
                           (q(mode.hook), q(field.name), q('desc'))"
-                src=":url('asc.svg')" class="clickable iconS"
+                src=":svg('asc')" class="clickable iconS"
                 style="transform:rotate(180deg)"/>
-          </x>
+          </div>
           <!-- Filter widget -->
           <x var="field,filterPx=field.getFilterPx()"
              if="filterPx and mode.showFilters">:filterPx</x>
@@ -246,7 +262,7 @@ class List(Mode):
       <input type="button"
              var="label=_('object_link_many'); css=ui.Button.getCss(label)"
              value=":label" class=":css"
-             style=":url('linkMany.svg', bg='18px 18px')"
+             style=":svg('linkMany', bg='18px 18px')"
              onclick=":uiSearch.initiator.jsSelectMany(\
                    q, mode.sortKey, mode.sortOrder, mode.getFiltersString())"/>
      </div>
@@ -280,7 +296,7 @@ class List(Mode):
         self.batch.hook = self.hook
         self.objects = self.batch.objects
         # Show sub-titles ? Show filters ?
-        self.showSubTitles = req.showSubTitles == 'True'
+        self.showSubTitles = req.showSubTitles in ('True', None)
         self.showFilters = self.filters or self.batch.total > 1
         # Every matched object may be selected via a checkbox
         self.rootHook = ui.getRootHook()
@@ -303,7 +319,7 @@ class List(Mode):
                                    ui.dir, addCheckboxes=self.checkboxes)
         # Get the Javascript code allowing to store IDs from batch objects in
         # the browser's session storage.
-        self.store = self.batch.store(search)
+        self.store = self.batch.store(search, name=ui.name)
 
     def updateAjaxParameters(self, params):
         '''List-specific ajax parameters'''
@@ -372,14 +388,15 @@ class Grid(List):
     '''Displays search results as a table containing one cell per object'''
 
     # Icon for switching to this mode
-    icon = 'grid.svg'
+    icon = 'grid' # .svg
 
     # Name for this mode
     name = 'grid'
 
     px = Px('''
      <!-- Filters -->
-     <div class="gridFilters" if="uiSearch.search.showFilters">
+     <div class="gridFilters" if="uiSearch.search.showFilters"
+          style=":'justify-content:%s' % uiSearch.search.gridFiltersAlign">
       <x for="column in [c for c in mode.columns if not c.special]"
          var2="field,filterPx=column.field.getFilterPx()">
        <div if="filterPx and mode.showFilters">
@@ -425,8 +442,8 @@ class Grid(List):
       <div if="not mode.objects">::_('query_no_filtered_result')</div>
      </div>''',
 
-     css='''.gridFilters { display: flex; flex-wrap: wrap;
-                           justify-content: center; margin-top: 20px }
+     css='''.gridFilters { display:flex; flex-wrap:wrap;
+                           margin:|gridFiltersMargin|}
             .gridFilters > div { margin: 10px 20px }''',
 
      js='''
@@ -455,7 +472,7 @@ class Calendar(Mode):
     '''Displays search results in a monthly calendar view'''
 
     # Icon for switching to this mode
-    icon = 'calendar.svg'
+    icon = 'calendar' # .svg
 
     # Name for this mode
     name = 'calendar'
@@ -463,10 +480,16 @@ class Calendar(Mode):
     px = Px('''<x var="layoutType='view';
                        field=tool.getField('calendar')">:field.view</x>''')
 
+    # Formats for keys representing dates
+    dayKey = '%Y%m%d'
+
+    # Format for representing hours in the UI
+    hourFormat = '%H:%M'
+
     def __init__(self, *args):
         Mode.__init__(self, *args)
         # For this calendar view to work properly, objects to show inside it
-        # must have the following attributes:
+        # must have the following fields:
         # ----------------------------------------------------------------------
         # date    | an indexed and required Date field with format =
         #         | Date.WITH_HOUR storing the object's start date and hour;
@@ -474,8 +497,12 @@ class Calendar(Mode):
         # endDate | a not necessarily required Date field with format =
         #         | Date.WITH_HOUR storing the object's end date and hour.
         # ----------------------------------------------------------------------
-        # Optionally, if objects define the following attributes, special icons
-        # indicating repeated events will be shown.
+        # Optionnally, if you want to use index "months" (we advice you to do
+        # so), an indexed Computed field with this name must also be defined
+        # (see details in m_init below).
+        # ----------------------------------------------------------------------
+        # Optionally, too, if objects define the following attributes, special
+        # icons indicating repeated events will be shown.
         # ----------------------------------------------------------------------
         # successor   | Ref field with multiplicity = (0,1), allowing to
         #             | navigate to the next object in the list (for a repeated
@@ -487,6 +514,14 @@ class Calendar(Mode):
         #             | hereabove.
         # ----------------------------------------------------------------------
 
+    def getMonthIndex(self):
+        '''Deduce, from p_self.class_, what is the month index to use'''
+        # Is there, on p_self.class_, a field name "months" ?
+        months = getattr(self.class_.python, 'months', None)
+        if not months: return 'date'
+        return 'months' if months.type == 'Computed' and months.indexed \
+                        else 'date'
+
     def init(self):
         '''Creates a stub calendar field'''
         Mode.init(self)
@@ -511,12 +546,65 @@ class Calendar(Mode):
         #           | day.
         # ----------------------------------------------------------------------
         self.objects = {} # ~{s_YYYYmmdd: [(s_entryType, Object)]}~
-        # Formats for keys representing dates
-        self.dayKey = '%Y%m%d'
-        # Format for representing hours in the UI
-        self.hourFormat = '%H:%M'
         # If filters are defined from a list mode, get it
-        self.filters = self.class_.getFilters(tool)
+        self.filters = self.class_.getFilters(self.tool)
+        # The name of the index to use to restrict the search to the currently
+        # shown month. If p_self.klass defines a Computed indexed TextIndex
+        # field named "months", this index will be used. Else, index "date" will
+        # be used. These 2 indexes are described hereafter.
+        # ----------------------------------------------------------------------
+        # "date"   | This index is expected to define the (start) date of the
+        #          | objects to search. So, the search will match any object
+        #          | whose (start) date is included in the range of dates
+        #          | defined by the currently shown month. Recall that this
+        #          | range may be larger than the month itself: because complete
+        #          | weeks are shown, some days from the previous and next
+        #          | months may be present, too.
+        #          |
+        #          | While this index is the easiest to implement, it has the
+        #          | following problem: objects spanning several days (by using
+        #          | fields "date" and "endate") and whose start date is not
+        #          | among the range of visible dates, will not be part of the
+        #          | result. This is why an alternate index named "month' is
+        #          | also proposed (see below).
+        # ----------------------------------------------------------------------
+        # "months" | When using index "months", every object is expected to
+        #          | hold, in a Computed indexed TextIndex field named "months",
+        #          | the space-separated list of months the object crosses
+        #          | (every crossing month must be represented as a string of
+        #          | the form YYYYmm). "Crossing a month" means: there must be a
+        #          | not-empty intersection between the range of visible days
+        #          | for this month, and the object's range of days. For objects
+        #          | defining no end date, this range is reduced to a unique day
+        #          | (defined by field "date").
+        #          |
+        #          | Because this list of crossing months is relatively complex
+        #          | to produce, Appy provides a function that computes it:
+        #          |
+        #          |           appy.shared.dates.getCalendarMonths
+        #          |
+        #          | Here is a complete example of a Appy class using index
+        #          | "months".
+        #          |
+        #          | from appy.shared.dates import getCalendarMonths
+        #          |
+        #          | class Event:
+        #          |     ...
+        #          |     date    = Date(format=Date.Date.WITH_HOUR,
+        #          |                    multiplicity=(1,1), indexed=True, ...)
+        #          |     endDate = Date(format=Date.Date.WITH_HOUR,
+        #          |                    indexed=True, ...)
+        #          |     ...
+        #          |     def computeMonths(self):
+        #          |         '''Compute the months crossed by p_self'''
+        #          |         # p_self.endDate may be None
+        #          |         return ' '.join(getCalendarMonths(self.date,
+        #          |                                           self.endDate))
+        #          |
+        #          |     months  = Computed(method=computeMonths,
+        #          |                        indexed='TextIndex', show=False)
+        # ----------------------------------------------------------------------
+        self.monthIndex = self.getMonthIndex()
 
     def updateAjaxParameters(self, params):
         '''Grid-specific ajax parameters'''
@@ -549,96 +637,120 @@ class Calendar(Mode):
         else:
             r[dateKey].append(entry)
 
-    def addEntries(self, obj):
-        '''Add, in self.objects, entries corresponding to p_obj. If p_obj spans
-           a single day, a single entry of the form ("start", p_obj) is added at
-           the key corresponding to this day. Else, a series of entries are
-           added, each of the form (s_entryType, p_obj), with the same object,
-           for every day in p_obj's timespan.
-
-           For example, for an p_obj starting at "1975/12/11 12:00" and ending
-           at "1975/12/13 14:00" will produce the following entries:
-              key "19751211"  >  value ("start+", obj)
-              key "19751212"  >  value ("pass", obj)
-              key "19751213"  >  value ("end", obj)
-        '''
+    def addEntries(self, o, gridFirst, gridLast):
+        '''Add, in self.objects, entries corresponding to p_o. If p_o spans a
+           single day, a single entry of the form ("start", p_o) is added at the
+           key corresponding to this day. Else, a series of entries are added,
+           each of the form (s_entryType, p_o), with the same object, for every
+           day in p_o's timespan.'''
+        # For example, for an p_o(bject) starting at "1975/12/11 12:00" and
+        # ending at "1975/12/13 14:00", m_addEntries will produce the following
+        # entries:
+        #      key "19751211"  >  value ("start+", obj)
+        #      key "19751212"  >  value ("pass", obj)
+        #      key "19751213"  >  value ("end", obj)
+        # ~~~
         # Get p_obj's start and end dates
-        start = obj.date
-        startKey = start.strftime(self.dayKey)
-        end = obj.endDate
-        endKey = end and end.strftime(self.dayKey) or None
+        fmt = Calendar.dayKey
+        start = o.date
+        startKey = start.strftime(fmt)
+        end = o.endDate
+        endKey = end.strftime(fmt) if end else None
         # Shorthand for self.objects
         r = self.objects
-        if not endKey or (endKey == startKey):
+        if not endKey or endKey == startKey:
             # A single entry must be added for p_obj, at the start date
-            self.addEntry(startKey, ("start", obj))
+            self.addEntry(startKey, ('start', o))
         else:
+            # Insert one event per day, provided the events are in the grid
+            # ~~~
             # Add one entry at the start day
-            self.addEntry(startKey, ("start+", obj))
-            # Add "pass" entries for every day between the start and end days
-            next = start + 1
-            nextKey = next.strftime(self.dayKey)
-            while nextKey != endKey:
+            if start >= gridFirst:
+                self.addEntry(startKey, ('start+', o))
+                next = start + 1
+            else:
+                # Ignore any day between v_start and p_gridFirst
+                next = gridFirst
+            # Add "pass" entries for every day between the event's start and end
+            # dates.
+            nextKey = next.strftime(fmt)
+            while nextKey != endKey and next <= gridLast:
                 # Add a "pass" event
-                self.addEntry(nextKey, ('pass', obj))
+                self.addEntry(nextKey, ('pass', o))
                 # Go the the next day
                 next += 1
-                nextKey = next.strftime(self.dayKey)
+                nextKey = next.strftime(fmt)
             # Add an "end" entry at the end day
-            self.addEntry(endKey, ('end', obj))
+            if end <= gridLast:
+                self.addEntry(endKey, ('end', o))
 
     def search(self, first, grid):
-        '''Performs the query, limited to the date range defined by p_grid'''
-        # Performs the query, restricted to the visible date range
-        last = DateTime(grid[-1][-1].strftime('%Y/%m/%d 23:59:59'))
-        dateSearch = Search(date=(first, last), sortBy='date', sortOrder='asc')
-        res = self.tool.executeQuery(self.className,
-          search=self.uiSearch.search, maxResults='NO_LIMIT',
-          search2=dateSearch, filters=self.filters)
+        '''Performs the search, restricted to the visible date range as defined
+           by p_grid.'''
+        # # The first date in the p_grid, that may be earlier than the p_first
+        # day of the month.
+        gridFirst = grid[0][0]
+        # The last date in the grid, calibrated
+        gridLast = DateTime(grid[-1][-1].strftime('%Y/%m/%d 23:59:59'))
+        # Get the search parameters being specific to the range of dates
+        params = {'sortBy':'date', 'sortOrder':'asc'}
+        if self.monthIndex == 'date':
+            # Define the search based on every event's (start) date.
+            # ~~~
+            # As first date, prefer the first visible date in the p_grid instead
+            # of the p_first day of the month.
+            params['date'] = in_(gridFirst, gridLast)
+        else: # p_self.monthIndex is "months"
+            # Define the search based on index "months"
+            params['months'] = first.strftime('%Y%m')
+        # Create a specific search with the parameters restricted to the visible
+        # date range, and execute the main search, merged with the specific one.
+        dateSearch = self.tool.Search(**params)
+        r = self.uiSearch.search.run(self.tool.H(), batch=False,
+                                     filters=self.filters, other=dateSearch)
         # Produce, in self.objects, the dict of matched objects
-        for zobj in res.objects:
-            self.addEntries(zobj.appy())
+        for o in r:
+            self.addEntries(o, gridFirst, gridLast)
 
     def dumpObjectsAt(self, date):
         '''Returns info about the object(s) that must be shown in the cell
            corresponding to p_date.'''
         # There may be no object dump at this date
-        dateStr = date.strftime(self.dayKey)
+        dateStr = date.strftime(Calendar.dayKey)
         if dateStr not in self.objects: return
         # Objects exist
         r = []
         types = self.entryTypes
-        url = self.tool.getIncludeUrl
-        for entryType, obj in self.objects[dateStr]:
+        for entryType, o in self.objects[dateStr]:
             # Dump the event hour and title. The back hook allows to refresh the
             # whole calendar view when coming back from the popup.
             eType = types[entryType]
             # What CSS class(es) to apply ?
-            css = eType.css and ('calEvt %s' % eType.css) or 'calEvt'
+            css = ('calEvt %s' % eType.css) if eType.css else 'calEvt'
             # Show start and/or end hour ?
             eHour = sHour = ''
             if eType.start:
                 sHour = '<td width="2em">%s</td>' % \
-                        obj.date.strftime(self.hourFormat)
+                        o.date.strftime(self.hourFormat)
             if eType.end:
-                endDate = obj.endDate
+                endDate = o.endDate
                 if endDate:
                     eHour = ' <abbr title="%s">¬</abbr>' % \
                             endDate.strftime(self.hourFormat)
             # Display indicators that the event spans more days
-            past = eType.past and '⇠ ' or ''
-            future = eType.future and ' ⇢' or ''
+            past = '⇠ ' if eType.past else ''
+            future = ' ⇢' if eType.future else ''
             # The event title
             title = Title.get(o, target=self.target, popup=True,
                               backHook='configcalendar', maxChars=24)
             # Display a "repetition" icon if the object is part of a series
-            hasSuccessor = obj.successor
-            hasPredecessor = obj.predecessor
+            hasSuccessor = o.successor
+            hasPredecessor = o.predecessor
             if hasSuccessor or hasPredecessor:
                 # For the last event of a series, show a more stressful icon
-                name = not hasSuccessor and 'repeated_last' or 'repeated'
+                name = 'repeated' if hasSuccessor else 'repeated_last'
                 icon = '<img src="%s" class="help" title="%s"/>' % \
-                       (url(name), obj.translate(name))
+                       (o.buildUrl(name), o.translate(name))
             else:
                 icon = ''
             # Produce the complete entry
@@ -651,7 +763,7 @@ class Custom(Mode):
     '''Displays search results via a custom PX'''
 
     # Icon for switching to this mode
-    icon = 'custom.svg'
+    icon = 'custom' # .svg
 
     # Name for this mode
     name = 'custom'
