@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -31,20 +31,24 @@ from appy.utils.dates import Date
 from appy.model.group import Group
 from appy.model.fields import Show
 from appy.model.query import Query
+from appy.model.mover import Mover
 from appy.database import Database
 from appy.ui.layout import Layouts
 from appy.utils.mail import sendMail
 from appy.ui.template import Template
 from appy.model.fields.ref import Ref
 from appy.server.backup import Backup
+from appy.model.searches import Search
 from appy.model.fields.rich import Rich
 from appy.model.fields import Initiator
 from appy.model.carousel import Carousel
 from appy.model.fields.phase import Page
+from appy.model.utils import Object as O
 from appy.model.page import Page as OPage
 from appy.database.catalog import Catalog
 from appy.model.fields.string import String
 from appy.model.fields.action import Action
+from appy.test.monitoring import Monitoring
 from appy.model.translation import Translation
 from appy.model.fields.calendar import Calendar
 from appy.model.fields.computed import Computed
@@ -53,6 +57,9 @@ from appy.server.context import AuthenticationContext
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 RS_FG_KO   = 'Server cannot be restarted if running in the foreground.'
 RESTART_S  = "Restarting server (wait %d'')..."
+SRV_CONF   = 'Server(s) %s configured.'
+USR_CREA   = 'User "%s" created.'
+GRP_CREA   = 'Group "admins" created.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Tool(Base):
@@ -107,7 +114,7 @@ class Tool(Base):
                 if status: status = ', ' + status
             servers.append('%s (%s%s)' % (cfg, enabled, status))
         if servers:
-            self.log('Server(s) %s configured.' % ', '.join(servers))
+            self.log(SRV_CONF % ', '.join(servers))
 
     def init(self, handler, poFiles):
         '''Ensure all required tool sub-objects are present and coherent'''
@@ -125,14 +132,14 @@ class Tool(Base):
                 # No. Create it.
                 user = self.create('users', secure=False, id=login, login=login,
                             password=login, roles=roles)
-                self.log('User "%s" created.' % login)
+                self.log(USR_CREA % login)
 
         # Ensure group "admins" exists. This group is granted role "Manager",
         # the role with the highest level of prerogatives in Appy.
         if 'admins' not in root.objects:
             self.create('groups', secure=False, id='admins', login='admins',
                         title='Administrators', roles=['Manager'])
-            self.log('Group "admins" created.')
+            self.log(GRP_CREA)
 
         # Load Appy "po" files if we need to inject their values into
         # Translation objects. app's "po" files are already in p_poFiles.
@@ -189,6 +196,7 @@ class Tool(Base):
         if self.allows('write'): return 'view'
 
     ta = {'label': 'Tool'}
+
     # Storing the Appy version on the tool (in the form of a string "x.y.z")
     # allows to trigger automatic migrations at server startup.
     appyVersion = String(show='view', **ta)
@@ -197,20 +205,10 @@ class Tool(Base):
     def calendarPreCompute(self, first, grid):
         '''Computes pre-computed information for the tool calendar'''
         # If this calendar is used as mode for a search, get this search
-        req = self.req
-        ctx = self.H().context
-        mode = None
-        if 'uiSearch' in ctx:
-            # The search and related mode are already in the PX context
-            mode = ctx.mode
-        elif req.has_key('search') and req.has_key('className'):
-            # Get the search and its mode from request keys
-            className = req['className']
-            tool = self.o
-            search = self.o.getSearch(className, req['search'], ui=True)
-            mode = search.Calendar(tool.getAppyClass(className), className,
-                                   search, ctx.popup, tool)
-            mode.init(req, ctx.dir)
+        ctx = self.traversal.context
+        mode = ctx.mode
+        if not mode:
+            Px.injectRequest(ctx, self.req, self)
         # Trigger the search via the mode
         if mode: mode.search(first, grid)
         return O(mode=mode)
@@ -241,18 +239,6 @@ class Tool(Base):
       show=forToolWriters, showHeaders=True, actionsDisplay='right',
       shownInfo=User.listColumns, **ta)
 
-    def getUserName(self, login=None, normalized=False):
-        '''Gets the user name corresponding to p_login (or the currently logged
-           user if None), or the p_login itself if the user does not exist
-           anymore. If p_normalized is True, special chars in the first and last
-           names are normalized.'''
-        if not login:
-            user = self.user
-        else:
-            user = self.search1('User', login=login)
-            if not user: return login
-        return user.getTitle(normalized=normalized)
-
     def doSynchronizeExternalUsers(self):
         '''Synchronizes the local User copies with a distant LDAP user base'''
         cfg, context = self.config.security.getLdap()
@@ -319,6 +305,10 @@ class Tool(Base):
                       show=lambda o: not o.isEmpty('pages'),
                       select=lambda o: o.pages, **ta)
 
+    # Update the base URL of all internal links within pages's rich fields
+    updateBaseUrl = Action(action=lambda o, options: options.run(),
+                           show='buttons', options=Mover, page=pp, **ta)
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Page "carousels"
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
@@ -364,6 +354,15 @@ class Tool(Base):
       page=Page('database', phase='admin', show=forAdmin,
                 label='Tool_page_database'), **ta)
 
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+    #  Monitoring
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+
+    traverse['check'] = True
+    def check(self):
+        '''Returns monitoring-related info about this instance'''
+        return Monitoring().get(self)
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Main methods
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
@@ -456,26 +455,28 @@ class Tool(Base):
 
      <!-- The home logo and text -->
      <div if="isAnon and _px_.name == 'home'" class="homeText">
-       <img src=":url('homeLogo')"/>
+       <img if="cfg.homeShowLogo" src=":url('homeLogo')"/>
        <!-- The home text must not be shown here, if shown on a public page -->
        <x if="not tool.isEmpty('defaultPage') and 
-              not cfg.discreetLogin">::tool.defaultPage.content</x>
+         not cfg.discreetLogin">::tool.defaultPage.getShownValue('content')</x>
      </div>
 
      <!-- The backgrounds for px "homes" -->
      <div if="_px_.name == 'homes'" class="homes">
-      <div for="fileName, info in cfg.homesBackgrounds.items()"
-           var2="base,width=info; bg=tool.buildUrl(fileName, base=base,bg=True)"
+      <div for="fileName, width in cfg.homesBackgrounds.items()"
+           var2="bg=tool.buildUrl(fileName, bg=True)"
            style=":'%s;width:%s' % (bg, width)"></div>
      </div>''',
 
      css='''
-      .homeText { position:fixed; left:45px; top:25px; color:|homeTextColor| }
-      .homeText h1 { font-size: 300%; margin-left: -5px }
-      .homeText h2 { font-size: 200% }
+      .homeText { position:fixed; left:|homeTextLeft|; top:|homeTextTop|;
+                  color:|homeTextColor| }
+      .homeText h1 { font-size:300%; margin-left:-5px; padding:0 }
+      .homeText h2 { font-size:200% }
       .homeText p { font-size: 120%; margin: 0 }
       .homeText a, .homeText a:visited { color: #ffd8eb }
-      .homeTop { position:fixed; top:20px; right:20px; color:|headerColor| }
+      .homeTop { position:fixed; top:20px; right:20px; color:|headerColor|;
+                 z-index:1 }
      ''',
 
      template=Template.px, hook='content', name='home')
