@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -54,6 +54,7 @@ class User(Base):
     searchAdvanced = Search('advanced', actionsDisplay='inline')
     listColumns = ('title', 'name*15%|', 'login*20%|', 'roles*20%|',
                    'state*10%|')
+    listColumnsShort = ('title', 'login')
 
     # The following users are always present in any Appy database
     defaultUsers = {'admin': ('Manager',), 'system': ('Manager',), 'anon': ()}
@@ -102,7 +103,7 @@ class User(Base):
     def showEncrypted(self):
         '''Allow to show encrypted password to admins only, via the xml
            layout.'''
-        if self.user.login == 'admin' and (self.source != 'sso'): return 'xml'
+        if self.user.login == 'admin' and self.source != 'sso': return 'xml'
 
     encrypted = Computed(show=showEncrypted, label='User',
                          method=lambda o: o.values['password'])
@@ -111,9 +112,11 @@ class User(Base):
     #  Title, name, first name
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
 
+    baseGroup = Group('main', style='grid', hasLabel=False)
+
     pm = {'page': Page('main', label='User_page_main', icon='pageProfile.svg'),
           'width': 28, 'layouts': Layouts.g, 'label': 'User',
-          'group': Group('main', style='grid', hasLabel=False)}
+          'group': baseGroup}
 
     def getTitle(self, normalized=False, useName=True, nameFirst=False,
                  nameTransform=None, firstNameTransform=None):
@@ -266,9 +269,8 @@ class User(Base):
         # will be added among user roles (again and again).
         r = roles[:]
         # Get the user logins
-        if self.login != 'anon':
-            for login in logins:
-                r.append('user:%s' % login)
+        for login in logins:
+            r.append('user:%s' % login)
         return or_(*r)
 
     def getAllowedValue(self):
@@ -333,7 +335,7 @@ class User(Base):
         '''
         guard = guard or self.guard
         # Return the cached value on the guard when appropriate
-        if not compute and (self.login == guard.userLogin):
+        if not compute and self.login == guard.userLogin:
             return guard.userRoles
         # Compute it
         r = list(self.roles)
@@ -397,6 +399,17 @@ class User(Base):
     # stored in an external LDAP (source='ldap').
     source = String(show='xml', default='zodb', layouts='f', label='User')
 
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+    #  Editable pages
+    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
+
+    def showEPages(self):
+        '''Show the list of editable pages to Managers and the user itself,
+           provided there is at least one such page.'''
+        if not self.editablePages: return
+        user = self.user
+        if self.login == user.login or user.hasRole('Manager'): return 'view'
+
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
     #  Actions
     #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -
@@ -420,9 +433,9 @@ class User(Base):
     def showResetPassword(self):
         '''Action "reset password" is available to anyone having write access to
            the user, excepted the user himself.'''
-        if (self.source == 'zodb') and self.guard.mayEdit(self) and \
-           (self.user != self) and not self.isSpecial(includeAdmin=False):
-            return 'buttons'
+        if self.source == 'zodb' and self.guard.mayEdit(self) and \
+           self.user != self and not self.isSpecial(includeAdmin=False):
+            return Show.BS
 
     resetPassword = Action(action=doResetPassword, show=showResetPassword,
                            confirm=True, label='User', icon='pagePassword.svg',
@@ -489,7 +502,7 @@ class User(Base):
     # Display, in the user strip, links to the User instance of the logged user
     pxUserLink = Px('''
      <a if="cfg.userLink" href=":user.url">
-      <img src=":url('user.svg')" class="icon"/>
+      <img src=":svg('user')" class="icon"/>
       <span class="topText">:user.getFirstName()</span>
      </a>
      <x if="not cfg.userLink">:user.getFirstName()</x>''')
@@ -602,10 +615,10 @@ class User(Base):
             # Authentication fails (and user "anon" is returned) if the user was
             # not found or inactive, its password was invalid or the required
             # authentication context was not found.
-            if (user is None) or user.isAnon() or (user.state=='inactive') or \
-               (not user.getField('password').check(user, password)) or \
-               (not ctx and authContext and authContext.isMandatory(tool) and \
-                (place != 'basic')):
+            if user is None or user.isAnon() or user.state == 'inactive' or \
+               not user.getField('password').check(user, password) or \
+               (not ctx and authContext and authContext.isMandatory(tool,True))\
+               and place != 'basic':
                 # Disable the authentication cookie
                 guard.Cookie.disable(handler)
                 user = handler.dbConnection.root.objects.get('anon')
@@ -624,8 +637,8 @@ class User(Base):
     def getOnline(class_, tool, expr=None, context=None):
         '''Returns a XHTML table showing users being currently connected'''
         # If p_expr is given, it must be an expression that will be evaluated on
-        # every connected user. The user will be part ofthe result only if the
-        # expression evaluates to True. The user is given to the expression as
+        # every connected user. The user will be part of the result only if the
+        # expression evaluates to True. The user is passed to the expression as
         # variable "user".
         # ~~~
         # Get and count connected users
@@ -675,16 +688,28 @@ class User(Base):
             password = self.getField('password').generate()
             # Store it, encrypted
             self.password = password
-            # Return the clear password to the UI
-            message = self.translate('new_password_text',
-                                     mapping={'password': password})
+            recipient = self.getMailRecipient()
+            _ = self.translate
+            if self.config.mail and recipient:
+                # Send the password by mail to the user
+                subject = _('first_password')
+                map = {'siteUrl': self.siteUrl, 'login': self.login,
+                       'password': password}
+                body = _('first_password_body', mapping=map, asText=True)
+                self.tool.sendMail(recipient, subject, body)
+                message = _('first_password_sent')
+            else:
+                # Return the clear password to the UI
+                message = _('new_password_text', mapping={'password': password})
+            # The user will need to change it at next login
+            self.changePasswordAtNextLogin = True
         page = self.req.page or 'main'
         if page == 'main':
             login = self.login
             # (re)-compute p_self's title, computed from other fields
             self.updateTitle()
             # Ensure correctness of some infos about this user
-            if isLocal and (self.id == 'admin'): self.ensureIsManager()
+            if isLocal and self.id == 'admin': self.ensureIsManager()
             # p_self must be owned by itself
             self.localRoles.add(login, 'Owner')
             # If the user was created by anon|system, anon|system can't stay its
