@@ -1,7 +1,7 @@
 '''This module defines classes allowing to store an object's history'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -32,6 +32,7 @@ from appy.px import Px
 from appy.model.batch import Batch
 from appy.xml.escape import Escape
 from appy.utils import string as sutils
+from appy.xml.cleaner import StringCleaner
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class EventIterator:
@@ -86,6 +87,8 @@ class EventIterator:
     def __iter__(self): return self
     def __next__(self):
         '''Return the next matching event'''
+        # Stop iterating if p_self.i is negative
+        if self.i < 0: raise StopIteration
         try:
             event = self.history[self.i]
         except IndexError:
@@ -104,6 +107,7 @@ class EventIterator:
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Event(persistent.Persistent):
     '''An object's history is made of events'''
+
     # A format for representing dates at various (non ui) places
     dateFormat = '%Y/%m/%d %H:%M'
 
@@ -112,13 +116,13 @@ class Event(persistent.Persistent):
 
     # The PX displaying details about an event (basically the event's comment)
     pxDetail = Px('''
-     <x var="eventId=event.getId()">
-      <img if="o.class_.mayEditEventComment(o, event, user, isManager)"
-           class="clickable iconS" style="float:right" src=":url('edit.svg')"
-           title=":_('object_edit')"
-           onclick=":'onHistoryEvent(%s,%s,%s,%s)' % \
-                    (q('Edit'), q(o.iid), q(event.date), q(eventId))"/>
-      <span id=":eventId">::event.getComment()</span>
+     <x var="eventId=event.getId();
+             editable=o.class_.mayEditEventComment(o,event, user, isManager)">
+      <img if="editable" class="clickable iconS" style="float:right"
+           src=":svg('edit')" title=":_('object_edit')"
+           onclick=":'onHistoryEvent(%s,%s,%s)' %
+                      (q('modify'), q(eventId), q(eventId))"/>
+      <span id=":eventId">::event.getComment(escaped=editable)</span>
      </x>''')
 
     def __init__(self, login, state, date, comment=None):
@@ -131,7 +135,7 @@ class Event(persistent.Persistent):
         # When did this event occur ?
         self.date = date
         # A textual optional comment for the event
-        self.comment = comment
+        self.comment = StringCleaner.clean(comment)
 
     def clone(self):
         '''Create an return a clone from myself'''
@@ -148,25 +152,42 @@ class Event(persistent.Persistent):
         else:
             self.comment =  '%s%s%s' % (self.comment, sep, comment)
 
-    def getTypeName(self):
+    def getTypeName(self, short=False):
         '''Return the class name, possibly completed with sub-class-specific
            information.'''
-        return self.__class__.__name__
+        # The p_short variant is used for searching events, in methods like
+        # History.getLast.
+        r = self.__class__.__name__
+        return r.lower() if short else r
 
     def getLabel(self, o):
         '''Returns the i18n label for this event'''
         # To be overridden
 
-    def getComment(self, empty='-'):
-        '''Returns the formatted version of p_self.comment'''
-        comment = self.comment
-        if not comment: return empty
-        return Escape.xhtml(comment, p=True)
+    def getComment(self, empty='-', escaped=True):
+        '''Returns p_self.comment, ready to be included in a chunk of XHTML'''
+        r = self.comment or empty
+        return Escape.xhtml(r) if escaped else r
 
     def getId(self):
         '''Return an ID for this history p_event, based on its date.'''
         return str(self.date.millis())
 
+    def getUser(self, o, title=True, expression=None):
+        '''Returns the complete name of the user with this p_self.login'''
+        login = self.login
+        if not login:
+            return '?' if title else None
+        user = o.search1('User', login=login)
+        if not user:
+            return '?' if title else None
+        # Return the found user or its p_title
+        if title:
+            r = eval(expression) if expression else (user.getTitle() or login)
+        else:
+            r = user
+        return r
+
     def __repr__(self):
         '''String representation'''
         date = self.date.strftime(Event.dateFormat)
@@ -182,9 +203,10 @@ class Trigger(Event):
         self.transition = params.pop('transition')
         Event.__init__(self, login, state, date, **params)
 
-    def getTypeName(self):
+    def getTypeName(self, short=False):
         '''Return the class name and the transition name'''
-        return 'Trigger %s' % self.transition
+        r = self.transition
+        return r if short else 'Trigger %s' % r
 
     def getLabel(self, o):
         '''Returns the label for the corresponding transition'''
@@ -204,9 +226,10 @@ class Action(Event):
         self.action = params.pop('action')
         Event.__init__(self, login, state, date, **params)
 
-    def getTypeName(self):
+    def getTypeName(self, short=False):
         '''Return the class name and the transition name.'''
-        return 'Action %s' % self.action
+        r = self.action
+        return r if short else 'Action %s' % r
 
     def getLabel(self, o):
         '''Returns the label for the corresponding action'''
@@ -238,8 +261,13 @@ class Change(Event):
                 field=o.getField(fname)">
        <td><x>::_(field.labelId) if field else fname</x>
            <x if="lg">:' (%s)' % ui.Language.getName(lg)</x></td>
-       <td>::field.getHistoryValue(o, value, history.data.index(event), lg) \
-             if field else value</td>
+       <td>
+        <x>::field.getHistoryValue(o, value, history.data.index(event), lg) \
+             if field else value</x>
+        <!-- A comment may coexist with the change in itself -->
+        <x var="comment=event.getComment(empty='', escaped=False)"
+           if="comment">::comment</x>
+       </td>
       </tr>
      </table>''')
 
@@ -261,8 +289,8 @@ class Change(Event):
         if name in self.changes: return True
         # Search for a key of the form (name, language)
         for key in self.changes.keys():
-            if isinstance(key, tuple) and (key[0] == name) and \
-               (not language or (language == key[1])):
+            if isinstance(key, tuple) and key[0] == name and \
+               (not language or language == key[1]):
                 return True
 
     def getValue(self, name, language=None):
@@ -272,19 +300,19 @@ class Change(Event):
         return self.changes.get(key)
 
     def getLabel(self, o):
-        '''Returns the translated label for a data change'''
+        '''Returns the i18n label for a data change'''
         return 'event_Change'
 
     def getDiffTexts(self, o):
         '''Returns a tuple (insertText, deleteText) containing texts to show on,
            respectively, inserted and deleted chunks of text in a XHTML diff.'''
-        user = o.search1('User', login=self.login, secure=False)
+        user = o.search1('User', login=self.login)
         mapping = {'userName': user.getTitle() if user else self.login}
         r = []
         tool = o.tool
         for type in self.diffEntries:
             msg = o.translate('history_%s' % type, mapping=mapping)
-            date = tool.formatDate(self.date)
+            date = tool.Date.format(tool, self.date, withHour=True)
             r.append('%s: %s' % (date, msg))
         return r
 
@@ -299,7 +327,7 @@ class Custom(Event):
         Event.__init__(self, login, state, date, **params)
 
     def getLabel(self, o):
-        '''Returns the translated label for a data change'''
+        '''Returns the i18n label for a data change'''
         return self.label
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -312,32 +340,56 @@ class Link(Event):
     # instead, the title of the newly linked object is set as comment in this
     # Link event.
 
+    # The name of specific attribute added by this class. It is not "hardcoded"
+    # in the constructor, because a sub-class may define another one. For class
+    # Link, boolean "addition" tells if we have linked an existing object
+    # (addition==False) or a newly created one (addition==True).
+    attribute = 'addition'
+
     def __init__(self, login, state, date, **params):
         # We may have linked an existing object or a newly created one
-        self.addition = params.pop('addition')
-        Event.__init__(self, login, state, date, **params)
+        attribute = self.attribute
+        setattr(self, attribute, params.pop(attribute))
+        # The name of the Ref field on which the operation was performed
+        self.field = params.pop('field')
+        # Call the base constructor
+        super().__init__(login, state, date, **params)
+
+    def getTypeName(self, short=False):
+        '''Return the class name + type-specific info'''
+        r = self.attribute if getattr(self, self.attribute) else ''
+        field = getattr(self, 'field', None) # May be absent from old DBs
+        cname = self.__class__.__name__
+        if short:
+            r = '%s%s' % (cname.lower(), r)
+        else:
+            suffix = ' (%s)' % field if field else ''
+            r = '%s%s%s' % (cname, r.capitalize(), suffix)
+        return r
 
     def getLabel(self, o):
-        '''Returns the translated label for a link'''
+        '''Returns the i18n label for a link'''
         return 'event_%s' % ('Addition' if self.addition else 'Link')
 
-class Unlink(Event):
+class Unlink(Link):
     '''Represents an object being unlinked from another one via a Ref field'''
 
-    def __init__(self, login, state, date, **params):
-        # We may have simply unlinked an object or it may have been deleted
-        self.deletion = params.pop('deletion')
-        Event.__init__(self, login, state, date, **params)
+    # The name of specific attribute added by this class. This boolean indicates
+    # if we have simply unlinked an object (deletion==False) or if it has been
+    # completely deleted (deletion==True).
+    attribute = 'deletion'
 
     def getLabel(self, o):
-        '''Returns the translated label for an unlink'''
-        return 'event_%s' % ('Deletion' if self.addition else 'Unlink')
+        '''Returns the i18n label for an unlink'''
+        return 'event_%s' % ('Deletion' if self.deletion else 'Unlink')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class History(PersistentList):
     '''Object history is implemented as a list, sorted in antichronological
        order, of history events.'''
 
+    traverse = {}
+
     # This PX displays history events
     events = Px('''
      <div var="history=o.history;
@@ -359,19 +411,32 @@ class History(PersistentList):
         <th align=":dleft">:_('action_comment')</th>
        </tr>
        <tr for="event in batch.objects"
-           class=":loop.event.odd and 'even' or 'odd'" valign="top">
+           class=":'even' if loop.event.odd else 'odd'" valign="top">
         <td><x>:_(event.getLabel(o))</x>
          <img if="isManager and event.deletable" class="clickable iconS"
-              src=":url('deleteS.svg')"
-              onclick=":'onHistoryEvent(%s,%s,%s)' % \
-                        (q('Delete'), q(o.iid), q(event.date))"/></td>
-        <td var="creator=o.search1('User', login=event.login)">
-         <x>:creator.getTitle() if creator else (event.login or '?')</x></td>
-        <td>:tool.formatDate(event.date)</td>
+              src=":svg('deleteS')"
+              onclick=":'onHistoryEvent(%s,%s)' % 
+                        (q('delete'), q(event.getId()))"/></td>
+        <td>:event.getUser(o)</td>
+        <td>:tool.Date.format(tool, event.date, withHour=True)</td>
         <td>:event.pxDetail</td>
        </tr>
       </table>
-     </div>''')
+     </div>''',
+
+     js='''
+      function onHistoryEvent(action, eventId, commentId) {
+        var showComment = action == 'modify',
+            comment = null,
+            params="'action':'history*"+action+"','eventId':'"+eventId+"'";
+        // Manage comment
+        if (commentId) {
+          comment = html2text(document.getElementById(commentId).innerHTML);
+          params = params + ",'comment':encodeURIComponent(comment)"
+        }
+        askConfirm('script', "askAjax('history',null,{" + params + "})",
+                   action_confirm, showComment, null, null, comment);
+      }''')
 
     view = Px('''
      <div if="not o.isTemp()"
@@ -428,7 +493,7 @@ class History(PersistentList):
         self.modified = None
 
     def add(self, type, state=None, **params):
-        ''''Adds a new event of p_type (=the name one an Event sub-class into
+        ''''Adds a new event of p_type (=the name of an Event sub-class) into
             the history.'''
         # Get the login of the user performing the action
         if 'login' in params:
@@ -509,7 +574,7 @@ class History(PersistentList):
             newer = getattr(self.o, name, None)
         return newer
 
-    def historize(self, previous):
+    def historize(self, previous, comment=None):
         '''Records, in self.o's history, potential changes on historized fields.
            p_previous contains the values, before an update, of the historized
            fields, while p_self.o already contains the (potentially) modified
@@ -520,15 +585,15 @@ class History(PersistentList):
             prev = previous[name]
             field = o.getField(name)
             curr = field.getValue(o, single=False)
-            if (prev == curr) or ((prev is None) and (curr == '')) or \
-               ((prev == '') and (curr is None)):
+            if prev == curr or (prev is None and curr == '') or \
+               (prev == '' and curr is None):
                 del(previous[name])
                 continue
             # The previous value may need to be formatted
             field.updateHistoryValue(o, previous)
         # Add the entry in the history (if not empty)
         if previous:
-            self.add('Change', changes=previous)
+            self.add('Change', changes=previous, comment=comment)
 
     def getEvents(self, type, notBefore=None):
         '''Gets a subset of history events of some p_type. If specified, p_type
@@ -538,6 +603,46 @@ class History(PersistentList):
         return [event for event in self.iter(eventType=type, condition=cond, \
                                              context={'notBefore':notBefore})]
 
+    def getEvent(self, id):
+        '''Get the event with this p_id or None if not found'''
+        for event in self:
+            if event.getId() == id:
+                return event
+
+    def getLast(self, name, notBefore=None, history=None, eventType='Trigger'):
+        '''Return the last event that occurred in p_self (or in p_history if
+           passed), having this p_name.'''
+
+        # See event methods m_getTypeName(short=True) to know what "name" refers
+        # to. For a Trigger event, it is the name of the fired transition.
+        #
+        # p_name can be a list: in that case, it returns the most recent
+        # occurrence of the corresponding events, being anti-chronologically
+        # walked. If p_notBefore is given (an event name, too), it corresponds
+        # to a kind of start event for the search: we will not search in the
+        # history preceding the last occurrence of this event. Note that
+        # p_notBefore can also hold a list.
+        #
+        # If p_eventType is passed, only events of this type will be taken into
+        # account. If it is none, all history events, of any type, are walked.
+        #
+        # If p_history is passed, it is used instead of p_self.o.history.
+        #
+        # Walk events, in anti-chronological order
+        for event in (history or self).iter(eventType=eventType):
+            ename = event.getTypeName(short=True)
+            if notBefore:
+                if isinstance(notBefore, str):
+                    condition = ename == notBefore
+                else:
+                    condition = ename in notBefore
+                if condition: return
+            if isinstance(name, str):
+                condition = ename == name
+            else:
+                condition = ename in name
+            if condition: return event
+
     def setEvents(self, other):
         '''Replace p_self's events with clones from p_other's events'''
         # Empty p_self's history first
@@ -573,4 +678,74 @@ class History(PersistentList):
                 event.login = new
                 r += 1
         return r
+
+    # The list of evvent types being, by default, excluded from the XHTML
+    # version of the history, as produced by m_asXhtml below.
+    xhtmlDefaultExclude = ('change', '_init_')
+
+    def asXhtml(self, exclude=xhtmlDefaultExclude, userExpression=None,
+                noLabelFor=('comment',), escapeComments=True):
+        '''Produce a XHTML version of this history, in the form of a discussion
+           thread, suitable to be injected as various places: in methods
+           getSubTitle, in PODs, etc.'''
+
+        # p_exclude lists the event types that will not be part of the result.
+
+        # If p_userExpression is None, user will be shown via standard method
+        # User::getTitle. Else, user will be shown according to this expression,
+        # that will be evaluated with name "user" in its context.
+
+        # p_noLabelFor gives a tuple of event types for which no label will be
+        # rendered. Typically, for a transition named "comment", it is useless
+        # to show its translated name.
+
+        # Browse history events
+        r = []
+        o = self.o
+        esc = Escape.xhtml
+        tool = o.tool
+        fmt = tool.Date.format
+        for event in self:
+            # Ignore inappropriate events
+            type = event.getTypeName(short=True)
+            if exclude and type in exclude: continue
+            # Define a prefix, being the name of the event
+            if noLabelFor and type in noLabelFor:
+                prefix = ''
+            else:
+                prefix = ' – <b>%s</b> ' % esc(o.translate(event.getLabel(o)))
+            # Get the comment when present
+            comment = event.comment or ''
+            if comment and escapeComments: comment = esc(comment)
+            # Create the message line
+            msg = '<i>%s</i>%s – %s' % \
+                  (fmt(tool, event.date, withHour=True), prefix,
+                   esc(event.getUser(o, expression=userExpression)))
+            if comment:
+                msg += ' – « %s »' % comment
+            r.append(msg)
+        return '<div class="sahist">%s</div>' % '<br/>'.join(r)
+
+    traverse['delete'] = 'Manager'
+    def delete(self, o):
+        '''Delete a history event whose ID is in the request'''
+        # Find the event given its ID being in the request
+        event = self.getEvent(o.req.eventId)
+        if not event: return o.translate('missing_event')
+        self.remove(event)
+        return o.translate('object_deleted')
+
+    traverse['modify'] = 'perm:read'
+    def modify(self, o):
+        '''Update a history event whose ID is in the request'''
+        event = self.getEvent(o.req.eventId)
+        if not event: return o.translate('missing_event')
+        # Is the user allowed to modify this event ?
+        user = o.user
+        allowed = o.class_.mayEditEventComment(o, event, user,
+                                               user.hasRole('Manager'))
+        
+        if not allowed: return o.translate('unauthorized')
+        event.comment = StringCleaner.clean(o.req.comment)
+        return o.translate('object_saved')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
