@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -21,9 +21,9 @@ from appy.utils import sequenceTypes
 from appy.model.workflow import Role
 
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-PERMISSION_NOT_FOUND = 'permission "%s" is not among permissions dict for ' \
-  'state %s in workflow %s.'
-                            
+PERM_N_F = 'permission "%s" is not among permissions dict for state %s in ' \
+           'workflow %s.'
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class State:
     '''Represents a workflow state'''
@@ -82,8 +82,12 @@ class State:
                 role = self.getRole(roles)
                 self.permissions[permission] = {role.name: role}
             elif isinstance(roles, dict):
-                for name, role in roles.iteritems():
+                for name, role in roles.items():
                     roles[name] = self.getRole(role)
+            elif callable(roles):
+                # v_roles is a method: it is not possible to standardize it. It
+                # will be called at check time.
+                pass
             else:
                 # "roles" is a list or tuple, or None (nobody may have this
                 # permission).
@@ -96,13 +100,17 @@ class State:
 
     def getUsedRoles(self): return self.usedRoles.values()
 
-    def getRolesFor(self, permission):
+    def getRolesFor(self, o, permission):
         '''Gets the roles that are granted p_permission on this state. r_ is a
            dict ~{s_roleName: Role}.'''
         if permission not in self.permissions:
-            raise Exception(PERMISSION_NOT_FOUND % (permission, self.name, \
-                                                    self.workflow.name))
-        return self.permissions[permission]
+            raise Exception(PERM_N_F % (permission, self.name,
+                                        self.workflow.name))
+        r = self.permissions[permission]
+        # If v_r is a method, it *must* return Role instances
+        if callable(r):
+            r = r(self.workflow, o, permission)
+        return r
 
     def addRoles(self, roles, permissions=()):
         '''Adds p_roles in self.permissions. p_roles can be a role name, a Role
@@ -114,6 +122,8 @@ class State:
         if isinstance(permissions, str): permissions = (permissions,)
         for perm, existingRoles in self.permissions.items():
             if permissions and (perm not in permissions): continue
+            # It is not possible to add p_roles if v_existingRoles is a method
+            if callable(existingRoles): continue
             for role in roles:
                 # Do nothing if "role" is already among existing roles
                 name = role if isinstance(role, str) else role.name
@@ -181,8 +191,8 @@ class State:
             else:
                 for start, end in tr.states:
                     # Bypass (start, end) pairs having nothing to do with self
-                    if (start != self) and (end != self): continue
-                    if (start != self) or (end != self): return
+                    if start != self and end != self: continue
+                    if start != self or  end != self: return
         # If we are here, either there was no transition starting from self,
         # either all transitions were auto-transitions: self is then isolated.
         return True
