@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -33,7 +33,7 @@ class Transition:
 
     def __init__(self, states, condition=True, action=None, show=True,
                  confirm=False, group=None, icon=None, sicon=None,
-                 redirect=None, historizeActionMessage=False):
+                 redirect=None, historizeActionMessage=False, iconOnly=False):
         # In its simpler form, "states" is a list of 2 states:
         # (fromState, toState). But it can also be a list of several
         # (fromState, toState) sub-lists. This way, you may define only 1
@@ -49,14 +49,16 @@ class Transition:
         # the transition. It will only be possible by code.
         self.confirm = confirm # If True, a confirm popup will show up.
         self.group = Group.get(group)
-        # If you want to specify an icon being in folder <yourApp>/static,
-        # specify "<yourApp>/<yourImage>". Else, default icon "appy/action.svg"
-        # will be used.
-        self.icon, self.iconBase = iconParts(icon or 'action.svg')
+        # If you want to specify a standard Appy SVG icon or one of your SVG
+        # icons being in folder <yourApp>/static, specify the name of the image,
+        # ie, "help.svg". If you want to specify one of your non-SVG icons,
+        # specify "<yourApp>/<yourImage>". Else, default Appy icon "action.svg"
+        # (from appy/ui/static) will be used.
+        self.icon, self.iconBase, self.iconRam = iconParts(icon or 'action.svg')
         # You may specify, in attribute "sicon", an alternate icon suitable when
         # rendered as a small icon.
         sicon = sicon or icon or 'action.svg'
-        self.sicon, self.siconBase = iconParts(sicon)
+        self.sicon, self.siconBase, self.siconRam = iconParts(sicon)
         # If redirect is None, once the transition will be triggered, Appy will
         # perform an automatic redirect:
         # (a) if you were on some "view" page, Appy will redirect you to this
@@ -66,17 +68,20 @@ class Transition:
         # Case (b) can be problematic if the transition modifies the list of
         # objects, or if it modifies other elements shown outside this list.
         # If you specify "redirect" being
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "page"     | case (a) will always apply;
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "referer"  | the entire page will be refreshed with the referer page.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.redirect = redirect
         # When a transition is triggered, the corresponding event is added in
         # the object's history. If p_historizeActionMessage is True, the message
         # returned by self.action (if any) will be appended to this event's
         # comment.
         self.historizeActionMessage = historizeActionMessage
+        # If p_iconOnly is True, the transition will be rendered as an icon (and
+        # not a button), even on the "buttons" layout.
+        self.iconOnly = iconOnly
 
     def init(self, workflow, name):
         '''Lazy initialisation'''
@@ -307,12 +312,12 @@ class Transition:
             raise Transition.Error(UNTRIGGERABLE % (name, o.url))
         # Identify the target state for this transition
         target = forceTarget or self.getTargetState(o)
-        # Add the event in the object history
-        event = o.history.add('Trigger', target.name, transition=name,
-                              comment=comment)
         # Remember the source state, it will be necessary for executing the
         # common action.
         fromState = o.state if not isInit else None
+        # Add the event in the object history
+        event = o.history.add('Trigger', target.name, transition=name,
+                              comment=comment)
         # Execute the action that is common to all transitions, if defined. It
         # is named "onTrigger" on the workflow class by convention. This common
         # action is executed before the transition-specific action (if any).
@@ -363,66 +368,72 @@ class Transition:
             back = tool.computeHomePage()
         o.goto(back)
 
-    @staticmethod
-    def getBack(workflow, transition):
-        '''Returns the name of the transition (in p_workflow) that "cancels" the
-           triggering of p_transition and allows to go back to p_transition's
-           start state.'''
-        # Get the end state(s) of p_transition
-        transition = getattr(workflow, transition)
-        # Browse all transitions and find the one starting at p_transition's end
-        # state and coming back to p_transition's start state.
-        for trName, tr in workflow.__dict__.items():
-            if not isinstance(tr, Transition) or (tr == transition): continue
-            if transition.isSingle():
-                if tr.hasState(transition.states[1], True) and \
-                   tr.hasState(transition.states[0], False): return trName
+    def getBack(self):
+        '''Returns, in p_self's workflow, the name of the transition that
+           "cancels" the triggering of this one and allows to go back to
+           p_self's start state.'''
+        single = self.isSingle()
+        # Browse all transitions and find the one starting at p_self's end
+        # state and coming back to p_self's start state.
+        for transition in self.workflow.transitions.values():
+            if transition == self: continue
+            if single:
+                if transition.hasState(self.states[1], True) and \
+                   transition.hasState(self.states[0], False):
+                       return transition.name
             else:
-                startOk = False
-                endOk = False
-                for start, end in transition.states:
-                    if (not startOk) and tr.hasState(end, True):
+                startOk = endOk = False
+                for start, end in self.states:
+                    if not startOk and transition.hasState(end, True):
                         startOk = True
-                    if (not endOk) and tr.hasState(start, False):
+                    if not endOk and transition.hasState(start, False):
                         endOk = True
-                    if startOk and endOk: return trName
+                    if startOk and endOk: return transition.name
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class UiTransition:
     '''Represents a widget that displays a transition'''
+
     px = Px('''
      <x var="label=transition.title;
              mayTrigger=transition.mayTrigger;
-             inButtons=layout == 'buttons'">
+             inSub=layout=='sub';
+             asIcon=not inSub or transition.iconOnly">
 
-      <!-- Picto -->
-      <a if="not inButtons" class=":'clickable' if mayTrigger else 'help fake'"
-         var="back=transition.getBackHook(o, inButtons, q, backHook)"
+      <!-- Picto or icon -->
+      <a if="asIcon" class=":'clickable' if mayTrigger else 'help fake'"
+         var="back=transition.getBackHook(_ctx_);
+              iconAttr='sicon' if inSub else 'icon';
+              iconBase='siconBase' if inSub else 'iconBase';
+              iconRam='siconRam' if inSub else 'iconRam'"
          name=":transition.name"
-         title=":'' if mayTrigger else transition.reason"
-         onclick=":'triggerTransition(%s,this.name,%s,%s)' % \
-                    (q(formId), q(transition.confirm), back) \
+         title=":transition.getIconTitle()"
+         onclick=":'triggerTransition(%s,this.name,%s,%s)' % 
+                    (q(formId), q(transition.confirm), back) 
                     if mayTrigger else ''">
-       <img src=":url(transition.icon, base=transition.iconBase)"
-            class=":picto"/>
-       <div style=":'display:%s' % config.ui.pageDisplay">::label</div>
+       <img src=":url(getattr(transition, iconAttr),
+                      base=getattr(transition, iconBase),
+                      ram=getattr(transition,iconRam))"
+            class=":picto|'iconS'"/>
+       <div style=":'display:%s' % config.ui.pageDisplay"
+            if="not inSub">::label</div>
       </a>
 
       <!-- Real button -->
-      <x if="inButtons" var2="css=ui.Button.getCss(label, inButtons)">
+      <x if="not asIcon" var2="css=ui.Button.getCss(label, inSub)">
        <input if="mayTrigger" type="button" class=":css" value=":label"
-              var="back=transition.getBackHook(o, inButtons, q, backHook)"
+              var="back=transition.getBackHook(_ctx_)"
               name=":transition.name"
-              style=":url(transition.sicon, base=transition.siconBase, \
-                          bg='18px 18px')"
+              style=":url(transition.sicon, base=transition.siconBase,
+                          ram=transition.siconRam, bg='18px 18px')"
               onclick=":'triggerTransition(%s,this.name,%s,%s)' % \
                          (q(formId), q(transition.confirm), back)"/>
 
        <!-- Fake button, explaining why the transition can't be triggered -->
        <input if="not mayTrigger" type="button"
               class=":'%s fake' % css"
-              style=":url(transition.icon, base=transition.siconBase, \
-                      bg='18px 18px')"
+              style=":url(transition.sicon, base=transition.siconBase,
+                          ram=transition.siconRam, bg='18px 18px')"
               value=":label" title=":transition.reason"/></x></x>''',
 
      js='''
@@ -434,7 +445,9 @@ class UiTransition:
       }''')
 
     # Fields to copy from Transition to UiTransition
-    copied = ('name', 'icon', 'iconBase', 'sicon', 'siconBase')
+    copied = ('name' , 'iconOnly' ,
+              'icon' , 'iconBase' , 'iconRam',
+              'sicon', 'siconBase', 'siconRam')
 
     def __init__(self, transition, o, mayTrigger):
         _ = o.translate
@@ -461,14 +474,25 @@ class UiTransition:
         # Required by the UiGroup
         self.colspan = 1
 
-    def getBackHook(self, o, inButtons, q, backHook=None):
+    def getBackHook(self, ctx):
         '''If, when the transition has been triggered, we must ajax-refresh some
            part of the page, this method will return the ID of the corresponding
            DOM node. Else (ie, the entire page needs to be refreshed), it
            returns None.'''
-        if inButtons and not self.transition.redirect:
-            r = q(backHook or o.iid)
+        if ctx.inSub and not self.transition.redirect:
+            r = ctx.q(ctx.backHook or ctx.ohook)
         else:
             r = 'null'
         return r
+
+    def getIconTitle(self):
+        '''Compute the title of the icon representing the transition, when this
+           latter is represented by an icon only.'''
+        if not self.mayTrigger:
+            r = self.reason
+        elif self.iconOnly:
+            r = self.title
+        else:
+            r = ''
+        return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
