@@ -1,7 +1,7 @@
 '''Import data from a Appy 0.x site to an Appy 1.x site'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -22,12 +22,26 @@ from appy.peer import Peer
 from appy.peer.importer import Importer, UserImporter
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-NOT_IMPL_YET = 'Not implemented yet.'
+LROLES_KO  = 'Instance of %s: no local roles found. Distant is: %s.'
+E_LC_KO    = 'No local roles found on distant object.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Importer0(Importer):
     '''Base object importer for Appy 0'''
 
+    # If, for some concrete importer, you want, for some transitions, use
+    # alternate names on the target site (Appy 1 is more strict and does not
+    # allow as much names as Appy 0), override static attribute "transitionMap"
+    # and specify a dict of the form ~{s_sourceName: s_targetName}~.
+    transitionsMap = None
+
+    def getTransitionName(self, name):
+        '''Gets, on the targte site, the name of the transition that must be
+           used for representing the distant transition whose name is p_name.'''
+        map = self.transitionsMap
+        if map is None: return name
+        return map.get(name) or name
+
     def setHistory(self):
         '''Imports Appy 0's history from p_self.distant and populate the
            corresponding Appy 1 data structure on p_self.local.'''
@@ -36,8 +50,13 @@ class Importer0(Importer):
         i = -1
         for event in self.distant.history:
             i += 1
-            # Creator, created & modified will be set by m_setFields
-            if i == 0: continue
+            if i == 0:
+                # Creator, created & modified will be set by m_setFields.
+                # Nevertheless, a potential comment can be retrieved from the
+                # initial distant transition.
+                if event.comments:
+                    history[-1].comment = event.comments
+                continue
             # Get parameters being common to all history events
             params = {'state': event.review_state, 'date': event.time,
                       'login': event.actor, 'comment': event.comments}
@@ -49,25 +68,38 @@ class Importer0(Importer):
                 for name, vals in event.changes.items():
                     changes[name] = vals[0]
                 params['changes'] = changes
-            elif action == '_datadelete_':
-                print('Data-delete event', event)
-                raise Exception(NOT_IMPL_YET)
             elif action == '_dataadd_':
-                print('Data-add event', event)
-                raise Exception(NOT_IMPL_YET)
+                eventType = 'Link'
+                # No idea if addition must be True or False: this subtelty was
+                # not present in Appy 0.
+                params['addition'] = False
+                params['field'] = 'unknown' # Idem
+            elif action == '_datadelete_':
+                eventType = 'Unlink'
+                # No idea if addition must be True or False: this subtelty was
+                # not present in Appy 0.
+                params['deletion'] = False
+                params['field'] = 'unknown' # Idem
             else:
                 # A workflow transition
-                params['transition'] = action
                 eventType = 'Trigger'
+                params['transition'] = self.getTransitionName(action)
             history.add(eventType, **params)
 
     def setLocalRoles(self):
         '''Sets local roles on p_self.local as defined on p_self.distant'''
+        o = self.local
+        localRoles = o.localRoles
         # Delete local role granted to technical user "system" when the local
         # object has been created.
-        localRoles = self.local.localRoles
         localRoles.delete('system')
-        for login, roles in self.distant.localRoles.items():
+        # Get the distant local roles
+        droles = self.distant.localRoles
+        if droles is None:
+            o.log(LROLES_KO % (o.class_.name, str(self.distant)), type='error')
+            raise Exception(E_LC_KO)
+        # Set the local roles on the local object
+        for login, roles in droles.items():
             localRoles.add(login, roles)
 
     def customize(self):
