@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -150,6 +150,13 @@ class Action:
                 # Trigger action-specific behaviour
                 self.do(result, context, eRes)
 
+    def getBufferSlice(self, context):
+        '''If only a slice of the tied buffer must be evaluated, this method
+           returns a tuple (startIndex, endIndex). In that case, only
+           p_self.buffer.content[startIndex, endIndex] will be dumped in the POD
+           result.'''
+        return
+
     def evaluateBuffer(self, result, context,
                        forceSource=None, ignoreMinus=False):
         '''Evaluates the buffer tied to this action and add the result in
@@ -160,7 +167,10 @@ class Action:
         # Determine "minus"
         minus = False if ignoreMinus else self.minus
         if source == 'buffer':
-            self.buffer.evaluate(result, context, removeMainElems=minus)
+            self.buffer.evaluate(result, context, removeMainElems=minus,
+                                 slice=self.getBufferSlice(context))
+            # m_getBufferSlice may determine if only a slice [start,end] of the
+            # buffer content must be dumped in the POD result.
         else:
             # Evaluate self.fromExpr in fromRes
             fromRes = None
@@ -194,6 +204,16 @@ class Action:
         else:
             self.subAction.addSubAction(action)
 
+    def getAction(self, type):
+        '''If p_self or one of its sub-actions (recursively) is of this p_type,
+           returns it.'''
+        # Return p_self itself, if it has this p_type
+        if self.__class__.__name__ == type: return self
+        # Check subAction
+        sub = self.subAction
+        if not sub: return
+        return sub.getAction(type)
+
     def check(self):
         '''Returns a tuple (success, message) indicating if the action is well
            formed or not.'''
@@ -367,7 +387,7 @@ class For(Action):
             loop.odd = not loop.even
             self.updateContext(context, item)
             # Cell: add a new row if we are at the end of a row
-            if isCell and (currentColIndex == nbOfColumns):
+            if isCell and currentColIndex == nbOfColumns:
                 result.dumpEndElement(Row.OD.nsName)
                 result.dumpStartElement(Row.OD.nsName, rowAttributes)
                 currentColIndex = 0
@@ -426,6 +446,31 @@ class For(Action):
                     # On error, name may not be in the context
                     del context[name]
 
+    def getBufferSlice(self, context):
+        '''When processing a "do doc" statement, depending on the current
+           iteration, only a specific slice of the buffer must be dumped.'''
+        if not isinstance(self.elem, Doc): return
+        # Get the current loop object
+        loop = getattr(context['loop'], self.iters[0])
+        # If there is a single iteration, do no alter anything
+        if loop.length == 1: return
+        # Dump only a part of the complete buffer, depending on the current
+        # iteration.
+        end = self.buffer.content.rfind(self.elem.END_TAG)
+        if loop.first:
+            # Dump the start of the buffer, but not the very last end tag
+            return 0, end
+        else:
+            # Remove the start of the doc: it must only be dumped once
+            sd = self.elem.SD_END_TAG
+            start = self.buffer.content.find(sd) + len(sd)
+            if loop.last:
+                # Dump the end tag
+                return start, len(self.buffer.content)
+            else:
+                # Still do not dump the end tag
+                return start, end
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Null(Action):
     '''Action that does nothing. Used in conjunction with a "from" clause, it
@@ -491,9 +536,10 @@ class MetaIf(Action):
 class Variables(Action):
     '''Action that allows to define a set of variables somewhere in the
        template.'''
-    def __init__(self, name, buff, elem, minus, variables):
-        # We do not use the default Buffer.expr attribute for storing the Python
-        # expression, because here we will have several expressions, one for
+
+    def __init__(self, name, buff, elem, minus, variables, lasting=False):
+        # The default Buffer.expr attribute is not used for storing the Python
+        # expression, because several expressions can exist here, one for
         # every defined variable.
         Action.__init__(self, name, buff, None, elem, minus)
         # Definitions of variables: ~[(s_name|[s_name], s_expr)]~. It allows a
@@ -503,6 +549,21 @@ class Variables(Action):
         # be defined, whose name is the key in the dict, and whose value is the
         # corresponding value at that key.
         self.variables = variables
+        # If p_lasting is:
+        # ----------------------------------------------------------------------
+        # False | (the default) every variable's scope will be the target
+        #       | element (as determined by the corresponding buffer), not more.
+        #       | Any homonym variable being defined in the outer scope will be
+        #       | hidden in the context of the target element, and will be
+        #       | visible again as soon as the walk leaves this sub-scope.
+        # ----------------------------------------------------------------------
+        # True  | From the moment such a "lasting" variable is defined, its
+        #       | scope will be the remaining of the document. So, its scope
+        #       | encompasses the target element + the remaining of the
+        #       | document. Any homonym variable being previously defined will
+        #       | be hidden forever, as soon as the lasting variable is defined.
+        # ----------------------------------------------------------------------
+        self.lasting = lasting
 
     def storeVariable(self, name, value, context, hidden):
         '''Adds a variable named p_name with this p_value in the p_context.
@@ -514,8 +575,9 @@ class Variables(Action):
         else:
             # Store variable p_name with p_value in the p_context. If this
             # variable already exists in the context, remember its previous
-            # value in p_hidden.
-            if name in context:
+            # value in p_hidden, excepted if the currently defined variables are
+            # lasting.
+            if not self.lasting and name in context:
                 if not hidden:
                     hidden = {name: context[name]}
                 else:
@@ -529,17 +591,22 @@ class Variables(Action):
         # Do not remove it if it is a global variable
         if name.startswith('@'): return
         # Do not remove it if it is an overridden variable
-        if hidden and (name in hidden): return
+        if hidden and name in hidden: return
         del context[name]
 
     def do(self, result, context, exprRes):
         '''Evaluate the variables' expressions'''
-        # Because there are several expressions, we do not use the standard,
-        # single-expression-minded Action code for evaluating expressions.
 
-        # v_hidden stores names and values of the variables that we will hide in
-        # the context. After execution of this buffer, values will be restored.
-        # v_multi stores (in reverse order) encountered multivariable values.
+        # Because there are several expressions, the standard,
+        # single-expression-minded Action code is not used for evaluating
+        # expressions.
+
+        # If the currently defined variables are not lasting, the names and
+        # values of the variables that will be hidden in the context will be
+        # stored in v_hidden. After execution of this buffer, their values
+        # will be restored. v_multi stores (in reverse order) encountered
+        # multivariable values.
+
         hidden = multi = None
         for names, expr in self.variables:
             # Evaluate variable expression in v_expr
@@ -573,16 +640,17 @@ class Variables(Action):
             self.evaluateBuffer(result, context)
         # Restore hidden variables if any
         if hidden: context.update(hidden)
-        # Delete not-hidden variables
-        for names, expr in self.variables:
-            if names == '*':
-                values = multi.pop()
-                if values:
-                    for name in values.keys():
+        # Delete not-hidden variables, if not lasting
+        if not self.lasting:
+            for names, expr in self.variables:
+                if names == '*':
+                    values = multi.pop()
+                    if values:
+                        for name in values.keys():
+                            self.removeVariable(name, context, hidden)
+                elif isinstance(names, str):
+                    self.removeVariable(names, context, hidden)
+                else:
+                    for name in names:
                         self.removeVariable(name, context, hidden)
-            elif isinstance(names, str):
-                self.removeVariable(names, context, hidden)
-            else:
-                for name in names:
-                    self.removeVariable(name, context, hidden)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
