@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -20,19 +20,19 @@ import os, os.path, re, stat, shutil, struct, urllib.parse, imghdr, base64
 
 import appy.pod
 from appy import utils
+from appy.utils import css
 from appy.pod import PodError, getUuid
 from appy.utils.client import Resource
 from appy.model.utils import Object as O
-from appy.ui.css import CssStyles, px2cm
 from appy.pod.odf_parser import OdfEnvironment
 from appy.utils.path import getOsTempFolder, getTempFileName
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-FILE_NOT_FOUND   = "'%s' does not exist or is not a file."
-PDF_TO_IMG_ERROR = 'A PDF file could not be converted into images. Please ' \
-                   'ensure that Ghostscript (gs) is installed on your ' \
-                   'system and the "gs" program is in the path.'
-TO_PDF_ERROR     = 'ConvertImporter error while converting a doc to PDF: %s.'
+FILE_KO  = "'%s' does not exist or is not a file."
+GS_KO    = 'A PDF file could not be converted into images. Please ensure ' \
+           'Ghostscript (gs) is installed on your system and the "gs" ' \
+           'program is in the path.'
+PDF_ERR  = 'ConvertImporter error while converting a doc to PDF: %s.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class DocImporter:
@@ -81,7 +81,7 @@ class DocImporter:
         if at.startswith('./'): at = os.path.join(os.getcwd(), at[2:])
         # Checks that p_at corresponds to an existing file if given
         if raiseOnError and not os.path.isfile(at):
-            raise PodError(FILE_NOT_FOUND % at)
+            raise PodError(FILE_KO % at)
         return at
 
     def getImportFolder(self):
@@ -285,7 +285,7 @@ class PdfImporter(DocImporter):
             if fileName == firstImage:
                 succeeded = True
                 break
-        if not succeeded: raise PodError(PDF_TO_IMG_ERROR)
+        if not succeeded: raise PodError(GS_KO)
         # Insert images into the result
         noMoreImages = False
         i = 0
@@ -333,7 +333,7 @@ class ConvertImporter(DocImporter):
         pdfFile = getTempFileName(extension='pdf')
         output = self.renderer.callLibreOffice(self.importPath,
                                                outputName=pdfFile)
-        if output: raise PodError(TO_PDF_ERROR % output)
+        if output: raise PodError(PDF_ERR % output)
         # Launch a PdfImporter to import this PDF into the POD result
         pdfImporter = PdfImporter(None, pdfFile, 'pdf', self.renderer)
         return pdfImporter.run()
@@ -388,7 +388,7 @@ class Image:
                 x, y, u = struct.unpack("<HHB", buf)
         f.close()
         if (x and y) and (unit == 'cm'):
-            return float(x)/px2cm, float(y)/px2cm
+            return float(x)/css.px2cm, float(y)/css.px2cm
         else:
             # Return the size, in pixels, or as a tuple (None, None) if
             # uncomputable.
@@ -511,14 +511,14 @@ class ImageImporter(DocImporter):
         '''Read, with ImageMagick, EXIF metadata in the image, and apply a
            rotation (still with ImageMagick) if required.'''
         # Indeed, LO will not interpret EXIF metadata when reading the image
-        cmd = ['identify', '-format', '%[EXIF:*]', self.importPath]
-        out, err = utils.executeCommand(cmd)
-        if err: raise Exception(CONVERT_ERROR)
+        path = self.importPath
+        cmd = ['identify', '-format', '%[EXIF:*]', path]
+        out = utils.ImageMagick.run(cmd)
         if out:
             match = self.orientationRex.search(out)
             if match:
                 #code = int(match.group(1)) # "auto-orient" manages all codes
-                utils.convert(self.importPath, '-auto-orient')
+                utils.ImageMagick.convert(path, '-auto-orient')
 
     def init(self, anchor, wrapInPara, size, sizeUnit, maxWidth, cssAttrs,
              keepRatio, convertOptions):
@@ -556,7 +556,7 @@ class ImageImporter(DocImporter):
                 image = Image(self.importPath, self.format)
                 options = self.convertOptions(image)
             if options:
-                utils.convert(self.importPath, options)
+                utils.ImageMagick.convert(self.importPath, options)
                 transformed = True
         # Avoid creating an Image instance twice if no transformation occurred
         if image and not transformed:
@@ -587,8 +587,8 @@ class ImageImporter(DocImporter):
                 # it to cm when relevant.
                 w, h = self.size
                 if self.sizeUnit == 'px':
-                    w = float(w) / px2cm
-                    h = float(h) / px2cm
+                    w = float(w) / css.px2cm
+                    h = float(h) / css.px2cm
                 # Use (w, h) as is if we don't care about keeping image ratio or
                 # if we couldn't determine image's width or height.
                 if (not width or not height) or not keepRatio:
