@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -19,20 +19,24 @@
 from xml.sax.saxutils import quoteattr
 
 from appy.xml import Tag
+from appy.utils import asDict
 from appy.pod import PodError, Evaluator
 from appy.pod.odf_parser import OdfEnvironment as ns
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class PodElement:
-    OD_TO_POD = {'p': 'Text', 'h': 'Title', 'list-item': 'Item',
-                 'section': 'Section', 'table': 'Table', 'table-row': 'Row',
-                 'table-cell': 'Cell', None: 'Expression', 'frame': 'Frame'}
-    POD_ELEMS = ('text', 'title', 'item', 'section', 'table', 'row', 'cell',
-                 'frame')
+    OD_TO_POD   = {'p': 'Text', 'h': 'Title', 'list-item': 'Item',
+                   'section': 'Section', 'table': 'Table', 'table-row': 'Row',
+                   'table-cell': 'Cell', None: 'Expression', 'frame': 'Frame',
+                   'text': 'Doc'}
+    POD_ELEMS   = ('text', 'title', 'item', 'section', 'table', 'row', 'cell',
+                   'frame', 'doc')
+    POD_ELEMS_D = asDict(POD_ELEMS)
     subTags = []
 
     # Elements for which the '-' operator can be applied
     MINUS_ELEMS = ('section', 'table')
+    MINUS_ELEMS_D = asDict(('section', 'table'))
 
     @staticmethod
     def create(elem):
@@ -90,6 +94,15 @@ class Table(PodElement):
 class Frame(PodElement):
     OD = Tag('frame', nsUri=ns.NS_DRAW)
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+class Doc(PodElement):
+    '''Represents the base tag for a complete ODT document'''
+    OD = Tag('text', nsUri=ns.NS_OFFICE)
+    # End tag for the main tag representing the document
+    END_TAG = '</office:text>'
+    # End tag for the sub-tag containing sequence declarations
+    SD_END_TAG = '</text:sequence-decls>'
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Expression(PodElement):
     '''Represents a Python expression that is found in a pod or px.'''
