@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -25,7 +25,7 @@ from appy.pod.odf_parser import OdfEnvironment, OdfParser
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class OdTable:
-    '''Informations about the currently parsed Open Document (Od)table.'''
+    '''Informations about the currently parsed Open Document (Od)table'''
     def __init__(self):
         self.nbOfColumns = 0
         self.nbOfRows = 0
@@ -33,7 +33,7 @@ class OdTable:
         self.curRowAttrs = None
 
     def isOneCell(self):
-        return (self.nbOfColumns == 1) and (self.nbOfRows == 1)
+        return self.nbOfColumns == 1 and self.nbOfRows == 1
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class OdInsert:
@@ -229,9 +229,10 @@ class PodEnvironment(OdfEnvironment):
                                for holder in self.expressionsHolders]
         self.exprEndElems = [self.exprEndTags[holder] \
                              for holder in self.expressionsHolders]
-        self.impactableTags = (Text.OD.nsName, Title.OD.nsName, Item.OD.nsName,
-                               Table.OD.nsName, Row.OD.nsName, Cell.OD.nsName,
-                               Section.OD.nsName, Frame.OD.nsName)
+        self.impactableTags = (
+          Text.OD.nsName, Title.OD.nsName, Item.OD.nsName   , Table.OD.nsName,
+          Row.OD.nsName , Cell.OD.nsName , Section.OD.nsName, Frame.OD.nsName,
+          Doc.OD.nsName)
         self.inserts = self.transformInserts()
 
     def getExpression(self, elem):
