@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -24,6 +24,7 @@ import appy.pod
 from appy import utils
 from appy.xml import Tag
 from appy.pod import PodError
+from appy.xml.escape import Escape
 from appy.pod.lo_pool import LoPool
 from appy.utils.zip import unzip, zip
 from appy.pod.buffers import FileBuffer
@@ -658,7 +659,8 @@ class Renderer:
             else:
                 css = otherCss
             css = ' class="%s"' % css if css else ''
-            r[i] = '<p%s>%s%s%s%s</p>' % (css, pre, opening, r[i], closing)
+            r[i] = '<p%s>%s%s%s%s</p>' % (css, pre, opening,
+                                          Escape.xhtml(r[i]), closing)
             i -= 1
         return self.renderXhtml(''.join(r), stylesMapping=stylesMapping)
 
