@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -23,9 +23,8 @@ import re, os.path, random, string
 
 from appy import utils
 from appy.pod import *
-from appy.pod.doc_importers import px2cm as px2cmRatio
+from appy.utils import css
 from appy.pod.odf_parser import OdfEnvironment, OdfParser
-from appy.ui.css import parseStyleAttribute, CssStyles, CssValue, px2cm
 
 # Possible states for the styles parser
 READING          = 0 # Default state
@@ -59,6 +58,7 @@ TXT_ODT_P  = 'For XHTML element "%s", you must associate an OpenDocument ' \
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Properties:
     '''Abstract base class for table and list properties'''
+
     # HTML elements whose styles are defined by Property instances instead of
     # Style instances.
     elems = ('table', 'ol', 'ul')
@@ -67,10 +67,11 @@ class Properties:
 class TableProperties(Properties):
     '''In a styles mapping, the value @key "table" must be an instance of this
        class.'''
+
     defaultMargins = (0.3, 0.0, 0.3, 0.0)
     columnModifiersPrefixes = {'optimize': 'OCW', 'distribute': 'DC'}
 
-    def __init__(self, pageWidth=None, px2cm=px2cm, cellPx2cm=10.0,
+    def __init__(self, pageWidth=None, px2cm=css.px2cm, cellPx2cm=10.0,
               wideAbove=495, minColumnWidth=0.07, columnModifier=None,
               minCellPadding=0.0, cellContentStyle='podCellContent',
               headerContentStyle='podHeaderCellContent', margins=defaultMargins,
@@ -82,9 +83,11 @@ class TableProperties(Properties):
         # with another width, specify it here (as a float value, in cm).
         self.pageWidth = pageWidth
         # Table widths expressed as pixels will use a "pixels to cm" ratio as
-        # defined in appy.ui.css.px2cm. If this is wrong for you, specify
-        # another ratio here. The width in cm will be computed as:
-        #             (table width in pixels) / px2cm
+        # defined in css.px2cm. If this is wrong for you, specify another ratio
+        # here. The width in cm will be computed as:
+        #
+        #                 (table width in pixels) / px2cm
+        #
         self.px2cm = px2cm
         # Table cell paddings may use another px / cm ratio. Indeed,
         # cellspacing="1" is converted to 0.02cm with the standard ratio, which
@@ -134,17 +137,17 @@ class TableProperties(Properties):
         self.prevails = prevails
 
     def getWidth(self, attrs, original=False):
-        '''Return the table width as a appy.shared.css.CssValue instance.
-           p_attrs is a CssStyles instance containing parsed table attributes.
-           If p_original is False, self.wideAbove is not taken into account.'''
+        '''Return the table width as a css.Value instance. p_attrs is a
+           css.Styles instance containing parsed table attributes. If p_original
+           is False, self.wideAbove is not taken into account.'''
         # Widths being "0" are simply ignored
-        if not hasattr(attrs, 'width') or (attrs.width.value == 0):
-            return CssValue('width', '100%')
+        if not hasattr(attrs, 'width') or attrs.width.value == 0:
+            return css.Value('width', '100%')
         res = attrs.width
         if original: return res
-        if (self.wideAbove is not None) and (res.unit == 'px') and \
-           (res.value > self.wideAbove):
-            return CssValue('width', '100%')
+        if self.wideAbove is not None and res.unit == 'px' and \
+           res.value > self.wideAbove:
+            return css.Value('width', '100%')
         return res
 
     def getCellPadding(self, value):
@@ -168,7 +171,7 @@ class TableProperties(Properties):
            empty string.'''
         r = ''
         i = -1
-        for direction in CssStyles.directions:
+        for direction in css.Styles.directions:
             i += 1
             # Get the value from CSS attributes
             cssValue = getattr(attrs, 'margin%s' % direction, None)
@@ -270,6 +273,7 @@ class ListProperties(Properties):
 class BulletedProperties(ListProperties):
     '''In a styles mapping, the value @key "ul" must be an instance of this
        class.'''
+
     type = 'bullet'
     defaultFormats = ('•', '◦', '▪')
     textStyle = 'podBulletStyle'
@@ -289,6 +293,7 @@ class BulletedProperties(ListProperties):
 class NumberedProperties(ListProperties):
     '''In a styles mapping, the value @key "ol" must be an instance of this
        class.'''
+
     type = 'number'
     defaultFormats = ('1',)
     defaultSuffixes = ('.',)
@@ -597,7 +602,7 @@ class PageLayout:
             defaultMargin = '2cm'
         else:
             defaultMargin = attrs[marginAttr]
-        for margin in CssStyles.directions:
+        for margin in css.Styles.directions:
             key = e.tags['margin-%s' % margin]
             value = (key in attrs) and attrs[key] or defaultMargin
             marginAttr = 'margin%s' % margin.capitalize()
@@ -817,7 +822,7 @@ class Css2odf:
     '''Allows to get a OpenDocument attribute from a CSS attribute'''
     # Map CSS attribute names to ODF attribute names. CSS attributes names
     # have lost their inner dashes (margin-left => marginleft) because they are
-    # used as Python attributes on CssStyles instances.
+    # used as Python attributes on css.Styles instances.
     namesMap = {
       'marginleft': 'fo:margin-left', 'marginright': 'fo:margin-right',
       'margintop': 'fo:margin-top', 'marginbottom': 'fo:margin-bottom',
@@ -863,7 +868,7 @@ class Css2odf:
     # CSS properties representing combinations of CSS properties
     combined = ('background', 'border')
     # For the following attributes, an alternative px2cm ratio can be defined
-    px2cmRatios = {'lineheight': px2cm * 2 }
+    px2cmRatios = {'lineheight': css.px2cm * 2 }
     # The following attributes will not be converted to ODF if their related
     # condition evaluates to True.
     notNegInTable = 'self.inTable() and (val < 0)'
@@ -898,7 +903,7 @@ class Css2odf:
         # Standardize the attribute for use within an ODF document
         unit = value.unit or ''
         if unit and (unit != '%'):
-            ratio = self.px2cmRatios.get(name, px2cm)
+            ratio = self.px2cmRatios.get(name, css.px2cm)
             try:
                 val = value.cm(value=val, ratio=ratio)
                 unit = 'cm'
@@ -1487,7 +1492,7 @@ class StylesManager:
                     match = False
                 else:
                     cssValue = cssStyles.get(name)
-                    match = cssValue and (value == cssValue.value)
+                    match = cssValue and value == cssValue.value
             # Negate the match when appropriate
             if negate: match = not match
             # Return None if there is no match
