@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -24,13 +24,13 @@
 import re, xml.sax, math
 
 from appy.pod import *
+from appy.utils import css
 from appy.xml.escape import Escape
 from appy.utils.inject import Injector
 from appy.xml import Environment, Parser
 from appy.utils import formatNumber, addPair
 from appy.pod.odf_parser import OdfEnvironment
 from appy.utils.string import WhitespaceCruncher
-from appy.ui.css import CssStyles, CssValue, px2cm
 from appy.pod.styles_manager import Style,BulletedProperties,NumberedProperties
 
 # Tags for which there is a direct correspondance between HTML and ODF
@@ -112,10 +112,10 @@ class HtmlElement(Element):
         # Store a reference to the parent element if known
         self.parent = parent
         # Extract style information from p_elem and p_attrs
-        self.cssStyles = CssStyles(elem, attrs)
+        self.cssStyles = css.Styles(elem, attrs)
         # When merging styles, "cssStyles" may temporarily hold a set of merged
-        # styles. In this case, the original CssStyles instance is stored in the
-        # following attribute.
+        # styles. In this case, the original css.Styles instance is stored in
+        # the following attribute.
         self.originalStyles = None
         # Extract other attributes depending on p_elem
         self.attrs = self.extractAttributes(elem, attrs)
@@ -139,9 +139,9 @@ class HtmlElement(Element):
         self.odStyle = None
         # For cells, the style to apply to their inner paragraphs (set later)
         self.innerStyle = None
-        # We must know if a style must be applied on inner paragraphs within
-        # "li" tags.
-        if (elem == 'li') and ('class' in attrs):
+        # Determine if a style must be applied on inner paragraphs within "li"
+        # tags.
+        if elem == 'li' and 'class' in attrs:
             self.paraStyle = attrs['class']
         else:
             self.paraStyle = None # Will also be set on "ol" and "ul" tags.
@@ -210,7 +210,7 @@ class HtmlElement(Element):
         # Special case: check elements that can't be found within a "li".
         # Instead of being noted as "conflictual", note that we must keep
         # these element's contents but remove the surrounding tags.
-        if (parent.elem == 'li') and (self.elem in PARA_TAGS):
+        if parent.elem == 'li' and self.elem in PARA_TAGS:
             self.removeTag = True
             parent.addInnerCssStyles(self)
             return ()
@@ -221,10 +221,10 @@ class HtmlElement(Element):
         if parent.elem == 'li':
             if self.elem in NOT_INSIDE_LI:
                 return (parent.parent.setConflictual(),parent.setConflictual(),)
-        elif (parent.elemType == 'para') and (self.elem in NOT_INSIDE_P_OR_P):
+        elif parent.elemType == 'para' and self.elem in NOT_INSIDE_P_OR_P:
             return (parent.setConflictual(),)
         # Check inner paragraphs
-        elif (parent.elem in INNER_TAGS) and (self.elemType == 'para'):
+        elif parent.elem in INNER_TAGS and self.elemType == 'para':
             res = [parent.setConflictual()]
             if len(env.currentTags) > 1:
                 i = 2
@@ -240,11 +240,11 @@ class HtmlElement(Element):
                         visitParents = False
             return res
         if parent.tagsToClose and \
-            (parent.tagsToClose[-1].elemType == 'para') and \
-            (self.elem in NOT_INSIDE_P_OR_P):
+           parent.tagsToClose[-1].elemType == 'para' and \
+           self.elem in NOT_INSIDE_P_OR_P:
             return (parent.tagsToClose[-1].setConflictual(),)
         # Check elements that can't be found within a list
-        if (parent.elemType == 'list') and (self.elem in NOT_INSIDE_LIST):
+        if parent.elemType == 'list' and self.elem in NOT_INSIDE_LIST:
             return (parent.setConflictual(),)
         return ()
 
@@ -276,9 +276,25 @@ class HtmlElement(Element):
                 dump(' text:style-name="%s"' % styleName)
             else:
                 # Check if a style must be applied on 'p' tags
-                if self.innerCssStyles:
-                    p.cssStyles.merge(self.innerCssStyles)
-                innerStyle = listElem.paraStyle or env.itemStyles[itemStyle]
+                innerStyles = self.innerCssStyles
+                paraStyle = None
+                if innerStyles:
+                    p.cssStyles.merge(innerStyles)
+                    class_ = getattr(innerStyles, 'classes', None)
+                    if class_:
+                        # Take the last CSS class if several are defined
+                        paraStyle = class_.split()[-1]
+                # Some remarks here.
+                # 1) The CSS class defined on the "p" tag, when present, takes
+                #    precedence over the one defined at the list level (itself
+                #    copied from the applicable ListProperties instance. This
+                #    precedence rule is hard-coded: it is not possible to choose
+                #    which one prevails, as opposed to what happens with
+                #    TableProperties instances via boolean attribute "prevails".
+                # 2) The CSS class potentially defined at the level of the "li"
+                #    tag is currently ignored.
+                innerStyle = paraStyle or listElem.paraStyle or \
+                             env.itemStyles[itemStyle]
                 if innerStyle:
                     p.cssStyles.addClass(innerStyle)
                 dump(env.getOdfAttributes(p))
@@ -393,7 +409,7 @@ class HtmlTable(Element):
         # p_xhtmlElem is the HtmlElement representing this table
         self.parent = xhtmlElem.parent
         self.elem = 'table'
-        self.cssStyles = CssStyles(self.elem, attrs)
+        self.cssStyles = css.Styles(self.elem, attrs)
         self.originalStyles = None
         self.removeTag = False
         self.contentDumped = False
@@ -480,10 +496,10 @@ class HtmlTable(Element):
         width = tableProps.getWidth(cssStyles)
         originalWidth = tableProps.getWidth(cssStyles, original=True)
         align = getattr(cssStyles, 'textalign', 'left')
-        # Get the page width, in cm, and the ratio "px2cm"
+        # Get the page width, in cm, and the ratio css.px2cm
         pageWidth, px2cmRatio = tableProps.pageWidth, tableProps.px2cm
         pageWidth = pageWidth or self.env.pageWidth
-        px2cmRatio = px2cmRatio or px2cm
+        px2cmRatio = px2cmRatio or css.px2cm
         # Compute the table attributes for setting its width
         s = self.styleNs
         if width.unit == '%':
@@ -728,13 +744,14 @@ class XhtmlEnvironment(Environment):
         self.listProperties = {}
         # The fusion of currently encountered styles corresponding to XHTML
         # "style-only" tags (STYLE_ONLY_TAGS).
-        self.mergedInnerStyles = CssStyles()
+        self.mergedInnerStyles = css.Styles()
         # How much STYLE_ONLY_TAGS are currently walked (whose fusion is in
         # self.mergedInnerStyles) ?
         self.mergedCount = 0
         # The width (in cm, as a float) of pages (margins excluded) within the
-        # current document.
-        self.pageWidth = renderer.stylesManager.pageLayout.getWidth()
+        # current document. Within ODS templates, such width seems not be
+        # present (may have no sense for a sheet).
+        self.pageWidth = renderer.stylesManager.pageLayout.getWidth() or 17.0
         # It may be required to enable the POD post-processor (=PPP). But did we
         # already do it ?
         self.pppEnabled = False
