@@ -3,7 +3,7 @@
    Python and XML.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -21,6 +21,8 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import xml.sax
+
+from appy.utils import css
 from appy.model.utils import Object as O
 from appy.pod.buffers import MemoryBuffer
 from appy.xml import xmlPrologue, xhtmlPrologue
@@ -49,19 +51,19 @@ class Px:
                  hook=None, prologue=None, css=None, js=None, name=None):
         '''Creates a PX object'''
 
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # If p_type is... | p_content is...
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #     STRING      | a chunk of PX code as a string 
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #      FILE       | the path to a file on disk, containing PX code
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         #     METHOD      | (cheat mode) a method that will accept the PX
         #                 | context as unique arg and must return XML code, just
         #                 | as a PX would do. This allows to "implement" a PX
         #                 | with Python code, which may, in some cases, be
         #                 | appropriate for performance reasons.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
         # If the PX code represents a complete XML file, p_partial is False.
         # Else, p_content will be surrounded by a root tag to be able to parse
@@ -141,13 +143,7 @@ class Px:
     def compact(self, s):
         '''Removes single-line comments and unnecessary spaces in p_s,
            representing CSS or JS code.'''
-        if not s: return
-        r = []
-        for line in s.split('\n'):
-            line = line.strip()
-            if line and not line.startswith('//'):
-                r.append(line)
-        return '\n'.join(r)
+        return css.File.compact(s)
 
     def addCssJs(self, code, type):
         '''Adds this p_code, being CSS or JS, depending on p_type'''
@@ -341,10 +337,10 @@ class Px:
         # Define and return the context
         r = O(
           traversal=traversal, handler=handler, guard=guard, tool=tool, req=req,
-          o=o, layout=layout, ui=tool.ui, _=o.translate, user=user,
+          o=o, layout=layout, ui=tool.ui, _=tool.translate, user=user,
           isAnon=user.isAnon(), Px=Px, config=config, field=traversal.field,
           appName=config.model.appName, q=Quote.js, url=tool.buildUrl,
-          lang=lang, dir=dir, dleft=dleft, dright=dright,
+          svg=tool.buildSvg, lang=lang, dir=dir, dleft=dleft, dright=dright,
           siteUrl=config.server.getUrl(handler), _css_=config.ui.patchCss)
         # Ensure keys "ajax" and "popup" are present
         for name in ('ajax', 'popup'):
@@ -359,14 +355,15 @@ class Px:
     # standard Appy PX.
     contextKeys = ('traversal', 'handler', 'guard', 'tool', 'req', 'o', 'home',
       'layout', 'popup', 'ajax', 'ui', '_', 'user', 'isAnon', 'Px', 'config',
-      'appName', 'q', 'url', 'lang', 'dir', 'dleft', 'dright', '_css_')
+      'appName', 'q', 'url', 'svg', 'lang', 'dir', 'dleft', 'dright', '_css_')
 
     @classmethod
-    def copyContext(class_, context, other):
+    def copyContext(class_, context, other, overwrite=False):
         '''Copy base variables whose names are defined in p_class_.contextKeys
            from p_other to p_context.'''
         for key in class_.contextKeys:
-            context[key] = other[key]
+            if overwrite or key not in context:
+                context[key] = other[key]
 
     # Names of the PXs being Appy home pages. Key indicates if "public" pages
     # are taken into account or not.
