@@ -1,7 +1,7 @@
 '''Appy HTTP server'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -26,6 +26,7 @@ from appy import utils, version
 from appy.database import Database
 from appy.utils import url as uutils
 from appy.server.pool import ThreadPool
+from appy.utils.path import getShownSize
 from appy.model.utils import Object as O
 from appy.server.scheduler import Scheduler
 from appy.server.static import Config as StaticConfig
@@ -40,8 +41,10 @@ STOP_CLASSIC  = ':: %s:%s stopped ::'
 STOP_CLEAN    = ':: Clean end ::'
 STOP_RUN      = ':: Run end ::'
 APPY_VERSION  = 'Appy is "%s".'
+RAM_LOAD      = '%d CSS + %d SVG files loaded in RAM (%s).'
 POLLING       = 'Polling (registered client sockets = %d)...'
 NEW_CLI       = 'Server socket accepted new client %s::%s'
+DB_C_ERR      = 'Error on closing database. %s.'
 
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CONN_RESET    = 'Connection reset by peer (client port %d).'
@@ -53,6 +56,7 @@ SPAWN_IF_KO   = 'spawnIfUnder (%d) should be < than the number of threads (%d)'
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
     '''HTTP server configuration for a Appy site'''
+
     def __init__(self):
         # The server address
         self.address = '0.0.0.0'
@@ -67,8 +71,6 @@ class Config:
         # "Strict". More info on https://developer.mozilla.org/en-US/docs/Web/
         #                         HTTP/Headers/Set-Cookie/SameSite
         self.sameSite = 'Lax'
-        # Configuration for static content (set by m_set below)
-        self.static = None
         # The size of the queue as defined on the server socket, passed as
         # parameter to the "listen" method.
         self.queueSize = 5
@@ -96,6 +98,27 @@ class Config:
         #   | less verbose output.
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.debugLevel = 0
+        # Configuration for static content (set by m_set below)
+        self.static = None
+        # Traversers. A "traverser" is a name that can be used at the root of a
+        # URL path, to traverse something else than an object. Indeed, the
+        # standard traversal is based on database objects. For instance,
+        #
+        #                      <appUrl>/666
+        #
+        # requires 666 to be the iid of a database object;
+        #
+        #                      <appUrl>/something
+        #
+        # requires "something" to point to an object stored in the database
+        # under that name in database dict root.objects.
+        # If you want to use starting URL paths not being database objects,
+        # define a traverser as an entry in the following dict.
+        # - The key is the name to be used in the URL path;
+        # - The value is a Python expression, receiving the tool as variable
+        #   "tool" in its context.
+        # Take care of security when defining traversers.
+        self.traversers = {}
         # The path to the site. Will be set by m_set below.
         self.sitePath = None
         # ~~~
@@ -133,6 +156,13 @@ class Config:
         # attribute), check for hung threads that need to be killed, or for
         # zombie threads that should cause a restart.
         self.hungCheckPeriod = 100
+        # On server shutdown, threads may still be busy. Determine here the
+        # maximum number of seconds the server will wait for all threads to
+        # terminate, before brutally killing them. By default, wait 1 minute.
+        # Set it to a higher value if your app frequently performs longer
+        # transactions and you want to give more chances to these latters to end
+        # up gracefully.
+        self.maxWait = 60
 
     def isIPv6(self):
         '''Is IP v6 in use ?'''
@@ -145,20 +175,32 @@ class Config:
         appPath = pathlib.Path(appFolder)
         self.static = StaticConfig(appPath)
 
+    def getHost(self, handler, withPort=True):
+        '''Gets this server's hostname'''
+        # In order to determine the host name, use, in that order, HTTP header
+        # keys "X-Forwarded-Host" or "Host".
+        headers = handler.headers
+        r = headers.get('X-Forwarded-Host') or headers.get('Host')
+        if r:
+            # v_r may contain the port. Remove it if it is present and
+            # p_withPort is False.
+            if not withPort and ':' in r:
+                r = r.split(':', 1)[0]
+        else:
+            # Use p_self.address as base
+            r = self.address
+            if withPort and self.port != 80:
+                r += ':%d' % self.port
+        return r
+
     def getUrl(self, handler, relative=False):
         '''Returns the base URL for URLs produced by this Appy server'''
+        headers = handler.headers
         if relative:
             base = ''
         else:
-            # Use, in that order, keys 'X-Forwarded-Host' or 'Host' if found
-            # among the HTTP header.
-            headers = handler.headers
-            host = headers.get('X-Forwarded-Host') or headers.get('Host')
-            if not host:
-                host = self.address
-                if self.port != 80:
-                    host += ':%d' % self.port
-            # Get protocol from header key 'X-Forwarded-Proto' when present
+            host = self.getHost(handler)
+            # Get protocol from header key "X-Forwarded-Proto" when present
             protocol = headers.get('X-Forwarded-Proto') or self.protocol
             # Build the base of the URL
             base = '%s://%s' % (protocol, host)
@@ -216,20 +258,24 @@ class Server:
         # Ensure the config is valid
         config.check()
         # p_mode can be:
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "fg"     | Server start, in the foreground (debug mode)
         # "bg"     | Server start, in the background
         # "clean"  | Special mode for cleaning the database
         # "run"    | Special mode for executing a single p_method on the
         #          | application tool.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # Modes "clean" and "run" misuse the server to perform a specific task.
         # In those modes, the server is not really started (it does not listen
         # to a port) and is shutdowned immediately after the task has been
         # performed.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.mode = mode
         self.classic = mode in ('fg', 'bg')
+        # Write, when appropriate, a file containing the server's process ID
+        if mode == 'bg':
+            path = config.database.getPidPath()
+            with path.open('w') as f: f.write(str(os.getpid()))
         # The following attributes will be initialized afterwards
         self.socket = None   # The server socket
         self.selector = None # The selector object
@@ -258,7 +304,9 @@ class Server:
             # performs the task linked to p_self.mode.
             config.database.getDatabase(self, handler, poFiles, method=method)
             # Initialise the static configuration
-            cfg.static.init(config.ui)
+            counts = cfg.static.init(config.ui)
+            self.loggers.app.info(RAM_LOAD % (counts.css, counts.svg,
+                                              getShownSize(counts.size)))
             # Unregister the virtual handler
             VirtualHandler.remove()
         except (Model.Error, Database.Error) as err:
@@ -326,18 +374,28 @@ class Server:
 
     def shutdown(self):
         '''Normal server shutdown'''
-        # Close the server socket if it is there
-        if self.socket: self.socket.close()
-        # Logs the shutdown
-        self.logShutdown()
         # Shutdown the pool of threads
         if self.pool:
             self.pool.shutdown()
-        # Shutdown the loggers
-        logging.shutdown()
+        # Close the server socket if it is there
+        if self.socket: self.socket.close()
         # Shutdown the database
         database = self.database
-        if database: database.close()
+        if database:
+            try:
+                database.close()
+            except Exception as err:
+                self.loggers.app.error(DB_C_ERR % str(err))
+                self.logTraceback()
+        # Logs the shutdown
+        self.logShutdown()
+        # Shutdown the loggers
+        logging.shutdown()
+        # Delete the "pid" file when it exists
+        if self.mode == 'bg':
+            pidFile = self.config.database.getPidPath()
+            if pidFile.exists():
+                pidFile.unlink()
 
     def abort(self, error=None):
         '''Server shutdown following an error'''
@@ -364,7 +422,8 @@ class Server:
         # Create a Handler to handle this request
         try:
             handler = HttpHandler(clientSocket, self)
-            closeSocket = handler.run()
+            handler.run()
+            closeSocket = handler.closeSocket
         except ConnectionResetError:
             clientPort = clientSocket.getpeername()[1]
             self.loggers.app.error(CONN_RESET % clientPort)
@@ -408,7 +467,7 @@ class Server:
         self.selector.unregister(clientSocket)
         # Depending on the server mode, manage the request in the main thread or
         # in a thread from the pool.
-        if (self.pool is None) or (self.mode == 'fg'):
+        if self.pool is None or self.mode == 'fg':
             # In debug mode, process the request in the single, main thread
             self.processClientRequest(clientSocket)
         else:
@@ -477,7 +536,7 @@ class Server:
         if name:
             # If no extension is found in p_name, we suppose it is a PNG image
             name = name if '.' in name else '%s.png' % name
-            if base is None:
+            if base is None and not ram:
                 # Get the base folder containing the resource
                 base = config.ui.images.get(name) or 'appy'
             name = '/%s' % name
@@ -503,14 +562,14 @@ class Server:
     def getUrl(self, o, sub=None, relative=False, **params):
         '''Gets the URL of some p_o(bject)'''
         # Parameters are as follows.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # sub      | If specified, it denotes a part that will be added to the
         #          | object base URL for getting one of its specific sub-pages,
         #          | like "view" or "edit".
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # relative | If True, the base URL <protocol>://<domain> will not be
         #          | part of the result.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # params   | Every entry in p_params will be added as-is as a parameter
         #          | to the URL, excepted if the value is None or key is
         #          | "unique": in that case, its value must be boolean: if
@@ -518,11 +577,11 @@ class Server:
         #          | replaced with a parameter whose value will be based on
         #          | time.time() in order to obtain a link that has never been
         #          | visited by the browser.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # The presence of parameter "popup=True" in the URL will open the
         # corresponding object-related page in the Appy iframe, in a
         # minimalistic way (ie, without portlet, header and footer).
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # The base app URL
         r = self.config.server.getUrl(o.H(), relative=relative)
         # Add the object ID
