@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -170,9 +170,13 @@ class AuthenticationContext:
         # The corresponding object, if any
         guard.authObject = self.getObject(tool, user, ctx)
 
-    def isMandatory(self, tool):
+    def isMandatory(self, tool, authenticate=False):
         '''When authentication contexts are activated, is the user forced to
            choose one ?'''
+        # Not if the user must be p_authenticate(d) and it is impossible to
+        # choose any context.
+        if authenticate and not tool.config.security.authContext.chooseOnLogin:
+            return
         return True
 
     def switchContext(self, tool):
@@ -221,7 +225,7 @@ class AuthenticationContext:
         req = tool.req
         ctx = req.ctx
         if not authContext or not authContext._maySwitchTo(tool, ctx):
-            self.raiseUnauthorized()
+            tool.raiseUnauthorized()
         # Perform the switch by updating the authentication cookie and
         # user-related data structures stored on the guard.
         Cookie.update(tool.H(), ctx)
