@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -38,12 +38,12 @@ class Error:
 
      # Error 403: unauthorized
      403: {'label': 'unauthorized', 'px':
-       Px('''<div><img src=":url('password.svg')" class="iconERR"/>
+       Px('''<div><img src=":svg('password')" class="iconERR"/>
                   <x>::msg</x></div>''', template=Template.px, hook='content')},
 
-     # Error 500 server error
+     # Error 500: server error
      500: {'label': 'server_error', 'px':
-       Px('''<div><img src=":url('warning.svg')" class="iconERR"/>
+       Px('''<div><img src=":svg('warning')" class="iconERR"/>
                   <x>::msg</x></div>''', template=Template.px, hook='content')},
 
      # Error 503: service unavailable. This error code is used by Appy when, due
@@ -55,7 +55,7 @@ class Error:
 
      # Return code 200: a logical error rendering a nice translated message to
      # the user, but not considered being an error at the HTTP level.
-     200: {'px': Px('''<img src=":url('warning.svg')" class="iconERR"/>
+     200: {'px': Px('''<img src=":svg('warning')" class="iconERR"/>
                        <x>::msg</x>''', template=Template.px, hook='content')}
     }
 
@@ -63,7 +63,7 @@ class Error:
     def getContent(class_, traversal, code, error):
         '''Get the textual content to render in the error page'''
         # Dump a traceback for a Manager
-        if traversal.user.hasRole('Manager') and (code != 200):
+        if traversal.user.hasRole('Manager') and code != 200:
             r = Traceback.get(html=True)
         else:
             if isinstance(error, MessageException):
