@@ -3,7 +3,7 @@
    respect of security rules defined: workflows, permissions, etc.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -35,7 +35,7 @@ from appy.model.fields.password import Password
 
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 KO_LOG_TFM  = 'Invalid value "%s" for attribute "loginTransform".'
-KO_PERM     = '%s "%s": "%s" disallowed.'
+KO_PERM     = '%s "%s": "%s" disallowed%s.'
 AUTH_KO     = 'Authentication failed with login %s.'
 AUTH_OK     = 'Logged in.'
 
@@ -49,9 +49,9 @@ class Config:
         # Activate or not the button on home page for asking a new password
         self.activateForgotPassword = True
         # By default, when creating a local user, Appy generates a password for
-        # him. If you want to disable this and manage yourself, in your app's
-        # User.onEdit method, generation and sending of passwords, set the
-        # following attribute to False.
+        # him and tries to send him by email. If you want to disable this and
+        # manage yourself, in your app's User.onEdit method, generation and
+        # sending of passwords, set the following attribute to False.
         self.generatePassword = True
         # On user log on, you may want to ask them to choose their
         # "authentication context" in a list. Such a context may have any
@@ -207,11 +207,16 @@ class Guard:
     # Security checks on objects
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-    def allows(self, o, permission='read', raiseError=False):
+    def stringInfo(self, info):
+        '''Returns this additional p_info about an error, as a string'''
+        return '' if info is None else (' (%s)' % info.asString())
+
+    def allows(self, o, permission='read', raiseError=False, info=None):
         '''Has the user p_permission on p_o ?'''
         r = self.user.hasPermission(permission, o)
         if not r and raiseError:
-            raise Unauthorized(KO_PERM % (o.class_.name, o.id, permission))
+            s = self.stringInfo(info)
+            raise Unauthorized(KO_PERM % (o.class_.name, o.id, permission, s))
         return r
 
     def mayAct(self, o):
@@ -230,35 +235,41 @@ class Guard:
         return multicall(o, 'mayDelete', True)
 
     def mayEdit(self, o, permission='write', permOnly='specific',
-                raiseError=False):
+                raiseError=False, info=None):
         '''May the user edit this object ? p_permission can be a field-specific
            permission. If p_permOnly is True, the additional user-defined
            condition (custom m_mayEdit) is not evaluated. If p_permOnly is
            "specific", this condition is evaluated only if the permission is
            specific (=not "write") If p_raiseError is True, if the user may not
            edit p_o, an error is raised.'''
-        r = self.allows(o, permission, raiseError=raiseError)
+        # p_info, if passed, represents additional contextual information that
+        # will be part of the error message in case p_o's edition can't be
+        # performed and p_raiseError is True.
+        r = self.allows(o, permission, raiseError=raiseError, info=info)
         if not r: return
-        if (permOnly == True) or \
-           ((permOnly == 'specific') and (permission != 'write')): return r
+        if permOnly == True or \
+           (permOnly == 'specific' and permission != 'write'): return r
         # An additional, user-defined condition, may refine the base permission
         r = multicall(o, 'mayEdit', True)
         if not r and raiseError:
             method = '%s::mayEdit' % o.class_.name
-            raise Unauthorized(KO_PERM % (method, o.id, permission))
+            s = self.stringInfo(info)
+            raise Unauthorized(KO_PERM % (method, o.id, permission, s))
         return r
 
-    def mayView(self, o, permission='read', raiseError=False):
+    def mayView(self, o, permission='read', raiseError=False, info=None):
         '''May the user view this p_o(bject) ? p_permission can be a field-
            specific permission. If p_raiseError is True, if the user may not
            view p_o, an error is raised.'''
-        r = self.allows(o, permission, raiseError=raiseError)
+        # Check m_mayEdit hereabove to know what p_info is all about
+        r = self.allows(o, permission, raiseError=raiseError, info=info)
         if not r: return
         # An additional, user-defined condition, may refine the base permission
         r = multicall(o, 'mayView', True)
         if not r and raiseError:
             method = '%s::mayView' % o.class_.name
-            raise Unauthorized(KO_PERM % (method, o.id, permission))
+            s = self.stringInfo(info)
+            raise Unauthorized(KO_PERM % (method, o.id, permission, s))
         return r
 
     #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -428,6 +439,17 @@ class Guard:
     # content.
     pxLoginBottom = Px('')
 
+    def getLoginLogo(self, discreet, url):
+        '''Returns the possibly clickable logo to include in the login box'''
+        # Compute the "img" tag representing the logo
+        r = '<img src="%s"/>' % url('loginLogo')
+        # Wrap it, when appropriate, in a link allowing to go back to the public
+        # page.
+        if discreet == 'home':
+            r = '<a onclick="goto(siteUrl+\'/tool/public\')" ' \
+                'class="clickable">%s</a>' % r
+        return r
+
     # The login form
     pxLogin = Px('''
      <!-- Popup for reinitializing the password -->
@@ -455,10 +477,12 @@ class Guard:
           style=":'display:%s' % display">
 
       <!-- Allow to hide the box when relevant -->
-      <img if="discreet == True" src=":url('close.svg')" style="float:right"
+      <img if="discreet == True" src=":svg('close')" style="float:right"
            onclick="toggleLoginBox(false)" class="clickable iconXS"/>
 
-      <div align="center"><img src=":url('loginLogo')"/></div>
+      <!-- A (possibly clickable) logo -->
+      <div align="center">::guard.getLoginLogo(discreet, url)</div>
+
       <h1>::_('app_name')</h1>
       <center class="sub">:_('please_connect')</center>
       <div class="loginSpace"></div>
@@ -520,18 +544,20 @@ class Guard:
 
      css='''
       .loginBox {
-         border:|loginBorder| |loginColorPh|; padding:35px; color:|darkColor|;
-         background-color:|boxBgColor|; position:absolute; top:|loginTop|;
-         left:50%; z-index:10; transform:translate(-50%,-50%);
+         border:|loginBorder| |loginColorPh|; padding:|loginBoxPadding|;
+         color:|loginColor|; background-color:|boxBgColor|; position:absolute;
+         top:|loginTop|; left:50%; z-index:10; transform:translate(-50%,-50%);
          box-shadow:|loginShadow|; border-radius: |loginBorderRadius|
       }
-      .loginBox a, .loginBox a:visited { font-weight:bold; letter-spacing:1px }
+      .loginBox a, .loginBox a:visited {
+        font-weight:bold; letter-spacing:1px; color:|loginColor| }
       .loginBox h1 { padding-bottom:0; margin-bottom:3px; text-align:center;
                      font-size:160%; font-weight: |loginTitleWeight| }
       .loginBox .sub { font-size:120% }
       #loginForm input {
         background-color:|loginBgColor|; width:|loginWidth|; border:none;
-        padding:|loginPadding|; margin:|loginMargin|; color:|loginColor| }
+        padding:|loginPadding|; margin:|loginMargin|; color:|loginColor|;
+        border-radius:|loginRadius|; }
       .submitLogin { text-align:|submitAlign|
       }
       #loginForm input[type=submit] {
@@ -545,10 +571,10 @@ class Guard:
       .autoRegister { text-align:|loginAlign| }
 
       /* Placeholder color for the login form */
-      .loginBox ::placeholder, .loginBox ::-webkit-input-placeholder {
-        color:|loginColorPh|; font-weight:bold; text-transform:uppercase }
+      .loginBox ::placeholder { color:|loginColorPh|;
+        font-weight:|loginWeightPh|; text-transform:|loginTransformPh| }
       .loginSpace { height:35px }
-      #rlogin { color:white }
+      #rlogin { color:|popupColor| }
      ''')
 
     # Warning about using cookies  - - - - - - - - - - - - - - - - - - - - - - -
