@@ -1,7 +1,7 @@
 '''A handler is responsible for handling a HTTP request'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -20,7 +20,7 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from http import HTTPStatus
 from http import client as hc
-import socket, threading, urllib.parse
+import socket, threading, urllib.parse, transaction
 
 from appy.model.base import Base
 from appy.server.guard import Guard
@@ -41,10 +41,12 @@ SOCK_TO     = 'Client socket timeout: %s.'
 RQ_LINE_KO  = 'Bad request line "%s".'
 HTTP_V_KO   = 'Wrong HTTP version "%s".'
 HTTP_V_UNS  = 'Unsupported HTTP version "%s".'
+H_METH_KO   = 'Wrong HTTP method "%s".'
 HEAD_TOO_LG = 'Line too long: %s...'
 HEAD_T_MANY = 'Too many headers: %s...'
 T_READ_SOCK = 'Reading request line on rfile...'
 T_READ_GOT  = 'Got request line: %s (length: %d)'
+T_AL_REG    = 'Thread %d was already in registry.'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class MethodsCache(dict):
@@ -75,7 +77,7 @@ class MethodsCache(dict):
         # This will allow to consider this method as if it was a static method
         # on the tool.
         cheat = False
-        if not class_ and (method.__code__.co_varnames[0] == 'tool'):
+        if not class_ and method.__code__.co_varnames[0] == 'tool':
             prefix = o.class_.name
             o = o.tool
             cheat = True
@@ -129,8 +131,13 @@ class Handler:
     def add(class_, handler):
         '''Adds a new p_handler into the registry'''
         id = threading.get_ident()
-        if id in class_.registry: return
+        # If the currently running thread is already in the registry, overwrite
+        # the entry with that new p_handler.
+        r = id in class_.registry
         class_.registry[id] = handler
+        # Return True if the thread was already mentioned in the registry
+        # (this should not happen).
+        return r
 
     @classmethod
     def remove(class_):
@@ -148,7 +155,7 @@ class Handler:
     def init(self):
         '''Define attributes which are common to any handler'''
         # A lot of object methods are executed while handling a request. Dict
-        # "method" hereafter caches method results: it avoids executing more
+        # "methods" hereafter caches method results: it avoids executing more
         # than once the same method. Only methods without args are cached, like
         # methods defined on field attributes such as "show".
         self.methods = MethodsCache()
@@ -220,6 +227,9 @@ class HttpHandler(Handler):
     def __init__(self, clientSocket, server):
         # The global config
         self.config = server.config
+        # The request path (will be parsed later, but may be mentioned in some
+        # errors before being initialized).
+        self.path = '-'
         # The client socket from which the client request will be read
         self.clientSocket = clientSocket
         self.clientHost, self.clientPort = clientSocket.getpeername()
@@ -255,10 +265,8 @@ class HttpHandler(Handler):
         Handler.init(self)
 
     def finish(self):
-        '''Closes p_self.rfile and returns the fact that p_self.clientSocket
-           must be closed or not.'''
+        '''Closes p_self.rfile'''
         self.rfile.close()
-        return self.closeSocket
 
     def getLayout(self):
         '''Try to deduce the current layout from the traversal, if present'''
@@ -287,9 +295,11 @@ class HttpHandler(Handler):
         # Return the response content. In the case of a redirect, None will be
         # returned.
         if self.guard.user.isAnon():
-            siteUrl = self.config.server.getUrl(self)
+            config = self.config
+            siteUrl = config.server.getUrl(self)
             gotoUrl = urllib.parse.quote('%s%s' % (siteUrl, self.path))
-            resp.goto(url='%s/tool/home?goto=%s' % (siteUrl, gotoUrl),
+            resp.goto(url='%s/tool/%s?goto=%s&stay=1' % \
+                      (siteUrl, config.ui.home, gotoUrl),
                       message=self.tool.translate('please_authenticate'))
             r = None
         else:
@@ -353,12 +363,12 @@ class HttpHandler(Handler):
                 transaction.abort()
                 # Return an error if is was the last attempt
                 if attempts == maxAttempts:
-                    self.log('site', 'error', TRANS_ERR % str(err))
+                    self.log('app', 'error', TRANS_ERR % str(err))
                     self.resp.code = HTTPStatus.SERVICE_UNAVAILABLE # 503
                     r = Error.get(self.resp, self.traversal)
                     break
                 else:
-                    self.log('site', 'warning',
+                    self.log('app', 'warning',
                              TRANS_RETRY % (str(err), attempts))
             except Exception as err:
                 self.resp.code = HTTPStatus.INTERNAL_SERVER_ERROR # 500
@@ -390,7 +400,7 @@ class HttpHandler(Handler):
             del parts[0]
         # If the unique part corresponds to a file (ie, any name containing a
         # dot), it is static content, too (favicon.ico, robots.txt...)
-        elif (len(parts) == 1) and ('.' in parts[0]):
+        elif len(parts) == 1 and '.' in parts[0]:
             self.static = True
             # Get this file at the place configured in the UI config
             base = config.ui.images.get(parts[0]) or 'appy'
@@ -401,8 +411,8 @@ class HttpHandler(Handler):
         self.parts = parts
         # Must the client socket be closed after the request has been handled ?
         httpVersion = config.server.httpVersion
-        self.closeSocket = (httpVersion == 1) or \
-                           (self.headers.get('Connection') == 'close')
+        self.closeSocket = httpVersion == 1 or \
+                           self.headers.get('Connection') == 'close'
 
     def handle(self):
         '''Called by m_run to handle a HTTP request'''
@@ -417,22 +427,24 @@ class HttpHandler(Handler):
             Static.get(self)
         else:
             # Add myself to the registry of handlers
-            Handler.add(self)
+            already = Handler.add(self)
+            if already:
+                self.log('app', 'warning', T_AL_REG % threading.get_ident())
             # Initialise a database connection
             database = self.server.database
             self.dbConnection = database.openConnection()
-            # Extract parameters and create a Request instance in p_self.req
-            self.req = Request.create(self)
-            # Compute dynamic content
-            r = self.attemptDatabaseRequest()   
-            # Build the HTTP response (if not done yet)
             try:
+                # Extract parameters and create a Request instance in p_self.req
+                self.req = Request.create(self)
+                # Compute dynamic content
+                r = self.attemptDatabaseRequest()   
+                # Build the HTTP response (if not done yet)
                 resp.build(r)
             finally:
-                # Close the DB connection
-                database.closeConnection(self.dbConnection)
                 # Remove myself from the registry (for now)
                 Handler.remove()
+                # Close the DB connection
+                database.closeConnection(self.dbConnection)
         # Log this hit and the response code on the site log
         self.log('site', 'info', str(resp.code.value))
 
@@ -444,7 +456,7 @@ class HttpHandler(Handler):
         parts = stripped.decode(Response.headersEncoding).split()
         count = len(parts)
         if count not in (2, 3):
-            return HTTPStatus.BAD_REQUEST, RQ_LINE_KO % decoded
+            return HTTPStatus.BAD_REQUEST, RQ_LINE_KO % stripped
         # Manage the request version
         if count == 3:
             # The version of the HTTP protocol requested by the client
@@ -457,7 +469,7 @@ class HttpHandler(Handler):
             try:
                 floatVersion = float(version[5:])
                 # Currently, HTTP 1.0 and 1.1 are supported
-                if (floatVersion < 1.0) or (floatVersion > 1.1):
+                if floatVersion < 1.0 or floatVersion > 1.1:
                     code = HTTPStatus.HTTP_VERSION_NOT_SUPPORTED
                     return code, HTTP_V_UNS % version
             except ValueError:
@@ -483,8 +495,7 @@ class HttpHandler(Handler):
                                 clientPort=self.clientPort)
 
     def run(self):
-        '''Handles a complete HTTP request. Returns True if p_self.clientSocket
-           must be closed afterwards.'''
+        '''Handles a complete HTTP request'''
         try:
             # Read the request line
             self.tlog(T_READ_SOCK)
@@ -511,10 +522,10 @@ class HttpHandler(Handler):
             # Handle the request
             self.handle()
         except socket.timeout as e:
-            # A read or a write timed out. Discard the client socket.
+            # A read or a write timed out: discard the client socket
             self.log('app', 'error', SOCK_TO % str(e))
         finally:
-            return self.finish()
+            self.finish()
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class VirtualHandler(Handler):
