@@ -1,7 +1,7 @@
 '''Pool of threads used by the Appy HTTP server'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -30,7 +30,7 @@ INITIAL_WK    = 'Initial worker'
 ADDED_WK      = 'Worker added in response to lack of idle workers'
 WK_KILLED     = 'Worker added for replacing killed thread %s'
 WK_REPLACED   = 'Voluntary replacement for thread %s'
-WORK_DONE     = ' > handled by thread %s (%i tasks queued).'
+WORK_DONE     = ' > handled by %s (%s) (tasks queued: %i).'
 CHECK_ADD_THR = 'No idle workers for task; checking if we need to make ' \
                 'more workers'
 SPAWNING      = 'No idle tasks and only %d busy task(s): adding %d more workers'
@@ -46,9 +46,9 @@ HUNG_STATUS   = "killHungThreads status: %d threads (%d working, %d idle, " \
                 "worker(s)."
 ZOMBIES_FOUND = 'Zombie thread(s) found: %s.'
 EXITING       = 'Exiting process because %s zombie threads is more than %s limit.'
-NEW_WORKER    = 'Started new worker %s: %s'
+NEW_WORKER    = 'Started new %s (%s): %s'
 STOPPING_THR  = 'Thread %s processed %i requests (limit %s); stopping it'
-SHUTD_RECV    = 'Worker %s is asked to shutdown'
+SHUTD_RECV    = '%s (%s) is asked to shutdown'
 SHUTD_POOL    = 'Shutting down the threadpool (%d threads)'
 SHUTD_HUNG    = "%s worker(s) didn't stop properly, and %s zombie(s)"
 SHUTD_OK      = 'All workers stopped'
@@ -62,7 +62,7 @@ class ThreadPool:
     # This excellent code was initially copied from the Paste project, many many
     # thanks. https://github.com/cdent/paste/blob/master/paste/httpserver.py
 
-    # This pool manages a list of thread workers managing cliet requests via a
+    # This pool manages a list of thread workers managing client requests via a
     # queue of callables.
 
     # The pool keeps a notion of the status of its worker threads.
@@ -82,7 +82,7 @@ class ThreadPool:
 
     # The pool also maintains, in attribute "tracked", a dictionary with these
     # keys and lists of thread IDs that fall in that status. All keys will be
-    # present, even if they point to emty lists.
+    # present, even if they point to empty lists.
 
     # Hung threads are threads that have been busy more than
     # config.hungThreadLimit seconds. Hung threads are killed when they live
@@ -109,7 +109,7 @@ class ThreadPool:
 
     # This flag specifies if, when the server terminates, all in-progress client
     # connections should be droppped.
-    daemon = False
+    daemon = True
 
     def __init__(self, server, config):
         '''Initialise the pool's data stuctures'''
@@ -177,8 +177,8 @@ class ThreadPool:
                 self.debug(NO_SPAWNING % busy)
         # Kill supernumerary workers when relevant
         idle = len(self.idleWorkers)
-        if (len(self.workers) > cfg.threads) and (idle > 3) and \
-           ((time.time()-self.lastAddedNewIdleWorkers) > cfg.hungThreadLimit):
+        if len(self.workers) > cfg.threads and idle > 3 and \
+           (time.time()-self.lastAddedNewIdleWorkers) > cfg.hungThreadLimit:
             # We've spawned workers in the past, but they aren't needed anymore;
             # kill off some.
             kill = len(self.workers) - cfg.threads
@@ -232,7 +232,7 @@ class ThreadPool:
                 '%s</table>' % '\n'.join(rows)
         return r
 
-    def killWorker(self, id):
+    def killWorker(self, id, recreate=True):
         '''Removes from the pool, the worker whose thread ID is p_id and
            replaces it with a new worker thread.'''
         # This should only be done for mis-behaving workers
@@ -257,7 +257,8 @@ class ThreadPool:
         if thread in self.workers:
             self.workers.remove(thread)
         self.dyingThreads[id] = (time.time(), thread)
-        self.addWorker(message=WK_KILLED % id)
+        if recreate:
+            self.addWorker(message=WK_KILLED % id)
 
     def addWorker(self, *args, **kwargs):
         index = next(self.workersCount)
@@ -325,17 +326,18 @@ class ThreadPool:
         '''Method executed by a worker thread'''
         cfg = self.config
         thread = threading.currentThread()
+        tname = thread.getName()
         id = thread.thread_id = threading.get_ident()
         self.workers.append(thread)
         self.idleWorkers.append(id)
         processed = 0 # The number of processed requests
         replace = False # Must we replace the thread ?
-        self.debug(NEW_WORKER % (id, message))
+        self.debug(NEW_WORKER % (tname, id, message))
         try:
             while True:
                 # Stop myself if I have managed the maximum number of requests
                 max = cfg.maxRequests
-                if max and (processed > max):
+                if max and processed > max:
                     # Replace me then die
                     self.debug(STOPPING_THR % (id, processed, max))
                     replace = True
@@ -344,7 +346,7 @@ class ThreadPool:
                 runnable = self.queue.get()
                 if runnable is ThreadPool.SHUTDOWN:
                     # This is not a job, but a shutdown command
-                    self.debug(SHUTD_RECV % id)
+                    self.debug(SHUTD_RECV % (tname, id))
                     break
                 # Note myself as working
                 try:
@@ -355,7 +357,7 @@ class ThreadPool:
                 processed += 1
                 # Perform my task
                 runnable()
-                self.debug(WORK_DONE % (id, self.queue.qsize()))
+                self.debug(WORK_DONE % (tname, id, self.queue.qsize()))
                 # Note myself again as being "idle"
                 try:
                     del(self.tracked[id])
@@ -389,7 +391,20 @@ class ThreadPool:
         # Add a shutdown request for every worker
         for i in range(count):
             self.queue.put(ThreadPool.SHUTDOWN)
-        # Wait for each thread to terminate
+        # Wait for each thread to terminate. v_wait is the maximum number of
+        # seconds we are allowed to wait before trying to kill threads.
+        wait = self.config.maxWait
+        waitOnce = 5
+        alive = len(self.workers)
+        while wait > 0 and alive > 0:
+            for worker in self.workers:
+                worker.join(waitOnce)
+                wait -= waitOnce
+                if not worker.is_alive():
+                    alive -= 1
+            # In the meanwhile, workers may have self-removed from the list
+            alive = min(alive, len(self.workers))
+        # Get threads still being busy after out wait
         hung = []
         for worker in self.workers:
             worker.join(0.5)
@@ -403,7 +418,7 @@ class ThreadPool:
             self.debug(SHUTD_HUNG % (len(hung), len(zombies)))
             if hung:
                 for worker in hung:
-                    self.killWorker(worker.thread_id)
+                    self.killWorker(worker.thread_id, recreate=False)
                 self.debug(KILLED_HUNGED)
             if forceQuitTimeout:
                 timedOut = False
