@@ -2,7 +2,7 @@
    values.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -21,6 +21,7 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 import re, urllib.parse
 
+from appy.utils import json
 from appy.model.utils import Object
 
 # Entries considered as empty in a multipart message - - - - - - - - - - - - - -
@@ -64,6 +65,8 @@ class Request(Object):
                 # elements are separated by some boundary text.
                 boundary = '--' + contentType[contentType.index('boundary=')+9:]
                 req.parseMultiPart(content, boundary.encode())
+            elif contentType == 'application/json':
+                req.parseJson(content)
         # Get Cookies
         cookieString = handler.headers['Cookie']
         if cookieString is not None:
@@ -90,13 +93,20 @@ class Request(Object):
 
     def parse(self, params, sep='&', post=False):
         '''Parse GET/POST p_param(eters) and add one attribute per parameter'''
+        method = 'unquote_plus' if post else 'unquote'
         for param in params.split(sep):
             # Every param is of the form <name>=<value> or simply <name>
             if '=' in param:
                 # <name>=<value>
                 name, value = param.split('=', 1)
-                method = 'unquote_plus' if post else 'unquote'
-                value = getattr(urllib.parse, method)(value)
+                # A parameter whose name ends with a "+" char means: do not
+                # escape "+" chars in its value.
+                if name.endswith('+'):
+                    name = name[:-1]
+                    meth = 'unquote'
+                else:
+                    meth = method
+                value = getattr(urllib.parse, meth)(value)
             else:
                 # <name>
                 name = param
@@ -140,6 +150,13 @@ class Request(Object):
             # Find the element value
             self.addValue(name, value)
 
+    def parseJson(self, content):
+        '''Parses JSON data string-encoded in p_content and add, on p_self, one
+           attribute for every entry from the parsed JSON dta structure.'''
+        parsed = json.Decoder.decode(content.decode('utf-8'))
+        if not parsed: return
+        self.update(parsed)
+
     def patchFromTemplate(self, o):
         '''p_o is a temp object that is going to be edited via px "edit". If a
            template object must be used to pre-fill the web form, patch the
