@@ -2,7 +2,7 @@
    values.'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -156,12 +156,19 @@ class Response:
         else:
             self.message = '%s<br/>%s' % (self.message, message)
 
-    def goto(self, url=None, message=None):
+    def goto(self, url=None, message=None, fromPopup=False):
         '''Redirect the user to p_url'''
         if message: self.addMessage(message)
-        self.code = HTTPStatus.SEE_OTHER # 303
-        # Redirect to p_url or to the referer URL if no p_url has been given
-        self.headers['Location'] = url or self.handler.headers['Referer']
+        req = self.handler.req
+        if fromPopup:
+            # Redirecting back to some URL from a popup requires an alternative
+            # technique, performed by the Appy IFrame.
+            req._back = url
+        else:
+            # The standard HTTP technique
+            self.code = HTTPStatus.SEE_OTHER # 303
+            # Redirect to p_url or to the referer URL if no p_url has been given
+            self.headers['Location'] = url or self.handler.headers['Referer']
 
     def write(self, s):
         '''Writes part p_s of the response to the client socket. Returns True if
@@ -242,7 +249,8 @@ class Response:
         toCompress = False
         type = headers.get('Content-Type')
         # Convert p_content to bytes if present
-        if content:
+        if content and not isinstance(content, bytes):
+            # p_content could also already be bytes
             content = content.encode(Response.encoding)
         elif path and (type in Response.compressableFiles):
             # Loads the file @ p_path in p_content if it must be compressed
