@@ -48,7 +48,7 @@
 #     there, ie, during the night.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -215,6 +215,11 @@ class TimePart:
             # Take the recurrence into account
             return (value - self.numA) % self.rec == 0
 
+    def isNum(self):
+        '''Return True if numA is an integer and numB and rec are None'''
+        return isinstance(self.numA, int) and \
+               self.numB is None and self.rec is None
+
     def __repr__(self):
         '''p_self's string representation'''
         rec = self.rec and ('/%d' % self.rec) or ''
@@ -313,6 +318,19 @@ class TimeDef:
             parts[i] = value
             i -= 1
 
+    def getHour(self):
+        '''Returns a tuple (i_hour, i_minute) if the hour and minute parts are
+           noth simply defined by integer numbers. Else, the method returns
+           None.'''
+        r = []
+        for i in (1, 0): # 1=hour, 0=minutes
+            parts = self.parts[i]
+            if len(parts) != 1: return
+            part = parts[0]
+            if not part.isNum(): return
+            r.append(part.numA)
+        return tuple(r)
+
     # Mapping between the parts of a timedef and the names of the corresponding
     # parts in the struct returned by time.localtime():
     #
@@ -371,7 +389,7 @@ class TimeDef:
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
-    '''Scheduler configuration, defining the list of planned jobs.'''
+    '''Scheduler configuration, defining the list of planned jobs'''
 
     def __init__(self):
         # The scheduler will be run every "minutes" minutes. It is not advised
@@ -452,7 +470,7 @@ class Scheduler:
         # is too much: perform a real execution every "self.minutes" minutes.
         if not self.minutes: return
         current = time.localtime().tm_min
-        if (current % self.minutes) or (current == self.last): return
+        if current % self.minutes or current == self.last: return
         # Run jobs that must be run
         self.last = current
         self.runJobs()
