@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -177,7 +177,7 @@ class Config:
         headers = req._orig_env
         encoding = self.extractEncoding(headers)
         decodeFunction = self.decodeFunction
-        for keyAttr, appyName in self.ssoAttributes.iteritems():
+        for keyAttr, appyName in self.ssoAttributes.items():
             if not appyName: continue
             # Get the name of the HTTP header corresponding to "keyAttr"
             keyName = getattr(self, keyAttr)
@@ -358,7 +358,7 @@ class Config:
         user.syncDate = DateTime()
         # Update basic user attributes and recompute the user title
         params, roles, groups = self.extractUserInfo(req)
-        for name, value in params.iteritems(): setattr(user, name, value)
+        for name, value in params.items(): setattr(user, name, value)
         user.updateTitle()
         # Update global roles (when roles are provided by the SSO server)
         if self.roleKey:
@@ -435,11 +435,4 @@ class Config:
             # user. Collect roles and groups this user has.
             user = self.createUser(tool, login, req)
         return user
-
-    def patchUrl(self, url):
-        '''If self.urlPrefix is not empty, check its presence in p_url and
-           remove it if found.'''
-        if not self.enabled or not self.urlPrefix: return url
-        part = '/%s/' % self.urlPrefix
-        return url.replace(part, '/', 1)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
