@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -18,18 +19,21 @@
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 from http import HTTPStatus
-import inspect, pathlib, mimetypes, os.path, email.utils, collections
+import re, inspect, pathlib, mimetypes, os.path, email.utils, collections
 
 from DateTime import DateTime
 
 import appy
+from appy.utils import css
 from appy.utils.string import Normalize
+from appy.model.utils import Object as O
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-MAP_VALUE_NOT_PATH = 'Values from the map must be pathlib.Path objects.'
-PATH_NOT_FOUND     = 'Path "%s" was not found or is not a folder.'
-RAM_ROOT_IN_MAP    = 'Ram root "%s" is also used as key for ' \
-                     'appy.server.static.Config.map.'
+MAP_VAL_KO  = 'Values from the map must be pathlib.Path objects.'
+PATH_KO     = 'Path "%s" was not found or is not a folder.'
+RAM_ROOT_KO = 'Ram root "%s" is also used as key for appy.server.static.' \
+              'config.map.'
+REPL_KO     = 'File %s: error while replacing variables. %s'
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Config:
@@ -59,7 +63,8 @@ class Config:
         # "static" part, allowing Appy to distinguish a RAM from a disk
         # resource. It defaults to "ram".
         self.ramRoot = 'ram'
-        # Let's illustrate this mechanism with an example. Suppose that:
+
+        # Here is an example. Suppose that:
         # * ramRoot = "ram" ;
         # * the content of appy/ui/static/appy.css is loaded in a string
         #   variable named "appyCss";
@@ -72,6 +77,9 @@ class Config:
         # not randomly chosen: Appy CSS files like appy.css are actually
         # template files containing variables that need to be replaced by their
         # actual values, coming from the app's (ui) config.
+        #
+        # Currently, such RAM resources include all CSS and SVG files.
+        #
         # When adding keys in Static.ram, ensure the key is a filename-like
         # name: the MIME type will be deduced from the file extension.
 
@@ -110,32 +118,71 @@ class Config:
         #
         # Keys must correspond to file names, not prefixed with any path-related
         # info.
-        self.versions = {'appy.css':2, 'appy.js':3}
+        self.versions = {'appy.css':8, 'appy.js':8}
 
     def check(self):
         '''Checks that every entry in p_self.map is valid'''
         for key, path in self.map.items():
             # Paths must be pathlib.Path instances
             if not isinstance(path, pathlib.Path):
-                raise Exception(MAP_VALUE_NOT_PATH)
+                raise Exception(MAP_VAL_KO)
             # Paths must exist and be folders
             if not path.is_dir():
-                raise Exception(PATH_NOT_FOUND % path)
+                raise Exception(PATH_KO % path)
         # Ensure the RAM root is not used as key in self.map
         if self.ramRoot in self.map:
-            raise Exception(RAM_ROOT_IN_MAP % self.ramRoot)
+            raise Exception(RAM_ROOT_KO % self.ramRoot)
+
+    def addFile(self, path, rex, fun, isCss):
+        '''Adds, in Static.ram, the file at p_path. Within the file content,
+           variables, if found, are replaced with values via a call to
+           p_rex.sub, with this p_fun(ction) used as replacement function.
+           Returns the size of the file in bytes.'''
+        # Read the content of this file
+        with path.open('r') as f:
+            # If the file is a CSS file, compact it
+            content = f.read()
+            if isCss:
+                content = css.File.compact(content)
+            # Add it to Static.ram, with variables replaced
+            try:
+                content = rex.sub(fun, content)
+                r = len(content)
+                Static.ram[path.name] = content
+                return r
+            except AttributeError as err:
+                raise Exception(REPL_KO % (path, str(err)))
 
     def init(self, uiConfig):
-        '''Reads all CSS files from all disk locations containing static content
-           (in p_self.map) and loads them in Static.ram, after having replaced
-           variables with their values from the app's (ui) config.'''
+        '''Reads all CSS and SVG files from all disk locations containing static
+           content (in p_self.map) and loads them in Static.ram, after having
+           replaced variables with their values from the app's config.'''
+        counts = O(css=0, svg=0, size=0)
+        # For CSS files, browse locations in forward order: that way, the order
+        # of inclusion of CSS files is: (1) Appy, (2) the app and, optionally,
+        # (3) the ext.
+        # ~~~
+        # The following function will, for every match (=every variable use
+        # found in the file), return the corresponding value as found on the UI
+        # config.
+        fun = lambda match: getattr(uiConfig, match.group(1))
+        rex = uiConfig.variable
         for path in self.map.values():
             for cssFile in path.glob('*.css'):
-                # Read the content of this file
-                with cssFile.open('r') as f:
-                    cssContent = f.read()
-                    # Add it to Static.ram, with variables replaced
-                    Static.ram[cssFile.name] = uiConfig.patchCss(cssContent)
+                counts.size += self.addFile(cssFile, rex, fun, True)
+                counts.css += 1
+        # For SVG files, browse location in reverse order. That way, if the same
+        # file exists at several levels, the one that exists at the deepest
+        # level will prevail: (1) ext, (2) app and finally (3) Appy.
+        # ~~~
+        # Values to replace are defined in a specific part of the UI config
+        fun = lambda match: getattr(uiConfig.svg, match.group(1))
+        for path in reversed(self.map.values()):
+            for svgFile in path.glob('*.svg'):
+                if svgFile.name not in Static.ram:
+                    counts.size += self.addFile(svgFile, rex, fun, False)
+                    counts.svg += 1
+        return counts
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Static:
@@ -184,7 +231,7 @@ class Static:
         browserDate = handler.headers.get('If-Modified-Since')
         modified = fileInfo.modified if fileInfo else modified
         smodified = email.utils.formatdate(modified, usegmt=True) # RFC 822
-        if not browserDate or (smodified > browserDate):
+        if not browserDate or smodified > browserDate:
             resp = handler.resp
             resp.code = HTTPStatus.OK
             # Identify MIME type
@@ -200,7 +247,7 @@ class Static:
             # ~~~ Manage caching ~~~
             if enableCache:
                 set('Last-Modified', smodified)
-                # Ensure there is no the cache-related header
+                # Ensure there is no cache-related header
                 resp.removeHeader('Cache-Control')
                 resp.removeHeader('Expires')
             else:
