@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -24,6 +24,7 @@ from appy.ui.js import Quote
 from appy.model.base import Base
 from appy.model.fields import Field
 from appy.server.request import Request
+from appy.xml.marshaller import Marshaller
 from appy.model.workflow.transition import Transition
 
 # Errors - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -77,6 +78,7 @@ class Traversal:
         # Perform standard initialisation
         self.handler = handler
         self.req = handler.req
+        self.tool = handler.tool
         # The path that will be traversed, splitted into parts
         self.parts = handler.parts
         # The currently logged user
@@ -133,7 +135,7 @@ class Traversal:
             if not r: raise NotFound(self.getPath())
         else:
             # If "part" is empty, (path is "/"), use the tool as default object
-            r = self.handler.tool
+            r = self.tool
         # Check if the user is allowed to consult this object
         self.handler.guard.allows(r, raiseError=True)
         return r
@@ -202,7 +204,7 @@ class Traversal:
         # field, a module...
         traverse = getattr(previous, 'traverse', None)
         # Is p_name mentioned in the traversal info ?
-        if traverse and (name in traverse):
+        if traverse and name in traverse:
             # Yes: retrieve "traverse info" from it
             r = traverse[name]
         else:
@@ -230,7 +232,7 @@ class Traversal:
         # element is protected by an instance-related info (=a permission),
         # convert this info to True: indeed, in that case, permission can't be
         # checked (we have no instance).
-        if (self.mode == Traversal.STATIC) and isinstance(previous, Field) and \
+        if self.mode == Traversal.STATIC and isinstance(previous, Field) and \
            isinstance(r, str) and r.startswith('perm:'):
             r = True
         return r
@@ -242,7 +244,7 @@ class Traversal:
             # We have a bound method. If the last walked object is not the bound
             # object, give the last walked object as unique arg.
             o = self.o
-            if o and (method.__self__ != o):
+            if o and method.__self__ != o:
                 r = method(o)
             else:
                 r = method()
@@ -334,6 +336,10 @@ class Traversal:
         if isinstance(r, Base):
             self.o = r
             self.field = self.fieldName = self.transition = None
+        elif isinstance(r, Field):
+            # A field may be directly specified from a class, not from an
+            # instance.
+            self.field = r
         return r
 
     def run(self):
@@ -360,9 +366,17 @@ class Traversal:
                     # Switch to "static" mode
                     self.mode = Traversal.STATIC
                 elif self.r is None:
-                    # If we are at the start of the traversal, and "r" is still
-                    # None, "part" must correspond to an object ID.
-                    self.r = self.o = self.getObject(part)
+                    # We are at the start of a traversal. p_self.r is still
+                    # None: v_part must either be a traverser, or correspond to
+                    # an object ID.
+                    expr = self.handler.config.server.traversers.get(part)
+                    if expr:
+                        tool = self.tool
+                        self.o = tool
+                        self.r = eval(expr)
+                    else:
+                        # v_part must be an object ID
+                        self.r = self.o = self.getObject(part)
                 # Manage any not-empty part
                 elif part:
                     # Try to get the attribute or method named "part" on self.r
@@ -378,6 +392,10 @@ class Traversal:
         # a 404 error in that case.
         r = self.r
         if r is not None and not isinstance(r, str):
-            raise NotFound(self.getPath())
+            # Can v_r be marshalled ? Currently, only XML marshalling is there
+            if self.handler.resp.contentType == 'xml':
+                r = Marshaller(rootTag=self.parts[-1]).marshall(r)
+            else:
+                raise NotFound(self.getPath())
         return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
