@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,23 +16,24 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import os
-from appy.utils import Version
+from appy import version
+from appy.utils import executeCommand
+from appy.model.fields.pod import Config as PodConfig
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Monitoring:
     '''Implements stuff allowing to perform monitoring on a Appy application.
-       * URL <yourapp>/config/check can be called to get monitoring info;
+       * URL <yourapp>/tool/check can be called to get monitoring info;
        * Configure monitoring parameters by updating attributes of the
          Monitoring instance defined in the Config class.'''
 
-    def __init__(self, forceComplete=False):
+    def __init__(self, forceComplete=False, checkLo=False):
         # When returning a success status code, what code to return ?
         self.ok = 'OK'
         # When returning a failure status code, what code to return ?
         self.ko = 'KO'
         # Do we check the presence of LibreOffice running in server mode ?
-        self.checkLo = True
+        self.checkLo = checkLo
         # Normally, producing a complete or summary status depends on a request
         # parameter. If p_forceComplete is True, however, we always return the
         # complete status.
@@ -50,30 +51,31 @@ class Monitoring:
         rows = ['<tr><th>%s</th><td>%s</td>' % (k, v) for k, v in r]
         return '<table class="grid">%s</table>' % '\n'.join(rows)
 
-    def get(self, request, config, html=False):
+    def get(self, tool, html=False):
         '''Returns monitoring-related info'''
         # The global monitoring status
         success = True
         # Check if LibreOffice is running
         if self.checkLo:
             loLine = ''
-            null, out = os.popen4('ps -ef | grep "soffice"')
-            for line in out.readlines():
-                if "accept=socket" in line:
-                    loLine = line
+            out, err = executeCommand('ps -ef | grep "soffice"', shell=True)
+            for line in out.split(b'\n'):
+                if b"accept=socket" in line:
+                    loLine = line.decode()
                     break
             if not loLine:
                 success = False
         # Do we need to return complete information or only a status code ?
-        status = success and self.ok or self.ko
-        if ('all' not in request) and not self.forceComplete: return status
+        status = self.ok if success else self.ko
+        if 'all' not in tool.req and not self.forceComplete: return status
         # Return complete information
-        r = [('Status', status), ('Appy version', appy.getVersion())]
+        cfg = tool.config.pod or PodConfig()
+        r = [('Status', status), ('Appy version', version.verbose)]
         if self.checkLo:
             # Appy parameters for connecting to LibreOffice in server mode
             r.append( ('UNO-enabled Python',
-                       config.unoEnabledPython or '<not specified>'))
-            r.append(('LibreOffice port', str(config.libreOfficePort)))
+                       cfg.unoEnabledPython or '<the one running this site>'))
+            r.append(('LibreOffice port', str(cfg.libreOfficePort)))
             # Info about the running LibreOffice server
             if not loLine:
                 r.append(('LibreOffice status', 'Not found'))
@@ -82,5 +84,6 @@ class Monitoring:
         # Add app-specific info when available
         if self.app: r += self.app
         # Return pure text of HTML when requested
-        return html and self.asHtml(r) or self.asText(r)
+        html = tool.req.html == '1' or html
+        return self.asHtml(r) if html else self.asText(r)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
