@@ -1,7 +1,7 @@
 '''Create or update translation "po" files for an app'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -56,7 +56,7 @@ class Migrator:
         for workflow in self.model.workflows.values():
             # This prefix was used, in Appy 0.9, for all i18n messages related
             # to this workflow.
-            moduleName = workflow.python.__module__.split('.')[0]
+            moduleName = workflow.python.__module__.split('.')[-1]
             oldPrefix = '%s_%s_%s_' % (self.appName, moduleName, workflow.name)
             r[oldPrefix.lower()] = '%s_' % workflow.name
         self.workflowNames = r
@@ -237,7 +237,9 @@ class Updater:
 
     def visitList(self, class_, field, label):
         '''Generate List-specific labels'''
-        # Generate labels for sub-fields
+        # Generate labels for sub-fields, excepted if they are dynamically
+        # computed.
+        if callable(field.fields): return
         walkedGroups = set()
         for name, sub in field.fields:
             if not self.mustGenerateLabels(sub): continue
@@ -309,8 +311,8 @@ class Updater:
         # only for not-over classes.
         if class_.type == 'app':
             # Class name, singular and plural
-            self.addLabel(name, name, nice=False)
-            self.addLabel('%s_plural' % name, name + 's', nice=False)
+            self.addLabel(name, name)
+            self.addLabel('%s_plural' % name, name + 's')
         # While walking fields and searches, groups will be encountered. In
         # order to avoid generating labels several times for the same group,
         # remember the already walked groups.
@@ -344,7 +346,7 @@ class Updater:
                 label = '%s_%s' % (name, element.name)
                 add(label, element.name)
                 # Add a label for a transition that must be confirmed
-                if (attribute == 'transitions') and element.confirm:
+                if attribute == 'transitions' and element.confirm:
                     add('%s_confirm' % label, self.CONFIRM, nice=False)
 
     def collectLabels(self):
