@@ -1,7 +1,7 @@
 '''User-interface module'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -36,6 +36,7 @@ from appy.ui.navigate import Siblings
 from appy.ui.includer import Includer
 from appy.ui.language import Language
 from appy.ui.validate import Validator
+from appy.ui.svg import Config as SvgConfig
 
 # Some elements in this module will be traversable - - - - - - - - - - - - - - -
 traverse = {'Language': True}
@@ -47,8 +48,9 @@ class Config:
     # Defaults fonts used in the web user interface
     defaultFonts = "Rajdhani, sans-serif"
 
-    # Regular expression for variables defined in CSS file templates
-    cssVariable = re.compile('\|(\w+?)\|', re.S)
+    # Regular expression for variables possibly defined in some (almost static)
+    # files like CSS and SVG.
+    variable = re.compile('\|(\w+?)\|', re.S)
 
     def __init__(self):
         '''Constructor for the UI configuration'''
@@ -57,10 +59,12 @@ class Config:
         # if you want to use an alternate image provided by your app or ext:
         # - create, in your app's or ext's "static" folder, an image having the
         #   same name and extension as the Appy file you want to override,
-        #   located in appy/ui/static;
-        # - add, in dict "images" hereafter, an entry whose key is the name of
-        #   the image and whose value is the name of the app or ext whose
-        #   "static" folder stores your variant.
+        #   located in appy/ui/static ;
+        # - if the file is a SVG file, you're already done! Indeed, SVG files
+        #   are managed in a special way, loaded in RAM ;
+        # - else, add, in dict "images" hereafter, an entry whose key is the
+        #   name of the image and whose value is the name of the app or ext
+        #   whose "static" folder stores your variant.
         
         # For example, suppose your app is named "MyApp" and you want to provide
         # your own home background picture. In folder MyApp/static, create your
@@ -76,7 +80,9 @@ class Config:
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # homeBG.jpg      | The home page's background image
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-        # baseBG.jpg      | The background image for every other page
+        # baseBG.jpg      | The background image for any other non-popup page
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+        # popupBG.jpg     | The background image for any page in the popup
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # homeLogo.png    | The logo shown on the home page for anonymous users,
         #                 | in the top left corner, on top of the home
@@ -101,28 +107,44 @@ class Config:
         #   names of the files from appy/ui/static.
         self.images = {}
 
+        # Appy comes with a set of standard SVG icons, stored in appy/ui/static.
+        # If you develop an app with an elaborate graphic design, you may end up
+        # with proposing an alternative for every SVG icon. That's fine, but for
+        # some projects, you may find it interesting to have a cheaper yet
+        # satisfying approach, consisting in using the base set of SVG icons,
+        # whose colors have been changed according to your needs. Appy allows to
+        # do that, via the SvgConfig instance as defined hereafter. Basically,
+        # within most Appy SVG icons, 3 different colors may be used; they are
+        # defined in a unique place, in this SvgConfig instance (see details in
+        # appy/ui/svg.py). At server startup, all SVG files will be pre-loaded
+        # in RAM and will be updated with these colors.
+        self.svg = SvgConfig()
+
         # Name of the home page. Here are the possibilites.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "home"  | (the default one) The default home page displays the login
         #         | box on top of a window-wide, single background image.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # "homes" | This page displays the login box on top of a space divided
         #         | in as much sections as there are defined background images
         #         | in attribute "homesBackgrounds", each one rendered besides
         #         | each other.
-        # ----------------------------------------------------------------------
+        #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.home = 'home'
 
         # The background images to use when the home page is "homes". Every
         # entry looks like:
         #
-        #            "<file_name>": ("<app_name>", "<image_width>")
+        #                "<file_name>": "<image_width>"
         #
         # Here is an example, for a triptych:
         #
-        #               {"bg1.jpg": ("MyApp", "30%"),
-        #                "bg2.jpg": ("MyApp", "40%"),
-        #                "bg3.jpg": ("MyApp", "30%")}
+        #                     {"bg1.jpg": "30%",
+        #                      "bg2.jpg": "40%",
+        #                      "bg3.jpg": "30%"}
+        #
+        # As usual, if the background files originate from an alternate place
+        # (ie, an ext), also add them in dict images as defined hereabove.
         self.homesBackgrounds = {}
 
         # Attribute "headerBackground" determines properties for the header's
@@ -136,12 +158,27 @@ class Config:
         # pages), it is possible to define another name for the background
         # image, rather than the default "headerBG.png". Attribute
         # "headerBackground" may hold a function, accepting the current root PX
-        # and its context as args, and must return a tuple or list as descrived
+        # and its context as args, and must return a tuple or list as described
         # hereabove. If name of the background image is None or the empty
         # string, no background will be rendered. Specifying None as global
         # value for attribute "headerBackground" is invalid.
         self.headerBackground = ['headerBG.png', 'no-repeat', 'center']
 
+        # Attributes "[home|base|popup]background determine properties for the
+        # corresponding backgrounds whose standard files are described
+        # hereabove, as tuples (or lists) of the form
+        # 
+        #                      (name, repetition, size)
+        #
+        # Each such attribute may also hold a function, accepting the current
+        # root PX and its context as args, and returning a tuple or list as
+        # described hereabove. If name of the background image is None or the
+        # empty string, no background will be rendered. Specifying None as
+        # global value for any attribute is invalid.
+        self.homeBackground  = ['homeBG.jpg' , 'no-repeat', 'cover']
+        self.baseBackground  = ['baseBG.jpg' , 'no-repeat', 'cover']
+        self.popupBackground = ['popupBG.jpg', 'no-repeat', 'cover']
+
         # The following attribute determines how the block of controls is
         # aligned within the header. If the margin is defined to be "left", the
         # block will be aligned to the right, and vice versa. The attribute may
@@ -179,18 +216,22 @@ class Config:
         # args, the current root PX and its context, and must return one of the
         # above-mentioned values.
         self.headerShow = lambda px, ctx: 'top' if not px.isHome(ctx) else None
+
         # Attribute "footerShow" determines when the page footer must be shown.
         # It can be a boolean value or a function. If a function is placed, it
         # will be called with, as args, the current root PX and its context, and
         # must return a boolean value.
         self.footerShow = False
+
         # Fonts in use
         self.fonts = Config.defaultFonts
         self.fontSize = '100%'
+
         # Among the fonts listed above, specify here, as a tuple, those being
         # Google fonts. That way, the corresponding "CSS include" will be
         # injected into all the pages from your app.
         self.googleFonts = ('Rajdhani',)
+
         # If you want to add specific CSS classes to some standard Appy parts,
         # specify a function in the following attribute. The function will
         # receive, as args, the name of the concerned part, the current root PX
@@ -202,15 +243,23 @@ class Config:
         #   main      | The main zone of any page
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         self.css = None
+
         # Color used when a bright color is required
         self.brightColor = 'white'
         # Color used when a dark background is required
         self.darkColor = '#002039'
+        # Alternate, lighter color
+        self.lightColor = '#f0f0f0'
         # Alternate text color, also used for button borders and other discreet
         # places.
         self.altColor = '#009aa4'
         # Variant being lighter
         self.altColorLight = '#e9f2f3'
+        # Color for some selected text
+        self.selectColor = self.brightColor
+        # Row background colors for most list-based data
+        self.evenColor = 'rgba(243,243,247,0.7)'
+        self.oddColor = 'rgba(255,255,255,0.5)'
         # In the popup, everything is smaller. If you want to achieve the same
         # result in the main window, set "compact" to True.
         self.compact = False
@@ -219,27 +268,45 @@ class Config:
         self.wrongTextColor = '#009ba4'
         # Text color representing a problem or warning (some flavour of red)
         self.warnColor = '#e15151'
-        # Text color for home text
-        self.homeTextColor = self.brightColor
         # Text color in the header
         self.headerColor = self.brightColor
         # Background color for the header
         self.headerBgColor = self.darkColor
+        self.headerSpanColor = self.brightColor # Color for standard text
+                                                # rendered within in the header
+        self.headerSpanWeight = 'bold'
+
         # Text color for links
         self.linkColor = self.darkColor
-        self.visitedColor = self.darkColor
+        self.visitedColor = self.linkColor
+        self.messageLinkColor = self.linkColor
         # Border bottom for input fields
         self.itextBottom = '1px solid'
         self.itextBottomColor = '#afb8d4'
         # Color used as "fill" color, ie, for the message box
         self.fillColor = '#dab823'
+        # Background color for the inline Python interpreter
+        self.pythonBgColor = self.darkColor
+
+        # Styling the home page
+        self.homeShowLogo = True # Logo on the top-left corner on the home page
+        self.homeTextColor = self.brightColor # Text color
+        self.homeTextLeft = '45px' # Distance from the left of the screen
+        self.homeTextTop = '25px'  # Distance from the top of the screen
+
         # Styling the login box
         self.loginTitleWeight = 'normal'
-        self.loginBgColor = '#f5f3f2' 
+        self.loginBgColor = '#f5f3f2'
+        self.loginBoxPadding = '35px'
         self.loginColor = self.darkColor
-        self.loginColorPh = '#a9b1b7'
+        self.loginColorPh = '#a9b1b7' # Color for the input *p*lace*h*older
+        self.loginWeightPh = 'bold' # Font-weight for the placeholder
+        self.loginTransformPh = 'uppercase' # Text transform for the
+                                            # placeholder. Other possible values
+                                            # include "none" or "capitalize".
         self.loginWidth = '240px'
         self.loginPadding = '12px'
+        self.loginRadius = '0'
         self.loginMargin = '8px 0'
         self.loginBorder = '0px solid' # Background excepted
         self.loginShadow = '0 4px 8px 0 rgba(160, 104, 132, 0.2), ' \
@@ -247,6 +314,7 @@ class Config:
         self.loginBorderRadius = '0'
         self.loginAlign = 'center'
         self.loginTop = '50%' # Position of the login box on the y axis
+
         # Styling the "connect" button
         self.submitTop = '30px' # Vertical space between login/password inputs
                                 # and the "connect" button
@@ -262,26 +330,47 @@ class Config:
         self.submitWeight = 'normal'
         self.submitPadding = '12px'
         self.boxBgColor = self.brightColor
+
+        # Popups
+        self.popupColor = self.brightColor
+
         # Portlet
+        self.portletShowLogo = True # Logo on top of the portlet
         self.portletWidth = '250px'
         self.portletMinWidth = '140px'
         self.portletBgColor = 'transparent'
         self.portletTextColor = self.darkColor
+        self.portletShadow = '2px 2px 5px #002039' # Set "unset" to remove  it
+        self.portletSep = '1px solid #002039' # Set "none" to remove it
+        self.pgMargin = '0.5em 2px 0.3em' # *p*ortlet *group margins
         self.asMargin = '3px 0 10px 30px' # For link "*a*dvanced *s*earch"
+
         # Live search (ls)
-        self.lsBgColor = 'transparent' # Bg color for the search field
+        self.lsBgColor = 'transparent' # Bg color for zone "search field + icon"
+        self.lsInputBgColor = 'transparent' # Bg color for the search field only
+        self.lsInputWidth = '7.5em'
+        self.lsInputHeight = '22px'
         self.lsrBgColor = self.brightColor # Bg color for search *r*esults
         self.lsPadding = '0'
         self.lsBottom = '1px solid' # Border bottom
         self.lsBottomColor = '#afb8d4'
+
+        # Other elements
+        self.gridFiltersMargin = '20px 0 0 0' # Filter in grid search results
+        self.weekEndBg = '#797979' # Background for calendar week-end cells
+
         # Header
         self.headerHeight = '60px'
+        self.burgerMargin = '0 5px'
+        self.topIconsMargin = '0 10px 0 0px'
+
         # Footer
         self.footerHeight = '20px'
         self.footerBgColor = '#ececec'
         self.footerBgTop = '1px solid #c4c4c4'
         self.footerAlign = 'right'
         self.footerFontSz = '65%'
+
         # Tabs
         self.tabSep = '1px' # Space between tabs and content
         self.tabBorder = '1px solid %s' % self.linkColor # Top, l, r borders
@@ -289,6 +378,7 @@ class Config:
         self.tabBorderRadius = '5px 5px 0 0'
         self.tabsBorderBottom = 'unset' # Define a border bottom for tabs only
                                         # if you define a tabSep > 0.
+
         # Basically, you may produce 2 families of tabs:
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # (1) | (the default) Tabs and content are glued: tabSep is 1px, a
@@ -304,18 +394,28 @@ class Config:
         # Styling phases (containing a picto per page)
         self.phaseBgColor = self.brightColor # When unselected
         self.phaseBgcColor = self.darkColor # When selected (*c*urrent)
-        self.phaseBorderColor = self.linkColor
-        self.phaseColor = self.linkColor # When unselected
-        self.phaseCcolor = self.brightColor # When selected (*c*urrent)
-        self.pageMargin = '0' # Margins between pages
-        # Pages' attributes, with alternate values when the UI is compact
-        # (see attribute "compact "hereabove).
+        self.phaseFilter = 'invert(66%)' # The CSS filter to apply when selected
+        self.phaseBorderColor = self.linkColor # Set "transparent" to hide it
+        self.phaseColor = self.linkColor # Font color unselected
+        self.phaseCcolor = self.brightColor # Font color, selected (*c*urrent)
+        self.pageGap = '0 0' # Gap between pages (vertical & horizontal).
+                             # Example: "0.6em ​1em"
+
+        # Pages' attributes, with alternate values when the UI is compact (see
+        # attribute "compact "hereabove).
         self.pageHeight = '115px'
         self.pageCHeight = '80px'
         self.pageWidth = '125px'
         self.pageCWidth = '110px'
         self.pagePadding = '0'
         self.pageCPadding = '0'
+        self.pageRadius = 'unset' # Border radius
+        self.pageCRadius = 'unset' # Example: "0 0 5px 5px"
+        self.pageTextTransform = 'uppercase'  # "none", "lowercase",
+        self.pageCTextTransform = 'uppercase' # "capitalize" ...
+        self.pageFontSize = '90%'
+        self.pageCFontSize = '9pt'
+
         #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         # If attribute          | The name of the page 
         # "pageDisplay" is ...  | will be rendered ...
@@ -331,16 +431,27 @@ class Config:
         self.pictoCSize = '35px'
         self.pictoMargin = '0 0 6px 0'
         self.pictoCMargin = '0 0 4px 0'
-        # Background color for buttons
+
+        # Buttons. Most buttons appear on top of dark backgrounds. Buttons
+        # appearing in standard bright backgrounds use p_self.darkColor and not
+        # this p_self.buttonColor.
+        self.buttonColor = self.brightColor
+        self.buttonCase = 'uppercase'
         self.buttonBgColor = 'transparent' # Standard buttons outside phases
         self.sbuttonBgColor = 'transparent' # *S*mall buttons
+
+        # Bottom space for every field
+        self.fieldBottom = '0.7em'
+
         # Within a page, by default, the "edit" icon has an "absolute" position.
         # If you want to use standard positioning, set value "inherit".
         self.epictoPosition = 'absolute'
         self.epictoPadding = '0'
+
         # Application-wide default formats for hours and dates
         self.dateFormat = '%d/%m/%Y'
         self.hourFormat = '%H:%M'
+
         # Fadeout duration for the message
         self.messageFadeout = '6s'
         # Application-wide maximum results on a single page of query results
@@ -385,7 +496,7 @@ class Config:
         # config.js, contents.css and styles.js stored in
         # appy/ui/static/ckeditor will be compatible with the version you want
         # to use.
-        self.ckVersion = '4.14.0'
+        self.ckVersion = '4.17.1'
         # ckDistribution can be "basic", "standard", "standard-all", "full" or
         # "full-all" (see doc in http://cdn.ckeditor.com).
         # CK toolbars are not configurable yet. So toolbar "Appy", defined in
@@ -498,9 +609,9 @@ class Config:
         if stop: return ''
         if type in ('home', 'base', 'popup'):
             # The background image for the home page or any other page
-            image = '%sBG.jpg' % type
-            attrs = 'background-size: cover'
-            repeat = 'no-repeat'
+            image, repeat, size = self.cget('%sBackground' % type, ctx)
+            if not image: return
+            attrs = 'background-size:%s' % size
         elif type == 'header':
             # The background for the page header
             image, repeat, position = self.cget('headerBackground', ctx)
@@ -563,14 +674,6 @@ class Config:
         families = '|'.join(self.googleFonts)
         return 'https://fonts.googleapis.com/css?family=%s' % families
 
-    def patchCss(self, cssContent):
-        '''p_cssContent is the content of some CSS file. Replace, in it,
-           variables with their values as defined on p_self.'''
-        # This function will, for every match (=every variable use found in
-        # p_cssContent), return the corresponding value on p_self.
-        fun = lambda match: getattr(self, match.group(1))
-        return Config.cssVariable.sub(fun, cssContent)
-
     def showTool(self, tool):
         '''Show the tool icon to anyone having write access to the tool,
            excepted if a specific function is defined.'''
@@ -583,6 +686,12 @@ class Config:
             r = tool.allows('write')
         return r
 
+    def patchCss(self, css):
+        '''Replaces variables possibly defined in this p_css code with values
+           from p_self.'''
+        fun = lambda match: getattr(self, match.group(1))
+        return self.variable.sub(fun, css)
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class LinkTarget:
     '''Represents information about the target of an HTML "a" tag'''
@@ -593,7 +702,8 @@ class LinkTarget:
            (depends on p_popup, if not None, or attribute p_class_.popup else),
            and if p_back is specified, when coming back from the popup, we will
            ajax-refresh a DOM node whose ID is specified in p_back.'''
-        # The link leads to a instance of some Python p_class_
+        # The link leads to a instance of some Python p_class_ (the true Python
+        # class, not the metaclass)
         self.class_ = class_
         # Does the link lead to a popup ?
         if popup or forcePopup:
@@ -603,7 +713,7 @@ class LinkTarget:
         else:
             toPopup = class_ and hasattr(class_, 'popup')
         # Determine the target of the "a" tag
-        self.target = toPopup and 'appyIFrame' or '_self'
+        self.target = 'appyIFrame' if toPopup else '_self'
         # If the link leads to a popup, a "onClick" attribute must contain the
         # JS code that opens the popup.
         if toPopup:
@@ -636,7 +746,7 @@ class LinkTarget:
 
     def __repr__(self):
         return '<LinkTarget for=%s,target=%s,onClick=%s>' % \
-               (self.class_.__name__, self.target, self.onClick or '-')
+               (self.__class__.__name__, self.target, self.onClick or '-')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Collapsible:
@@ -907,9 +1017,9 @@ def getActionsCss(display):
        the specified p_display.'''
     if display == 'block':
         r = 'objectActions'
+    elif display == 'right':
+        r = 'alignedActions'
     else:
-        r = 'inline'
-        if display == 'right':
-            r += ' alignedActions'
+        r = 'inlineActions'
     return r
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
