@@ -3,7 +3,7 @@
 '''Manages accounts-related data in the user interface'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -132,7 +132,7 @@ class Account:
 
     def addAmount(self, field=None, value=None, expression=None, label=None,
                   mapping=None, positive=True, style=None, unit=None,
-                  sep=False, fieldValue=None, cellTag='td'):
+                  sep=False, fieldValue=None, cellTag='td', context=None):
         '''Adds an entry of type "amount" in the account'''
 
         # ----------------------------------------------------------------------
@@ -149,10 +149,11 @@ class Account:
         # ----------------------------------------------------------------------
         # p_expression | it must be a string containing a Python expression that
         #              | will be evaluated with "o" in its context ("o" being
-        #              | p_self.o). The expression must return the float or int
-        #              | amount; in that case, similarly to the previous case,
-        #              | an i18n p_label must be present, with its optional
-        #              | p_mapping.
+        #              | p_self.o). p_context, if passed, can also be used in
+        #              | the expression. The expression must return the float or
+        #              | int amount ; in that case, similarly to the previous
+        #              | case, an i18n p_label must be present, with its
+        #              | optional p_mapping.
         # ----------------------------------------------------------------------
         # p_positive is False, the amount will be negated. A specific p_style
         # can be applied to the entry "b" (bold) or "i" (italic).
