@@ -1,7 +1,7 @@
 '''This module is about disposing graphical elements in the user interface'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -365,17 +365,18 @@ class Layouts(dict):
     # concept of "layout type". Appy currently defines the following layout
     # types.
     types = (
-      'view',    # a read-only page for an object;
-      'edit',    # the editable page for an object;
-      'result',  # a cell within a list of objects;
+      'view',    # a read-only page for an object ;
+      'edit',    # the editable page for an object ;
+      'result',  # a cell within a list of objects ;
       'query',   # the top-level zone on the page displaying results of some
                  # search, typically used for rendering Pod fields allowing
-                 # to export documents including these results;
-      'buttons', # the series of buttons rendered on a view/edit page or
-                 # besides the object title on a list of objects;
+                 # to export documents including these results ;
+      'sub',     # the series of buttons rendered besides the object title on a
+                 # list of objects ;
+      'buttons', # the series of buttons rendered on a view/edit page ;
       'xml',     # similar to 'view', but in XML format, readable by a peer
                  # system, the 'view' layout being in XHTML format, readable
-                 # by a human being;
+                 # by a human being ;
       'sidebar'  # The zone in the UI situated, for left-to-right languages,
                  # at the right of the screen.
     )
@@ -394,7 +395,7 @@ class Layouts(dict):
     # account, while, for most of them, this is implicit. For example, when
     # determining visibility of a field, boolean "True" implicitly denotes any
     # layout type, excepted those listed in the following attribute.
-    explicitTypes = ('buttons', 'sidebar')
+    explicitTypes = ('sub', 'buttons', 'sidebar')
 
     def __init__(self, edit=None, view=None, search=None):
         '''Initialise layouts based on the specified attributes'''
@@ -541,6 +542,7 @@ Layouts.nd  = Layouts(Layout(d, width=None))
 Layouts.w   = Layouts(Layout(b))
 # Layouts with content only (no label)
 Layouts.f   = Layouts(Layout('f', width=None))
+Layouts.fr  = Layouts(Layout('frv', width=None))
 Layouts.wf  = Layouts(Layout('f'))
 Layouts.fv  = Layouts(edit=Layout(b), view=Layout('f'))
 Layouts.fvd = Layouts(edit=Layout(d), view=Layout('f'))
