@@ -1,7 +1,7 @@
 '''Navigation management'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -21,6 +21,9 @@
 from appy.px import Px
 from appy.model.batch import Batch
 
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+WRONG_NAV  = 'Wrong nav key "%s".'
+
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Sibling:
     '''Represents a sibling element, accessible via navigation from
@@ -29,8 +32,8 @@ class Sibling:
     types = ('previous', 'next', 'first', 'last')
 
     # Names of icons corresponding to sibling types
-    icons = {'previous': 'arrow.svg',  'next': 'arrow.svg',
-             'first':    'arrows.svg', 'last': 'arrows.svg'}
+    icons = {'previous': 'arrow',  'next': 'arrow',
+             'first':    'arrows', 'last': 'arrows'}
 
     # Rotations for icons
     rotate = {'previous': 90, 'next': 270, 'first': 90, 'last': 270}
@@ -73,16 +76,17 @@ class Sibling:
             # We only know the index of the object to go to. The object ID is in
             # the browser's session storage.
             container = self.container
+            searchName = container.uiSearch.name
+            sessionKey = container.search.getSessionKey(searchName)
             url = "buildSiblingUrl('%s','%s',%d)" % \
-                  (container.tool.siteUrl, container.search.getSessionKey(),
-                   self.getSiblingIndex())
+                  (container.tool.siteUrl, sessionKey, self.getSiblingIndex())
         # Build the JS code allowing to navigate to the sibling element
         js = "gotoSibling(%s,'%s','%s','%s')" % \
              (url, self.nav, self.page, self.popup)
         type = self.type
         return '<img src="%s" title="%s" class="clickable iconXS" ' \
                'style="transform: rotate(%ddeg)" onclick="%s"/>' % \
-               (ctx.url(Sibling.icons[type]), ctx._('goto_%s' % type),
+               (ctx.svg(Sibling.icons[type]), ctx._('goto_%s' % type),
                 Sibling.rotate[type], js)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -99,7 +103,7 @@ class Siblings:
     # Icons for going to the current object's siblings
     pxNavigate = Px('''<div>
       <!-- Go to the source URL (search or referred object) -->
-      <div if="not popup">::self.getGotoSource(url, _)</div>
+      <div if="not popup" class="pagePicto">::self.getGotoSource(svg, _)</div>
 
       <!-- Show other navigation icons only when relevant -->
       <div if="self.total" class="navSib">
@@ -120,12 +124,12 @@ class Siblings:
        <x>::self.getIcons(_ctx_, previous=False)</x>
 
        <!-- Go to the element number... -->
-       <div if="self.showGotoNumber()"
+       <div if="self.showGotoNumber()" class="pagePicto"
             var2="field=self.field; sourceUrl=self.sourceObject.url;
                   total=self.total">:self.pxGotoNumber</div>
       </div></div>''',
 
-     css='''.navSib { padding: |navsPadding|; font-size: 90% }''',
+     css='''.navSib { padding:|navsPadding|; font-size:90%; display:inline }''',
 
      js='''
        function gotoCustomSearch() {
@@ -178,8 +182,14 @@ class Siblings:
         '''Analyse the navigation info p_nav and returns the corresponding
           concrete Siblings instance.'''
         elems = nav.split('.')
-        Siblings = (elems[0] == 'ref') and RefSiblings or SearchSiblings
-        return Siblings(tool, ctx, *elems[1:])
+        Siblings = RefSiblings if elems[0] == 'ref' else SearchSiblings
+        try:
+            return Siblings(tool, ctx, *elems[1:])
+        except KeyError:
+            # Param "nav" may be broken. Example: the silly MSN bot replays Appy
+            # URLs, all lowercased.
+            tool.log(WRONG_NAV % nav, type='error')
+            return
 
     def computeStartNumber(self):
         '''Returns the start number of the batch where the current element
@@ -207,7 +217,7 @@ class Siblings:
         # Do I need to navigate to first, previous, next and/or last sibling ?
         self.previousNeeded = False # Previous ?
         self.previousIndex = self.number - 2
-        if (self.previousIndex > -1) and (self.total > self.previousIndex):
+        if self.previousIndex > -1 and self.total > self.previousIndex:
             self.previousNeeded = True
         self.nextNeeded = False     # Next ?
         self.nextIndex = self.number
@@ -261,11 +271,11 @@ class Siblings:
             if sibling: r += sibling.get(ctx)
         return r
 
-    def getGotoSource(self, url, _):
+    def getGotoSource(self, svg, _):
         '''Get the link allowing to return to the source URL'''
         text, message = self.getBackTexts(_)
         return '<a href="%s" title="%s"><img src="%s" class="back"/>%s</a>'% \
-               (self.sourceUrl, message, url('arrowsA.svg'), text)
+               (self.sourceUrl, message, svg('arrowsA'), text)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class RefSiblings(Siblings):
@@ -385,10 +395,10 @@ class SearchSiblings(Siblings):
             if ref: params += '&ref=%s' % ref
             return '%s?%s' % (url, params)
 
-    def getGotoSource(self, url, _):
+    def getGotoSource(self, svg, _):
         '''Get the link or form allowing to return to the source URL'''
         if self.searchName != 'customSearch':
-            return Siblings.getGotoSource(self, url, _)
+            return Siblings.getGotoSource(self, svg, _)
         # For a custom search, post a form with the search criteria retrieved
         # from the browser's session.
         text, message = self.getBackTexts(_)
@@ -401,7 +411,7 @@ class SearchSiblings(Siblings):
           '<a class="clickable" title="%s" onclick="gotoCustomSearch()">' \
           '<img src="%s" class="back"/>%s</a>' % \
           (self.sourceUrl, self.className, self.computeStartNumber(),
-           self.req.ref or '', message, url('arrowsA.svg'), text)
+           self.req.ref or '', message, svg('arrowsA'), text)
 
     def showGotoNumber(self): return
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
