@@ -1,7 +1,7 @@
 '''Portlet management'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -50,8 +50,8 @@ class Portlet:
            rootClasses=handler.server.model.getRootClasses()"
       id="appyPortlet" class="portlet" style=":collapse.style">
 
-      <!-- The portlet logo when present -->
-      <center>
+      <!-- The portlet logo -->
+      <center if="cfg.portletShowLogo">
        <img src=":url('portletLogo')" class="portletLogo"/></center>
 
       <!-- One section for every searchable root class -->
@@ -59,7 +59,8 @@ class Portlet:
          var2="className=class_.name">
 
        <!-- A separator -->
-       <div class="portletSep"></div>
+       <div class="portletSep"
+            if="not loop.class_.first or cfg.portletShowLogo"></div>
 
        <!-- Section title (link triggers the default search) -->
        <div class="portletContent"
@@ -73,9 +74,8 @@ class Portlet:
             href=":'%s?className=%s&amp;search=%s' % \
                    (queryUrl, className, queryParam)"
             onclick="clickOn(this)"
-            class=":(not currentSearch and (currentClass==className) and \
-                    (currentPage == 'pxResults')) and \
-                    'current' or ''">::labelPlural</a>
+            class=":'current' if not currentSearch and currentClass==className
+                      and currentPage == 'pxResults' else ''">::labelPlural</a>
          <x if="not indexable">::labelPlural</x>
         </div>
 
@@ -91,8 +91,8 @@ class Portlet:
          <x var="pre=className; liveCss='lsSearch'">:tool.Search.live</x>
 
          <!-- Advanced search -->
-         <div var="highlighted=(currentClass == className) and \
-                               (req.search == 'customSearch')"
+         <div var="highlighted=currentClass == className and \
+                               req.search == 'customSearch'"
               class=":'portletAdv current' if highlighted else 'portletAdv'">
           <a var="text=_('search_title')"
              href=":'%s/Search/advanced?className=%s' % (toolUrl, className)"
@@ -112,19 +112,19 @@ class Portlet:
      </div>''',
 
      css='''
-       .portlet { padding:30px 50px 0 0; box-shadow:2px 2px 5px #002039;
+       .portlet { padding:30px 50px 0 0; box-shadow:|portletShadow|;
                   position:sticky; top:0; overflow-y:auto; overflow-x:auto;
                   width:|portletWidth|; min-width:|portletMinWidth|;
                   font-size:98%; z-index:4; background-color:|portletBgColor| }
        .portlet a, .portlet a:visited { color: |portletTextColor| }
-       .portletSearch a:hover { background-color:|linkColor|;
-                                color:|brightColor| }
+       .portletSearch a:hover { background-color:|brightColor|;
+                                color:|linkColor| }
        .portletContent { padding-left: 18px; width: 100% }
        .portletTitle { padding: 5px 0; text-transform: uppercase }
        .portletLogo { margin: 0 0 15px 25px }
-       .portletSep { border-top: 1px solid #002039; margin: 25px 0 20px 0 }
+       .portletSep { border-top:|portletSep|; margin:25px 0 20px 0 }
        .portletGroup { text-transform: uppercase; padding: 5px 0 0 0;
-                       margin: 0.5em 2px 0.3em; color: |portletTextColor| }
+                       margin:|pgMargin|; color: |portletTextColor| }
        .portletAdv { margin: |asMargin|; font-size: 90% }
        .portletSearch { text-align: left }
        .portletCurrent { font-weight: bold }
