@@ -4,16 +4,9 @@ var lsTimeout,  // Timout for the live search
 // Builds the URL to a static resource named p_name
 function buildUrl(name) { return siteUrl + '/static/appy/' + name; }
 
-// Compute a chunk of XHTML code containing a "loading" animated icon
-function loading(name, icon) {
- // Get the name of the icon to use: p_icon or a default one, based on p_name
- if (!icon) icon = 'loading_' + name + '.gif';
- var img = `<img src="${buildUrl(icon)}"`, r=null;
- if      (name == 'link')   r = `${img}/>`;
- else if (name == 'button') r = `${img} align="center"/>`;
- else if (name == 'icon')   r = `${img} align="center"/>`;
- else if (name == 'zone')   r = `<div align="center">${img}/></div>`;
- return r
+// Chunk of XHTML code containing a "loading" animated icon
+function loading(icon) {
+ return `<div align="center"><img src="${buildUrl(icon)}"/></div>`;
 }
 
 // Add to form p_f a hidden field named p_name with this p_value
@@ -177,8 +170,8 @@ function showPreloader(hook, waiting) {
      currently shown, as when inline-editing a rich text field). */
   if (!hook || (waiting == 'none')) return;
   // What waiting icon to show ?
-  if (!waiting) waiting = 'loading_big.gif';
-  injectChunk(getNode(hook), loading('zone', waiting), true);
+  if (!waiting) waiting = 'loading.gif';
+  injectChunk(getNode(hook), loading(waiting), true);
 }
 
 function askAjaxChunk(url, mode, params, hook, beforeSend, onGet, waiting,
@@ -204,9 +197,8 @@ function askAjaxChunk(url, mode, params, hook, beforeSend, onGet, waiting,
      HTML node element into which the result has been inserted.
 
      p_waiting is the name of the animated icon that will be shown while waiting
-     for the ajax result. If null, it will be "loading_big.gif". Other values
-     can be "loading_link.gif", "loading_button.gif" or "loading_icon.gif".
-     If string "none" is passed, there will be no icon at all.
+     for the ajax result. If null, it will be "loading.gif". If string "none" is
+     passed, there will be no icon at all.
 
      If p_append is False or null, the hook content will be replaced with the
      Ajax response. Else, it will be appended after the existing hook content.
@@ -281,9 +273,9 @@ function AjaxData(url, mode, params, hook, parentHook, beforeSend, onGet,
   this.append = append;
   // Inject this AjaxData instance into p_hook
   getNode(hook, true)['ajax'] = this;
-  if (params && ('criteria' in params)) {
+  if (params && ('criteria+' in params)) {
     // Remember these search criteria in the browser's session
-    sessionStorage.setItem(params['className'], params['criteria']);
+    sessionStorage.setItem(params['className'], params['criteria+']);
   }
 }
 
@@ -297,7 +289,7 @@ function askAjax(hook, form, params, waiting) {
     if (hook[0] == ':') parentHook = ':' + parentHook;
     var parent = getNode(parentHook)['ajax'];
     for (var key in parent) {
-      if (key == 'params') continue; // Will get a specific treatment herafter
+      if (key == 'params') continue; // Will get a specific treatment hereafter
       if (!d[key]) d[key] = parent[key]; // Override if no value on child
     }
     // Merge parameters
@@ -336,7 +328,7 @@ function askAjax(hook, form, params, waiting) {
     // Get the other params
     var name, value, elems = f.elements;
     for (var i=0; i < elems.length; i++) {
-      // Ignore unnamed  form elements
+      // Ignore unnamed form elements
       name = elems[i].name;
       if (!name) continue;
       // Escape the value when relevant
@@ -431,17 +423,11 @@ function clickOn(node) {
     return;
   }
   // Disable any click on p_node to be protected against double-click
-  var cn = (node.className)? 'unclickable ' + node.className : 'unclickable';
-  node.className = cn;
-  /* For a button, show the preloader directly. For a link, show it only after
-     a while, if the target page is still not there. */
-  if (node.tagName == 'A') {
-    setTimeout(function(){injectChunk(node, loading('link'))}, 700);
-  }
-  else {
-    var name = (cn.search('buttonIcon') == -1)? 'button' : 'icon';
-    injectChunk(node, loading(name));
-  }
+  var cn = node.className,
+      anim = (cn && (cn.search('buttonIcon') != -1))? 'waiting': 'blinkT',
+      acn = 'unclickable ' + anim,
+      ncn = (cn)? acn + ' ' + cn : acn;
+  node.className = ncn;
 }
 
 function gotoTied(objectUrl, field, numberWidget, total, popup) {
@@ -821,7 +807,7 @@ function getFormData() {
   for (var i=0; i < appyForm.elements.length; i++) {
     elem = appyForm.elements[i];
     name = elem.name
-    if (name == 'action') continue;
+    if ((name == 'action') || (name == 'referer')) continue;
     if (name.startsWith('w_')) name = name.substr(2);
     r[name] = elem.value;
   }
@@ -985,22 +971,6 @@ function onDeleteObject(objectUrl, back) {
   else askConfirm('url', objectUrl + '/remove', action_confirm);
 }
 
-function onHistoryEvent(action, objectId, eventTime, commentId) {
-  var f = document.getElementById('eventForm'),
-      showComment = action == 'Edit',
-      comment = null;
-  f.action = 'on' + action + 'Event';
-  f.objectId.value = objectId;
-  f.eventTime.value = eventTime;
-  // Manage comment
-  if (commentId) {
-    comment = html2text(document.getElementById(commentId).innerHTML);
-  }
-  askConfirm('script', "askAjax('appyHistory','" + f.id +
-             "',{'comment':encodeURIComponent(comment)})",
-             action_confirm, showComment, null, null, comment);
-}
-
 function onLink(action, url, fieldName, targetId, hook, start, semantics) {
   var params = {'linkAction': action, 'targetId': targetId};
   if (hook) params[hook + '_start'] = start;
@@ -1142,6 +1112,13 @@ function toggleCookie(cookieId,display,defaultValue,expandIcon,collapseIcon) {
   createCookie(cookieId, newState);
 }
 
+// Set the mask preventing clicking behind the iframe
+function setMask() {
+  var mask = document.getElementById('iframeMask');
+  mask.style.opacity = 0.7;
+  mask.style.zIndex = 99;
+}
+
 // Functions for opening and closing a popup
 function openPopup(popupId, msg, width, height, css, back, commentLabel) {
   // Put the message into the popup
@@ -1166,13 +1143,11 @@ function openPopup(popupId, msg, width, height, css, back, commentLabel) {
   if (height) popup.style.height = height.toFixed() + 'px';
   if (frame) {
     // Set the enclosed iframe dimensions and show the mask
-    var iframe = document.getElementById('appyIFrame'),
-        imask = document.getElementById('iframeMask');
+    var iframe = document.getElementById('appyIFrame');
     iframe.style.width = (width - 20).toFixed() + 'px';
     iframe.style.height = (height - 20).toFixed() + 'px';
     popup['back'] = back;
-    imask.style.opacity = 0.7;
-    imask.style.zIndex = 99;
+    setMask();
   }
   // Apply the CSS class to the popup
   popup.className = (css)? 'popup ' + css: 'popup';
