@@ -164,12 +164,12 @@ function onCheckCbCell(cb, hook, totalRows, totalCols) {
     if (totalRows) {
       params['totalType'] = 'rows';
       params['mode'] = 'POST'; // askAjax removes key 'mode' from params
-      askAjax(hook + '_trs', null, params, 'loading_icon.gif');
+      askAjax(hook + '_trs', null, params);
     }
     if (totalCols) {
       params['totalType'] = 'cols';
       params['mode'] = 'POST'; // askAjax removes key 'mode' from params
-      askAjax(hook + '_tcs', null, params, 'loading_icon.gif');
+      askAjax(hook + '_tcs', null, params);
     }
   }
 }
