@@ -1,7 +1,7 @@
 '''Base template for any UI page'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -49,20 +49,6 @@ class Template:
                 r += text
         return Px.truncateValue(r, width=100)
 
-    @classmethod
-    def getContentCss(class_, ctx):
-        '''Return the CSS class to apply to the zone of the template that will
-           host the effective page content.'''
-        # In most situations, class "content" must be used. In situations where
-        # a more compact design must be applied (ie, in the popup), class
-        # "contentP" must be used instead.
-        if ctx.popup or (not ctx.showPortlet and \
-                         (ctx._px_.name in ('public', 'homes'))):
-            r = 'contentP'
-        else:
-            r = 'content'
-        return r
-
     # PX for which compact CSS must be used
     compactPx = {'payload': ('homes',), 'content': ('public', 'homes')}
 
@@ -155,13 +141,13 @@ class Template:
     # Global links, that can be shown within the template header but also from
     # other pages, like "home[s]", without the base header container.
     pxTemplateLinks = Px('''
-     <!-- Custom links (I) -->
-     <x>:Template.pxLinksBefore</x>
-
      <!-- The burger button for collapsing the portlet -->
      <a if="showPortlet" class="burger"
         onclick="toggleCookie('appyPortlet','block','expanded',\
-              'show','hide')"><img src=":url('burger.svg')" class="icon"/></a>
+              'show','hide')"><img src=":svg('burger')" class="icon"/></a>
+
+     <!-- Custom links (I) -->
+     <x>:Template.pxLinksBefore</x>
 
      <!-- Links and icons -->
      <div class="topIcons" style=":Template.getIconsStyle(_ctx_)">
@@ -176,12 +162,12 @@ class Template:
       <!-- The home icon -->
       <a if="not isAnon and cfg.tget('showHomeIcon', tool)"
          href=":tool.computeHomePage()">
-        <img src=":url('home.svg')" class="icon"/></a>
+        <img src=":svg('home')" class="icon"/></a>
 
       <!-- Connect link if discreet login -->
       <a if="Template.showConnect(_ctx_)" id="loginIcon" name="loginIcon"
          href=":guard.getLoginUrl(_ctx_)" class="clickable">
-       <img if="cfg.showLoginIcon" src=":url('login.svg')" class="icon"/>
+       <img if="cfg.showLoginIcon" src=":svg('login')" class="icon"/>
        <span class="topText">:_('app_connect')</span>
       </a>
 
@@ -194,18 +180,14 @@ class Template:
       <x if="ui.Language.showSelector(cfg,layout)">:ui.Language.pxSelector</x>
 
       <!-- User info and controls for authenticated users -->
-      <x if="not isAnon" var2="logoutText=_('app_logout')">
+      <x if="not isAnon">
        <!-- Config -->
        <a if="cfg.showTool(tool)" href=":'%s/view' % tool.url"
               title=":_('Tool')">
-        <img src=":url('config.svg')" class="icon"/></a>
+        <img src=":svg('config')" class="icon"/></a>
        <x if="cfg.tget('showUserLink', tool)">:user.pxUserLink</x>
        <!-- Authentication context selector -->
        <x var="ctx=config.security.authContext" if="ctx">:ctx.pxLogged</x>
-       <!-- Log out -->
-       <a href=":guard.getLogoutUrl(tool, user)" title=":logoutText">
-        <img src=":url('logout.svg')" class="icon"/>
-        <span if="cfg.logoutText">:logoutText</span></a>
       </x>
 
       <!-- Custom links (III) -->
@@ -215,15 +197,25 @@ class Template:
      <!-- The burger button for collapsing the sidebar -->
      <a if="showSidebar" class="burger"
         onclick="toggleCookie('appySidebar','block','expanded',\
-           'show','hide')"><img src=":url('burger.svg')" class="icon"/></a>
-     ''',
+           'show','hide')"><img src=":svg('burger')" class="icon"/></a>
+
+     <!-- Logout -->
+     <a if="not isAnon" var2="logoutText=_('app_logout')"
+        href=":guard.getLogoutUrl(tool, user)"
+        title=":logoutText">
+      <img src=":svg('logout')" class="icon"/>
+      <span>:logoutText if cfg.logoutText else ''</span>
+     </a>''',
 
      css='''
-      .topIcons > a, .topIcons > select { margin:0 5px; color:|headerColor| }
-      .topIcons span { text-transform: uppercase; font-weight: bold;
-                       padding: 0 15px; vertical-align: -2px }
-      .topText { letter-spacing:1px; padding: 0 8px }
-      .burger { cursor:pointer; margin:0 5px }''')
+      .topIcons > a,
+      .topIcons > select { margin:|topIconsMargin|; color:|headerColor| }
+      .top span { text-transform:uppercase; font-weight:|headerSpanWeight|;
+                  color:|headerSpanColor|; padding:0 5px 0 10px;
+                  vertical-align:-2px }
+      .topText { letter-spacing:1px; padding: 0 1em; font-weight:bold;
+                 text-transform:uppercase }
+      .burger { cursor:pointer; margin:|burgerMargin| }''')
 
     # The global page header
     pxHeader = Px('''
@@ -269,7 +261,7 @@ class Template:
        <x if="config.analytics">::config.analytics.get(tool)</x>
 
        <!-- Popups -->
-       <x>::ui.Globals.getPopups(tool, url, _, dleft, dright, popup)</x>
+       <x>::ui.Globals.getPopups(tool, svg, _, dleft, dright, popup)</x>
 
        <div class=":cfg.getClass('main', _px_, _ctx_)"
             style=":Template.getMainStyle(_ctx_)">
