@@ -1,7 +1,7 @@
 '''Utility modules and functions for Appy'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -22,30 +22,6 @@ import traceback, mimetypes, subprocess
 
 from persistent.list import PersistentList
 
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-class Version:
-    '''Appy version'''
-    short = '1.0.0'
-    verbose = 'Appy %s' % short
-
-    @classmethod
-    def get(class_):
-        '''Returns a string containing the short and verbose Appy version'''
-        if class_.short == 'dev': return 'dev'
-        return '%s (%s)' % (class_.short, class_.verbose)
-
-    @classmethod
-    def isGreaterThanOrEquals(class_, v):
-        '''This method returns True if the current Appy version is greater than
-           or equals p_v. p_v must have a format like "0.5.0".'''
-        if class_.short == 'dev':
-            # We suppose that a developer knows what he is doing
-            return True
-        else:
-            paramVersion = [int(i) for i in v.split('.')]
-            currentVersion = [int(i) for i in class_.short.split('.')]
-            return currentVersion >= paramVersion
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def asDict(seq):
     '''Returns a dict whose keys are elements from p_seq ad values are None'''
@@ -260,30 +236,43 @@ class Traceback:
         return r
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-def executeCommand(cmd, stdin=None):
+def executeCommand(cmd, stdin=None, env=None, shell=False):
     '''Executes command p_cmd and returns a tuple (s_stdout, s_stderr)
        containing the data output by the subprocesss on stdout and stderr. p_cmd
        should be a list of args (the 1st arg being the command in itself, the
        remaining args being the parameters), but it can also be a string, too
        (see subprocess.Popen doc).'''
     proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stdin=stdin,
-                            stderr=subprocess.PIPE)
+                            stderr=subprocess.PIPE, env=env, shell=shell)
     return proc.communicate()
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-CONVERT_ERROR = 'Program "convert" (ImageMagick) must be installed and ' \
-                'in the path for performing this operation (%s).'
+class ImageMagick:
+    '''Allows to perform command-line calls to ImageMagick'''
 
-def convert(path, options):
-    '''Calls the "convert" executable to apply some transform to an image whose
-       p_path is passed.'''
-    # p_options can be a list, or a string containing blank-separated options
-    options = options.split() if isinstance(options, str) else options
-    cmd = ['convert', path] + options
-    cmd.append(path)
-    out, err = executeCommand(cmd)
-    if err: raise Exception(CONVERT_ERROR % str(err))
-    return out
+    # The wrapper around any error message
+    error = 'Error while executing ImageMagick command: %s. %s'
+
+    @classmethod
+    def run(class_, command):
+        '''Executes this ImageMagick p_command, passed a list'''
+        out, err = executeCommand(command)
+        if err: raise Exception(class_.error % (' '.join(command), err))
+        return out
+
+    @classmethod
+    def convert(class_, path, options):
+        '''Calls the "convert" executable to apply some transform to an image
+           whose p_path is passed.'''
+        # p_options can be a list, or a string containing blank-separated
+        # options.
+        options = options.split() if isinstance(options, str) else options
+        # Build the command. p_path could be a Pathlib.Path instance.
+        path = str(path) if not isinstance(path, str) else path
+        cmd = ['convert', path] + options
+        cmd.append(path)
+        # Execute it
+        return class_.run(cmd)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 def formatNumber(n, sep=',', precision=2, tsep=' ', removeTrailingZeros=False):
@@ -373,6 +362,5 @@ def iconParts(icon):
         base, icon = icon.split('/', 1)
     else:
         base = None
-    return icon, base
-
+    return icon, base, icon.endswith('.svg')
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
