@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -21,9 +21,9 @@ from mimetypes import guess_type
 import xml.sax, http.client, urllib, urllib.parse, ssl
 import os, sys, re, stat, time, socket, random, hashlib, gzip
 
+from appy.utils import json
 from appy.utils import copyData
 from appy.model.utils import Object
-from appy.utils.string import Normalize
 from appy.xml.marshaller import Marshaller
 from appy.xml.unmarshaller import Unmarshaller
 
@@ -81,58 +81,6 @@ class SoapDataEncoder:
                                 namespacedTags=self.namespacedTags)
         return marshaller.marshall(soap)
 
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-class JsonDecoder:
-    '''Converts JSON data into Pyton data structures'''
-    # Boolean and None values are different in JSON and Python
-    context = {'true': True, 'false': False, 'null': None}
-    UNREADABLE = 'Unreadable JSON string: %s'
-
-    @classmethod
-    def convertDict(class_, d):
-        '''Returns a appy.Object instance representing dict p_d'''
-        res = Object()
-        for name, value in d.items():
-            # Ensure "name" will be a valid attribute name for a Python object
-            n = Normalize.alphanum(name, keepUnderscore=True)
-            setattr(res, n, class_.convertValue(value))
-        return res
-
-    @classmethod
-    def convertList(class_, l):
-        '''Every item being a dict in p_l is converted to an object'''
-        i = len(l) - 1
-        while i >= 0:
-            l[i] = class_.convertValue(l[i])
-            i -= 1
-
-    @classmethod
-    def convertValue(class_, val):
-        '''Converts a JSON p_val into a Python value'''
-        if isinstance(val, list):
-            class_.convertList(val)
-            res = val
-        elif isinstance(val, dict):
-            res = class_.convertDict(val)
-        else:
-            # In all other cases, no conversion is needed
-            res = val
-        return res
-
-    @classmethod
-    def decode(class_, jsonData):
-        '''Converts JSON data received in a string (p_jsonData) to a Python data
-           structure. JSON dicts are converted to Python objects.'''
-        try:
-            return class_.convertValue(eval(jsonData, class_.context))
-        except SyntaxError:
-            # The presence of char "\r" may pose problem
-            jsonData = jsonData.replace('\r', '')
-            try:
-                return class_.convertValue(eval(jsonData, class_.context))
-            except SyntaxError as err:
-                raise ResourceError(JsonDecoder.UNREADABLE % jsonData)
-
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class DigestRealm:
     '''Represents information delivered by a server requiring Digest-based
@@ -217,8 +165,12 @@ class DigestRealm:
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class HttpResponse:
     '''Stores information about a HTTP response'''
+
+    # Useful empty dict
+    emptyDict = {}
+
     def __init__(self, resource, response, body, duration=None, utf8=True,
-                 responseType=None):
+                 responseType=None, unmarshallParams=None):
         self.resource = resource
         self.code = response.status # The return code, ie 404, 200, 500...
         self.text = response.reason # Textual description of the code
@@ -228,6 +180,9 @@ class HttpResponse:
         # getting this response after having sent the request.
         self.duration = duration
         self.utf8 = utf8
+        # If data must be unmarshalled, customizing the unmarshalling process
+        # can be done by specifying a dict of p_unmarshallParams.
+        self.unmarshallParams = unmarshallParams
         # The following attribute may contain specific data extracted from
         # the previous fields. For example, when response if 302 (Redirect),
         # self.data contains the URI where we must redirect the user to.
@@ -304,14 +259,15 @@ class HttpResponse:
         contentType = self.extractContentType(responseType)
         # Manage JSON content
         if contentType == 'application/json':
-            return JsonDecoder.decode(self.body)
+            return json.Decoder.decode(self.body)
         # Manage XML content
         for xmlHeader in self.xmlHeaders:
             # Ensure this is XML
             if not contentType.startswith(xmlHeader): continue
             # Return an unmarshalled version of the XML content, for easy use
             try:
-                parser = Unmarshaller()
+                params = self.unmarshallParams or HttpResponse.emptyDict
+                parser = Unmarshaller(**params)
                 res = parser.parse(self.body)
                 if parser.rootTag == 'exception':
                     # This is an exception: "res" contains the traceback
@@ -422,7 +378,7 @@ class Resource:
         return r
 
     def send(self, method, uri, body=None, headers=None, bodyType=None,
-             responseType=None):
+             responseType=None, unmarshallParams=None):
         '''Sends a HTTP request with p_method, for p_uri'''
         hc = http.client
         if self.protocol == 'https':
@@ -466,7 +422,8 @@ class Resource:
             duration = endTime - startTime
             self.serverTime += duration
         return HttpResponse(self, response, body, duration=duration,
-                            utf8=self.utf8, responseType=responseType)
+                            utf8=self.utf8, responseType=responseType,
+                            unmarshallParams=unmarshallParams)
 
     def mkdir(self, name):
         '''Creates a folder named p_name in this resource.'''
@@ -497,25 +454,28 @@ class Resource:
         return r
 
     def get(self, uri=None, headers=None, params=None, followRedirect=True,
-            responseType=None):
+            responseType=None, unmarshallParams=None):
         '''Perform a HTTP GET on the server. Parameters can be given as a dict
            in p_params. p_responseType will be used if no "Content-Type" key is
-           found on the HTTP response.'''
+           found on the HTTP response. In the processs of unmarshalling received
+           data, specific parameters can be passed in dict
+           p_unmarshallParams. '''
         if not uri: uri = self.uri
         # Encode and append params if given
         if params:
             sep = ('?' in uri) and '&' or '?'
             uri = '%s%s%s' % (uri, sep, urllib.urlencode(params))
-        r = self.send('GET', uri, headers=headers, responseType=responseType)
+        r = self.send('GET', uri, headers=headers, responseType=responseType,
+                      unmarshallParams=unmarshallParams)
         # Follow redirect when relevant
-        if (r.code == 302) and followRedirect:
+        if r.code == 302 and followRedirect:
             # Addition durations when "measure" is True
             duration = r.duration
             r = self.get(r.data, headers=headers)
             if self.measure: r.duration += duration
             return r
         # Perform Digest-based authentication when relevant
-        if (r.code == 401) and (self.authMethod == 'Digest') and r.data:
+        if r.code == 401 and self.authMethod == 'Digest' and r.data:
             # Re-trigger the request with the correct authentication headers
             headers = headers or {}
             headers['Authorization'] = r.data.buildCredentials(self, uri)
