@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -16,7 +16,8 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-import re, difflib
+import re, sys, difflib
+from appy.xml.escape import Escape
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 innerDiff = re.compile('<span name="(insert|delete)".*? title="(.*?)">' \
@@ -390,7 +391,7 @@ class HtmlDiff:
             # Return the line without the surrounding tag
             action = 'insert'
             outerTag = htmlTag.match(line)
-            line = outerTag.group(3)
+            line = outerTag.group(3) if outerTag else '?'
         else:
             action = None
             outerTag = None
@@ -402,7 +403,7 @@ class HtmlDiff:
             # I found one.
             innerDiffs.append(match)
             line = self.applyDiff(line, match)
-        return (action, line, innerDiffs, outerTag)
+        return action, line, innerDiffs, outerTag
 
     def computeTag(self, regexTag, content):
         '''p_regexTag is a re.MatchObject from regex htmlTag. p_content is a
@@ -682,4 +683,61 @@ class HtmlDiff:
         # the next diff. Overlaps are hard to manage, so we avoid to get them
         # as a starting point when computing cumulative diffs.
         return self.getHtmlDiff(self.old, self.new, '\n')
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+class ColoredDiff:
+    '''Produce a XHTML version of the output of a the Unix diff (or derivates,
+       like git diff).'''
+
+    # Styles to define for representing added and removed lines
+    styles = '<style>.a_d_d {color:green}\n.d_e_l {color:red}</style>'
+
+    def __init__(self, rdiff, splitAbove=100):
+        # The output of the diff command, as a string
+        self.rdiff = rdiff
+        # Visually split a line if above this number of chars
+        self.splitAbove = splitAbove
+
+    def splitLine(self, line, lmax=None):
+        '''Split this p_line into several sub-lines, each one being at most
+           p_lmax, or, if None, p_self.splitAbove chars.'''
+        # Return the p_line unchanged if shorter enough
+        max = lmax or self.splitAbove
+        if len(line) <= max: return line
+        # Split the line, recursively
+        if lmax is None:
+            lmax = self.splitAbove - 1
+        return '%s⤶\n  %s' % (line[:max], self.splitLine(line[max:],lmax=lmax))
+
+    def run(self):
+        '''Compute and returns a XHTML chunk, as a string, containing the
+           colored version of p_self.rdiff.'''
+        r = []
+        esc = Escape.xhtml
+        # m_splitLine will split a line into chunks of max p_self.splitAbove
+        # chars, but is coded as a recursive method. If the line is too long,
+        # maximum recursion depth could be reached. In that case, the line is
+        # truncated.
+        sabove = self.splitAbove
+        maxLine = sys.getrecursionlimit() * (sabove-3)
+        for line in self.rdiff.split('\n'):
+            # Get the line, escaped, potentially split into several lines, if
+            # too long.
+            if len(line) > maxLine:
+                line = '%s...' % line[:sabove]
+            else:
+                line = self.splitLine(line)
+            line = esc(line)
+            # Compute the CSS class to potentially apply to this v_line
+            if line.startswith('+'):
+                css = 'a_d_d'
+            elif line.startswith('-'):
+                css = 'd_e_l'
+            else:
+                css = None
+            # Wrap the line around a "span" tag if it must be styled
+            if css:
+                line = '<span class="%s">%s</span>' % (css, line)
+            r.append(line)
+        return '%s<pre>%s</pre>' % (ColoredDiff.styles, '\n'.join(r))
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
