@@ -1,7 +1,7 @@
 '''Functions for (un)zipping files'''
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -27,19 +27,7 @@ from appy.utils import mimeTypes
 odfInnerFiles = ('content.xml', 'styles.xml', 'meta.xml', 'mimetype')
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-def fixFileName(fileName):
-    '''Fix potential encoding problems in p_fileName and return a fixed version
-       of it.'''
-    # Ensure the name can be converted to utf-8
-    try:
-        fileName.decode('utf-8')
-    except UnicodeDecodeError:
-        # CP 437 is the DOS encoding
-        fileName = fileName.decode('cp437').encode('utf-8')
-    return fileName
-
-#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-def subUnzip(folder, zipName, odf=False, fixNames=False):
+def subUnzip(folder, zipName, odf=False):
     '''Unzip the zip file named p_zipName into this p_folder, by creating,
        within p_folder, a sub-folder with a unique name, into which p_zipName
        content will be unzippped. p_zipName is deleted at the end of the
@@ -63,15 +51,15 @@ def subUnzip(folder, zipName, odf=False, fixNames=False):
     os.mkdir(subFolder)
     # Unzip p_zipName into it
     absZipName = os.path.join(folder, zipName)
-    unzip(absZipName, subFolder, odf=odf, unzipSubZips=True, fixNames=fixNames)
+    unzip(absZipName, subFolder, odf=odf, unzipSubZips=True)
     # Delete p_zipName
     os.remove(absZipName)
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-def unzip(f, folder, odf=False, unzipSubZips=False, fixNames=False):
+def unzip(f, folder, odf=False, unzipSubZips=False):
     '''Unzips file p_f into p_folder. p_f can be any anything accepted by the
        zipfile.ZipFile constructor. p_folder must exist.
-       
+
        If p_odf is True, p_f is considered to be an odt or ods file and this
        function will return a dict containing the content of content.xml,
        styles.xml, meta.xml and metadata from the zipped file.'''
@@ -97,8 +85,6 @@ def unzip(f, folder, odf=False, unzipSubZips=False, fixNames=False):
                     os.makedirs(fullFolderName)
         # Unzip the file in folder
         if fileName:
-            # Fix fileName when requested
-            if fixNames: fileName = fixFileName(fileName)
             fullFileName = os.path.join(fullFolderName, fileName)
             f = open(fullFileName, 'wb')
             fileContent = zipFile.read(zippedFile)
@@ -112,7 +98,7 @@ def unzip(f, folder, odf=False, unzipSubZips=False, fixNames=False):
             # If the unzipped file is itself a zip file, and p_unzipSubZips is
             # True, replace it with a folder containing its unzipped content.
             if unzipSubZips and fileName.endswith('.zip'):
-                subUnzip(fullFolderName, fileName, odf=odf, fixNames=fixNames)
+                subUnzip(fullFolderName, fileName, odf=odf)
     zipFile.close()
     return res
 
