@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -18,6 +18,8 @@
 # Appy. If not, see <http://www.gnu.org/licenses/>.
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+import re
+
 from appy.xml import Parser
 from appy.xml.escape import Escape
 
@@ -37,11 +39,11 @@ class Cleaner(Parser):
 
     # Attrs to add, if not present, to ensure good formatting, be it at the web
     # or ODT levels.
-    attrsToAdd = {'table': {'cellspacing':'0', 'cellpadding':'6', 'border':'1'},
+    attrsToAdd = {'table': {'cellpadding':'6', 'cellspacing':'0', 'border':'1'},
                   'tr':    {'valign': 'top'}}
 
     # Tags that require a line break to be inserted after them
-    lineBreakTags = ('p', 'li', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'td')
+    lineBreakTags = ('p', 'li', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'td', 'th')
 
     # No-end tags
     noEndTags = ('br', 'img')
@@ -162,4 +164,32 @@ class Cleaner(Parser):
         # also ignore its content.
         self.env.ignoreContent = False
         return self.parse('<x>%s</x>' % s)
+
+#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+class StringCleaner:
+    '''Ensure a string does not contain any char that would provoke SAX parser
+       errors. Also propose a method for producing clean strings, for which
+       those chars were removed.'''
+
+    # Chars (some of the ASCII control characters) provoking SAX parse errors in
+    # strings, once being part of a XML data structure that must be parsed with
+    # a SAX parser.
+    numbers = tuple(range(1,20)) + ('0e',)
+    chars = '|'.join(['\\x%s' % str(n).zfill(2) for n in numbers])
+    illegal = re.compile(chars)
+
+    @classmethod
+    def isParsable(class_, s):
+        '''Returns True if p_s is SAX-parsable'''
+        try:
+            Parser().parse('<x>%s</x>' % s)
+            return True
+        except Exception:
+            return False
+
+    @classmethod
+    def clean(class_, s):
+        '''Return p_s whose illegal chars were removed'''
+        if not s: return s
+        return class_.illegal.sub('', s)
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
