@@ -1,5 +1,5 @@
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-# Copyright (C) 2007-2021 Gaetan Delannay
+# Copyright (C) 2007-2022 Gaetan Delannay
 
 # This file is part of Appy.
 
@@ -20,12 +20,13 @@ from appy.xml import Parser
 from appy.utils.string import Normalize
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-EXTRACTOR_WRONG_PARAMS = "You can't have both 'keepCRs' and 'normalize' " \
-  "being True. Normalizing implies CRs to be removed."
+PARAMS_KO = "You can't have both 'keepCRs' and 'normalize' being True. " \
+            "Normalizing implies CRs to be removed."
 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 class Extractor(Parser):
     '''Produces a text version of XML/XHTML content'''
+
     # There are 2 main usages to this parser. It can extract text:
     # - to produce a complete but textual version of some XHTML chunk, keeping
     #   accented chars and carriage returns;
@@ -44,7 +45,7 @@ class Extractor(Parser):
         # to its non-accented counterpart.
         self.normalize = normalize
         # Is is not possible to have both p_keepCRs and p_normalize being True
-        if keepCRs and normalize: raise Exception(EXTRACTOR_WRONG_PARAMS)
+        if keepCRs and normalize: raise Exception(PARAMS_KO)
         # Must be lowerise text ? (only if p_normalize is True)
         self.lower = lower
         # Must we keep dashes ? (only if p_normalize is True)
